@@ -316,7 +316,7 @@ sub mca_process_framework {
             $mca_found->{$pname}->{$framework}->{found} = 1;
             opendir(DIR, $dir) ||
                 my_die "Can't open $dir directory";
-            foreach my $d (readdir(DIR)) {
+            foreach my $d (sort(readdir(DIR))) {
                 # Skip any non-directory, "base", or any dir that
                 # begins with "."
                 next
@@ -628,7 +628,7 @@ sub mpiext_run_global {
     my $dir = "$topdir/$ext_prefix";
     opendir(DIR, $dir) ||
         my_die "Can't open $dir directory";
-    foreach my $d (readdir(DIR)) {
+    foreach my $d (sort(readdir(DIR))) {
         # Skip any non-directory, "base", or any dir that begins with "."
         next
             if (! -d "$dir/$d" || $d eq "base" || substr($d, 0, 1) eq ".");
@@ -715,13 +715,13 @@ sub mpicontrib_run_global {
     my $dir = "$topdir/$contrib_prefix";
     opendir(DIR, $dir) ||
         my_die "Can't open $dir directory";
-    foreach my $d (readdir(DIR)) {
+    foreach my $d (sort(readdir(DIR))) {
         # Skip any non-directory, "base", or any dir that begins with "."
         next
             if (! -d "$dir/$d" || $d eq "base" || substr($d, 0, 1) eq ".");
 
         # If this directory has a configure.m4, then it's an
-        # extension.
+        # contrib.
         if (-f "$dir/$d/configure.m4") {
             verbose "=== Found $d MPI contrib";
 
@@ -735,7 +735,7 @@ sub mpicontrib_run_global {
         }
     }
     closedir(DIR);
-    debug_dump($mpiext_found);
+    debug_dump($mpicontrib_found);
 
     #-----------------------------------------------------------------------
 
@@ -760,7 +760,7 @@ $dnl_line\n\n";
 
     $m4_config_contrib_list =~ s/^, //;
 
-    # List the M4 and no configure exts
+    # List the M4 and no configure contribs
     $m4 .= "dnl List of all MPI contribs
 m4_define([ompi_mpicontrib_list], [$m4_config_contrib_list])\n";
     # List out all the m4_include
@@ -918,10 +918,15 @@ sub patch_autotools_output {
     # enough Libtool that dosn't need this patch.  But don't alarm the
     # user and make them think that autogen failed if this patch fails --
     # make the errors be silent.
+    # Also patch ltmain.sh for NAG compiler
     if (-f "config/ltmain.sh") {
         verbose "$indent_str"."Patching PGI compiler version numbers in ltmain.sh\n";
         system("$patch_prog -N -p0 < $topdir/config/ltmain_pgi_tp.diff >/dev/null 2>&1");
         unlink("config/ltmain.sh.rej");
+
+        verbose "$indent_str"."Patching \"-pthread\" option for NAG compiler in ltmain.sh\n";
+        system("$patch_prog -N -p0 < $topdir/config/ltmain_nag_pthread.diff >/dev/null 2>&1");
+        unlink("config/ltmain.sh.rej");
     }
 
     # If there's no configure script, there's nothing else to do.
@@ -970,6 +975,28 @@ sub patch_autotools_output {
     # Below is essentially an upstream patch for Libtool which we want
     # made available to Open MPI users running older versions of Libtool
 
+    foreach my $tag (("", "_FC")) {
+
+        # We have to change the search pattern and substitution on each
+        # iteration to take into account the tag changing
+        my $search_string = '# icc used to be incompatible with GCC.\n\s+' .
+                            '# ICC 10 doesn\047t accept -KPIC any more.\n.*\n\s+' .
+	                    "lt_prog_compiler_wl${tag}=";
+        my $replace_string = "# Flang compiler
+      *flang)
+	lt_prog_compiler_wl${tag}='-Wl,'
+	lt_prog_compiler_pic${tag}='-fPIC -DPIC'
+	lt_prog_compiler_static${tag}='-static'
+        ;;
+      # icc used to be incompatible with GCC.
+      # ICC 10 doesn't accept -KPIC any more.
+      icc* | ifort*)
+	lt_prog_compiler_wl${tag}=";
+
+        push(@verbose_out, $indent_str . "Patching configure for flang Fortran ($tag)\n");
+        $c =~ s/$search_string/$replace_string/;
+    }
+
     foreach my $tag (("", "_FC")) {
 
         # We have to change the search pattern and substitution on each
@@ -994,6 +1021,25 @@ sub patch_autotools_output {
         $c =~ s/$search_string/$replace_string/;
     }
 
+    foreach my $tag (("", "_FC")) {
+
+        # We have to change the search pattern and substitution on each
+        # iteration to take into account the tag changing
+        my $search_string = 'lf95\052.*# Lahey Fortran 8.1\n\s+' .
+            "whole_archive_flag_spec${tag}=" . '\n\s+' .
+            "tmp_sharedflag='--shared' ;;" . '\n\s+' .
+            'xl';
+        my $replace_string = "lf95*)				# Lahey Fortran 8.1
+	  whole_archive_flag_spec${tag}=
+	  tmp_sharedflag='--shared' ;;
+	nagfor*)			# NAGFOR 5.3
+	  tmp_sharedflag='-Wl,-shared';;
+	xl";
+
+        push(@verbose_out, $indent_str . "Patching configure for NAG compiler ($tag)\n");
+        $c =~ s/$search_string/$replace_string/;
+    }
+
     # Oracle has apparently begun (as of 12.5-beta) removing the "Sun" branding.
     # So this patch (cumulative over the previous one) is required.
     push(@verbose_out, $indent_str . "Patching configure for Oracle Studio Fortran version strings\n");
