@@ -13,8 +13,9 @@
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2013-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2015-2016 Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -40,7 +41,7 @@ static inline int
 __ompi_datatype_pack_description( ompi_datatype_t* datatype,
                                   void** packed_buffer, int* next_index );
 static ompi_datatype_t*
-__ompi_datatype_create_from_args( int32_t* i, OPAL_PTRDIFF_TYPE * a,
+__ompi_datatype_create_from_args( int32_t* i, ptrdiff_t * a,
                                   ompi_datatype_t** d, int32_t type );
 
 typedef struct __dt_args {
@@ -51,7 +52,7 @@ typedef struct __dt_args {
     int32_t            ca;
     int32_t            cd;
     int*               i;
-    OPAL_PTRDIFF_TYPE* a;
+    ptrdiff_t* a;
     ompi_datatype_t**  d;
 } ompi_datatype_args_t;
 
@@ -65,7 +66,7 @@ typedef struct __dt_args {
  */
 #if OPAL_ALIGN_WORD_SIZE_INTEGERS
 #define OMPI_DATATYPE_ALIGN_PTR(PTR, TYPE) \
-    (PTR) = OPAL_ALIGN_PTR((PTR), sizeof(OPAL_PTRDIFF_TYPE), TYPE)
+    (PTR) = OPAL_ALIGN_PTR((PTR), sizeof(ptrdiff_t), TYPE)
 #else
 #define OMPI_DATATYPE_ALIGN_PTR(PTR, TYPE)
 #endif  /* OPAL_ALIGN_WORD_SIZE_INTEGERS */
@@ -80,7 +81,7 @@ typedef struct __dt_args {
 #define ALLOC_ARGS(PDATA, IC, AC, DC)                                   \
     do {                                                                \
         int length = sizeof(ompi_datatype_args_t) + (IC) * sizeof(int) + \
-            (AC) * sizeof(OPAL_PTRDIFF_TYPE) + (DC) * sizeof(MPI_Datatype); \
+            (AC) * sizeof(ptrdiff_t) + (DC) * sizeof(MPI_Datatype); \
         char* buf = (char*)malloc( length );                            \
         ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)buf;       \
         pArgs->ci = (IC);                                               \
@@ -89,8 +90,8 @@ typedef struct __dt_args {
         buf += sizeof(ompi_datatype_args_t);                            \
         if( pArgs->ca == 0 ) pArgs->a = NULL;                           \
         else {                                                          \
-            pArgs->a = (OPAL_PTRDIFF_TYPE*)buf;                         \
-            buf += pArgs->ca * sizeof(OPAL_PTRDIFF_TYPE);               \
+            pArgs->a = (ptrdiff_t*)buf;                         \
+            buf += pArgs->ca * sizeof(ptrdiff_t);               \
         }                                                               \
         if( pArgs->cd == 0 ) pArgs->d = NULL;                           \
         else {                                                          \
@@ -101,7 +102,7 @@ typedef struct __dt_args {
         else pArgs->i = (int*)buf;                                      \
         pArgs->ref_count = 1;                                           \
         pArgs->total_pack_size = (4 + (IC) + (DC)) * sizeof(int) +      \
-            (AC) * sizeof(OPAL_PTRDIFF_TYPE);                           \
+            (AC) * sizeof(ptrdiff_t);                                   \
         (PDATA)->args = (void*)pArgs;                                   \
         (PDATA)->packed_description = NULL;                             \
     } while(0)
@@ -109,7 +110,7 @@ typedef struct __dt_args {
 
 int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
                                 int32_t ci, const int32_t** i,
-                                int32_t ca, const OPAL_PTRDIFF_TYPE* a,
+                                int32_t ca, const ptrdiff_t* a,
                                 int32_t cd, ompi_datatype_t* const * d, int32_t type)
 {
     int pos;
@@ -220,9 +221,9 @@ int32_t ompi_datatype_set_args( ompi_datatype_t* pData,
         break;
     }
 
-    /* copy the array of MPI_Aint, aka OPAL_PTRDIFF_TYPE */
+    /* copy the array of MPI_Aint, aka ptrdiff_t */
     if( pArgs->a != NULL )
-        memcpy( pArgs->a, a, ca * sizeof(OPAL_PTRDIFF_TYPE) );
+        memcpy( pArgs->a, a, ca * sizeof(ptrdiff_t) );
 
     for( pos = 0; pos < cd; pos++ ) {
         pArgs->d[pos] = d[pos];
@@ -317,7 +318,7 @@ int32_t ompi_datatype_print_args( const ompi_datatype_t* pData )
 
 int32_t ompi_datatype_get_args( const ompi_datatype_t* pData, int32_t which,
                                 int32_t* ci, int32_t* i,
-                                int32_t* ca, OPAL_PTRDIFF_TYPE* a,
+                                int32_t* ca, ptrdiff_t* a,
                                 int32_t* cd, ompi_datatype_t** d, int32_t* type)
 {
     ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)pData->args;
@@ -354,7 +355,7 @@ int32_t ompi_datatype_get_args( const ompi_datatype_t* pData, int32_t which,
             memcpy( i, pArgs->i, pArgs->ci * sizeof(int) );
         }
         if( (NULL != a) && (NULL != pArgs->a) ) {
-            memcpy( a, pArgs->a, pArgs->ca * sizeof(OPAL_PTRDIFF_TYPE) );
+            memcpy( a, pArgs->a, pArgs->ca * sizeof(ptrdiff_t) );
         }
         if( (NULL != d) && (NULL != pArgs->d) ) {
             memcpy( d, pArgs->d, pArgs->cd * sizeof(MPI_Datatype) );
@@ -372,12 +373,14 @@ int32_t ompi_datatype_copy_args( const ompi_datatype_t* source_data,
 {
     ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)source_data->args;
 
-    /* If required then increase the reference count of the arguments. This avoid us
-     * to make one more copy for a read only piece of memory.
+    /* Increase the reference count of the datatype enveloppe. This
+     * prevent us from making extra copies for the enveloppe (which is mostly
+     * a read only memory).
      */
-    assert( NULL != source_data->args );
-    pArgs->ref_count++;
-    dest_data->args = pArgs;
+    if( NULL != pArgs ) {
+        OPAL_THREAD_ADD32(&pArgs->ref_count, 1);
+        dest_data->args = pArgs;
+    }
     return OMPI_SUCCESS;
 }
 
@@ -393,7 +396,7 @@ int32_t ompi_datatype_release_args( ompi_datatype_t* pData )
     ompi_datatype_args_t* pArgs = (ompi_datatype_args_t*)pData->args;
 
     assert( 0 < pArgs->ref_count );
-    pArgs->ref_count--;
+    OPAL_THREAD_ADD32(&pArgs->ref_count, -1);
     if( 0 == pArgs->ref_count ) {
         /* There are some duplicated datatypes around that have a pointer to this
          * args. We will release them only when the last datatype will dissapear.
@@ -447,8 +450,8 @@ static inline int __ompi_datatype_pack_description( ompi_datatype_t* datatype,
         /* description of the displacements must be 64 bits aligned */
         OMPI_DATATYPE_ALIGN_PTR(next_packed, char*);
 
-        memcpy( next_packed, args->a, sizeof(OPAL_PTRDIFF_TYPE) * args->ca );
-        next_packed += sizeof(OPAL_PTRDIFF_TYPE) * args->ca;
+        memcpy( next_packed, args->a, sizeof(ptrdiff_t) * args->ca );
+        next_packed += sizeof(ptrdiff_t) * args->ca;
     }
     position = (int*)next_packed;
     next_packed += sizeof(int) * args->cd;
@@ -555,7 +558,7 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
     int* position;
     ompi_datatype_t* datatype = NULL;
     ompi_datatype_t** array_of_datatype;
-    OPAL_PTRDIFF_TYPE* array_of_disp;
+    ptrdiff_t* array_of_disp;
     int* array_of_length;
     int number_of_length, number_of_disp, number_of_datatype, data_id;
     int create_type, i;
@@ -607,13 +610,13 @@ static ompi_datatype_t* __ompi_datatype_create_from_packed_description( void** p
     next_buffer += (4 * sizeof(int));  /* move after the header */
 
     /* description of the displacements (if ANY !)  should always be aligned
-       on MPI_Aint, aka OPAL_PTRDIFF_TYPE */
+       on MPI_Aint, aka ptrdiff_t */
     if (number_of_disp > 0) {
         OMPI_DATATYPE_ALIGN_PTR(next_buffer, char*);
     }
 
-    array_of_disp   = (OPAL_PTRDIFF_TYPE*)next_buffer;
-    next_buffer    += number_of_disp * sizeof(OPAL_PTRDIFF_TYPE);
+    array_of_disp   = (ptrdiff_t*)next_buffer;
+    next_buffer    += number_of_disp * sizeof(ptrdiff_t);
     /* the other datatypes */
     position        = (int*)next_buffer;
     next_buffer    += number_of_datatype * sizeof(int);
@@ -835,16 +838,19 @@ ompi_datatype_t* ompi_datatype_get_single_predefined_type_from_args( ompi_dataty
                 return NULL;
             }
         }
-        if( NULL == predef ) {  /* This is the first iteration */
-            predef = current_predef;
-        } else {
-            /**
-             *  What exactly should we consider as identical types? If they are
-             * the same MPI level type, or if they map to the same OPAL datatype?
-             * In other words, MPI_FLOAT and MPI_REAL4 are they identical?
-             */
-            if( predef != current_predef ) {
-                return NULL;
+        if (current_predef != MPI_LB && current_predef != MPI_UB) {
+            if( NULL == predef ) {  /* This is the first iteration */
+                predef = current_predef;
+            } else {
+                /**
+                 *  What exactly should we consider as identical types?
+                 *  If they are the same MPI level type, or if they map
+                 *  to the same OPAL datatype? In other words, MPI_FLOAT
+                 *  and MPI_REAL4 are they identical?
+                 */
+                if( predef != current_predef ) {
+                    return NULL;
+                }
             }
         }
     }
