@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2015 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -66,24 +66,24 @@ int ompi_coll_base_reduce_scatter_intra_nonoverlapping(const void *sbuf, void *r
     if (MPI_IN_PLACE == sbuf) {
         /* rbuf on root (0) is big enough to hold whole data */
         if (root == rank) {
-            err = comm->c_coll.coll_reduce (MPI_IN_PLACE, tmprbuf, total_count,
-                                            dtype, op, root, comm, comm->c_coll.coll_reduce_module);
+            err = comm->c_coll->coll_reduce (MPI_IN_PLACE, tmprbuf, total_count,
+                                            dtype, op, root, comm, comm->c_coll->coll_reduce_module);
         } else {
-            err = comm->c_coll.coll_reduce(tmprbuf, NULL, total_count,
-                                           dtype, op, root, comm, comm->c_coll.coll_reduce_module);
+            err = comm->c_coll->coll_reduce(tmprbuf, NULL, total_count,
+                                           dtype, op, root, comm, comm->c_coll->coll_reduce_module);
         }
     } else {
         if (root == rank) {
             /* We must allocate temporary receive buffer on root to ensure that
                rbuf is big enough */
-            ptrdiff_t dsize, gap;
+            ptrdiff_t dsize, gap = 0;
             dsize = opal_datatype_span(&dtype->super, total_count, &gap);
 
             tmprbuf_free = (char*) malloc(dsize);
             tmprbuf = tmprbuf_free - gap;
         }
-        err = comm->c_coll.coll_reduce (sbuf, tmprbuf, total_count,
-                                        dtype, op, root, comm, comm->c_coll.coll_reduce_module);
+        err = comm->c_coll->coll_reduce (sbuf, tmprbuf, total_count,
+                                        dtype, op, root, comm, comm->c_coll->coll_reduce_module);
     }
     if (MPI_SUCCESS != err) {
         if (NULL != tmprbuf_free) free(tmprbuf_free);
@@ -96,13 +96,13 @@ int ompi_coll_base_reduce_scatter_intra_nonoverlapping(const void *sbuf, void *r
         displs[i] = displs[i-1] + rcounts[i-1];
     }
     if (MPI_IN_PLACE == sbuf && root == rank) {
-        err =  comm->c_coll.coll_scatterv (tmprbuf, rcounts, displs, dtype,
+        err =  comm->c_coll->coll_scatterv (tmprbuf, rcounts, displs, dtype,
                                            MPI_IN_PLACE, 0, MPI_DATATYPE_NULL,
-                                           root, comm, comm->c_coll.coll_scatterv_module);
+                                           root, comm, comm->c_coll->coll_scatterv_module);
     } else {
-        err =  comm->c_coll.coll_scatterv (tmprbuf, rcounts, displs, dtype,
+        err =  comm->c_coll->coll_scatterv (tmprbuf, rcounts, displs, dtype,
                                            rbuf, rcounts[rank], dtype,
-                                           root, comm, comm->c_coll.coll_scatterv_module);
+                                           root, comm, comm->c_coll->coll_scatterv_module);
     }
     free(displs);
     if (NULL != tmprbuf_free) free(tmprbuf_free);
@@ -138,7 +138,7 @@ ompi_coll_base_reduce_scatter_intra_basic_recursivehalving( const void *sbuf,
 {
     int i, rank, size, count, err = OMPI_SUCCESS;
     int tmp_size, remain = 0, tmp_rank, *disps = NULL;
-    ptrdiff_t extent, buf_size, gap;
+    ptrdiff_t extent, buf_size, gap = 0;
     char *recv_buf = NULL, *recv_buf_free = NULL;
     char *result_buf = NULL, *result_buf_free = NULL;
 
@@ -462,7 +462,7 @@ ompi_coll_base_reduce_scatter_intra_ring( const void *sbuf, void *rbuf, const in
     int inbi, *displs = NULL;
     char *tmpsend = NULL, *tmprecv = NULL, *accumbuf = NULL, *accumbuf_free = NULL;
     char *inbuf_free[2] = {NULL, NULL}, *inbuf[2] = {NULL, NULL};
-    ptrdiff_t extent, max_real_segsize, dsize, gap;
+    ptrdiff_t extent, max_real_segsize, dsize, gap = 0;
     ompi_request_t *reqs[2] = {NULL, NULL};
 
     size = ompi_comm_size(comm);
@@ -613,6 +613,7 @@ ompi_coll_base_reduce_scatter_intra_ring( const void *sbuf, void *rbuf, const in
  error_hndl:
     OPAL_OUTPUT((ompi_coll_base_framework.framework_output, "%s:%4d\tRank %d Error occurred %d\n",
                  __FILE__, line, rank, ret));
+    (void)line;  // silence compiler warning
     if (NULL != displs) free(displs);
     if (NULL != accumbuf_free) free(accumbuf_free);
     if (NULL != inbuf_free[0]) free(inbuf_free[0]);
