@@ -56,6 +56,7 @@ typedef struct {
 } mca_coll_hcoll_dtype_t;
 OBJ_CLASS_DECLARATION(mca_coll_hcoll_dtype_t);
 
+extern mca_coll_hcoll_dtype_t zero_dte_mapping;
 struct mca_coll_hcoll_component_t {
     /** Base coll component */
     mca_coll_base_component_2_0_0_t super;
@@ -180,7 +181,7 @@ int mca_coll_hcoll_bcast(void *buff, int count,
                         struct ompi_communicator_t *comm,
                         mca_coll_base_module_t *module);
 
-int mca_coll_hcoll_allgather(void *sbuf, int scount,
+int mca_coll_hcoll_allgather(const void *sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
                             void *rbuf, int rcount,
                             struct ompi_datatype_t *rdtype,
@@ -203,7 +204,7 @@ int mca_coll_hcoll_gather(const void *sbuf, int scount,
                           struct ompi_communicator_t *comm,
                           mca_coll_base_module_t *module);
 
-int mca_coll_hcoll_allreduce(void *sbuf, void *rbuf, int count,
+int mca_coll_hcoll_allreduce(const void *sbuf, void *rbuf, int count,
                             struct ompi_datatype_t *dtype,
                             struct ompi_op_t *op,
                             struct ompi_communicator_t *comm,
@@ -223,18 +224,18 @@ int mca_coll_hcoll_alltoall(const void *sbuf, int scount,
                            struct ompi_communicator_t *comm,
                            mca_coll_base_module_t *module);
 
-int mca_coll_hcoll_alltoallv(void *sbuf, int *scounts,
-                            int *sdisps,
+int mca_coll_hcoll_alltoallv(const void *sbuf, const int *scounts,
+                            const int *sdisps,
                             struct ompi_datatype_t *sdtype,
-                            void *rbuf, int *rcounts,
-                            int *rdisps,
+                            void *rbuf, const int *rcounts,
+                            const int *rdisps,
                             struct ompi_datatype_t *rdtype,
                             struct ompi_communicator_t *comm,
                             mca_coll_base_module_t *module);
 
-int mca_coll_hcoll_gatherv(void* sbuf, int scount,
+int mca_coll_hcoll_gatherv(const void* sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
-                            void* rbuf, int *rcounts, int *displs,
+                            void* rbuf, const int *rcounts, const int *displs,
                             struct ompi_datatype_t *rdtype,
                             int root,
                             struct ompi_communicator_t *comm,
@@ -250,7 +251,7 @@ int mca_coll_hcoll_ibcast(void *buff, int count,
                             ompi_request_t** request,
                             mca_coll_base_module_t *module);
 
-int mca_coll_hcoll_iallgather(void *sbuf, int scount,
+int mca_coll_hcoll_iallgather(const void *sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
                             void *rbuf, int rcount,
                             struct ompi_datatype_t *rdtype,
@@ -291,7 +292,7 @@ int mca_coll_hcoll_ialltoall(const void *sbuf, int scount,
                             mca_coll_base_module_t *module);
 
 #if HCOLL_API >= HCOLL_VERSION(3,7)
-int mca_coll_hcoll_ialltoallv(void *sbuf, int *scounts,
+int mca_coll_hcoll_ialltoallv(const void *sbuf, int *scounts,
                             int *sdisps,
                             struct ompi_datatype_t *sdtype,
                             void *rbuf, int *rcounts,
@@ -304,7 +305,7 @@ int mca_coll_hcoll_ialltoallv(void *sbuf, int *scounts,
 
 int mca_coll_hcoll_igatherv(const void* sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
-                            void* rbuf, int *rcounts, int *displs,
+                            void* rbuf, const int *rcounts, const int *displs,
                             struct ompi_datatype_t *rdtype,
                             int root,
                             struct ompi_communicator_t *comm,
