@@ -2,6 +2,7 @@
   Copyright (c) 2011      Mellanox Technologies. All rights reserved.
   Copyright (c) 2015      Research Organization for Information Science
                           and Technology (RIST). All rights reserved.
+  Copyright (c) 2018      Cisco Systems, Inc.  All rights reserved
   $COPYRIGHT$
 
   Additional copyrights may follow
@@ -21,7 +22,7 @@ int mca_coll_hcoll_barrier(struct ompi_communicator_t *comm,
     mca_coll_hcoll_module_t *hcoll_module = (mca_coll_hcoll_module_t*)module;
     HCOL_VERBOSE(20,"RUNNING HCOL BARRIER");
 
-    if (OPAL_UNLIKELY(ompi_mpi_finalize_started)) {
+    if (OPAL_UNLIKELY(ompi_mpi_state >= OMPI_MPI_STATE_FINALIZE_STARTED)) {
         HCOL_VERBOSE(5, "In finalize, reverting to previous barrier");
         goto orig_barrier;
     }
@@ -64,7 +65,7 @@ int mca_coll_hcoll_bcast(void *buff, int count,
     return rc;
 }
 
-int mca_coll_hcoll_allgather(void *sbuf, int scount,
+int mca_coll_hcoll_allgather(const void *sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
                             void *rbuf, int rcount,
                             struct ompi_datatype_t *rdtype,
@@ -94,7 +95,7 @@ int mca_coll_hcoll_allgather(void *sbuf, int scount,
                                              hcoll_module->previous_allgather_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_allgather(sbuf,scount,stype,rbuf,rcount,rtype,hcoll_module->hcoll_context);
+    rc = hcoll_collectives.coll_allgather((void *)sbuf,scount,stype,rbuf,rcount,rtype,hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK ALLGATHER");
         rc = hcoll_module->previous_allgather(sbuf,scount,sdtype,
@@ -175,10 +176,10 @@ int mca_coll_hcoll_gather(const void *sbuf, int scount,
                                            hcoll_module->previous_allgather_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_gather(sbuf,scount,stype,rbuf,rcount,rtype,root,hcoll_module->hcoll_context);
+    rc = hcoll_collectives.coll_gather((void *)sbuf,scount,stype,rbuf,rcount,rtype,root,hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK GATHER");
-        rc = hcoll_module->previous_gather(sbuf,scount,sdtype,
+        rc = hcoll_module->previous_gather((void *)sbuf,scount,sdtype,
                                               rbuf,rcount,rdtype,root,
                                               comm,
                                               hcoll_module->previous_allgather_module);
@@ -187,7 +188,7 @@ int mca_coll_hcoll_gather(const void *sbuf, int scount,
 
 }
 
-int mca_coll_hcoll_allreduce(void *sbuf, void *rbuf, int count,
+int mca_coll_hcoll_allreduce(const void *sbuf, void *rbuf, int count,
                             struct ompi_datatype_t *dtype,
                             struct ompi_op_t *op,
                             struct ompi_communicator_t *comm,
@@ -224,7 +225,7 @@ int mca_coll_hcoll_allreduce(void *sbuf, void *rbuf, int count,
         return rc;
     }
 
-    rc = hcoll_collectives.coll_allreduce(sbuf,rbuf,count,Dtype,Op,hcoll_module->hcoll_context);
+    rc = hcoll_collectives.coll_allreduce((void *)sbuf,rbuf,count,Dtype,Op,hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK ALLREDUCE");
         rc = hcoll_module->previous_allreduce(sbuf,rbuf,
@@ -312,7 +313,7 @@ int mca_coll_hcoll_alltoall(const void *sbuf, int scount,
                                             hcoll_module->previous_alltoall_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_alltoall(sbuf,scount,stype,rbuf,rcount,rtype,hcoll_module->hcoll_context);
+    rc = hcoll_collectives.coll_alltoall((void *)sbuf,scount,stype,rbuf,rcount,rtype,hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK ALLTOALL");
         rc = hcoll_module->previous_alltoall(sbuf,scount,sdtype,
@@ -323,9 +324,9 @@ int mca_coll_hcoll_alltoall(const void *sbuf, int scount,
     return rc;
 }
 
-int mca_coll_hcoll_alltoallv(void *sbuf, int *scounts, int *sdisps,
+int mca_coll_hcoll_alltoallv(const void *sbuf, const int *scounts, const int *sdisps,
                             struct ompi_datatype_t *sdtype,
-                            void *rbuf, int *rcounts, int *rdisps,
+                            void *rbuf, const int *rcounts, const int *rdisps,
                             struct ompi_datatype_t *rdtype,
                             struct ompi_communicator_t *comm,
                             mca_coll_base_module_t *module)
@@ -346,8 +347,8 @@ int mca_coll_hcoll_alltoallv(void *sbuf, int *scounts, int *sdisps,
                                             comm, hcoll_module->previous_alltoallv_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_alltoallv(sbuf, scounts, sdisps, stype,
-                                            rbuf, rcounts, rdisps, rtype,
+    rc = hcoll_collectives.coll_alltoallv((void *)sbuf, (int *)scounts, (int *)sdisps, stype,
+                                            rbuf, (int *)rcounts, (int *)rdisps, rtype,
                                                 hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK ALLTOALLV");
@@ -358,9 +359,9 @@ int mca_coll_hcoll_alltoallv(void *sbuf, int *scounts, int *sdisps,
     return rc;
 }
 
-int mca_coll_hcoll_gatherv(void* sbuf, int scount,
+int mca_coll_hcoll_gatherv(const void* sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
-                            void* rbuf, int *rcounts, int *displs,
+                            void* rbuf, const int *rcounts, const int *displs,
                             struct ompi_datatype_t *rdtype,
                             int root,
                             struct ompi_communicator_t *comm,
@@ -385,7 +386,7 @@ int mca_coll_hcoll_gatherv(void* sbuf, int scount,
                                            comm, hcoll_module->previous_gatherv_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_gatherv(sbuf,scount,stype,rbuf,rcounts,displs, rtype, root, hcoll_module->hcoll_context);
+    rc = hcoll_collectives.coll_gatherv((void *)sbuf, scount, stype, rbuf, (int *)rcounts, (int *)displs, rtype, root, hcoll_module->hcoll_context);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK GATHERV");
         rc = hcoll_module->previous_gatherv(sbuf,scount,sdtype,
@@ -444,7 +445,7 @@ int mca_coll_hcoll_ibcast(void *buff, int count,
     return rc;
 }
 
-int mca_coll_hcoll_iallgather(void *sbuf, int scount,
+int mca_coll_hcoll_iallgather(const void *sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
                             void *rbuf, int rcount,
                             struct ompi_datatype_t *rdtype,
@@ -475,7 +476,7 @@ int mca_coll_hcoll_iallgather(void *sbuf, int scount,
                                              hcoll_module->previous_iallgather_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_iallgather(sbuf, scount, stype, rbuf, rcount, rtype, hcoll_module->hcoll_context, rt_handle);
+    rc = hcoll_collectives.coll_iallgather((void *)sbuf, scount, stype, rbuf, rcount, rtype, hcoll_module->hcoll_context, rt_handle);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK NON-BLOCKING ALLGATHER");
         rc = hcoll_module->previous_iallgather(sbuf,scount,sdtype,
@@ -486,7 +487,6 @@ int mca_coll_hcoll_iallgather(void *sbuf, int scount,
     }
     return rc;
 }
-
 #if HCOLL_API >= HCOLL_VERSION(3,5)
 int mca_coll_hcoll_iallgatherv(const void *sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
@@ -576,7 +576,7 @@ int mca_coll_hcoll_iallreduce(const void *sbuf, void *rbuf, int count,
         return rc;
     }
 
-    rc = hcoll_collectives.coll_iallreduce(sbuf, rbuf, count, Dtype, Op, hcoll_module->hcoll_context, rt_handle);
+    rc = hcoll_collectives.coll_iallreduce((void *)sbuf, rbuf, count, Dtype, Op, hcoll_module->hcoll_context, rt_handle);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK NON-BLOCKING ALLREDUCE");
         rc = hcoll_module->previous_iallreduce(sbuf,rbuf,
@@ -644,7 +644,7 @@ int mca_coll_hcoll_ireduce(const void *sbuf, void *rbuf, int count,
 #endif
 int mca_coll_hcoll_igatherv(const void* sbuf, int scount,
                             struct ompi_datatype_t *sdtype,
-                            void* rbuf, int *rcounts, int *displs,
+                            void* rbuf, const int *rcounts, const int *displs,
                             struct ompi_datatype_t *rdtype,
                             int root,
                             struct ompi_communicator_t *comm,
@@ -673,7 +673,7 @@ int mca_coll_hcoll_igatherv(const void* sbuf, int scount,
                                            hcoll_module->previous_igatherv_module);
         return rc;
     }
-    rc = hcoll_collectives.coll_igatherv(sbuf,scount,stype,rbuf,rcounts,displs, rtype, root, hcoll_module->hcoll_context, rt_handle);
+    rc = hcoll_collectives.coll_igatherv((void *)sbuf, scount, stype, rbuf, (int *)rcounts, (int *)displs, rtype, root, hcoll_module->hcoll_context, rt_handle);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(20,"RUNNING FALLBACK IGATHERV");
         rc = hcoll_module->previous_igatherv(sbuf,scount,sdtype,
@@ -687,7 +687,7 @@ int mca_coll_hcoll_igatherv(const void* sbuf, int scount,
 
 
 #if HCOLL_API >= HCOLL_VERSION(3,7)
-int mca_coll_hcoll_ialltoallv(void *sbuf, int *scounts, int *sdisps,
+int mca_coll_hcoll_ialltoallv(const void *sbuf, int *scounts, int *sdisps,
                               struct ompi_datatype_t *sdtype,
                               void *rbuf, int *rcounts, int *rdisps,
                               struct ompi_datatype_t *rdtype,
