@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -10,8 +10,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,6 +26,7 @@
 #include "mpi.h"
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
+#include "ompi/mca/fcoll/base/fcoll_base_coll_array.h"
 #include "ompi/mca/io/ompio/io_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "math.h"
@@ -151,7 +153,7 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     double write_time = 0.0, start_write_time = 0.0, end_write_time = 0.0;
     double comm_time = 0.0, start_comm_time = 0.0, end_comm_time = 0.0;
     double exch_write = 0.0, start_exch = 0.0, end_exch = 0.0;
-    mca_io_ompio_print_entry nentry;
+    mca_common_ompio_print_entry nentry;
 #endif
     
     
@@ -263,26 +265,26 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     start_comm_time = MPI_Wtime();
 #endif
     if ( 1 == mca_fcoll_dynamic_gen2_num_groups ) {
-        ret = fh->f_comm->c_coll.coll_allgather (broken_total_lengths,
+        ret = fh->f_comm->c_coll->coll_allgather (broken_total_lengths,
                                                  dynamic_gen2_num_io_procs,
                                                  MPI_LONG,
                                                  total_bytes_per_process,
                                                  dynamic_gen2_num_io_procs,
                                                  MPI_LONG,
                                                  fh->f_comm,
-                                                 fh->f_comm->c_coll.coll_allgather_module);
+                                                 fh->f_comm->c_coll->coll_allgather_module);
     }
     else {
-        ret = fh->f_allgather_array (broken_total_lengths,
-                                     dynamic_gen2_num_io_procs,
-                                     MPI_LONG,
-                                     total_bytes_per_process,
-                                     dynamic_gen2_num_io_procs,
-                                     MPI_LONG,
-                                     0,
-                                     fh->f_procs_in_group,
-                                     fh->f_procs_per_group,
-                                     fh->f_comm);
+        ret = ompi_fcoll_base_coll_allgather_array (broken_total_lengths,
+                                               dynamic_gen2_num_io_procs,
+                                               MPI_LONG,
+                                               total_bytes_per_process,
+                                               dynamic_gen2_num_io_procs,
+                                               MPI_LONG,
+                                               0,
+                                               fh->f_procs_in_group,
+                                               fh->f_procs_per_group,
+                                               fh->f_comm);
     }
     
     if( OMPI_SUCCESS != ret){
@@ -322,26 +324,26 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     start_comm_time = MPI_Wtime();
 #endif
     if ( 1 == mca_fcoll_dynamic_gen2_num_groups ) {
-        ret = fh->f_comm->c_coll.coll_allgather(broken_counts,
+        ret = fh->f_comm->c_coll->coll_allgather(broken_counts,
                                                 dynamic_gen2_num_io_procs,
                                                 MPI_INT,
                                                 result_counts,
                                                 dynamic_gen2_num_io_procs,
                                                 MPI_INT,
                                                 fh->f_comm,
-                                                fh->f_comm->c_coll.coll_allgather_module);            
+                                                fh->f_comm->c_coll->coll_allgather_module);            
     }
     else {
-        ret = fh->f_allgather_array (broken_counts,
-                                     dynamic_gen2_num_io_procs,
-                                     MPI_INT,
-                                     result_counts,
-                                     dynamic_gen2_num_io_procs,
-                                     MPI_INT,
-                                     0,
-                                     fh->f_procs_in_group,
-                                     fh->f_procs_per_group,
-                                     fh->f_comm);
+        ret = ompi_fcoll_base_coll_allgather_array (broken_counts,
+                                               dynamic_gen2_num_io_procs,
+                                               MPI_INT,
+                                               result_counts,
+                                               dynamic_gen2_num_io_procs,
+                                               MPI_INT,
+                                               0,
+                                               fh->f_procs_in_group,
+                                               fh->f_procs_per_group,
+                                               fh->f_comm);
     }
     if( OMPI_SUCCESS != ret){
         goto exit;
@@ -408,7 +410,7 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         start_comm_time = MPI_Wtime();
 #endif
         if ( 1 == mca_fcoll_dynamic_gen2_num_groups ) {
-            ret = fh->f_comm->c_coll.coll_allgatherv (broken_iov_arrays[i],
+            ret = fh->f_comm->c_coll->coll_allgatherv (broken_iov_arrays[i],
                                                       broken_counts[i],
                                                       fh->f_iov_type,
                                                       aggr_data[i]->global_iov_array,
@@ -416,20 +418,20 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
                                                       displs,
                                                       fh->f_iov_type,
                                                       fh->f_comm,
-                                                      fh->f_comm->c_coll.coll_allgatherv_module );
+                                                      fh->f_comm->c_coll->coll_allgatherv_module );
         }
         else {
-            ret = fh->f_allgatherv_array (broken_iov_arrays[i],
-                                          broken_counts[i],
-                                          fh->f_iov_type,
-                                          aggr_data[i]->global_iov_array,
-                                          aggr_data[i]->fview_count,
-                                          displs,
-                                          fh->f_iov_type,
-                                          aggregators[i],
-                                          fh->f_procs_in_group,
-                                          fh->f_procs_per_group,
-                                          fh->f_comm);
+            ret = ompi_fcoll_base_coll_allgatherv_array (broken_iov_arrays[i],
+                                                    broken_counts[i],
+                                                    fh->f_iov_type,
+                                                    aggr_data[i]->global_iov_array,
+                                                    aggr_data[i]->fview_count,
+                                                    displs,
+                                                    fh->f_iov_type,
+                                                    aggregators[i],
+                                                    fh->f_procs_in_group,
+                                                    fh->f_procs_per_group,
+                                                    fh->f_comm);
         }
         if (OMPI_SUCCESS != ret){
             goto exit;
@@ -454,7 +456,7 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
                 ret = OMPI_ERR_OUT_OF_RESOURCE;
                 goto exit;
             }
-            fh->f_sort_iovec (aggr_data[i]->global_iov_array, total_fview_count, aggr_data[i]->sorted);
+            ompi_fcoll_base_sort_iovec (aggr_data[i]->global_iov_array, total_fview_count, aggr_data[i]->sorted);
         }
         
         if (NULL != local_iov_array){
@@ -469,7 +471,7 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     
     
 #if DEBUG_ON
-        if (my_aggregator == fh->f_rank) {
+        if (aggregators[i] == fh->f_rank) {
             uint32_t tv=0;
             for (tv=0 ; tv<total_fview_count ; tv++) {
                 printf("%d: OFFSET: %lld   LENGTH: %ld\n",
@@ -590,10 +592,17 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         
         /* Write data for iteration i-1 */
         for ( i=0; i<dynamic_gen2_num_io_procs; i++ ) {
+#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+            start_write_time = MPI_Wtime();
+#endif
             ret = write_init (fh, aggregators[i], aggr_data[i], write_chunksize );
             if (OMPI_SUCCESS != ret){
                 goto exit;
             }            
+#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+            end_write_time = MPI_Wtime();
+            write_time += end_write_time - start_write_time;
+#endif
 
             if (!aggr_data[i]->prev_sendbuf_is_contiguous && aggr_data[i]->prev_bytes_sent) {
                 free (aggr_data[i]->prev_send_buf);
@@ -616,10 +625,17 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
         
         /* Write data for iteration i=cycles-1 */
         for ( i=0; i<dynamic_gen2_num_io_procs; i++ ) {
+#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+            start_write_time = MPI_Wtime();
+#endif
             ret = write_init (fh, aggregators[i], aggr_data[i], write_chunksize );
             if (OMPI_SUCCESS != ret){
                 goto exit;
             }                    
+#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+            end_write_time = MPI_Wtime();
+            write_time += end_write_time - start_write_time;
+#endif
             
             if (!aggr_data[i]->prev_sendbuf_is_contiguous && aggr_data[i]->prev_bytes_sent) {
                 free (aggr_data[i]->prev_send_buf);
@@ -634,14 +650,15 @@ int mca_fcoll_dynamic_gen2_file_write_all (mca_io_ompio_file_t *fh,
     nentry.time[0] = write_time;
     nentry.time[1] = comm_time;
     nentry.time[2] = exch_write;
-    if (my_aggregator == fh->f_rank)
+    nentry.aggregator = 0;
+    for ( i=0; i<dynamic_gen2_num_io_procs; i++ ) {
+        if (aggregators[i] == fh->f_rank)
 	nentry.aggregator = 1;
-    else
-	nentry.aggregator = 0;
+    }
     nentry.nprocs_for_coll = dynamic_gen2_num_io_procs;
-    if (!fh->f_full_print_queue(WRITE_PRINT_QUEUE)){
-        fh->f_register_print_entry(WRITE_PRINT_QUEUE,
-                                   nentry);
+    if (!mca_common_ompio_full_print_queue(fh->f_coll_write_time)){
+        mca_common_ompio_register_print_entry(fh->f_coll_write_time,
+                                               nentry);
     }
 #endif
     
@@ -724,19 +741,12 @@ static int write_init (mca_io_ompio_file_t *fh, int aggregator, mca_io_ompio_agg
                                                                                       aggr_data->prev_num_io_entries, 
                                                                                       &last_array_pos, &last_pos,
                                                                                       write_chunksize );
-#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
-            start_write_time = MPI_Wtime();
-#endif
             if ( 0 >  fh->f_fbtl->fbtl_pwritev (fh)) {
                 free ( aggr_data->prev_io_array);
                 opal_output (1, "dynamic_gen2_write_all: fbtl_pwritev failed\n");
                 ret = OMPI_ERROR;
                 goto exit;
             }
-#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
-            end_write_time = MPI_Wtime();
-            write_time += end_write_time - start_write_time;
-#endif
         }
         free ( fh->f_io_array );
         free ( aggr_data->prev_io_array);
@@ -860,7 +870,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 if (aggregator == rank) {
                     data->blocklen_per_process[data->n][data->disp_index[data->n] - 1] = data->bytes_remaining;
                     data->displs_per_process[data->n][data->disp_index[data->n] - 1] =
-                        (OPAL_PTRDIFF_TYPE)data->global_iov_array[data->sorted[data->current_index]].iov_base +
+                        (ptrdiff_t)data->global_iov_array[data->sorted[data->current_index]].iov_base +
                         (data->global_iov_array[data->sorted[data->current_index]].iov_len
                          - data->bytes_remaining);
                     
@@ -888,7 +898,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 if (aggregator == rank) {
                     data->blocklen_per_process[data->n][data->disp_index[data->n] - 1] = data->bytes_to_write_in_cycle;
                     data->displs_per_process[data->n][data->disp_index[data->n] - 1] =
-                        (OPAL_PTRDIFF_TYPE)data->global_iov_array[data->sorted[data->current_index]].iov_base +
+                        (ptrdiff_t)data->global_iov_array[data->sorted[data->current_index]].iov_base +
                         (data->global_iov_array[data->sorted[data->current_index]].iov_len
                          - data->bytes_remaining);
                 }
@@ -909,7 +919,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 if (aggregator == rank) {
                     data->blocklen_per_process[data->n][data->disp_index[data->n] - 1] = data->bytes_to_write_in_cycle;
                     data->displs_per_process[data->n][data->disp_index[data->n] - 1] =
-                        (OPAL_PTRDIFF_TYPE)data->global_iov_array[data->sorted[data->current_index]].iov_base ;
+                        (ptrdiff_t)data->global_iov_array[data->sorted[data->current_index]].iov_base ;
                 }
                 if (data->procs_in_group[data->n] == rank) {
                     bytes_sent += data->bytes_to_write_in_cycle;
@@ -925,7 +935,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
                 if (aggregator == rank) {
                     data->blocklen_per_process[data->n][data->disp_index[data->n] - 1] =
                         data->global_iov_array[data->sorted[data->current_index]].iov_len;
-                    data->displs_per_process[data->n][data->disp_index[data->n] - 1] = (OPAL_PTRDIFF_TYPE)
+                    data->displs_per_process[data->n][data->disp_index[data->n] - 1] = (ptrdiff_t)
                         data->global_iov_array[data->sorted[data->current_index]].iov_base;
                     
                     /*realloc for next blocklength
@@ -1091,9 +1101,9 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
             printf("%d : global_count : %ld, bytes_sent : %d\n",
                    rank,global_count, bytes_sent);
 #endif
-#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
-            start_comm_time = MPI_Wtime();
-#endif
+//#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+//            start_comm_time = MPI_Wtime();
+//#endif
             /*************************************************************************
              *** 7e. Perform the actual communication
              *************************************************************************/
@@ -1133,7 +1143,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
         /* allocate a send buffer and copy the data that needs
            to be sent into it in case the data is non-contigous
            in memory */
-        OPAL_PTRDIFF_TYPE mem_address;
+        ptrdiff_t mem_address;
         size_t remaining = 0;
         size_t temp_position = 0;
         
@@ -1147,7 +1157,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
         remaining = bytes_sent;
         
         while (remaining) {
-            mem_address = (OPAL_PTRDIFF_TYPE)
+            mem_address = (ptrdiff_t)
                 (data->decoded_iov[data->iov_index].iov_base) + data->current_position;
             
             if (remaining >=
@@ -1202,10 +1212,10 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
     }
 #endif
     
-#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
-    end_comm_time = MPI_Wtime();
-    comm_time += (end_comm_time - start_comm_time);
-#endif
+//#if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
+//    end_comm_time = MPI_Wtime();
+//    comm_time += (end_comm_time - start_comm_time);
+//#endif
     /**********************************************************
      *** 7f. Create the io array, and pass it to fbtl
      *********************************************************/
@@ -1257,7 +1267,7 @@ static int shuffle_init ( int index, int cycles, int aggregator, int rank, mca_i
         for (i=0 ; i<num_of_io_entries ; i++) {
             printf(" ADDRESS: %p  OFFSET: %ld   LENGTH: %ld\n",
                    io_array[i].memory_address,
-                   (OPAL_PTRDIFF_TYPE)io_array[i].offset,
+                   (ptrdiff_t)io_array[i].offset,
                    io_array[i].length);
         }
         
