@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,7 +11,10 @@
  *                         All rights reserved.
  * Copyright (c) 2008-2016 University of Houston. All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  *
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,6 +29,7 @@
 #include "mpi.h"
 #include "ompi/constants.h"
 #include "ompi/mca/fcoll/fcoll.h"
+#include "ompi/mca/fcoll/base/fcoll_base_coll_array.h"
 #include "ompi/mca/io/ompio/io_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "math.h"
@@ -88,7 +92,7 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
 
     int blocklen[3] = {1, 1, 1};
     int static_num_io_procs=1;
-    OPAL_PTRDIFF_TYPE d[3], base;
+    ptrdiff_t d[3], base;
     ompi_datatype_t *types[3];
     ompi_datatype_t *io_array_type=MPI_DATATYPE_NULL;
     ompi_datatype_t **sendtype = NULL;
@@ -96,13 +100,13 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
     int my_aggregator=-1;
     bool recvbuf_is_contiguous=false;
     size_t ftype_size;
-    OPAL_PTRDIFF_TYPE ftype_extent, lb;
+    ptrdiff_t ftype_extent, lb;
 
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     double read_time = 0.0, start_read_time = 0.0, end_read_time = 0.0;
     double rcomm_time = 0.0, start_rcomm_time = 0.0, end_rcomm_time = 0.0;
     double read_exch = 0.0, start_rexch = 0.0, end_rexch = 0.0;
-    mca_io_ompio_print_entry nentry;
+    mca_common_ompio_print_entry nentry;
 #endif
 #if DEBUG_ON
     MPI_Aint gc_in;
@@ -113,7 +117,7 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
     /**************************************************************************
      ** 1.  In case the data is not contigous in memory, decode it into an iovec
      **************************************************************************/
-    if ( ( ftype_extent == (OPAL_PTRDIFF_TYPE) ftype_size)             &&
+    if ( ( ftype_extent == (ptrdiff_t) ftype_size)             &&
          opal_datatype_is_contiguous_memory_layout(&datatype->super,1) &&
          0 == lb ) {
         recvbuf_is_contiguous = true;
@@ -185,9 +189,9 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
         local_iov_array[0].process_id = fh->f_rank;
     }
 
-    d[0] = (OPAL_PTRDIFF_TYPE)&local_iov_array[0];
-    d[1] = (OPAL_PTRDIFF_TYPE)&local_iov_array[0].length;
-    d[2] = (OPAL_PTRDIFF_TYPE)&local_iov_array[0].process_id;
+    d[0] = (ptrdiff_t)&local_iov_array[0];
+    d[1] = (ptrdiff_t)&local_iov_array[0].length;
+    d[2] = (ptrdiff_t)&local_iov_array[0].process_id;
     base = d[0];
     for (i=0 ; i<3 ; i++) {
         d[i] -= base;
@@ -212,13 +216,13 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     start_rexch = MPI_Wtime();
 #endif
-    ret = fh->f_comm->c_coll.coll_allreduce (&local_cycles,
+    ret = fh->f_comm->c_coll->coll_allreduce (&local_cycles,
                                              &cycles,
                                              1,
                                              MPI_INT,
                                              MPI_MAX,
                                              fh->f_comm,
-                                             fh->f_comm->c_coll.coll_allreduce_module);
+                                             fh->f_comm->c_coll->coll_allreduce_module);
 
     if (OMPI_SUCCESS != ret){
         goto exit;
@@ -274,14 +278,14 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
     }
 
 
-    iovec_count_per_process = (int *) malloc (fh->f_procs_per_group * sizeof(int));
+    iovec_count_per_process = (int *) calloc (fh->f_procs_per_group, sizeof(int));
     if (NULL == iovec_count_per_process){
         opal_output (1, "OUT OF MEMORY\n");
         ret = OMPI_ERR_OUT_OF_RESOURCE;
         goto exit;
     }
 
-    displs = (int *) malloc (fh->f_procs_per_group * sizeof(int));
+    displs = (int *) calloc (fh->f_procs_per_group, sizeof(int));
     if (NULL == displs){
         opal_output (1, "OUT OF MEMORY\n");
         ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -291,16 +295,16 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     start_rexch = MPI_Wtime();
 #endif
-    ret = fh->f_allgather_array (&iov_size,
-                                 1,
-                                 MPI_INT,
-                                 iovec_count_per_process,
-                                 1,
-                                 MPI_INT,
-                                 fh->f_aggregator_index,
-                                 fh->f_procs_in_group,
-                                 fh->f_procs_per_group,
-                                 fh->f_comm);
+    ret = ompi_fcoll_base_coll_allgather_array (&iov_size,
+                                           1,
+                                           MPI_INT,
+                                           iovec_count_per_process,
+                                           1,
+                                           MPI_INT,
+                                           fh->f_aggregator_index,
+                                           fh->f_procs_in_group,
+                                           fh->f_procs_per_group,
+                                           fh->f_comm);
 
     if( OMPI_SUCCESS != ret){
         goto exit;
@@ -334,17 +338,17 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     start_rexch = MPI_Wtime();
 #endif
-    ret = fh->f_gatherv_array (local_iov_array,
-                               iov_size,
-                               io_array_type,
-                               global_iov_array,
-                               iovec_count_per_process,
-                               displs,
-                               io_array_type,
-                               fh->f_aggregator_index,
-                               fh->f_procs_in_group,
-                               fh->f_procs_per_group,
-                               fh->f_comm);
+    ret = ompi_fcoll_base_coll_gatherv_array (local_iov_array,
+                                         iov_size,
+                                         io_array_type,
+                                         global_iov_array,
+                                         iovec_count_per_process,
+                                         displs,
+                                         io_array_type,
+                                         fh->f_aggregator_index,
+                                         fh->f_procs_in_group,
+                                         fh->f_procs_per_group,
+                                         fh->f_comm);
 
     if (OMPI_SUCCESS != ret){
         fprintf(stderr,"global_iov_array gather error!\n");
@@ -440,7 +444,7 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
                 free(memory_displacements);
                 memory_displacements= NULL;
             }
-            
+
             if ( NULL != sendtype ) {
                 for ( i=0; i<fh->f_procs_per_group; i++ ) {
                     if ( MPI_DATATYPE_NULL != sendtype[i] ) {
@@ -493,16 +497,16 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
     start_rexch = MPI_Wtime();
 #endif
-        fh->f_gather_array (&bytes_to_read_in_cycle,
-                            1,
-                            MPI_INT,
-                            bytes_per_process,
-                            1,
-                            MPI_INT,
-                            fh->f_aggregator_index,
-                            fh->f_procs_in_group,
-                            fh->f_procs_per_group,
-                            fh->f_comm);
+        ompi_fcoll_base_coll_gather_array (&bytes_to_read_in_cycle,
+                                      1,
+                                      MPI_INT,
+                                      bytes_per_process,
+                                      1,
+                                      MPI_INT,
+                                      fh->f_aggregator_index,
+                                      fh->f_procs_in_group,
+                                      fh->f_procs_per_group,
+                                      fh->f_comm);
 
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
         end_rcomm_time = MPI_Wtime();
@@ -767,7 +771,7 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
             for (i=0 ; i<fh->f_num_of_io_entries ; i++) {
                 printf(" ADDRESS: %p  OFFSET: %ld   LENGTH: %ld\n",
                        fh->f_io_array[i].memory_address,
-                       (OPAL_PTRDIFF_TYPE)fh->f_io_array[i].offset,
+                       (ptrdiff_t)fh->f_io_array[i].offset,
                        fh->f_io_array[i].length);
             }
 #endif
@@ -870,14 +874,14 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
         position += bytes_to_read_in_cycle;
 
         if (!recvbuf_is_contiguous) {
-            OPAL_PTRDIFF_TYPE mem_address;
+            ptrdiff_t mem_address;
             size_t remaining = 0;
             size_t temp_position = 0;
 
             remaining = bytes_to_read_in_cycle;
 
             while (remaining && (iov_count > iov_index)){
-                mem_address = (OPAL_PTRDIFF_TYPE)
+                mem_address = (ptrdiff_t)
                     (decoded_iov[iov_index].iov_base) + current_position;
 
                 if (remaining >=
@@ -918,9 +922,9 @@ mca_fcoll_static_file_read_all (mca_io_ompio_file_t *fh,
     else
         nentry.aggregator = 0;
     nentry.nprocs_for_coll = static_num_io_procs;
-    if (!fh->f_full_print_queue(READ_PRINT_QUEUE)){
-        fh->f_register_print_entry(READ_PRINT_QUEUE,
-                                   nentry);
+    if (!mca_common_ompio_full_print_queue(fh->f_coll_read_time)){
+        mca_common_ompio_register_print_entry(fh->f_coll_read_time,
+                                              nentry);
     }
 #endif
 
