@@ -8,7 +8,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2017 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2009-2011 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012-2013 Sandia National Laboratories.  All rights reserved.
@@ -75,7 +75,7 @@ static void osc_pt2pt_accumulate_data_destructor (osc_pt2pt_accumulate_data_t *a
     }
 
     if (acc_data->datatype) {
-        OBJ_RELEASE(acc_data->datatype);
+        OMPI_DATATYPE_RELEASE(acc_data->datatype);
     }
 }
 
@@ -102,6 +102,7 @@ struct osc_pt2pt_pending_acc_t {
     void *data;
     size_t data_len;
     ompi_datatype_t *datatype;
+    bool active_target;
 };
 typedef struct osc_pt2pt_pending_acc_t osc_pt2pt_pending_acc_t;
 
@@ -118,7 +119,7 @@ static void osc_pt2pt_pending_acc_destructor (osc_pt2pt_pending_acc_t *pending)
     }
 
     if (NULL != pending->datatype) {
-        OBJ_RELEASE(pending->datatype);
+        OMPI_DATATYPE_RELEASE(pending->datatype);
     }
 }
 
@@ -370,7 +371,7 @@ static inline int process_put(ompi_osc_pt2pt_module_t* module, int source,
 
     osc_pt2pt_copy_on_recv (target, data, data_len, proc, put_header->count, datatype);
 
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return put_header->len;
 }
@@ -407,7 +408,7 @@ static inline int process_put_long(ompi_osc_pt2pt_module_t* module, int source,
         return OMPI_ERROR;
     }
 
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return put_header->len;
 }
@@ -532,7 +533,7 @@ static inline int process_get (ompi_osc_pt2pt_module_t* module, int target,
     ret = osc_pt2pt_get_post_send (module, source, get_header->count, datatype,
                                   target, tag_to_origin(get_header->tag));
 
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return OMPI_SUCCESS == ret ? (int) get_header->len : ret;
 }
@@ -634,7 +635,7 @@ static int osc_pt2pt_accumulate_allocate (ompi_osc_pt2pt_module_t *module, int p
     acc_data->proc = proc;
     acc_data->count = count;
     acc_data->datatype = datatype;
-    OBJ_RETAIN(datatype);
+    OMPI_DATATYPE_RETAIN(datatype);
     acc_data->op = op;
     acc_data->request_count = request_count;
 
@@ -666,8 +667,6 @@ static int accumulate_cb (ompi_request_t *request)
         rank = acc_data->peer;
     }
 
-    mark_incoming_completion (module, rank);
-
     if (0 == OPAL_THREAD_ADD32(&acc_data->request_count, -1)) {
         /* no more requests needed before the buffer can be accumulated */
 
@@ -693,13 +692,15 @@ static int accumulate_cb (ompi_request_t *request)
         osc_pt2pt_gc_add_buffer (module, &acc_data->super);
     }
 
+    mark_incoming_completion (module, rank);
+
     ompi_request_free (&request);
     return ret;
 }
 
 
 static int ompi_osc_pt2pt_acc_op_queue (ompi_osc_pt2pt_module_t *module, ompi_osc_pt2pt_header_t *header, int source,
-                                       char *data, size_t data_len, ompi_datatype_t *datatype)
+                                        char *data, size_t data_len, ompi_datatype_t *datatype, bool active_target)
 {
     ompi_osc_pt2pt_peer_t *peer = ompi_osc_pt2pt_peer_lookup (module, source);
     osc_pt2pt_pending_acc_t *pending_acc;
@@ -714,8 +715,13 @@ static int ompi_osc_pt2pt_acc_op_queue (ompi_osc_pt2pt_module_t *module, ompi_os
 
     /* NTH: ensure we don't leave wait/process_flush/etc until this
      * accumulate operation is complete. */
-    OPAL_THREAD_ADD32(&peer->passive_incoming_frag_count, -1);
+    if (active_target) {
+        OPAL_THREAD_ADD32(&module->active_incoming_frag_count, -1);
+    } else {
+        OPAL_THREAD_ADD32(&peer->passive_incoming_frag_count, -1);
+    }
 
+    pending_acc->active_target = active_target;
     pending_acc->source = source;
 
     /* save any inline data (eager acc, gacc only) */
@@ -728,7 +734,7 @@ static int ompi_osc_pt2pt_acc_op_queue (ompi_osc_pt2pt_module_t *module, ompi_os
 
     /* save the datatype */
     pending_acc->datatype = datatype;
-    OBJ_RETAIN(datatype);
+    OMPI_DATATYPE_RETAIN(datatype);
 
     /* save the header */
     switch (header->base.type) {
@@ -747,7 +753,7 @@ static int ompi_osc_pt2pt_acc_op_queue (ompi_osc_pt2pt_module_t *module, ompi_os
     }
 
     /* add to the pending acc queue */
-    OPAL_THREAD_SCOPED_LOCK(&module->lock, opal_list_append (&module->pending_acc, &pending_acc->super));
+    OPAL_THREAD_SCOPED_LOCK(&module->pending_acc_lock, opal_list_append (&module->pending_acc, &pending_acc->super));
 
     return OMPI_SUCCESS;
 }
@@ -1090,7 +1096,9 @@ int ompi_osc_pt2pt_progress_pending_acc (ompi_osc_pt2pt_module_t *module)
         return OMPI_SUCCESS;
     }
 
+    OPAL_THREAD_LOCK(&module->pending_acc_lock);
     pending_acc = (osc_pt2pt_pending_acc_t *) opal_list_remove_first (&module->pending_acc);
+    OPAL_THREAD_UNLOCK(&module->pending_acc_lock);
     if (OPAL_UNLIKELY(NULL == pending_acc)) {
         /* called without any pending accumulation operations */
         ompi_osc_pt2pt_accumulate_unlock (module);
@@ -1127,7 +1135,7 @@ int ompi_osc_pt2pt_progress_pending_acc (ompi_osc_pt2pt_module_t *module)
     }
 
     /* signal that an operation is complete */
-    mark_incoming_completion (module, pending_acc->source);
+    mark_incoming_completion (module, pending_acc->active_target ? MPI_PROC_NULL : pending_acc->source);
 
     pending_acc->data = NULL;
     OBJ_RELEASE(pending_acc);
@@ -1138,6 +1146,7 @@ int ompi_osc_pt2pt_progress_pending_acc (ompi_osc_pt2pt_module_t *module)
 static inline int process_acc (ompi_osc_pt2pt_module_t *module, int source,
                                ompi_osc_pt2pt_header_acc_t *acc_header)
 {
+    bool active_target = !(acc_header->tag & 0x1);
     char *data = (char *) (acc_header + 1);
     struct ompi_datatype_t *datatype;
     uint64_t data_len;
@@ -1162,11 +1171,11 @@ static inline int process_acc (ompi_osc_pt2pt_module_t *module, int source,
     } else {
         /* couldn't aquire the accumulate lock so queue up the accumulate operation */
         ret = ompi_osc_pt2pt_acc_op_queue (module, (ompi_osc_pt2pt_header_t *) acc_header,
-                                          source, data, data_len, datatype);
+                                           source, data, data_len, datatype, active_target);
     }
 
     /* Release datatype & op */
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return (OMPI_SUCCESS == ret) ? (int) acc_header->len : ret;
 }
@@ -1174,6 +1183,7 @@ static inline int process_acc (ompi_osc_pt2pt_module_t *module, int source,
 static inline int process_acc_long (ompi_osc_pt2pt_module_t* module, int source,
                                     ompi_osc_pt2pt_header_acc_t* acc_header)
 {
+    bool active_target = !(acc_header->tag & 0x1);
     char *data = (char *) (acc_header + 1);
     struct ompi_datatype_t *datatype;
     int ret;
@@ -1193,11 +1203,11 @@ static inline int process_acc_long (ompi_osc_pt2pt_module_t* module, int source,
     } else {
         /* queue the operation */
         ret = ompi_osc_pt2pt_acc_op_queue (module, (ompi_osc_pt2pt_header_t *) acc_header, source,
-                                          NULL, 0, datatype);
+                                           NULL, 0, datatype, active_target);
     }
 
     /* Release datatype & op */
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return (OMPI_SUCCESS == ret) ? (int) acc_header->len : ret;
 }
@@ -1205,6 +1215,7 @@ static inline int process_acc_long (ompi_osc_pt2pt_module_t* module, int source,
 static inline int process_get_acc(ompi_osc_pt2pt_module_t *module, int source,
                                   ompi_osc_pt2pt_header_acc_t *acc_header)
 {
+    bool active_target = !(acc_header->tag & 0x1);
     char *data = (char *) (acc_header + 1);
     struct ompi_datatype_t *datatype;
     void *buffer = NULL;
@@ -1231,7 +1242,7 @@ static inline int process_get_acc(ompi_osc_pt2pt_module_t *module, int source,
             uint32_t primitive_count;
             buffer = malloc (data_len);
             if (OPAL_UNLIKELY(NULL == buffer)) {
-                OBJ_RELEASE(datatype);
+                OMPI_DATATYPE_RELEASE(datatype);
                 return OMPI_ERR_OUT_OF_RESOURCE;
             }
 
@@ -1246,11 +1257,11 @@ static inline int process_get_acc(ompi_osc_pt2pt_module_t *module, int source,
     } else {
         /* queue the operation */
         ret = ompi_osc_pt2pt_acc_op_queue (module, (ompi_osc_pt2pt_header_t *) acc_header,
-                                          source, data, data_len, datatype);
+                                           source, data, data_len, datatype, active_target);
     }
 
     /* Release datatype & op */
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return (OMPI_SUCCESS == ret) ? (int) acc_header->len : ret;
 }
@@ -1258,6 +1269,7 @@ static inline int process_get_acc(ompi_osc_pt2pt_module_t *module, int source,
 static inline int process_get_acc_long(ompi_osc_pt2pt_module_t *module, int source,
                                        ompi_osc_pt2pt_header_acc_t *acc_header)
 {
+    bool active_target = !(acc_header->tag & 0x1);
     char *data = (char *) (acc_header + 1);
     struct ompi_datatype_t *datatype;
     int ret;
@@ -1277,11 +1289,11 @@ static inline int process_get_acc_long(ompi_osc_pt2pt_module_t *module, int sour
     } else {
         /* queue the operation */
         ret = ompi_osc_pt2pt_acc_op_queue (module, (ompi_osc_pt2pt_header_t *) acc_header,
-                                          source, NULL, 0, datatype);
+                                           source, NULL, 0, datatype, active_target);
     }
 
     /* Release datatype & op */
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return OMPI_SUCCESS == ret ? (int) acc_header->len : ret;
 }
@@ -1290,6 +1302,7 @@ static inline int process_get_acc_long(ompi_osc_pt2pt_module_t *module, int sour
 static inline int process_cswap (ompi_osc_pt2pt_module_t *module, int source,
                                  ompi_osc_pt2pt_header_cswap_t *cswap_header)
 {
+    bool active_target = !(cswap_header->tag & 0x1);
     char *data = (char*) (cswap_header + 1);
     struct ompi_datatype_t *datatype;
     int ret;
@@ -1309,11 +1322,11 @@ static inline int process_cswap (ompi_osc_pt2pt_module_t *module, int source,
     } else {
         /* queue the operation */
         ret = ompi_osc_pt2pt_acc_op_queue (module, (ompi_osc_pt2pt_header_t *) cswap_header, source,
-                                          data, 2 * datatype->super.size, datatype);
+                                           data, 2 * datatype->super.size, datatype, active_target);
     }
 
     /* Release datatype */
-    OBJ_RELEASE(datatype);
+    OMPI_DATATYPE_RELEASE(datatype);
 
     return (OMPI_SUCCESS == ret) ? (int) cswap_header->len : ret;
 }
@@ -1392,7 +1405,7 @@ static inline int process_unlock (ompi_osc_pt2pt_module_t *module, int source,
         osc_pt2pt_add_pending (pending);
     }
 
-    /* signal incomming will increment this counter */
+    /* signal incoming will increment this counter */
     OPAL_THREAD_ADD32(&peer->passive_incoming_frag_count, -1);
 
     return sizeof (*unlock_header);
@@ -1632,6 +1645,7 @@ int ompi_osc_pt2pt_process_receive (ompi_osc_pt2pt_receive_t *recv)
     int rc;
 
     assert(incoming_length >= sizeof(ompi_osc_pt2pt_header_base_t));
+    (void)incoming_length;  // silence compiler warning
 
     OPAL_OUTPUT_VERBOSE((50, ompi_osc_base_framework.framework_output,
                          "received pt2pt callback for fragment. source = %d, count = %u, type = 0x%x",
