@@ -8,11 +8,15 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2018 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2012-2013 Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2017-2018 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -45,6 +49,28 @@ typedef struct ompi_osc_rdma_pending_post_t ompi_osc_rdma_pending_post_t;
 
 static OBJ_CLASS_INSTANCE(ompi_osc_rdma_pending_post_t, opal_list_item_t, NULL, NULL);
 
+static void ompi_osc_rdma_pending_op_construct (ompi_osc_rdma_pending_op_t *pending_op)
+{
+    pending_op->op_frag = NULL;
+    pending_op->op_buffer = NULL;
+    pending_op->op_result = NULL;
+    pending_op->op_complete = false;
+    pending_op->cbfunc = NULL;
+}
+
+static void ompi_osc_rdma_pending_op_destruct (ompi_osc_rdma_pending_op_t *pending_op)
+{
+    if (NULL != pending_op->op_frag) {
+        ompi_osc_rdma_frag_complete (pending_op->op_frag);
+    }
+
+    ompi_osc_rdma_pending_op_construct (pending_op);
+}
+
+OBJ_CLASS_INSTANCE(ompi_osc_rdma_pending_op_t, opal_list_item_t,
+                   ompi_osc_rdma_pending_op_construct,
+                   ompi_osc_rdma_pending_op_destruct);
+
 /**
  * Dummy completion function for atomic operations
  */
@@ -52,11 +78,25 @@ void ompi_osc_rdma_atomic_complete (mca_btl_base_module_t *btl, struct mca_btl_b
                                     void *local_address, mca_btl_base_registration_handle_t *local_handle,
                                     void *context, void *data, int status)
 {
-    volatile bool *atomic_complete = (volatile bool *) context;
+    ompi_osc_rdma_pending_op_t *pending_op = (ompi_osc_rdma_pending_op_t *) context;
 
-    if (atomic_complete) {
-        *atomic_complete = true;
+    OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_INFO, "pending atomic %p complete with status %d", (void*)pending_op, status);
+
+    if (pending_op->op_result) {
+        memmove (pending_op->op_result, pending_op->op_buffer, pending_op->op_size);
     }
+
+    if (NULL != pending_op->cbfunc) {
+        pending_op->cbfunc (pending_op->cbdata, pending_op->cbcontext, status);
+    }
+
+    if (NULL != pending_op->op_frag) {
+        ompi_osc_rdma_frag_complete (pending_op->op_frag);
+        pending_op->op_frag = NULL;
+    }
+
+    pending_op->op_complete = true;
+    OBJ_RELEASE(pending_op);
 }
 
 /**
@@ -161,7 +201,8 @@ static void ompi_osc_rdma_handle_post (ompi_osc_rdma_module_t *module, int rank,
         if (rank == peers[j]->rank) {
             OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_INFO, "got expected post from %d. still expecting posts from %d processes",
                              rank, (int) (npeers - state->num_post_msgs - 1));
-            ++state->num_post_msgs;
+            /* an atomic is not really necessary as this function is currently used but it doesn't hurt */
+            ompi_osc_rdma_counter_add (&state->num_post_msgs, 1);
             return;
         }
     }
@@ -173,16 +214,90 @@ static void ompi_osc_rdma_handle_post (ompi_osc_rdma_module_t *module, int rank,
     OPAL_THREAD_SCOPED_LOCK(&module->lock, opal_list_append (&module->pending_posts, &pending_post->super));
 }
 
+static void ompi_osc_rdma_check_posts (ompi_osc_rdma_module_t *module)
+{
+    ompi_osc_rdma_state_t *state = module->state;
+    ompi_osc_rdma_sync_t *sync = &module->all_sync;
+    int count = 0;
+
+    if (OMPI_OSC_RDMA_SYNC_TYPE_PSCW == sync->type) {
+        count = sync->num_peers;
+    }
+
+    for (int i = 0 ; i < OMPI_OSC_RDMA_POST_PEER_MAX ; ++i) {
+        /* no post at this index (yet) */
+        if (0 == state->post_peers[i]) {
+            continue;
+        }
+
+        ompi_osc_rdma_handle_post (module, state->post_peers[i] - 1, sync->peer_list.peers, count);
+        state->post_peers[i] = 0;
+    }
+}
+
+static int ompi_osc_rdma_post_peer (ompi_osc_rdma_module_t *module, ompi_osc_rdma_peer_t *peer)
+{
+    uint64_t target = (uint64_t) (intptr_t) peer->state + offsetof (ompi_osc_rdma_state_t, post_index);
+    ompi_osc_rdma_lock_t post_index, result, _tmp_value;
+    int my_rank = ompi_comm_rank (module->comm);
+    int ret;
+
+    if (peer->rank == my_rank) {
+        ompi_osc_rdma_handle_post (module, my_rank, NULL, 0);
+        return OMPI_SUCCESS;
+    }
+
+    /* get a post index */
+    if (!ompi_osc_rdma_peer_local_state (peer)) {
+        ret = ompi_osc_rdma_lock_btl_fop (module, peer, target, MCA_BTL_ATOMIC_ADD, 1, &post_index, true);
+        if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+            return ret;
+        }
+    } else {
+        post_index = ompi_osc_rdma_counter_add ((osc_rdma_counter_t *) (intptr_t) target, 1) - 1;
+    }
+
+    post_index &= OMPI_OSC_RDMA_POST_PEER_MAX - 1;
+
+    target = (uint64_t) (intptr_t) peer->state + offsetof (ompi_osc_rdma_state_t, post_peers) +
+        sizeof (osc_rdma_counter_t) * post_index;
+
+    do {
+        OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "attempting to post to index %d @ rank %d", (int)post_index, peer->rank);
+
+        _tmp_value = 0;
+
+        /* try to post. if the value isn't 0 then another rank is occupying this index */
+        if (!ompi_osc_rdma_peer_local_state (peer)) {
+            ret = ompi_osc_rdma_lock_btl_cswap (module, peer, target, 0, 1 + (int64_t) my_rank, &result);
+            if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+                return ret;
+            }
+        } else {
+            result = !ompi_osc_rdma_lock_compare_exchange ((osc_rdma_counter_t *) target, &_tmp_value,
+                                                           1 + (osc_rdma_counter_t) my_rank);
+        }
+
+        if (OPAL_LIKELY(0 == result)) {
+            break;
+        }
+
+        /* prevent circular wait by checking for post messages received */
+        ompi_osc_rdma_check_posts (module);
+
+        /* zzzzzzzzzzzzz */
+        nanosleep (&(struct timespec) {.tv_sec = 0, .tv_nsec = 100}, NULL);
+    } while (1);
+
+    return OMPI_SUCCESS;
+}
+
 int ompi_osc_rdma_post_atomic (ompi_group_t *group, int assert, ompi_win_t *win)
 {
     ompi_osc_rdma_module_t *module = GET_MODULE(win);
     ompi_osc_rdma_peer_t **peers;
-    int my_rank = ompi_comm_rank (module->comm);
     ompi_osc_rdma_state_t *state = module->state;
-    volatile bool atomic_complete;
-    ompi_osc_rdma_frag_t *frag;
-    osc_rdma_counter_t *temp;
-    int ret;
+    int ret = OMPI_SUCCESS;
 
     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "post: %p, %d, %s", (void*) group, assert, win->w_name);
 
@@ -209,21 +324,13 @@ int ompi_osc_rdma_post_atomic (ompi_group_t *group, int assert, ompi_win_t *win)
     state->num_complete_msgs = 0;
     OPAL_THREAD_UNLOCK(&module->lock);
 
-    /* allocate a temporary buffer for atomic response */
-    ret = ompi_osc_rdma_frag_alloc (module, 8, &frag, (char **) &temp);
-
     if ((assert & MPI_MODE_NOCHECK) || 0 == ompi_group_size (group)) {
         return OMPI_SUCCESS;
     }
 
-    if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
-        return OMPI_ERR_OUT_OF_RESOURCE;
-    }
-
     /* translate group ranks into the communicator */
     peers = ompi_osc_rdma_get_peers (module, module->pw_group);
     if (OPAL_UNLIKELY(NULL == peers)) {
-        ompi_osc_rdma_frag_complete (frag);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
@@ -231,92 +338,17 @@ int ompi_osc_rdma_post_atomic (ompi_group_t *group, int assert, ompi_win_t *win)
 
     /* send a hello counter to everyone in group */
     for (int i = 0 ; i < ompi_group_size(module->pw_group) ; ++i) {
-        ompi_osc_rdma_peer_t *peer = peers[i];
-        uint64_t target = (uint64_t) (intptr_t) peer->state + offsetof (ompi_osc_rdma_state_t, post_index);
-        int post_index;
-
-        if (peer->rank == my_rank) {
-            ompi_osc_rdma_handle_post (module, my_rank, NULL, 0);
-            continue;
+        ret = ompi_osc_rdma_post_peer (module, peers[i]);
+        if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
+            break;
         }
-
-        /* get a post index */
-        atomic_complete = false;
-        if (!ompi_osc_rdma_peer_local_state (peer)) {
-            do {
-                ret = module->selected_btl->btl_atomic_fop (module->selected_btl, peer->state_endpoint, temp, target, frag->handle,
-                                                            peer->state_handle, MCA_BTL_ATOMIC_ADD, 1, 0, MCA_BTL_NO_ORDER,
-                                                            ompi_osc_rdma_atomic_complete, (void *) &atomic_complete, NULL);
-                assert (OPAL_SUCCESS >= ret);
-
-                if (OMPI_SUCCESS == ret) {
-                    while (!atomic_complete) {
-                        ompi_osc_rdma_progress (module);
-                    }
-
-                    break;
-                }
-
-                ompi_osc_rdma_progress (module);
-            } while (1);
-        } else {
-            *temp = ompi_osc_rdma_counter_add ((osc_rdma_counter_t *) (intptr_t) target, 1) - 1;
-        }
-        post_index = (*temp) & (OMPI_OSC_RDMA_POST_PEER_MAX - 1);
-
-        target = (uint64_t) (intptr_t) peer->state + offsetof (ompi_osc_rdma_state_t, post_peers) +
-            sizeof (osc_rdma_counter_t) * post_index;
-
-        do {
-            OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "attempting to post to index %d @ rank %d", post_index, peer->rank);
-
-            /* try to post. if the value isn't 0 then another rank is occupying this index */
-            if (!ompi_osc_rdma_peer_local_state (peer)) {
-                atomic_complete = false;
-                ret = module->selected_btl->btl_atomic_cswap (module->selected_btl, peer->state_endpoint, temp, target, frag->handle, peer->state_handle,
-                                                              0, 1 + (int64_t) my_rank, 0, MCA_BTL_NO_ORDER, ompi_osc_rdma_atomic_complete,
-                                                              (void *) &atomic_complete, NULL);
-                assert (OPAL_SUCCESS >= ret);
-
-                if (OMPI_SUCCESS == ret) {
-                    while (!atomic_complete) {
-                        ompi_osc_rdma_progress (module);
-                    }
-                } else {
-                    ompi_osc_rdma_progress (module);
-                    continue;
-                }
-
-            } else {
-                *temp = !ompi_osc_rdma_lock_cmpset ((osc_rdma_counter_t *) target, 0, 1 + (osc_rdma_counter_t) my_rank);
-            }
-
-            if (OPAL_LIKELY(0 == *temp)) {
-                break;
-            }
-
-            /* prevent circular wait by checking for post messages received */
-            for (int j = 0 ; j < OMPI_OSC_RDMA_POST_PEER_MAX ; ++j) {
-                /* no post at this index (yet) */
-                if (0 == state->post_peers[j]) {
-                    continue;
-                }
-
-                ompi_osc_rdma_handle_post (module, state->post_peers[j] - 1, NULL, 0);
-                state->post_peers[j] = 0;
-            }
-
-            usleep (100);
-        } while (1);
     }
 
-    ompi_osc_rdma_frag_complete (frag);
-
     ompi_osc_rdma_release_peers (peers, ompi_group_size(module->pw_group));
 
     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "post complete");
 
-    return OMPI_SUCCESS;
+    return ret;
 }
 
 int ompi_osc_rdma_start_atomic (ompi_group_t *group, int assert, ompi_win_t *win)
@@ -382,8 +414,7 @@ int ompi_osc_rdma_start_atomic (ompi_group_t *group, int assert, ompi_win_t *win
                                      "from %d processes", peer->rank, (int) (group_size - state->num_post_msgs - 1));
                     opal_list_remove_item (&module->pending_posts, &pending_post->super);
                     OBJ_RELEASE(pending_post);
-                    /* only one thread can process post messages so there is no need of atomics here */
-                    ++state->num_post_msgs;
+                    ompi_osc_rdma_counter_add (&state->num_post_msgs, 1);
                     break;
                 }
             }
@@ -393,16 +424,7 @@ int ompi_osc_rdma_start_atomic (ompi_group_t *group, int assert, ompi_win_t *win
         while (state->num_post_msgs != group_size) {
             OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "waiting for post messages. have %d of %d",
                              (int) state->num_post_msgs, group_size);
-            for (int i = 0 ; i < OMPI_OSC_RDMA_POST_PEER_MAX ; ++i) {
-                /* no post at this index (yet) */
-                if (0 == state->post_peers[i]) {
-                    continue;
-                }
-
-                ompi_osc_rdma_handle_post (module, state->post_peers[i] - 1, sync->peer_list.peers, group_size);
-                state->post_peers[i] = 0;
-            }
-
+            ompi_osc_rdma_check_posts (module);
             ompi_osc_rdma_progress (module);
         }
     } else {
@@ -419,9 +441,7 @@ int ompi_osc_rdma_complete_atomic (ompi_win_t *win)
 {
     ompi_osc_rdma_module_t *module = GET_MODULE(win);
     ompi_osc_rdma_sync_t *sync = &module->all_sync;
-    ompi_osc_rdma_frag_t *frag = NULL;
     ompi_osc_rdma_peer_t **peers;
-    void *scratch_lock = NULL;
     ompi_group_t *group;
     int group_size, ret;
 
@@ -456,45 +476,19 @@ int ompi_osc_rdma_complete_atomic (ompi_win_t *win)
 
     ompi_osc_rdma_sync_rdma_complete (sync);
 
-    if (!(MCA_BTL_FLAGS_ATOMIC_OPS & module->selected_btl->btl_flags)) {
-        /* need a temporary buffer for performing fetching atomics */
-        ret = ompi_osc_rdma_frag_alloc (module, 8, &frag, (char **) &scratch_lock);
-        if (OPAL_UNLIKELY(OPAL_SUCCESS != ret)) {
-            return ret;
-        }
-    }
-
     /* for each process in the group increment their number of complete messages */
     for (int i = 0 ; i < group_size ; ++i) {
         ompi_osc_rdma_peer_t *peer = peers[i];
         intptr_t target = (intptr_t) peer->state + offsetof (ompi_osc_rdma_state_t, num_complete_msgs);
 
         if (!ompi_osc_rdma_peer_local_state (peer)) {
-            do {
-                if (MCA_BTL_FLAGS_ATOMIC_OPS & module->selected_btl->btl_flags) {
-                    ret = module->selected_btl->btl_atomic_op (module->selected_btl, peer->state_endpoint, target, peer->state_handle,
-                                                               MCA_BTL_ATOMIC_ADD, 1, 0, MCA_BTL_NO_ORDER,
-                                                               ompi_osc_rdma_atomic_complete, NULL, NULL);
-                } else {
-                    /* don't care about the read value so use the scratch lock */
-                    ret = module->selected_btl->btl_atomic_fop (module->selected_btl, peer->state_endpoint, scratch_lock,
-                                                                target, frag->handle, peer->state_handle, MCA_BTL_ATOMIC_ADD, 1,
-                                                                0, MCA_BTL_NO_ORDER, ompi_osc_rdma_atomic_complete, NULL, NULL);
-                }
-
-                if (OPAL_LIKELY(OMPI_SUCCESS == ret)) {
-                    break;
-                }
-            } while (1);
+            ret = ompi_osc_rdma_lock_btl_op (module, peer, target, MCA_BTL_ATOMIC_ADD, 1, true);
+            assert (OMPI_SUCCESS == ret);
         } else {
             (void) ompi_osc_rdma_counter_add ((osc_rdma_counter_t *) target, 1);
         }
     }
 
-    if (frag) {
-        ompi_osc_rdma_frag_complete (frag);
-    }
-
     /* release our reference to peers in this group */
     ompi_osc_rdma_release_peers (peers, group_size);
 
@@ -531,7 +525,6 @@ int ompi_osc_rdma_wait_atomic (ompi_win_t *win)
     }
 
     OPAL_THREAD_LOCK(&module->lock);
-    state->num_complete_msgs = 0;
     group = module->pw_group;
     module->pw_group = NULL;
     OPAL_THREAD_UNLOCK(&module->lock);
@@ -582,6 +575,8 @@ int ompi_osc_rdma_test_atomic (ompi_win_t *win, int *flag)
 
     OBJ_RELEASE(group);
 
+    OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "test complete. returning flag: true");
+
     return OMPI_SUCCESS;
 }
 
@@ -598,6 +593,8 @@ int ompi_osc_rdma_fence_atomic (int assert, ompi_win_t *win)
         return OMPI_ERR_RMA_SYNC;
     }
 
+    /* NTH: locking here isn't really needed per-se but it may make user synchronization errors more
+     * predicable. if the user is using RMA correctly then there will be no contention on this lock. */
     OPAL_THREAD_LOCK(&module->lock);
 
     /* active sends are now active (we will close the epoch if NOSUCCEED is specified) */
@@ -609,23 +606,18 @@ int ompi_osc_rdma_fence_atomic (int assert, ompi_win_t *win)
     }
 
     /* technically it is possible to enter a lock epoch (which will close the fence epoch) if
-     * no communication has occurred. this flag will be set on the next put, get, accumulate, etc. */
+     * no communication has occurred. this flag will be set to true on the next put, get,
+     * accumulate, etc if no other synchronization call is made. <sarcasm> yay fence </sarcasm> */
     module->all_sync.epoch_active = false;
 
-    /* short-circuit the noprecede case */
-    if (0 != (assert & MPI_MODE_NOPRECEDE)) {
-        OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "fence complete (short circuit)");
-        /* no communication can occur until a peer has entered the same fence epoch. for now
-         * a barrier is used to ensure this is the case. */
-        ret = module->comm->c_coll.coll_barrier(module->comm, module->comm->c_coll.coll_barrier_module);
-        OPAL_THREAD_UNLOCK(&module->lock);
-        return ret;
-    }
+    /* there really is no practical difference between NOPRECEDE and the normal case. in both cases there
+     * may be local stores that will not be visible as they should if we do not barrier. since that is the
+     * case there is no optimization for NOPRECEDE */
 
     ompi_osc_rdma_sync_rdma_complete (&module->all_sync);
 
-    /* ensure all writes to my memory are complete */
-    ret = module->comm->c_coll.coll_barrier(module->comm, module->comm->c_coll.coll_barrier_module);
+    /* ensure all writes to my memory are complete (both local stores, and RMA operations) */
+    ret = module->comm->c_coll->coll_barrier(module->comm, module->comm->c_coll->coll_barrier_module);
 
     if (assert & MPI_MODE_NOSUCCEED) {
         /* as specified in MPI-3 p 438 3-5 the fence can end an epoch. it isn't explicitly
