@@ -2,20 +2,22 @@
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University.
  *                         All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2018 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2008 University of Houston.  All rights reserved.
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012-2015 Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2015      NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2015      Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
+ * Copyright (c) 2018      Cisco Systems, Inc.  All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -43,6 +45,7 @@
 #if OPAL_CUDA_SUPPORT
 #include "opal/datatype/opal_datatype_cuda.h"
 #endif /* OPAL_CUDA_SUPPORT */
+#include "opal/util/info_subscriber.h"
 
 #include "ompi/info/info.h"
 #include "ompi/communicator/communicator.h"
@@ -53,23 +56,34 @@
 #include "opal/mca/btl/base/base.h"
 #include "opal/mca/base/mca_base_pvar.h"
 #include "ompi/mca/bml/base/base.h"
+#include "ompi/mca/mtl/base/base.h"
 
 static int ompi_osc_rdma_component_register (void);
 static int ompi_osc_rdma_component_init (bool enable_progress_threads, bool enable_mpi_threads);
 static int ompi_osc_rdma_component_finalize (void);
 static int ompi_osc_rdma_component_query (struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                                          struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                                          struct ompi_communicator_t *comm, struct opal_info_t *info,
                                           int flavor);
 static int ompi_osc_rdma_component_select (struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                                           struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                                           struct ompi_communicator_t *comm, struct opal_info_t *info,
                                            int flavor, int *model);
-
-static int ompi_osc_rdma_set_info (struct ompi_win_t *win, struct ompi_info_t *info);
-static int ompi_osc_rdma_get_info (struct ompi_win_t *win, struct ompi_info_t **info_used);
-
+#if 0  // stale code?
+static int ompi_osc_rdma_set_info (struct ompi_win_t *win, struct opal_info_t *info);
+static int ompi_osc_rdma_get_info (struct ompi_win_t *win, struct opal_info_t **info_used);
+#endif
 static int ompi_osc_rdma_query_btls (ompi_communicator_t *comm, struct mca_btl_base_module_t **btl);
+static int ompi_osc_rdma_query_mtls (void);
+
+static char* ompi_osc_rdma_set_no_lock_info(opal_infosubscriber_t *obj, char *key, char *value);
 
 static char *ompi_osc_rdma_btl_names;
+static char *ompi_osc_rdma_mtl_names;
+
+static const mca_base_var_enum_value_t ompi_osc_rdma_locking_modes[] = {
+    {.value = OMPI_OSC_RDMA_LOCKING_TWO_LEVEL, .string = "two_level"},
+    {.value = OMPI_OSC_RDMA_LOCKING_ON_DEMAND, .string = "on_demand"},
+    {.string = NULL},
+};
 
 ompi_osc_rdma_component_t mca_osc_rdma_component = {
     .super = {
@@ -126,21 +140,18 @@ ompi_osc_base_module_t ompi_osc_rdma_module_rdma_template = {
     .osc_flush_all = ompi_osc_rdma_flush_all,
     .osc_flush_local = ompi_osc_rdma_flush_local,
     .osc_flush_local_all = ompi_osc_rdma_flush_local_all,
-
-    .osc_set_info = ompi_osc_rdma_set_info,
-    .osc_get_info = ompi_osc_rdma_get_info
 };
 
 /* look up parameters for configuring this window.  The code first
    looks in the info structure passed by the user, then it checks
    for a matching MCA variable. */
-static bool check_config_value_bool (char *key, ompi_info_t *info)
+static bool check_config_value_bool (char *key, opal_info_t *info)
 {
     int ret, flag, param;
     bool result = false;
     const bool *flag_value = &result;
 
-    ret = ompi_info_get_bool (info, key, &result, &flag);
+    ret = opal_info_get_bool (info, key, &result, &flag);
     if (OMPI_SUCCESS == ret && flag) {
         return result;
     }
@@ -166,62 +177,109 @@ static int ompi_osc_rdma_pvar_read (const struct mca_base_pvar_t *pvar, void *va
 
 static int ompi_osc_rdma_component_register (void)
 {
+    char *description_str;
+    mca_base_var_enum_t *new_enum;
+
     mca_osc_rdma_component.no_locks = false;
-    (void) mca_base_component_var_register(&mca_osc_rdma_component.super.osc_version,
-                                           "no_locks", "Enable optimizations available only if MPI_LOCK is "
-                                           "not used. Info key of same name overrides this value (default: false)",
+    asprintf(&description_str, "Enable optimizations available only if MPI_LOCK is "
+             "not used. Info key of same name overrides this value (default: %s)",
+             mca_osc_rdma_component.no_locks  ? "true" : "false");
+    (void) mca_base_component_var_register(&mca_osc_rdma_component.super.osc_version, "no_locks", description_str,
                                            MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_5,
                                            MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.no_locks);
+    free(description_str);
 
     mca_osc_rdma_component.acc_single_intrinsic = false;
+    asprintf(&description_str, "Enable optimizations for MPI_Fetch_and_op, MPI_Accumulate, etc for codes "
+             "that will not use anything more than a single predefined datatype (default: %s)",
+             mca_osc_rdma_component.acc_single_intrinsic  ? "true" : "false");
     (void) mca_base_component_var_register(&mca_osc_rdma_component.super.osc_version, "acc_single_intrinsic",
-                                           "Enable optimizations for MPI_Fetch_and_op, MPI_Accumulate, etc for codes "
-                                           "that will not use anything more than a single predefined datatype (default: false)",
-                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_5,
+                                           description_str, MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_5,
                                            MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.acc_single_intrinsic);
+    free(description_str);
 
     mca_osc_rdma_component.acc_use_amo = true;
-    (void) mca_base_component_var_register(&mca_osc_rdma_component.super.osc_version, "acc_use_amo",
-                                           "Enable the use of network atomic memory operations when using single "
-                                           "intrinsic optimizations. If not set network compare-and-swap will be "
-                                           "used instread (default: true)", MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_5,
-                                           MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.acc_use_amo);
+    asprintf(&description_str, "Enable the use of network atomic memory operations when using single "
+             "intrinsic optimizations. If not set network compare-and-swap will be "
+             "used instread (default: %s)", mca_osc_rdma_component.acc_use_amo ? "true" : "false");
+    (void) mca_base_component_var_register(&mca_osc_rdma_component.super.osc_version, "acc_use_amo", description_str,
+                                           MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_GROUP,
+                                           &mca_osc_rdma_component.acc_use_amo);
+    free(description_str);
 
     mca_osc_rdma_component.buffer_size = 32768;
-    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "buffer_size",
-                                            "Size of temporary buffers (default: 32k)", MCA_BASE_VAR_TYPE_UNSIGNED_INT,
-                                            NULL, 0, 0, OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_LOCAL,
-                                            &mca_osc_rdma_component.buffer_size);
+    asprintf(&description_str, "Size of temporary buffers (default: %d)", mca_osc_rdma_component.buffer_size);
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "buffer_size", description_str,
+                                            MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                            MCA_BASE_VAR_SCOPE_LOCAL, &mca_osc_rdma_component.buffer_size);
+    free(description_str);
 
     mca_osc_rdma_component.max_attach = 32;
-    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "max_attach",
-                                            "Maximum number of buffers that can be attached to a dynamic window. "
-                                            "Keep in mind that each attached buffer will use a potentially limited "
-                                            "resource (default: 32)", MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0,
-                                            OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.max_attach);
+    asprintf(&description_str, "Maximum number of buffers that can be attached to a dynamic window. "
+             "Keep in mind that each attached buffer will use a potentially limited "
+             "resource (default: %d)", mca_osc_rdma_component.max_attach);
+   (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "max_attach", description_str,
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                           MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.max_attach);
+    free(description_str);
 
     mca_osc_rdma_component.aggregation_limit = 1024;
+    asprintf(&description_str, "Maximum size of an aggregated put/get. Messages are aggregated for consecutive"
+             "put and get operations. In some cases this may lead to higher latency but "
+             "should also lead to higher bandwidth utilization. Set to 0 to disable (default: %d)",
+             mca_osc_rdma_component.aggregation_limit);
     (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "aggregation_limit",
-                                            "Maximum size of an aggregated put/get. Messages are aggregated for consecutive"
-                                            "put and get operations. In some cases this may lead to higher latency but "
-                                            "should also lead to higher bandwidth utilization. Set to 0 to disable (default:"
-                                            " 1k)", MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                            description_str, MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0, OPAL_INFO_LVL_3,
                                             MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.aggregation_limit);
+    free(description_str);
 
-    mca_osc_rdma_component.priority = 90;
-    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "priority",
-                                            "Priority of the osc/rdma component (default: 90)",
+    mca_osc_rdma_component.priority = 101;
+    asprintf(&description_str, "Priority of the osc/rdma component (default: %d)",
+             mca_osc_rdma_component.priority);
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "priority", description_str,
                                             MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0, OPAL_INFO_LVL_3,
                                             MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.priority);
+    free(description_str);
 
-    ompi_osc_rdma_btl_names = "openib,ugni";
-    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "btls",
-                                            "Comma-delimited list of BTL component names to allow without verifying "
-                                            "connectivity. Do not add a BTL to to this list unless it can reach all "
-                                            "processes in any communicator used with an MPI window (default: openib,ugni)",
+    (void) mca_base_var_enum_create ("osc_rdma_locking_mode", ompi_osc_rdma_locking_modes, &new_enum);
+
+    mca_osc_rdma_component.locking_mode = OMPI_OSC_RDMA_LOCKING_TWO_LEVEL;
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "locking_mode",
+                                            "Locking mode to use for passive-target synchronization (default: two_level)",
+                                            MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0, OPAL_INFO_LVL_3,
+                                            MCA_BASE_VAR_SCOPE_GROUP, &mca_osc_rdma_component.locking_mode);
+    OBJ_RELEASE(new_enum);
+
+    ompi_osc_rdma_btl_names = "openib,ugni,uct,ucp";
+    asprintf(&description_str, "Comma-delimited list of BTL component names to allow without verifying "
+             "connectivity. Do not add a BTL to to this list unless it can reach all "
+             "processes in any communicator used with an MPI window (default: %s)",
+             ompi_osc_rdma_btl_names);
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "btls", description_str,
                                             MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
                                             MCA_BASE_VAR_SCOPE_GROUP, &ompi_osc_rdma_btl_names);
+    free(description_str);
 
+    ompi_osc_rdma_mtl_names = "psm2";
+    asprintf(&description_str, "Comma-delimited list of MTL component names to lower the priority of rdma "
+             "osc component favoring pt2pt osc (default: %s)", ompi_osc_rdma_mtl_names);
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "mtls", description_str,
+                                            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                            MCA_BASE_VAR_SCOPE_GROUP, &ompi_osc_rdma_mtl_names);
+    free(description_str);
+
+    if (0 == access ("/dev/shm", W_OK)) {
+        mca_osc_rdma_component.backing_directory = "/dev/shm";
+    } else {
+        mca_osc_rdma_component.backing_directory = ompi_process_info.proc_session_dir;
+    }
+
+    (void) mca_base_component_var_register (&mca_osc_rdma_component.super.osc_version, "backing_directory",
+                                            "Directory to place backing files for memory windows. "
+                                            "This directory should be on a local filesystem such as /tmp or "
+                                            "/dev/shm (default: (linux) /dev/shm, (others) session directory)",
+                                            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                            MCA_BASE_VAR_SCOPE_READONLY, &mca_osc_rdma_component.backing_directory);
 
     /* register performance variables */
 
@@ -322,7 +380,7 @@ int ompi_osc_rdma_component_finalize (void)
 
 
 static int ompi_osc_rdma_component_query (struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                                          struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                                          struct ompi_communicator_t *comm, struct opal_info_t *info,
                                           int flavor)
 {
 
@@ -339,6 +397,10 @@ static int ompi_osc_rdma_component_query (struct ompi_win_t *win, void **base, s
     }
 #endif /* OPAL_CUDA_SUPPORT */
 
+    if (OMPI_SUCCESS == ompi_osc_rdma_query_mtls ()) {
+        return 5; /* this has to be lower that osc pt2pt default priority */
+    }
+
     if (OMPI_SUCCESS != ompi_osc_rdma_query_btls (comm, NULL)) {
         return -1;
     }
@@ -393,7 +455,8 @@ static int allocate_state_single (ompi_osc_rdma_module_t *module, void **base, s
 
     /* allocate anything that will be accessed remotely in the same region. this cuts down on the number of
      * registration handles needed to access this data. */
-    total_size = module->state_size + local_rank_array_size + leader_peer_data_size;
+    total_size = local_rank_array_size + module->region_size +
+        module->state_size + leader_peer_data_size;
 
     if (MPI_WIN_FLAVOR_ALLOCATE == module->flavor) {
         total_size += size;
@@ -408,7 +471,11 @@ static int allocate_state_single (ompi_osc_rdma_module_t *module, void **base, s
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
 
-    module->state_offset = local_rank_array_size;
+// Note, the extra module->region_size space added after local_rank_array_size
+// is unused but is there to match what happens in allocte_state_shared()
+// This allows module->state_offset to be uniform across the ranks which
+// is part of how they pull peer info from each other.
+    module->state_offset = local_rank_array_size + module->region_size;
 
     module->state = (ompi_osc_rdma_state_t *) ((intptr_t) module->rank_array + module->state_offset);
     module->node_comm_info = (unsigned char *) ((intptr_t) module->state + module->state_size);
@@ -442,11 +509,12 @@ static int allocate_state_single (ompi_osc_rdma_module_t *module, void **base, s
         return ret;
     }
 
+    module->my_peer = my_peer;
     module->free_after = module->rank_array;
     my_peer->flags |= OMPI_OSC_RDMA_PEER_LOCAL_BASE;
     my_peer->state = (uint64_t) (uintptr_t) module->state;
 
-    if (module->selected_btl->btl_flags & MCA_BTL_ATOMIC_SUPPORTS_GLOB) {
+    if (module->use_cpu_atomics) {
         /* all peers are local or it is safe to mix cpu and nic atomics */
         my_peer->flags |= OMPI_OSC_RDMA_PEER_LOCAL_STATE;
     } else {
@@ -464,8 +532,13 @@ static int allocate_state_single (ompi_osc_rdma_module_t *module, void **base, s
             ex_peer->size = size;
         }
 
-        if (MPI_WIN_FLAVOR_ALLOCATE == module->flavor) {
-            ex_peer->super.base_handle = module->state_handle;
+        if (!module->use_cpu_atomics) {
+            if (MPI_WIN_FLAVOR_ALLOCATE == module->flavor) {
+                /* base is local and cpu atomics are available */
+                ex_peer->super.base_handle = module->state_handle;
+            } else {
+                ex_peer->super.base_handle = module->base_handle;
+            }
         }
     }
 
@@ -483,11 +556,10 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
     unsigned long offset, total_size;
     unsigned long state_base, data_base;
     int local_rank, local_size, ret;
-    size_t local_rank_array_size, leader_peer_data_size;
+    size_t local_rank_array_size, leader_peer_data_size, my_base_offset = 0;
     int my_rank = ompi_comm_rank (module->comm);
     int global_size = ompi_comm_size (module->comm);
     ompi_osc_rdma_region_t *state_region;
-    int my_base_offset = 0;
     struct _local_data *temp;
     char *data_file;
 
@@ -496,6 +568,9 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
     local_rank = ompi_comm_rank (shared_comm);
     local_size = ompi_comm_size (shared_comm);
 
+    /* CPU atomics can be used if every process is on the same node or the NIC allows mixing CPU and NIC atomics */
+    module->use_cpu_atomics = local_size == global_size || (module->selected_btl->btl_flags & MCA_BTL_ATOMIC_SUPPORTS_GLOB);
+
     if (1 == local_size) {
         /* no point using a shared segment if there are no other processes on this node */
         return allocate_state_single (module, base, size);
@@ -521,8 +596,8 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
         temp[local_rank].size = size;
 
         /* gather the local sizes and ranks */
-        ret = shared_comm->c_coll.coll_allgather (MPI_IN_PLACE, sizeof (*temp), MPI_BYTE, temp, sizeof (*temp),
-                                                  MPI_BYTE, shared_comm, shared_comm->c_coll.coll_allgather_module);
+        ret = shared_comm->c_coll->coll_allgather (MPI_IN_PLACE, sizeof (*temp), MPI_BYTE, temp, sizeof (*temp),
+                                                  MPI_BYTE, shared_comm, shared_comm->c_coll->coll_allgather_module);
         if (OMPI_SUCCESS != ret) {
             break;
         }
@@ -539,9 +614,9 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
         }
 
         /* allocate the shared memory segment */
-        ret = asprintf (&data_file, "%s"OPAL_PATH_SEP"window_%d.%s",
-                        ompi_process_info.job_session_dir, ompi_comm_get_cid (module->comm),
-                        ompi_process_info.nodename);
+        ret = asprintf (&data_file, "%s" OPAL_PATH_SEP "osc_rdma.%s.%x.%d",
+                        mca_osc_rdma_component.backing_directory, ompi_process_info.nodename,
+                        OMPI_PROC_MY_NAME->jobid, ompi_comm_get_cid(module->comm));
         if (0 > ret) {
             ret = OMPI_ERR_OUT_OF_RESOURCE;
             break;
@@ -557,8 +632,8 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
             }
         }
 
-        ret = module->comm->c_coll.coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
-                                               shared_comm, shared_comm->c_coll.coll_bcast_module);
+        ret = module->comm->c_coll->coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
+                                               shared_comm, shared_comm->c_coll->coll_bcast_module);
         if (OMPI_SUCCESS != ret) {
             break;
         }
@@ -607,7 +682,7 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
         }
 
         /* barrier to make sure all ranks have attached */
-        shared_comm->c_coll.coll_barrier(shared_comm, shared_comm->c_coll.coll_barrier_module);
+        shared_comm->c_coll->coll_barrier(shared_comm, shared_comm->c_coll->coll_barrier_module);
 
         /* unlink the shared memory backing file */
         if (0 == local_rank) {
@@ -625,13 +700,15 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
             }
         }
 
-        /* barrier to make sure all ranks have attached */
-        shared_comm->c_coll.coll_barrier(shared_comm, shared_comm->c_coll.coll_barrier_module);
+        /* barrier to make sure all ranks have set up their region data */
+        shared_comm->c_coll->coll_barrier(shared_comm, shared_comm->c_coll->coll_barrier_module);
 
         offset = data_base;
         for (int i = 0 ; i < local_size ; ++i) {
+            /* local pointer to peer's state */
+            ompi_osc_rdma_state_t *peer_state = (ompi_osc_rdma_state_t *) ((uintptr_t) module->segment_base + state_base + module->state_size * i);
+            ompi_osc_rdma_region_t *peer_region = (ompi_osc_rdma_region_t *) peer_state->regions;
             ompi_osc_rdma_peer_extended_t *ex_peer;
-            ompi_osc_rdma_state_t *peer_state;
             ompi_osc_rdma_peer_t *peer;
             int peer_rank = temp[i].rank;
 
@@ -642,13 +719,12 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
 
             ex_peer = (ompi_osc_rdma_peer_extended_t *) peer;
 
-            /* peer state local pointer */
-            peer_state = (ompi_osc_rdma_state_t *) ((uintptr_t) module->segment_base + state_base + module->state_size * i);
-
-            if (local_size == global_size || (module->selected_btl->btl_flags & MCA_BTL_ATOMIC_SUPPORTS_GLOB)) {
+            /* set up peer state */
+            if (module->use_cpu_atomics) {
                 /* all peers are local or it is safe to mix cpu and nic atomics */
                 peer->flags |= OMPI_OSC_RDMA_PEER_LOCAL_STATE;
                 peer->state = (osc_rdma_counter_t) peer_state;
+                peer->state_endpoint = NULL;
             } else {
                 /* use my endpoint handle to modify the peer's state */
                 if (module->selected_btl->btl_register_mem) {
@@ -658,38 +734,43 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
                 peer->state_endpoint = ompi_osc_rdma_peer_btl_endpoint (module, temp[0].rank);
             }
 
-            /* finish setting up the local peer structure */
-            if (MPI_WIN_FLAVOR_DYNAMIC != module->flavor) {
-                if (!module->same_disp_unit) {
-                    ex_peer->disp_unit = peer_state->disp_unit;
-                }
+            if (my_rank == peer_rank) {
+	        module->my_peer = peer;
+            }
 
-                if (!module->same_size) {
-                    ex_peer->size = temp[i].size;
-                }
+            if (MPI_WIN_FLAVOR_DYNAMIC == module->flavor || MPI_WIN_FLAVOR_CREATE == module->flavor) {
+                /* use the peer's BTL endpoint directly */
+                peer->data_endpoint = ompi_osc_rdma_peer_btl_endpoint (module, peer_rank);
+            } else if (!module->use_cpu_atomics && temp[i].size) {
+                /* use the local leader's endpoint */
+                peer->data_endpoint = ompi_osc_rdma_peer_btl_endpoint (module, temp[0].rank);
+            }
 
-                if (my_rank == peer_rank) {
-                    peer->flags |= OMPI_OSC_RDMA_PEER_LOCAL_BASE;
-                }
+            ompi_osc_module_add_peer (module, peer);
 
-                if (MPI_WIN_FLAVOR_ALLOCATE == module->flavor) {
-                    if (temp[i].size) {
-                        ex_peer->super.base = state_region->base + offset;
-                        offset += temp[i].size;
-                    } else {
-                        ex_peer->super.base = 0;
-                    }
-                }
+            if (MPI_WIN_FLAVOR_DYNAMIC == module->flavor || 0 == temp[i].size) {
+                /* nothing more to do */
+                continue;
+            }
 
-                ompi_osc_rdma_region_t *peer_region = (ompi_osc_rdma_region_t *) peer_state->regions;
+            /* finish setting up the local peer structure for win allocate/create */
+            if (!(module->same_disp_unit && module->same_size)) {
+                ex_peer->disp_unit = peer_state->disp_unit;
+                ex_peer->size = temp[i].size;
+            }
 
+            if (module->use_cpu_atomics && MPI_WIN_FLAVOR_ALLOCATE == module->flavor) {
+                /* base is local and cpu atomics are available */
+                ex_peer->super.base = (uintptr_t) module->segment_base + offset;
+                peer->flags |= OMPI_OSC_RDMA_PEER_LOCAL_BASE;
+                offset += temp[i].size;
+            } else {
                 ex_peer->super.base = peer_region->base;
+
                 if (module->selected_btl->btl_register_mem) {
                     ex_peer->super.base_handle = (mca_btl_base_registration_handle_t *) peer_region->btl_handle_data;
                 }
             }
-
-            ompi_osc_module_add_peer (module, peer);
         }
     } while (0);
 
@@ -698,6 +779,23 @@ static int allocate_state_shared (ompi_osc_rdma_module_t *module, void **base, s
     return ret;
 }
 
+static int ompi_osc_rdma_query_mtls (void)
+{
+    char **mtls_to_use;
+
+    mtls_to_use = opal_argv_split (ompi_osc_rdma_mtl_names, ',');
+    if (mtls_to_use && ompi_mtl_base_selected_component) {
+	for (int i = 0 ; mtls_to_use[i] ; ++i) {
+	    if (0 == strcmp (mtls_to_use[i], ompi_mtl_base_selected_component->mtl_version.mca_component_name)) {
+                opal_argv_free(mtls_to_use);
+		return OMPI_SUCCESS;
+	    }
+	}
+    }
+    opal_argv_free(mtls_to_use);
+    return -1;
+}
+
 static int ompi_osc_rdma_query_btls (ompi_communicator_t *comm, struct mca_btl_base_module_t **btl)
 {
     struct mca_btl_base_module_t **possible_btls = NULL;
@@ -810,11 +908,18 @@ static int ompi_osc_rdma_query_btls (ompi_communicator_t *comm, struct mca_btl_b
     }
 
     for (int i = 0 ; i < max_btls ; ++i) {
+        int btl_count = btl_counts[i];
+
         if (NULL == possible_btls[i]) {
             break;
         }
 
-        if (btl_counts[i] == comm_size && possible_btls[i]->btl_latency < selected_latency) {
+        if (possible_btls[i]->btl_atomic_flags & MCA_BTL_ATOMIC_SUPPORTS_GLOB) {
+            /* do not need to use the btl for self communication */
+            btl_count++;
+        }
+
+        if (btl_count >= comm_size && possible_btls[i]->btl_latency < selected_latency) {
             selected_btl = possible_btls[i];
             selected_latency = possible_btls[i]->btl_latency;
         }
@@ -858,8 +963,8 @@ static int ompi_osc_rdma_share_data (ompi_osc_rdma_module_t *module)
         temp[my_rank].node_id = module->node_id;
         temp[my_rank].rank = ompi_comm_rank (module->shared_comm);
 
-        ret = module->comm->c_coll.coll_allgather (MPI_IN_PLACE, 1, MPI_2INT, temp, 1, MPI_2INT,
-                                                   module->comm, module->comm->c_coll.coll_allgather_module);
+        ret = module->comm->c_coll->coll_allgather (MPI_IN_PLACE, 1, MPI_2INT, temp, 1, MPI_2INT,
+                                                   module->comm, module->comm->c_coll->coll_allgather_module);
         if (OMPI_SUCCESS != ret) {
             break;
         }
@@ -879,9 +984,9 @@ static int ompi_osc_rdma_share_data (ompi_osc_rdma_module_t *module)
 
             /* gather state data at each node leader */
             if (ompi_comm_size (module->local_leaders) > 1) {
-                ret = module->local_leaders->c_coll.coll_allgather (MPI_IN_PLACE, module->region_size, MPI_BYTE, module->node_comm_info,
+                ret = module->local_leaders->c_coll->coll_allgather (MPI_IN_PLACE, module->region_size, MPI_BYTE, module->node_comm_info,
                                                                     module->region_size, MPI_BYTE, module->local_leaders,
-                                                                    module->local_leaders->c_coll.coll_allgather_module);
+                                                                    module->local_leaders->c_coll->coll_allgather_module);
                 if (OMPI_SUCCESS != ret) {
                     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_ERROR, "leader allgather failed with ompi error code %d", ret);
                     break;
@@ -905,8 +1010,8 @@ static int ompi_osc_rdma_share_data (ompi_osc_rdma_module_t *module)
     } while (0);
 
 
-    ret = module->comm->c_coll.coll_allreduce (&ret, &global_result, 1, MPI_INT, MPI_MIN, module->comm,
-                                               module->comm->c_coll.coll_allreduce_module);
+    ret = module->comm->c_coll->coll_allreduce (&ret, &global_result, 1, MPI_INT, MPI_MIN, module->comm,
+                                               module->comm->c_coll->coll_allreduce_module);
 
     if (OMPI_SUCCESS != ret) {
         global_result = ret;
@@ -953,8 +1058,8 @@ static int ompi_osc_rdma_create_groups (ompi_osc_rdma_module_t *module)
     }
 
     if (ompi_comm_size (module->shared_comm) > 1) {
-        ret = module->shared_comm->c_coll.coll_bcast (values, 2, MPI_INT, 0, module->shared_comm,
-                                                      module->shared_comm->c_coll.coll_bcast_module);
+        ret = module->shared_comm->c_coll->coll_bcast (values, 2, MPI_INT, 0, module->shared_comm,
+                                                      module->shared_comm->c_coll->coll_bcast_module);
         if (OMPI_SUCCESS != ret) {
             OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_ERROR, "failed to broadcast local data. error code %d", ret);
             return ret;
@@ -993,8 +1098,8 @@ static int ompi_osc_rdma_check_parameters (ompi_osc_rdma_module_t *module, int d
     values[2] = size;
     values[3] = -(ssize_t) size;
 
-    ret = module->comm->c_coll.coll_allreduce (MPI_IN_PLACE, values, 4, MPI_LONG, MPI_MIN, module->comm,
-                                               module->comm->c_coll.coll_allreduce_module);
+    ret = module->comm->c_coll->coll_allreduce (MPI_IN_PLACE, values, 4, MPI_LONG, MPI_MIN, module->comm,
+                                               module->comm->c_coll->coll_allreduce_module);
     if (OMPI_SUCCESS != ret) {
         return ret;
     }
@@ -1014,7 +1119,7 @@ static int ompi_osc_rdma_check_parameters (ompi_osc_rdma_module_t *module, int d
 
 
 static int ompi_osc_rdma_component_select (struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                                           struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                                           struct ompi_communicator_t *comm, struct opal_info_t *info,
                                            int flavor, int *model)
 {
     ompi_osc_rdma_module_t *module = NULL;
@@ -1045,7 +1150,8 @@ static int ompi_osc_rdma_component_select (struct ompi_win_t *win, void **base,
     module->same_disp_unit = check_config_value_bool ("same_disp_unit", info);
     module->same_size      = check_config_value_bool ("same_size", info);
     module->no_locks       = check_config_value_bool ("no_locks", info);
-    module->acc_single_intrinsic = check_config_value_bool ("ompi_single_accumulate", info);
+    module->locking_mode   = mca_osc_rdma_component.locking_mode;
+    module->acc_single_intrinsic = check_config_value_bool ("acc_single_intrinsic", info);
     module->acc_use_amo = mca_osc_rdma_component.acc_use_amo;
 
     module->all_sync.module = module;
@@ -1117,6 +1223,15 @@ static int ompi_osc_rdma_component_select (struct ompi_win_t *win, void **base,
     } else {
         module->state_size += mca_osc_rdma_component.max_attach * module->region_size;
     }
+/*
+ * These are the info's that this module is interested in
+ */
+    opal_infosubscribe_subscribe(&win->super, "no_locks", "false", ompi_osc_rdma_set_no_lock_info);
+
+/*
+ * TODO: same_size, same_disp_unit have w_flag entries, but do not appear
+ * to be used anywhere.  If that changes, they should be subscribed
+ */
 
     /* fill in the function pointer part */
     memcpy(&module->super, &ompi_osc_rdma_module_rdma_template, sizeof(module->super));
@@ -1201,7 +1316,43 @@ static int ompi_osc_rdma_component_select (struct ompi_win_t *win, void **base,
 }
 
 
-static int ompi_osc_rdma_set_info (struct ompi_win_t *win, struct ompi_info_t *info)
+static char* ompi_osc_rdma_set_no_lock_info(opal_infosubscriber_t *obj, char *key, char *value)
+{
+
+    struct ompi_win_t *win = (struct ompi_win_t*) obj;
+    ompi_osc_rdma_module_t *module = GET_MODULE(win);
+    bool temp;
+
+    temp = opal_str_to_bool(value);
+    if (temp && !module->no_locks) {
+        /* clean up the lock hash. it is up to the user to ensure no lock is
+         * outstanding from this process when setting the info key */
+        OBJ_DESTRUCT(&module->outstanding_locks);
+        OBJ_CONSTRUCT(&module->outstanding_locks, opal_hash_table_t);
+
+        module->no_locks = true;
+    } else if (!temp && module->no_locks) {
+        int world_size = ompi_comm_size (module->comm);
+        int init_limit = world_size > 256 ? 256 : world_size;
+        int ret;
+
+        ret = opal_hash_table_init (&module->outstanding_locks, init_limit);
+        if (OPAL_SUCCESS != ret) {
+            module->no_locks = true;
+        }
+
+        module->no_locks = false;
+    }
+    /* enforce collectiveness... */
+    module->comm->c_coll->coll_barrier(module->comm, module->comm->c_coll->coll_barrier_module);
+/*
+ * Accept any value
+ */
+    return module->no_locks ? "true" : "false";
+}
+
+#if 0  // stale code?
+static int ompi_osc_rdma_set_info (struct ompi_win_t *win, struct opal_info_t *info)
 {
     ompi_osc_rdma_module_t *module = GET_MODULE(win);
     bool temp;
@@ -1230,14 +1381,14 @@ static int ompi_osc_rdma_set_info (struct ompi_win_t *win, struct ompi_info_t *i
     }
 
     /* enforce collectiveness... */
-    return module->comm->c_coll.coll_barrier(module->comm,
-                                             module->comm->c_coll.coll_barrier_module);
+    return module->comm->c_coll->coll_barrier(module->comm,
+                                             module->comm->c_coll->coll_barrier_module);
 }
 
 
-static int ompi_osc_rdma_get_info (struct ompi_win_t *win, struct ompi_info_t **info_used)
+static int ompi_osc_rdma_get_info (struct ompi_win_t *win, struct opal_info_t **info_used)
 {
-    ompi_info_t *info = OBJ_NEW(ompi_info_t);
+    opal_info_t *info = OBJ_NEW(opal_info_t);
 
     if (NULL == info) {
         return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
@@ -1247,5 +1398,5 @@ static int ompi_osc_rdma_get_info (struct ompi_win_t *win, struct ompi_info_t **
 
     return OMPI_SUCCESS;
 }
-
+#endif
 OBJ_CLASS_INSTANCE(ompi_osc_rdma_aggregation_t, opal_list_item_t, NULL, NULL);
