@@ -61,7 +61,8 @@ int ompi_osc_rdma_new_peer (struct ompi_osc_rdma_module_t *module, int peer_id,
     *peer_out = NULL;
 
     endpoint = ompi_osc_rdma_peer_btl_endpoint (module, peer_id);
-    if (OPAL_UNLIKELY(NULL == endpoint)) {
+    if (OPAL_UNLIKELY(NULL == endpoint && !((module->selected_btl->btl_atomic_flags & MCA_BTL_ATOMIC_SUPPORTS_GLOB) &&
+                                            peer_id == ompi_comm_rank (module->comm)))) {
         return OMPI_ERR_UNREACH;
     }
 
@@ -193,6 +194,7 @@ static int ompi_osc_rdma_peer_setup (ompi_osc_rdma_module_t *module, ompi_osc_rd
     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_DEBUG, "peer %d: remote base region: 0x%" PRIx64 ", size: %" PRId64
                      ", flags: 0x%x, disp_unit: %d", peer->rank, base_region->base, base_region->len,
                      peer->flags, disp_unit);
+    (void)disp_unit;  // silence compiler warning
 
     if (ompi_osc_rdma_peer_local_base (peer)) {
         /* for now we store the local address in the standard place. do no overwrite it */
@@ -301,7 +303,7 @@ static void ompi_osc_rdma_peer_destruct (ompi_osc_rdma_peer_t *peer)
     }
 }
 
-OBJ_CLASS_INSTANCE(ompi_osc_rdma_peer_t, opal_object_t,
+OBJ_CLASS_INSTANCE(ompi_osc_rdma_peer_t, opal_list_item_t,
                    ompi_osc_rdma_peer_construct,
                    ompi_osc_rdma_peer_destruct);
 
