@@ -1,12 +1,16 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2012      Sandia National Laboratories.  All rights reserved.
- * Copyright (c) 2014-2015 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2014-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -22,6 +26,7 @@
 #include "ompi/request/request.h"
 #include "opal/util/sys_limits.h"
 #include "opal/include/opal/align.h"
+#include "opal/util/info_subscriber.h"
 
 #include "osc_sm.h"
 
@@ -29,11 +34,14 @@ static int component_open(void);
 static int component_init(bool enable_progress_threads, bool enable_mpi_threads);
 static int component_finalize(void);
 static int component_query(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                           struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                           struct ompi_communicator_t *comm, struct opal_info_t *info,
                            int flavor);
+static int component_register (void);
 static int component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                            struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                            struct ompi_communicator_t *comm, struct opal_info_t *info,
                             int flavor, int *model);
+static char* component_set_blocking_fence_info(opal_infosubscriber_t *obj, char *key, char *val);
+static char* component_set_alloc_shared_noncontig_info(opal_infosubscriber_t *obj, char *key, char *val);
 
 
 ompi_osc_sm_component_t mca_osc_sm_component = {
@@ -44,6 +52,7 @@ ompi_osc_sm_component_t mca_osc_sm_component = {
             MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
                                   OMPI_RELEASE_VERSION),
             .mca_open_component = component_open,
+            .mca_register_component_params = component_register,
         },
         .osc_data = { /* mca_base_component_data */
             /* The component is not checkpoint ready */
@@ -95,12 +104,26 @@ ompi_osc_sm_module_t ompi_osc_sm_module_template = {
         .osc_flush_all = ompi_osc_sm_flush_all,
         .osc_flush_local = ompi_osc_sm_flush_local,
         .osc_flush_local_all = ompi_osc_sm_flush_local_all,
-
-        .osc_set_info = ompi_osc_sm_set_info,
-        .osc_get_info = ompi_osc_sm_get_info
     }
 };
 
+static int component_register (void)
+{
+    if (0 == access ("/dev/shm", W_OK)) {
+        mca_osc_sm_component.backing_directory = "/dev/shm";
+    } else {
+        mca_osc_sm_component.backing_directory = ompi_process_info.proc_session_dir;
+    }
+
+    (void) mca_base_component_var_register (&mca_osc_sm_component.super.osc_version, "backing_directory",
+                                            "Directory to place backing files for shared memory windows. "
+                                            "This directory should be on a local filesystem such as /tmp or "
+                                            "/dev/shm (default: (linux) /dev/shm, (others) session directory)",
+                                            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
+                                            MCA_BASE_VAR_SCOPE_READONLY, &mca_osc_sm_component.backing_directory);
+
+    return OPAL_SUCCESS;
+}
 
 static int
 component_open(void)
@@ -143,7 +166,7 @@ check_win_ok(ompi_communicator_t *comm, int flavor)
 
 static int
 component_query(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                struct ompi_communicator_t *comm, struct opal_info_t *info,
                 int flavor)
 {
     int ret;
@@ -160,11 +183,12 @@ component_query(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
 
 static int
 component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
-                 struct ompi_communicator_t *comm, struct ompi_info_t *info,
+                 struct ompi_communicator_t *comm, struct opal_info_t *info,
                  int flavor, int *model)
 {
     ompi_osc_sm_module_t *module = NULL;
     int comm_size = ompi_comm_size (comm);
+    bool unlink_needed = false;
     int ret = OMPI_ERROR;
 
     if (OMPI_SUCCESS != (ret = check_win_ok(comm, flavor))) {
@@ -176,8 +200,14 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         calloc(1, sizeof(ompi_osc_sm_module_t));
     if (NULL == module) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
+    win->w_osc_module = &module->super;
+
     OBJ_CONSTRUCT(&module->lock, opal_mutex_t);
 
+    ret = opal_infosubscribe_subscribe(&(win->super), "alloc_shared_contig", "false", component_set_alloc_shared_noncontig_info);
+
+    if (OPAL_SUCCESS != ret) goto error;
+
     /* fill in the function pointer part */
     memcpy(module, &ompi_osc_sm_module_template,
            sizeof(ompi_osc_base_module_t));
@@ -204,9 +234,9 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (NULL == module->global_state) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         module->node_states = malloc(sizeof(ompi_osc_sm_node_state_t));
         if (NULL == module->node_states) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-        module->posts = calloc (1, sizeof(module->posts[0]) + sizeof (uint64_t));
+        module->posts = calloc (1, sizeof(module->posts[0]) + sizeof (module->posts[0][0]));
         if (NULL == module->posts) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-        module->posts[0] = (uint64_t *) (module->posts + 1);
+        module->posts[0] = (osc_sm_post_type_t *) (module->posts + 1);
     } else {
         unsigned long total, *rbuf;
         int i, flag;
@@ -224,7 +254,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (NULL == rbuf) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
         module->noncontig = false;
-        if (OMPI_SUCCESS != ompi_info_get_bool(info, "alloc_shared_noncontig",
+        if (OMPI_SUCCESS != opal_info_get_bool(info, "alloc_shared_noncontig",
                                                &module->noncontig, &flag)) {
             goto error;
         }
@@ -234,10 +264,10 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         } else {
             total = size;
         }
-        ret = module->comm->c_coll.coll_allgather(&total, 1, MPI_UNSIGNED_LONG,
+        ret = module->comm->c_coll->coll_allgather(&total, 1, MPI_UNSIGNED_LONG,
                                                   rbuf, 1, MPI_UNSIGNED_LONG,
                                                   module->comm,
-                                                  module->comm->c_coll.coll_allgather_module);
+                                                  module->comm->c_coll->coll_allgather_module);
         if (OMPI_SUCCESS != ret) return ret;
 
         total = 0;
@@ -248,14 +278,14 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 	/* user opal/shmem directly to create a shared memory segment */
 	state_size = sizeof(ompi_osc_sm_global_state_t) + sizeof(ompi_osc_sm_node_state_t) * comm_size;
         state_size += OPAL_ALIGN_PAD_AMOUNT(state_size, 64);
-        posts_size = comm_size * post_size * sizeof (uint64_t);
+        posts_size = comm_size * post_size * sizeof (module->posts[0][0]);
         posts_size += OPAL_ALIGN_PAD_AMOUNT(posts_size, 64);
         if (0 == ompi_comm_rank (module->comm)) {
             char *data_file;
-            if (asprintf(&data_file, "%s"OPAL_PATH_SEP"shared_window_%d.%s",
-                         ompi_process_info.proc_session_dir,
-                         ompi_comm_get_cid(module->comm),
-                         ompi_process_info.nodename) < 0) {
+            ret = asprintf (&data_file, "%s" OPAL_PATH_SEP "osc_sm.%s.%x.%d.%d",
+                            mca_osc_sm_component.backing_directory, ompi_process_info.nodename,
+                            OMPI_PROC_MY_NAME->jobid, (int) OMPI_PROC_MY_NAME->vpid, ompi_comm_get_cid(module->comm));
+            if (ret < 0) {
                 return OMPI_ERR_OUT_OF_RESOURCE;
             }
 
@@ -264,10 +294,12 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
             if (OPAL_SUCCESS != ret) {
                 goto error;
             }
+
+            unlink_needed = true;
         }
 
-	ret = module->comm->c_coll.coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
-					       module->comm, module->comm->c_coll.coll_bcast_module);
+	ret = module->comm->c_coll->coll_bcast (&module->seg_ds, sizeof (module->seg_ds), MPI_BYTE, 0,
+					       module->comm, module->comm->c_coll->coll_bcast_module);
 	if (OMPI_SUCCESS != ret) {
 	    goto error;
 	}
@@ -277,6 +309,17 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 	    goto error;
 	}
 
+        /* wait for all processes to attach */
+	ret = module->comm->c_coll->coll_barrier (module->comm, module->comm->c_coll->coll_barrier_module);
+	if (OMPI_SUCCESS != ret) {
+	    goto error;
+	}
+
+        if (0 == ompi_comm_rank (module->comm)) {
+            opal_shmem_unlink (&module->seg_ds);
+            unlink_needed = false;
+        }
+
         module->sizes = malloc(sizeof(size_t) * comm_size);
         if (NULL == module->sizes) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
         module->bases = malloc(sizeof(void*) * comm_size);
@@ -285,7 +328,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (NULL == module->posts) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
         /* set module->posts[0] first to ensure 64-bit alignment */
-        module->posts[0] = (uint64_t *) (module->segment_base);
+        module->posts[0] = (osc_sm_post_type_t *) (module->segment_base);
         module->global_state = (ompi_osc_sm_global_state_t *) (module->posts[0] + comm_size * post_size);
         module->node_states = (ompi_osc_sm_node_state_t *) (module->global_state + 1);
 
@@ -312,14 +355,14 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     *base = module->bases[ompi_comm_rank(module->comm)];
 
-    opal_atomic_init(&module->my_node_state->accumulate_lock, OPAL_ATOMIC_UNLOCKED);
+    opal_atomic_lock_init(&module->my_node_state->accumulate_lock, OPAL_ATOMIC_LOCK_UNLOCKED);
 
     /* share everyone's displacement units. */
     module->disp_units = malloc(sizeof(int) * comm_size);
-    ret = module->comm->c_coll.coll_allgather(&disp_unit, 1, MPI_INT,
+    ret = module->comm->c_coll->coll_allgather(&disp_unit, 1, MPI_INT,
                                               module->disp_units, 1, MPI_INT,
                                               module->comm,
-                                              module->comm->c_coll.coll_allgather_module);
+                                              module->comm->c_coll->coll_allgather_module);
     if (OMPI_SUCCESS != ret) goto error;
 
     module->start_group = NULL;
@@ -341,7 +384,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         bool blocking_fence=false;
         int flag;
 
-        if (OMPI_SUCCESS != ompi_info_get_bool(info, "blocking_fence",
+        if (OMPI_SUCCESS != opal_info_get_bool(info, "blocking_fence",
                                                &blocking_fence, &flag)) {
             goto error;
         }
@@ -375,18 +418,25 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 #endif
     }
 
-    ret = module->comm->c_coll.coll_barrier(module->comm,
-                                            module->comm->c_coll.coll_barrier_module);
+    ret = opal_infosubscribe_subscribe(&(win->super), "blocking_fence", "false",
+        component_set_blocking_fence_info);
+
+    if (OPAL_SUCCESS != ret) goto error;
+
+    ret = module->comm->c_coll->coll_barrier(module->comm,
+                                            module->comm->c_coll->coll_barrier_module);
     if (OMPI_SUCCESS != ret) goto error;
 
     *model = MPI_WIN_UNIFIED;
 
-    win->w_osc_module = &module->super;
-
     return OMPI_SUCCESS;
 
  error:
-    win->w_osc_module = &module->super;
+
+    if (0 == ompi_comm_rank (module->comm) && unlink_needed) {
+        opal_shmem_unlink (&module->seg_ds);
+    }
+
     ompi_osc_sm_free (win);
 
     return ret;
@@ -462,18 +512,16 @@ ompi_osc_sm_free(struct ompi_win_t *win)
     /* free memory */
     if (NULL != module->segment_base) {
         /* synchronize */
-        module->comm->c_coll.coll_barrier(module->comm,
-                                          module->comm->c_coll.coll_barrier_module);
-
-        if (0 == ompi_comm_rank (module->comm)) {
-            opal_shmem_unlink (&module->seg_ds);
-        }
+        module->comm->c_coll->coll_barrier(module->comm,
+                                          module->comm->c_coll->coll_barrier_module);
 
 	opal_shmem_segment_detach (&module->seg_ds);
     } else {
         free(module->node_states);
         free(module->global_state);
-        free(module->bases[0]);
+        if (NULL != module->bases) {
+            free(module->bases[0]);
+        }
     }
     free(module->disp_units);
     free(module->outstanding_locks);
@@ -494,30 +542,53 @@ ompi_osc_sm_free(struct ompi_win_t *win)
 
 
 int
-ompi_osc_sm_set_info(struct ompi_win_t *win, struct ompi_info_t *info)
+ompi_osc_sm_set_info(struct ompi_win_t *win, struct opal_info_t *info)
 {
     ompi_osc_sm_module_t *module =
         (ompi_osc_sm_module_t*) win->w_osc_module;
 
     /* enforce collectiveness... */
-    return module->comm->c_coll.coll_barrier(module->comm,
-                                             module->comm->c_coll.coll_barrier_module);
+    return module->comm->c_coll->coll_barrier(module->comm,
+                                             module->comm->c_coll->coll_barrier_module);
+}
+
+
+static char*
+component_set_blocking_fence_info(opal_infosubscriber_t *obj, char *key, char *val)
+{
+    ompi_osc_sm_module_t *module = (ompi_osc_sm_module_t*) ((struct ompi_win_t*) obj)->w_osc_module;
+/*
+ * Assuming that you can't change the default.  
+ */
+    return module->global_state->use_barrier_for_fence ? "true" : "false";
+}
+
+
+static char*
+component_set_alloc_shared_noncontig_info(opal_infosubscriber_t *obj, char *key, char *val)
+{
+
+    ompi_osc_sm_module_t *module = (ompi_osc_sm_module_t*) ((struct ompi_win_t*) obj)->w_osc_module;
+/*
+ * Assuming that you can't change the default.  
+ */
+    return module->noncontig ? "true" : "false";
 }
 
 
 int
-ompi_osc_sm_get_info(struct ompi_win_t *win, struct ompi_info_t **info_used)
+ompi_osc_sm_get_info(struct ompi_win_t *win, struct opal_info_t **info_used)
 {
     ompi_osc_sm_module_t *module =
         (ompi_osc_sm_module_t*) win->w_osc_module;
 
-    ompi_info_t *info = OBJ_NEW(ompi_info_t);
+    opal_info_t *info = OBJ_NEW(opal_info_t);
     if (NULL == info) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
     if (module->flavor == MPI_WIN_FLAVOR_SHARED) {
-        ompi_info_set(info, "blocking_fence",
+        opal_info_set(info, "blocking_fence",
                       (1 == module->global_state->use_barrier_for_fence) ? "true" : "false");
-        ompi_info_set(info, "alloc_shared_noncontig",
+        opal_info_set(info, "alloc_shared_noncontig",
                       (module->noncontig) ? "true" : "false");
     }
 
