@@ -68,13 +68,17 @@ mca_pml_ucx_module_t ompi_pml_ucx = {
         mca_pml_ucx_mrecv,
         mca_pml_ucx_dump,
         NULL, /* FT */
-        1ul << (PML_UCX_TAG_BITS - 1),
         1ul << (PML_UCX_CONTEXT_BITS),
+        1ul << (PML_UCX_TAG_BITS - 1),
     },
     NULL,   /* ucp_context */
     NULL    /* ucp_worker */
 };
 
+#define PML_UCX_REQ_ALLOCA() \
+    ((char *)alloca(ompi_pml_ucx.request_size) + ompi_pml_ucx.request_size);
+
+
 static int mca_pml_ucx_send_worker_address(void)
 {
     ucp_address_t *address;
@@ -108,9 +112,10 @@ static int mca_pml_ucx_recv_worker_address(ompi_proc_t *proc,
 
     *address_p = NULL;
     OPAL_MODEX_RECV(ret, &mca_pml_ucx_component.pmlm_version, &proc->super.proc_name,
-                              (void**)address_p, addrlen_p);
+                    (void**)address_p, addrlen_p);
     if (ret < 0) {
-        PML_UCX_ERROR("Failed to receive EP address");
+        PML_UCX_ERROR("Failed to receive UCX worker address: %s (%d)",
+                      opal_strerror(ret), ret);
     }
     return ret;
 }
@@ -135,12 +140,17 @@ int mca_pml_ucx_open(void)
                              UCP_PARAM_FIELD_REQUEST_SIZE |
                              UCP_PARAM_FIELD_REQUEST_INIT |
                              UCP_PARAM_FIELD_REQUEST_CLEANUP |
-                             UCP_PARAM_FIELD_TAG_SENDER_MASK;
+                             UCP_PARAM_FIELD_TAG_SENDER_MASK |
+                             UCP_PARAM_FIELD_MT_WORKERS_SHARED |
+                             UCP_PARAM_FIELD_ESTIMATED_NUM_EPS;
     params.features        = UCP_FEATURE_TAG;
     params.request_size    = sizeof(ompi_request_t);
     params.request_init    = mca_pml_ucx_request_init;
     params.request_cleanup = mca_pml_ucx_request_cleanup;
     params.tag_sender_mask = PML_UCX_SPECIFIC_SOURCE_MASK;
+    params.mt_workers_shared = 0; /* we do not need mt support for context
+                                     since it will be protected by worker */
+    params.estimated_num_eps = ompi_proc_world_size();
 
     status = ucp_init(&params, config, &ompi_pml_ucx.ucp_context);
     ucp_config_release(config);
@@ -178,6 +188,7 @@ int mca_pml_ucx_init(void)
 {
     ucp_worker_params_t params;
     ucs_status_t status;
+    ucp_worker_attr_t attr;
     int rc;
 
     PML_UCX_VERBOSE(1, "mca_pml_ucx_init");
@@ -185,10 +196,34 @@ int mca_pml_ucx_init(void)
     /* TODO check MPI thread mode */
     params.field_mask  = UCP_WORKER_PARAM_FIELD_THREAD_MODE;
     params.thread_mode = UCS_THREAD_MODE_SINGLE;
+    if (ompi_mpi_thread_multiple) {
+        params.thread_mode = UCS_THREAD_MODE_MULTI;
+    } else {
+        params.thread_mode = UCS_THREAD_MODE_SINGLE;
+    }
 
     status = ucp_worker_create(ompi_pml_ucx.ucp_context, &params,
                                &ompi_pml_ucx.ucp_worker);
     if (UCS_OK != status) {
+        PML_UCX_ERROR("Failed to create UCP worker");
+        return OMPI_ERROR;
+    }
+
+    attr.field_mask = UCP_WORKER_ATTR_FIELD_THREAD_MODE;
+    status = ucp_worker_query(ompi_pml_ucx.ucp_worker, &attr);
+    if (UCS_OK != status) {
+        ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
+        ompi_pml_ucx.ucp_worker = NULL;
+        PML_UCX_ERROR("Failed to query UCP worker thread level");
+        return OMPI_ERROR;
+    }
+
+    if (ompi_mpi_thread_multiple && attr.thread_mode != UCS_THREAD_MODE_MULTI) {
+        /* UCX does not support multithreading, disqualify current PML for now */
+        /* TODO: we should let OMPI to fallback to THREAD_SINGLE mode */
+        ucp_worker_destroy(ompi_pml_ucx.ucp_worker);
+        ompi_pml_ucx.ucp_worker = NULL;
+        PML_UCX_ERROR("UCP worker does not support MPI_THREAD_MULTIPLE");
         return OMPI_ERROR;
     }
 
@@ -234,7 +269,7 @@ int mca_pml_ucx_cleanup(void)
     return OMPI_SUCCESS;
 }
 
-ucp_ep_h mca_pml_ucx_add_proc(ompi_communicator_t *comm, int dst)
+static ucp_ep_h mca_pml_ucx_add_proc_common(ompi_proc_t *proc)
 {
     ucp_ep_params_t ep_params;
     ucp_address_t *address;
@@ -243,23 +278,12 @@ ucp_ep_h mca_pml_ucx_add_proc(ompi_communicator_t *comm, int dst)
     ucp_ep_h ep;
     int ret;
 
-    ompi_proc_t *proc0      = ompi_comm_peer_lookup(comm, 0);
-    ompi_proc_t *proc_peer = ompi_comm_peer_lookup(comm, dst);
-
-    /* Note, mca_pml_base_pml_check_selected, doesn't use 3rd argument */
-    if (OMPI_SUCCESS != (ret = mca_pml_base_pml_check_selected("ucx",
-                                                              &proc0,
-                                                              dst))) {
-        return NULL;
-    }
-
-    ret = mca_pml_ucx_recv_worker_address(proc_peer, &address, &addrlen);
+    ret = mca_pml_ucx_recv_worker_address(proc, &address, &addrlen);
     if (ret < 0) {
-        PML_UCX_ERROR("Failed to receive worker address from proc: %d", proc_peer->super.proc_name.vpid);
         return NULL;
     }
 
-    PML_UCX_VERBOSE(2, "connecting to proc. %d", proc_peer->super.proc_name.vpid);
+    PML_UCX_VERBOSE(2, "connecting to proc. %d", proc->super.proc_name.vpid);
 
     ep_params.field_mask = UCP_EP_PARAM_FIELD_REMOTE_ADDRESS;
     ep_params.address    = address;
@@ -267,68 +291,80 @@ ucp_ep_h mca_pml_ucx_add_proc(ompi_communicator_t *comm, int dst)
     status = ucp_ep_create(ompi_pml_ucx.ucp_worker, &ep_params, &ep);
     free(address);
     if (UCS_OK != status) {
-        PML_UCX_ERROR("Failed to connect to proc: %d, %s", proc_peer->super.proc_name.vpid,
-                                                           ucs_status_string(status));
+        PML_UCX_ERROR("ucp_ep_create(proc=%d) failed: %s",
+                      proc->super.proc_name.vpid,
+                      ucs_status_string(status));
         return NULL;
     }
 
-    proc_peer->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML] = ep;
-
+    proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML] = ep;
     return ep;
 }
 
+static ucp_ep_h mca_pml_ucx_add_proc(ompi_communicator_t *comm, int dst)
+{
+    ompi_proc_t *proc0     = ompi_comm_peer_lookup(comm, 0);
+    ompi_proc_t *proc_peer = ompi_comm_peer_lookup(comm, dst);
+    int ret;
+
+    /* Note, mca_pml_base_pml_check_selected, doesn't use 3rd argument */
+    if (OMPI_SUCCESS != (ret = mca_pml_base_pml_check_selected("ucx",
+                                                               &proc0,
+                                                               dst))) {
+        return NULL;
+    }
+
+    return mca_pml_ucx_add_proc_common(proc_peer);
+}
+
 int mca_pml_ucx_add_procs(struct ompi_proc_t **procs, size_t nprocs)
 {
-    ucp_ep_params_t ep_params;
-    ucp_address_t *address;
-    ucs_status_t status;
     ompi_proc_t *proc;
-    size_t addrlen;
     ucp_ep_h ep;
     size_t i;
     int ret;
 
     if (OMPI_SUCCESS != (ret = mca_pml_base_pml_check_selected("ucx",
-                                                              procs,
-                                                              nprocs))) {
+                                                               procs,
+                                                               nprocs))) {
         return ret;
     }
 
     for (i = 0; i < nprocs; ++i) {
         proc = procs[(i + OMPI_PROC_MY_NAME->vpid) % nprocs];
-
-        ret = mca_pml_ucx_recv_worker_address(proc, &address, &addrlen);
-        if (ret < 0) {
-            PML_UCX_ERROR("Failed to receive worker address from proc: %d",
-                          proc->super.proc_name.vpid);
-            return ret;
-        }
-
-        if (proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML]) {
-            PML_UCX_VERBOSE(3, "already connected to proc. %d", proc->super.proc_name.vpid);
-            continue;
-        }
-
-        PML_UCX_VERBOSE(2, "connecting to proc. %d", proc->super.proc_name.vpid);
-
-        ep_params.field_mask = UCP_EP_PARAM_FIELD_REMOTE_ADDRESS;
-        ep_params.address    = address;
-
-        status = ucp_ep_create(ompi_pml_ucx.ucp_worker, &ep_params, &ep);
-        free(address);
-
-        if (UCS_OK != status) {
-            PML_UCX_ERROR("Failed to connect to proc: %d, %s", proc->super.proc_name.vpid,
-                          ucs_status_string(status));
+        ep = mca_pml_ucx_add_proc_common(proc);
+        if (ep == NULL) {
             return OMPI_ERROR;
         }
-
-        proc->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML] = ep;
     }
 
     return OMPI_SUCCESS;
 }
 
+static inline ucp_ep_h mca_pml_ucx_get_ep(ompi_communicator_t *comm, int rank)
+{
+    ucp_ep_h ep;
+
+    ep = ompi_comm_peer_lookup(comm, rank)->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_PML];
+    if (OPAL_LIKELY(ep != NULL)) {
+        return ep;
+    }
+
+    ep = mca_pml_ucx_add_proc(comm, rank);
+    if (OPAL_LIKELY(ep != NULL)) {
+        return ep;
+    }
+
+    if (rank >= ompi_comm_size(comm)) {
+        PML_UCX_ERROR("Rank number (%d) is larger than communicator size (%d)",
+                      rank, ompi_comm_size(comm));
+    } else {
+        PML_UCX_ERROR("Failed to resolve UCX endpoint for rank %d", rank);
+    }
+
+    return NULL;
+}
+
 static void mca_pml_ucx_waitall(void **reqs, size_t *count_p)
 {
     ucs_status_t status;
@@ -351,8 +387,14 @@ static void mca_pml_ucx_waitall(void **reqs, size_t *count_p)
     *count_p = 0;
 }
 
+static void mca_pml_fence_complete_cb(int status, void *fenced)
+{
+    *(int*)fenced = 1;
+}
+
 int mca_pml_ucx_del_procs(struct ompi_proc_t **procs, size_t nprocs)
 {
+    int fenced = 0;
     ompi_proc_t *proc;
     size_t num_reqs, max_reqs;
     void *dreq, **dreqs;
@@ -400,7 +442,10 @@ int mca_pml_ucx_del_procs(struct ompi_proc_t **procs, size_t nprocs)
     mca_pml_ucx_waitall(dreqs, &num_reqs);
     free(dreqs);
 
-    opal_pmix.fence(NULL, 0);
+    opal_pmix.fence_nb(NULL, 0, mca_pml_fence_complete_cb, &fenced);
+    while (!fenced) {
+        ucp_worker_progress(ompi_pml_ucx.ucp_worker);
+    }
 
     return OMPI_SUCCESS;
 }
@@ -495,12 +540,11 @@ int mca_pml_ucx_recv(void *buf, size_t count, ompi_datatype_t *datatype, int src
     PML_UCX_TRACE_RECV("%s", buf, count, datatype, src, tag, comm, "recv");
 
     PML_UCX_MAKE_RECV_TAG(ucp_tag, ucp_tag_mask, tag, src, comm);
-    req = (char *)alloca(ompi_pml_ucx.request_size) + ompi_pml_ucx.request_size;
+    req = PML_UCX_REQ_ALLOCA();
     status = ucp_tag_recv_nbr(ompi_pml_ucx.ucp_worker, buf, count,
                               mca_pml_ucx_get_datatype(datatype),
                               ucp_tag, ucp_tag_mask, req);
 
-    ucp_worker_progress(ompi_pml_ucx.ucp_worker);
     for (;;) {
         status = ucp_request_test(req, &info);
         if (status != UCS_INPROGRESS) {
@@ -549,7 +593,6 @@ int mca_pml_ucx_isend_init(const void *buf, size_t count, ompi_datatype_t *datat
 
     ep = mca_pml_ucx_get_ep(comm, dst);
     if (OPAL_UNLIKELY(NULL == ep)) {
-        PML_UCX_ERROR("Failed to get ep for rank %d", dst);
         return OMPI_ERROR;
     }
 
@@ -571,7 +614,7 @@ int mca_pml_ucx_isend_init(const void *buf, size_t count, ompi_datatype_t *datat
     return OMPI_SUCCESS;
 }
 
-static int 
+static ucs_status_ptr_t
 mca_pml_ucx_bsend(ucp_ep_h ep, const void *buf, size_t count, 
                   ompi_datatype_t *datatype, uint64_t pml_tag)
 {
@@ -593,21 +636,21 @@ mca_pml_ucx_bsend(ucp_ep_h ep, const void *buf, size_t count,
     if (OPAL_UNLIKELY(NULL == packed_data)) {
         OBJ_DESTRUCT(&opal_conv);
         PML_UCX_ERROR("bsend: failed to allocate buffer");
-        return OMPI_ERR_OUT_OF_RESOURCE;
+        return UCS_STATUS_PTR(OMPI_ERROR);
     }
     
     iov_count    = 1;
     iov.iov_base = packed_data;
     iov.iov_len  = packed_length;
 
-    PML_UCX_VERBOSE(8, "bsend of packed buffer %p len %d", packed_data, packed_length);
+    PML_UCX_VERBOSE(8, "bsend of packed buffer %p len %zu", packed_data, packed_length);
     offset = 0;
     opal_convertor_set_position(&opal_conv, &offset);
     if (0 > opal_convertor_pack(&opal_conv, &iov, &iov_count, &packed_length)) {
         mca_pml_base_bsend_request_free(packed_data);
         OBJ_DESTRUCT(&opal_conv);
         PML_UCX_ERROR("bsend: failed to pack user datatype");
-        return OMPI_ERROR;
+        return UCS_STATUS_PTR(OMPI_ERROR);
     }
 
     OBJ_DESTRUCT(&opal_conv);
@@ -618,17 +661,34 @@ mca_pml_ucx_bsend(ucp_ep_h ep, const void *buf, size_t count,
     if (NULL == req) {
         /* request was completed in place */
         mca_pml_base_bsend_request_free(packed_data);
-        return OMPI_SUCCESS;
+        return NULL;
     }
 
     if (OPAL_UNLIKELY(UCS_PTR_IS_ERR(req))) {
         mca_pml_base_bsend_request_free(packed_data);
         PML_UCX_ERROR("ucx bsend failed: %s", ucs_status_string(UCS_PTR_STATUS(req)));
-        return OMPI_ERROR;
+        return UCS_STATUS_PTR(OMPI_ERROR);
     }
 
     req->req_complete_cb_data = packed_data;
-    return OMPI_SUCCESS;
+    return NULL;
+}
+
+static inline ucs_status_ptr_t mca_pml_ucx_common_send(ucp_ep_h ep, const void *buf,
+                                                       size_t count,
+                                                       ompi_datatype_t *datatype,
+                                                       ucp_datatype_t ucx_datatype,
+                                                       ucp_tag_t tag,
+                                                       mca_pml_base_send_mode_t mode,
+                                                       ucp_send_callback_t cb)
+{
+    if (OPAL_UNLIKELY(MCA_PML_BASE_SEND_BUFFERED == mode)) {
+        return mca_pml_ucx_bsend(ep, buf, count, datatype, tag);
+    } else if (OPAL_UNLIKELY(MCA_PML_BASE_SEND_SYNCHRONOUS == mode)) {
+        return ucp_tag_send_sync_nb(ep, buf, count, ucx_datatype, tag, cb);
+    } else {
+        return ucp_tag_send_nb(ep, buf, count, ucx_datatype, tag, cb);
+    }
 }
 
 int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
@@ -644,25 +704,16 @@ int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
                        mode == MCA_PML_BASE_SEND_BUFFERED ? "b" : "",
                        (void*)request)
 
-    /* TODO special care to sync/buffered send */
-
     ep = mca_pml_ucx_get_ep(comm, dst);
     if (OPAL_UNLIKELY(NULL == ep)) {
-        PML_UCX_ERROR("Failed to get ep for rank %d", dst);
         return OMPI_ERROR;
     }
 
-    /* Special care to sync/buffered send */
-    if (OPAL_UNLIKELY(MCA_PML_BASE_SEND_BUFFERED == mode)) {
-        *request = &ompi_pml_ucx.completed_send_req;
-        return mca_pml_ucx_bsend(ep, buf, count, datatype, 
-                                 PML_UCX_MAKE_SEND_TAG(tag, comm));
-    }
+    req = (ompi_request_t*)mca_pml_ucx_common_send(ep, buf, count, datatype,
+                                                   mca_pml_ucx_get_datatype(datatype),
+                                                   PML_UCX_MAKE_SEND_TAG(tag, comm), mode,
+                                                   mca_pml_ucx_send_completion);
 
-    req = (ompi_request_t*)ucp_tag_send_nb(ep, buf, count,
-                                           mca_pml_ucx_get_datatype(datatype),
-                                           PML_UCX_MAKE_SEND_TAG(tag, comm),
-                                           mca_pml_ucx_send_completion);
     if (req == NULL) {
         PML_UCX_VERBOSE(8, "returning completed request");
         *request = &ompi_pml_ucx.completed_send_req;
@@ -677,32 +728,19 @@ int mca_pml_ucx_isend(const void *buf, size_t count, ompi_datatype_t *datatype,
     }
 }
 
-int mca_pml_ucx_send(const void *buf, size_t count, ompi_datatype_t *datatype, int dst,
-                     int tag, mca_pml_base_send_mode_t mode,
-                     struct ompi_communicator_t* comm)
+static inline __opal_attribute_always_inline__ int
+mca_pml_ucx_send_nb(ucp_ep_h ep, const void *buf, size_t count,
+                    ompi_datatype_t *datatype, ucp_datatype_t ucx_datatype,
+                    ucp_tag_t tag, mca_pml_base_send_mode_t mode,
+                    ucp_send_callback_t cb)
 {
     ompi_request_t *req;
-    ucp_ep_h ep;
 
-    PML_UCX_TRACE_SEND("%s", buf, count, datatype, dst, tag, mode, comm, 
-                       mode == MCA_PML_BASE_SEND_BUFFERED ? "bsend" : "send");
+    req = (ompi_request_t*)mca_pml_ucx_common_send(ep, buf, count, datatype,
+                                                   mca_pml_ucx_get_datatype(datatype),
+                                                   tag, mode,
+                                                   mca_pml_ucx_send_completion);
 
-    ep = mca_pml_ucx_get_ep(comm, dst);
-    if (OPAL_UNLIKELY(NULL == ep)) {
-        PML_UCX_ERROR("Failed to get ep for rank %d", dst);
-        return OMPI_ERROR;
-    }
-
-    /* Special care to sync/buffered send */
-    if (OPAL_UNLIKELY(MCA_PML_BASE_SEND_BUFFERED == mode)) {
-        return mca_pml_ucx_bsend(ep, buf, count, datatype,
-                                 PML_UCX_MAKE_SEND_TAG(tag, comm));
-    }
-
-    req = (ompi_request_t*)ucp_tag_send_nb(ep, buf, count,
-                                           mca_pml_ucx_get_datatype(datatype),
-                                           PML_UCX_MAKE_SEND_TAG(tag, comm),
-                                           mca_pml_ucx_send_completion);
     if (OPAL_LIKELY(req == NULL)) {
         return OMPI_SUCCESS;
     } else if (!UCS_PTR_IS_ERR(req)) {
@@ -716,6 +754,59 @@ int mca_pml_ucx_send(const void *buf, size_t count, ompi_datatype_t *datatype, i
     }
 }
 
+#if HAVE_DECL_UCP_TAG_SEND_NBR
+static inline __opal_attribute_always_inline__ int
+mca_pml_ucx_send_nbr(ucp_ep_h ep, const void *buf, size_t count,
+                     ucp_datatype_t ucx_datatype, ucp_tag_t tag)
+
+{
+    void *req;
+    ucs_status_t status;
+
+    req    = PML_UCX_REQ_ALLOCA();
+    status = ucp_tag_send_nbr(ep, buf, count, ucx_datatype, tag, req);
+    if (OPAL_LIKELY(status == UCS_OK)) {
+        return OMPI_SUCCESS;
+    }
+
+    ucp_worker_progress(ompi_pml_ucx.ucp_worker);
+    while ((status = ucp_request_check_status(req)) == UCS_INPROGRESS) {
+        opal_progress();
+    }
+
+    return OPAL_LIKELY(UCS_OK == status) ? OMPI_SUCCESS : OMPI_ERROR;
+}
+#endif
+
+int mca_pml_ucx_send(const void *buf, size_t count, ompi_datatype_t *datatype, int dst,
+                     int tag, mca_pml_base_send_mode_t mode,
+                     struct ompi_communicator_t* comm)
+{
+    ucp_ep_h ep;
+
+    PML_UCX_TRACE_SEND("%s", buf, count, datatype, dst, tag, mode, comm,
+                       mode == MCA_PML_BASE_SEND_BUFFERED ? "bsend" : "send");
+
+    ep = mca_pml_ucx_get_ep(comm, dst);
+    if (OPAL_UNLIKELY(NULL == ep)) {
+        return OMPI_ERROR;
+    }
+
+#if HAVE_DECL_UCP_TAG_SEND_NBR
+    if (OPAL_LIKELY((MCA_PML_BASE_SEND_BUFFERED != mode) &&
+                    (MCA_PML_BASE_SEND_SYNCHRONOUS != mode))) {
+        return mca_pml_ucx_send_nbr(ep, buf, count,
+                                    mca_pml_ucx_get_datatype(datatype),
+                                    PML_UCX_MAKE_SEND_TAG(tag, comm));
+    }
+#endif
+
+    return mca_pml_ucx_send_nb(ep, buf, count, datatype,
+                               mca_pml_ucx_get_datatype(datatype),
+                               PML_UCX_MAKE_SEND_TAG(tag, comm), mode,
+                               mca_pml_ucx_send_completion);
+}
+
 int mca_pml_ucx_iprobe(int src, int tag, struct ompi_communicator_t* comm,
                          int *matched, ompi_status_public_t* mpi_status)
 {
@@ -861,7 +952,6 @@ int mca_pml_ucx_start(size_t count, ompi_request_t** requests)
     mca_pml_ucx_persistent_request_t *preq;
     ompi_request_t *tmp_req;
     size_t i;
-    int rc;
 
     for (i = 0; i < count; ++i) {
         preq = (mca_pml_ucx_persistent_request_t *)requests[i];
@@ -876,22 +966,14 @@ int mca_pml_ucx_start(size_t count, ompi_request_t** requests)
         mca_pml_ucx_request_reset(&preq->ompi);
 
         if (preq->flags & MCA_PML_UCX_REQUEST_FLAG_SEND) {
-            if (OPAL_UNLIKELY(MCA_PML_BASE_SEND_BUFFERED == preq->send.mode)) {
-                PML_UCX_VERBOSE(8, "start bsend request %p", (void*)preq);
-                rc = mca_pml_ucx_bsend(preq->send.ep, preq->buffer, preq->count,
-                                       preq->ompi_datatype,  preq->tag);
-                if (OMPI_SUCCESS != rc) {
-                    return rc;
-                }
-                /* pretend that we got immediate completion */
-                tmp_req = NULL;
-            } else {
-                PML_UCX_VERBOSE(8, "start send request %p", (void*)preq);
-                tmp_req = (ompi_request_t*)ucp_tag_send_nb(preq->send.ep, preq->buffer,
-                                                           preq->count, preq->datatype,
-                                                           preq->tag,
-                                                           mca_pml_ucx_psend_completion);
-            }
+            tmp_req = (ompi_request_t*)mca_pml_ucx_common_send(preq->send.ep,
+                                                               preq->buffer,
+                                                               preq->count,
+                                                               preq->ompi_datatype,
+                                                               preq->datatype,
+                                                               preq->tag,
+                                                               preq->send.mode,
+                                                               mca_pml_ucx_psend_completion);
         } else {
             PML_UCX_VERBOSE(8, "start recv request %p", (void*)preq);
             tmp_req = (ompi_request_t*)ucp_tag_recv_nb(ompi_pml_ucx.ucp_worker,
