@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -12,6 +12,7 @@
  * Copyright (c) 2013-2017 University of Houston. All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -34,11 +35,12 @@
 #include <sys/stat.h>
 #endif
 #include <fcntl.h>
+#include <unistd.h>
 
 int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
 				       const char* filename,
 				       int amode,
-				       struct ompi_info_t *info,
+				       struct opal_info_t *info,
 				       mca_io_ompio_file_t *fh)
 {
     int err = MPI_SUCCESS;
@@ -49,11 +51,14 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
     mca_io_ompio_file_t * shfileHandle, *ompio_fh;
     mca_io_ompio_data_t *data;
 
+    pid_t my_pid;
+    int int_pid;
+
     /*------------------------------------------------------------*/
     /*Open the same file again without shared file pointer support*/
     /*------------------------------------------------------------*/
     shfileHandle =  (mca_io_ompio_file_t *)malloc(sizeof(mca_io_ompio_file_t));
-    err = ompio_io_ompio_file_open(comm,filename,amode,info,shfileHandle,false);
+    err = mca_common_ompio_file_open(comm,filename,amode,info,shfileHandle,false);
     if ( OMPI_SUCCESS != err)  {
         opal_output(0, "mca_sharedfp_lockedfile_file_open: Error during file open\n");
         return err;
@@ -62,13 +67,13 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
     data = (mca_io_ompio_data_t *) fh->f_fh->f_io_selected_data;
     ompio_fh = &data->ompio_fh;
 
-    err = mca_io_ompio_set_view_internal (shfileHandle,
-                                          ompio_fh->f_disp,
-                                          ompio_fh->f_etype,
-                                          ompio_fh->f_orig_filetype,
-                                          ompio_fh->f_datarep,
-                                          MPI_INFO_NULL);
-
+    err = mca_common_ompio_set_view (shfileHandle,
+                                     ompio_fh->f_disp,
+                                     ompio_fh->f_etype,
+                                     ompio_fh->f_orig_filetype,
+                                     ompio_fh->f_datarep,
+                                     &(MPI_INFO_NULL->super));
+    
 
     /*Memory is allocated here for the sh structure*/
     sh = (struct mca_sharedfp_base_data_t*)malloc(sizeof(struct mca_sharedfp_base_data_t));
@@ -106,10 +111,22 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
         ompi_proc_t *masterproc = ompi_group_peer_lookup(comm->c_local_group, 0 );
         masterjobid = OMPI_CAST_RTE_NAME(&masterproc->super.proc_name)->jobid;
     }
-    comm->c_coll.coll_bcast ( &masterjobid, 1, MPI_UNSIGNED, 0, comm, 
-                              comm->c_coll.coll_bcast_module );
+    comm->c_coll->coll_bcast ( &masterjobid, 1, MPI_UNSIGNED, 0, comm, 
+                               comm->c_coll->coll_bcast_module );
  
-    size_t filenamelen = strlen(filename) + 16;
+    if ( 0 == fh->f_rank ) {
+        my_pid = getpid();
+        int_pid = (int) my_pid;
+    }
+    err = comm->c_coll->coll_bcast (&int_pid, 1, MPI_INT, 0, comm, comm->c_coll->coll_bcast_module );
+    if ( OMPI_SUCCESS != err ) {
+        opal_output(0, "[%d]mca_sharedfp_lockedfile_file_open: Error in bcast operation\n", fh->f_rank);
+        free (sh);
+        free(module_data);
+        return err;
+    }
+
+    size_t filenamelen = strlen(filename) + 24;
     lockedfilename = (char*)malloc(sizeof(char) * filenamelen);
     if ( NULL == lockedfilename ) {
 	free (shfileHandle);
@@ -117,7 +134,7 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
         free (module_data);
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
-    snprintf(lockedfilename, filenamelen, "%s-%u%s",filename,masterjobid,".lock");
+    snprintf(lockedfilename, filenamelen, "%s-%u-%d%s",filename,masterjobid,int_pid,".lock");
     module_data->filename = lockedfilename;
 
     /*-------------------------------------------------*/
@@ -132,7 +149,7 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
 	write ( handle, &position, sizeof(OMPI_MPI_OFFSET_TYPE) );
 	close ( handle );
     }
-    comm->c_coll.coll_barrier ( comm, comm->c_coll.coll_barrier_module );
+    comm->c_coll->coll_barrier ( comm, comm->c_coll->coll_barrier_module );
 
     handle = open ( lockedfilename, O_RDWR, 0644  );
     if ( -1 == handle ) {
@@ -150,7 +167,7 @@ int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
     /*remember the shared file handle*/
     fh->f_sharedfp_data = sh;
 
-    comm->c_coll.coll_barrier ( comm, comm->c_coll.coll_barrier_module );
+    comm->c_coll->coll_barrier ( comm, comm->c_coll->coll_barrier_module );
 
     return err;
 }
@@ -187,7 +204,7 @@ int mca_sharedfp_lockedfile_file_close (mca_io_ompio_file_t *fh)
     }
 
     /* Close the main file opened by this component*/
-    err = ompio_io_ompio_file_close(sh->sharedfh);
+    err = mca_common_ompio_file_close(sh->sharedfh);
 
     /*free shared file pointer data struct*/
     free(sh);
