@@ -11,6 +11,8 @@
 ! Copyright (c) 2004-2005 The Regents of the University of California.
 !                         All rights reserved.
 ! Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
+! Copyright (c) 2016-2018 Research Organization for Information Science
+!                         and Technology (RIST).  All rights reserved.
 ! $COPYRIGHT$
 !
 ! Additional copyrights may follow
@@ -668,307 +670,6 @@ end subroutine MPI_Error_string
 end interface
 
 
-interface MPI_File_call_errhandler
-
-subroutine MPI_File_call_errhandler(fh, errorcode, ierror)
-  integer, intent(in) :: fh
-  integer, intent(in) :: errorcode
-  integer, intent(out) :: ierror
-end subroutine MPI_File_call_errhandler
-
-end interface
-
-
-interface MPI_File_close
-
-subroutine MPI_File_close(fh, ierror)
-  integer, intent(inout) :: fh
-  integer, intent(out) :: ierror
-end subroutine MPI_File_close
-
-end interface
-
-
-interface MPI_File_create_errhandler
-
-subroutine MPI_File_create_errhandler(function, errhandler, ierror)
-  external :: function
-  integer, intent(out) :: errhandler
-  integer, intent(out) :: ierror
-end subroutine MPI_File_create_errhandler
-
-end interface
-
-
-interface MPI_File_delete
-
-subroutine MPI_File_delete(filename, info, ierror)
-  character(len=*), intent(in) :: filename
-  integer, intent(in) :: info
-  integer, intent(out) :: ierror
-end subroutine MPI_File_delete
-
-end interface
-
-
-interface MPI_File_get_amode
-
-subroutine MPI_File_get_amode(fh, amode, ierror)
-  integer, intent(in) :: fh
-  integer, intent(out) :: amode
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_amode
-
-end interface
-
-
-interface MPI_File_get_atomicity
-
-subroutine MPI_File_get_atomicity(fh, flag, ierror)
-  integer, intent(in) :: fh
-  logical, intent(out) :: flag
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_atomicity
-
-end interface
-
-
-interface MPI_File_get_byte_offset
-
-subroutine MPI_File_get_byte_offset(fh, offset, disp, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: offset
-  integer(kind=MPI_OFFSET_KIND), intent(out) :: disp
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_byte_offset
-
-end interface
-
-
-interface MPI_File_get_errhandler
-
-subroutine MPI_File_get_errhandler(file, errhandler, ierror)
-  integer, intent(in) :: file
-  integer, intent(out) :: errhandler
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_errhandler
-
-end interface
-
-
-interface MPI_File_get_group
-
-subroutine MPI_File_get_group(fh, group, ierror)
-  integer, intent(in) :: fh
-  integer, intent(out) :: group
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_group
-
-end interface
-
-
-interface MPI_File_get_info
-
-subroutine MPI_File_get_info(fh, info_used, ierror)
-  integer, intent(in) :: fh
-  integer, intent(out) :: info_used
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_info
-
-end interface
-
-
-interface MPI_File_get_position
-
-subroutine MPI_File_get_position(fh, offset, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(out) :: offset
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_position
-
-end interface
-
-
-interface MPI_File_get_position_shared
-
-subroutine MPI_File_get_position_shared(fh, offset, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(out) :: offset
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_position_shared
-
-end interface
-
-
-interface MPI_File_get_size
-
-subroutine MPI_File_get_size(fh, size, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_size
-
-end interface
-
-
-interface MPI_File_get_type_extent
-
-subroutine MPI_File_get_type_extent(fh, datatype, extent, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer, intent(in) :: datatype
-  integer(kind=MPI_ADDRESS_KIND), intent(out) :: extent
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_type_extent
-
-end interface
-
-
-interface MPI_File_get_view
-
-subroutine MPI_File_get_view(fh, disp, etype, filetype, datarep&
-        , ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(out) :: disp
-  integer, intent(out) :: etype
-  integer, intent(out) :: filetype
-  character(len=*), intent(out) :: datarep
-  integer, intent(out) :: ierror
-end subroutine MPI_File_get_view
-
-end interface
-
-
-interface MPI_File_open
-
-subroutine MPI_File_open(comm, filename, amode, info, fh&
-        , ierror)
-  integer, intent(in) :: comm
-  character(len=*), intent(in) :: filename
-  integer, intent(in) :: amode
-  integer, intent(in) :: info
-  integer, intent(out) :: fh
-  integer, intent(out) :: ierror
-end subroutine MPI_File_open
-
-end interface
-
-
-interface MPI_File_preallocate
-
-subroutine MPI_File_preallocate(fh, size, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_File_preallocate
-
-end interface
-
-
-interface MPI_File_seek
-
-subroutine MPI_File_seek(fh, offset, whence, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: offset
-  integer, intent(in) :: whence
-  integer, intent(out) :: ierror
-end subroutine MPI_File_seek
-
-end interface
-
-
-interface MPI_File_seek_shared
-
-subroutine MPI_File_seek_shared(fh, offset, whence, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: offset
-  integer, intent(in) :: whence
-  integer, intent(out) :: ierror
-end subroutine MPI_File_seek_shared
-
-end interface
-
-
-interface MPI_File_set_atomicity
-
-subroutine MPI_File_set_atomicity(fh, flag, ierror)
-  integer, intent(in) :: fh
-  logical, intent(in) :: flag
-  integer, intent(out) :: ierror
-end subroutine MPI_File_set_atomicity
-
-end interface
-
-
-interface MPI_File_set_errhandler
-
-subroutine MPI_File_set_errhandler(file, errhandler, ierror)
-  integer, intent(in) :: file
-  integer, intent(in) :: errhandler
-  integer, intent(out) :: ierror
-end subroutine MPI_File_set_errhandler
-
-end interface
-
-
-interface MPI_File_set_info
-
-subroutine MPI_File_set_info(fh, info, ierror)
-  integer, intent(in) :: fh
-  integer, intent(in) :: info
-  integer, intent(out) :: ierror
-end subroutine MPI_File_set_info
-
-end interface
-
-
-interface MPI_File_set_size
-
-subroutine MPI_File_set_size(fh, size, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_File_set_size
-
-end interface
-
-
-interface MPI_File_set_view
-
-subroutine MPI_File_set_view(fh, disp, etype, filetype, datarep, &
-        info, ierror)
-  include 'mpif-config.h'
-  integer, intent(in) :: fh
-  integer(kind=MPI_OFFSET_KIND), intent(in) :: disp
-  integer, intent(in) :: etype
-  integer, intent(in) :: filetype
-  character(len=*), intent(in) :: datarep
-  integer, intent(in) :: info
-  integer, intent(out) :: ierror
-end subroutine MPI_File_set_view
-
-end interface
-
-
-interface MPI_File_sync
-
-subroutine MPI_File_sync(fh, ierror)
-  integer, intent(in) :: fh
-  integer, intent(out) :: ierror
-end subroutine MPI_File_sync
-
-end interface
-
-
 interface MPI_Finalize
 
 subroutine MPI_Finalize(ierror)
@@ -1650,570 +1351,6 @@ end subroutine MPI_Request_get_status
 end interface
 
 
-interface MPI_Sizeof
-
-subroutine MPI_Sizeof0DCH(x, size, ierror)
-  character, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DCH
-
-
-subroutine MPI_Sizeof0DL(x, size, ierror)
-  logical, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DL
-
-
-subroutine MPI_Sizeof0DI1(x, size, ierror)
-  integer*1, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DI1
-
-
-subroutine MPI_Sizeof0DI2(x, size, ierror)
-  integer*2, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DI2
-
-
-subroutine MPI_Sizeof0DI4(x, size, ierror)
-  integer*4, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DI4
-
-
-subroutine MPI_Sizeof0DI8(x, size, ierror)
-  integer*8, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DI8
-
-
-subroutine MPI_Sizeof0DR4(x, size, ierror)
-  real*4, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DR4
-
-
-subroutine MPI_Sizeof0DR8(x, size, ierror)
-  real*8, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DR8
-
-
-subroutine MPI_Sizeof0DC8(x, size, ierror)
-  complex*8, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DC8
-
-
-subroutine MPI_Sizeof0DC16(x, size, ierror)
-  complex*16, intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof0DC16
-
-
-subroutine MPI_Sizeof1DCH(x, size, ierror)
-  character, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DCH
-
-
-subroutine MPI_Sizeof1DL(x, size, ierror)
-  logical, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DL
-
-
-subroutine MPI_Sizeof1DI1(x, size, ierror)
-  integer*1, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DI1
-
-
-subroutine MPI_Sizeof1DI2(x, size, ierror)
-  integer*2, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DI2
-
-
-subroutine MPI_Sizeof1DI4(x, size, ierror)
-  integer*4, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DI4
-
-
-subroutine MPI_Sizeof1DI8(x, size, ierror)
-  integer*8, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DI8
-
-
-subroutine MPI_Sizeof1DR4(x, size, ierror)
-  real*4, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DR4
-
-
-subroutine MPI_Sizeof1DR8(x, size, ierror)
-  real*8, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DR8
-
-
-subroutine MPI_Sizeof1DC8(x, size, ierror)
-  complex*8, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DC8
-
-
-subroutine MPI_Sizeof1DC16(x, size, ierror)
-  complex*16, dimension(*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof1DC16
-
-
-subroutine MPI_Sizeof2DCH(x, size, ierror)
-  character, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DCH
-
-
-subroutine MPI_Sizeof2DL(x, size, ierror)
-  logical, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DL
-
-
-subroutine MPI_Sizeof2DI1(x, size, ierror)
-  integer*1, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DI1
-
-
-subroutine MPI_Sizeof2DI2(x, size, ierror)
-  integer*2, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DI2
-
-
-subroutine MPI_Sizeof2DI4(x, size, ierror)
-  integer*4, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DI4
-
-
-subroutine MPI_Sizeof2DI8(x, size, ierror)
-  integer*8, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DI8
-
-
-subroutine MPI_Sizeof2DR4(x, size, ierror)
-  real*4, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DR4
-
-
-subroutine MPI_Sizeof2DR8(x, size, ierror)
-  real*8, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DR8
-
-
-subroutine MPI_Sizeof2DC8(x, size, ierror)
-  complex*8, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DC8
-
-
-subroutine MPI_Sizeof2DC16(x, size, ierror)
-  complex*16, dimension(1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof2DC16
-
-
-subroutine MPI_Sizeof3DCH(x, size, ierror)
-  character, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DCH
-
-
-subroutine MPI_Sizeof3DL(x, size, ierror)
-  logical, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DL
-
-
-subroutine MPI_Sizeof3DI1(x, size, ierror)
-  integer*1, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DI1
-
-
-subroutine MPI_Sizeof3DI2(x, size, ierror)
-  integer*2, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DI2
-
-
-subroutine MPI_Sizeof3DI4(x, size, ierror)
-  integer*4, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DI4
-
-
-subroutine MPI_Sizeof3DI8(x, size, ierror)
-  integer*8, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DI8
-
-
-subroutine MPI_Sizeof3DR4(x, size, ierror)
-  real*4, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DR4
-
-
-subroutine MPI_Sizeof3DR8(x, size, ierror)
-  real*8, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DR8
-
-
-subroutine MPI_Sizeof3DC8(x, size, ierror)
-  complex*8, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DC8
-
-
-subroutine MPI_Sizeof3DC16(x, size, ierror)
-  complex*16, dimension(1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof3DC16
-
-
-subroutine MPI_Sizeof4DCH(x, size, ierror)
-  character, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DCH
-
-
-subroutine MPI_Sizeof4DL(x, size, ierror)
-  logical, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DL
-
-
-subroutine MPI_Sizeof4DI1(x, size, ierror)
-  integer*1, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DI1
-
-
-subroutine MPI_Sizeof4DI2(x, size, ierror)
-  integer*2, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DI2
-
-
-subroutine MPI_Sizeof4DI4(x, size, ierror)
-  integer*4, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DI4
-
-
-subroutine MPI_Sizeof4DI8(x, size, ierror)
-  integer*8, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DI8
-
-
-subroutine MPI_Sizeof4DR4(x, size, ierror)
-  real*4, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DR4
-
-
-subroutine MPI_Sizeof4DR8(x, size, ierror)
-  real*8, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DR8
-
-
-subroutine MPI_Sizeof4DC8(x, size, ierror)
-  complex*8, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DC8
-
-
-subroutine MPI_Sizeof4DC16(x, size, ierror)
-  complex*16, dimension(1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof4DC16
-
-
-subroutine MPI_Sizeof5DCH(x, size, ierror)
-  character, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DCH
-
-
-subroutine MPI_Sizeof5DL(x, size, ierror)
-  logical, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DL
-
-
-subroutine MPI_Sizeof5DI1(x, size, ierror)
-  integer*1, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DI1
-
-
-subroutine MPI_Sizeof5DI2(x, size, ierror)
-  integer*2, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DI2
-
-
-subroutine MPI_Sizeof5DI4(x, size, ierror)
-  integer*4, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DI4
-
-
-subroutine MPI_Sizeof5DI8(x, size, ierror)
-  integer*8, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DI8
-
-
-subroutine MPI_Sizeof5DR4(x, size, ierror)
-  real*4, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DR4
-
-
-subroutine MPI_Sizeof5DR8(x, size, ierror)
-  real*8, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DR8
-
-
-subroutine MPI_Sizeof5DC8(x, size, ierror)
-  complex*8, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DC8
-
-
-subroutine MPI_Sizeof5DC16(x, size, ierror)
-  complex*16, dimension(1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof5DC16
-
-
-subroutine MPI_Sizeof6DCH(x, size, ierror)
-  character, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DCH
-
-
-subroutine MPI_Sizeof6DL(x, size, ierror)
-  logical, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DL
-
-
-subroutine MPI_Sizeof6DI1(x, size, ierror)
-  integer*1, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DI1
-
-
-subroutine MPI_Sizeof6DI2(x, size, ierror)
-  integer*2, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DI2
-
-
-subroutine MPI_Sizeof6DI4(x, size, ierror)
-  integer*4, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DI4
-
-
-subroutine MPI_Sizeof6DI8(x, size, ierror)
-  integer*8, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DI8
-
-
-subroutine MPI_Sizeof6DR4(x, size, ierror)
-  real*4, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DR4
-
-
-subroutine MPI_Sizeof6DR8(x, size, ierror)
-  real*8, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DR8
-
-
-subroutine MPI_Sizeof6DC8(x, size, ierror)
-  complex*8, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DC8
-
-
-subroutine MPI_Sizeof6DC16(x, size, ierror)
-  complex*16, dimension(1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof6DC16
-
-
-subroutine MPI_Sizeof7DCH(x, size, ierror)
-  character, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DCH
-
-
-subroutine MPI_Sizeof7DL(x, size, ierror)
-  logical, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DL
-
-
-subroutine MPI_Sizeof7DI1(x, size, ierror)
-  integer*1, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DI1
-
-
-subroutine MPI_Sizeof7DI2(x, size, ierror)
-  integer*2, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DI2
-
-
-subroutine MPI_Sizeof7DI4(x, size, ierror)
-  integer*4, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DI4
-
-
-subroutine MPI_Sizeof7DI8(x, size, ierror)
-  integer*8, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DI8
-
-
-subroutine MPI_Sizeof7DR4(x, size, ierror)
-  real*4, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DR4
-
-
-subroutine MPI_Sizeof7DR8(x, size, ierror)
-  real*8, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DR8
-
-
-subroutine MPI_Sizeof7DC8(x, size, ierror)
-  complex*8, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DC8
-
-
-subroutine MPI_Sizeof7DC16(x, size, ierror)
-  complex*16, dimension(1,1,1,1,1,1,*), intent(in) :: x
-  integer, intent(out) :: size
-  integer, intent(out) :: ierror
-end subroutine MPI_Sizeof7DC16
-
-end interface
-
-
 interface MPI_Start
 
 subroutine MPI_Start(request, ierror)
@@ -2344,8 +1481,8 @@ end interface
 
 interface MPI_Type_commit
 
-subroutine MPI_Type_commit(type, ierror)
-  integer, intent(inout) :: type
+subroutine MPI_Type_commit(datatype, ierror)
+  integer, intent(inout) :: datatype
   integer, intent(out) :: ierror
 end subroutine MPI_Type_commit
 
@@ -2529,8 +1666,8 @@ end interface
 
 interface MPI_Type_delete_attr
 
-subroutine MPI_Type_delete_attr(type, type_keyval, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_delete_attr(datatype, type_keyval, ierror)
+  integer, intent(in) :: datatype
   integer, intent(in) :: type_keyval
   integer, intent(out) :: ierror
 end subroutine MPI_Type_delete_attr
@@ -2540,8 +1677,8 @@ end interface
 
 interface MPI_Type_dup
 
-subroutine MPI_Type_dup(type, newtype, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_dup(oldtype, newtype, ierror)
+  integer, intent(in) :: oldtype
   integer, intent(out) :: newtype
   integer, intent(out) :: ierror
 end subroutine MPI_Type_dup
@@ -2551,8 +1688,8 @@ end interface
 
 interface MPI_Type_extent
 
-subroutine MPI_Type_extent(type, extent, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_extent(datatype, extent, ierror)
+  integer, intent(in) :: datatype
   integer, intent(out) :: extent
   integer, intent(out) :: ierror
 end subroutine MPI_Type_extent
@@ -2562,8 +1699,8 @@ end interface
 
 interface MPI_Type_free
 
-subroutine MPI_Type_free(type, ierror)
-  integer, intent(inout) :: type
+subroutine MPI_Type_free(datatype, ierror)
+  integer, intent(inout) :: datatype
   integer, intent(out) :: ierror
 end subroutine MPI_Type_free
 
@@ -2582,9 +1719,9 @@ end interface
 
 interface MPI_Type_get_attr
 
-subroutine MPI_Type_get_attr(type, type_keyval, attribute_val, flag, ierror)
+subroutine MPI_Type_get_attr(datatype, type_keyval, attribute_val, flag, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer, intent(in) :: type_keyval
   integer(kind=MPI_ADDRESS_KIND), intent(out) :: attribute_val
   logical, intent(out) :: flag
@@ -2596,10 +1733,10 @@ end interface
 
 interface MPI_Type_get_contents
 
-subroutine MPI_Type_get_contents(mtype, max_integers, max_addresses, max_datatypes, array_of_integers, &
+subroutine MPI_Type_get_contents(datatype, max_integers, max_addresses, max_datatypes, array_of_integers, &
         array_of_addresses, array_of_datatypes, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: mtype
+  integer, intent(in) :: datatype
   integer, intent(in) :: max_integers
   integer, intent(in) :: max_addresses
   integer, intent(in) :: max_datatypes
@@ -2614,9 +1751,9 @@ end interface
 
 interface MPI_Type_get_envelope
 
-subroutine MPI_Type_get_envelope(type, num_integers, num_addresses, num_datatypes, combiner&
+subroutine MPI_Type_get_envelope(datatype, num_integers, num_addresses, num_datatypes, combiner&
         , ierror)
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer, intent(out) :: num_integers
   integer, intent(out) :: num_addresses
   integer, intent(out) :: num_datatypes
@@ -2629,9 +1766,9 @@ end interface
 
 interface MPI_Type_get_extent
 
-subroutine MPI_Type_get_extent(type, lb, extent, ierror)
+subroutine MPI_Type_get_extent(datatype, lb, extent, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer(kind=MPI_ADDRESS_KIND), intent(out) :: lb
   integer(kind=MPI_ADDRESS_KIND), intent(out) :: extent
   integer, intent(out) :: ierror
@@ -2642,9 +1779,9 @@ end interface
 
 interface MPI_Type_get_extent_x
 
-subroutine MPI_Type_get_extent_x(type, lb, extent, ierror)
+subroutine MPI_Type_get_extent_x(datatype, lb, extent, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer(kind=MPI_COUNT_KIND), intent(out) :: lb
   integer(kind=MPI_COUNT_KIND), intent(out) :: extent
   integer, intent(out) :: ierror
@@ -2655,8 +1792,8 @@ end interface
 
 interface MPI_Type_get_name
 
-subroutine MPI_Type_get_name(type, type_name, resultlen, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_get_name(datatype, type_name, resultlen, ierror)
+  integer, intent(in) :: datatype
   character(len=*), intent(out) :: type_name
   integer, intent(out) :: resultlen
   integer, intent(out) :: ierror
@@ -2738,8 +1875,8 @@ end interface
 
 interface MPI_Type_lb
 
-subroutine MPI_Type_lb(type, lb, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_lb(datatype, lb, ierror)
+  integer, intent(in) :: datatype
   integer, intent(out) :: lb
   integer, intent(out) :: ierror
 end subroutine MPI_Type_lb
@@ -2749,10 +1886,10 @@ end interface
 
 interface MPI_Type_match_size
 
-subroutine MPI_Type_match_size(typeclass, size, type, ierror)
+subroutine MPI_Type_match_size(typeclass, size, datatype, ierror)
   integer, intent(in) :: typeclass
   integer, intent(in) :: size
-  integer, intent(out) :: type
+  integer, intent(out) :: datatype
   integer, intent(out) :: ierror
 end subroutine MPI_Type_match_size
 
@@ -2761,9 +1898,9 @@ end interface
 
 interface MPI_Type_set_attr
 
-subroutine MPI_Type_set_attr(type, type_keyval, attr_val, ierror)
+subroutine MPI_Type_set_attr(datatype, type_keyval, attr_val, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer, intent(in) :: type_keyval
   integer(kind=MPI_ADDRESS_KIND), intent(in) :: attr_val
   integer, intent(out) :: ierror
@@ -2774,8 +1911,8 @@ end interface
 
 interface MPI_Type_set_name
 
-subroutine MPI_Type_set_name(type, type_name, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_set_name(datatype, type_name, ierror)
+  integer, intent(in) :: datatype
   character(len=*), intent(in) :: type_name
   integer, intent(out) :: ierror
 end subroutine MPI_Type_set_name
@@ -2785,8 +1922,8 @@ end interface
 
 interface MPI_Type_size
 
-subroutine MPI_Type_size(type, size, ierror)
-  integer, intent(in) :: type
+subroutine MPI_Type_size(datatype, size, ierror)
+  integer, intent(in) :: datatype
   integer, intent(out) :: size
   integer, intent(out) :: ierror
 end subroutine MPI_Type_size
@@ -2796,9 +1933,9 @@ end interface
 
 interface MPI_Type_size_x
 
-subroutine MPI_Type_size_x(type, size, ierror)
+subroutine MPI_Type_size_x(datatype, size, ierror)
   include 'mpif-config.h'
-  integer, intent(in) :: type
+  integer, intent(in) :: datatype
   integer(kind=MPI_COUNT_KIND), intent(out) :: size
   integer, intent(out) :: ierror
 end subroutine MPI_Type_size_x
@@ -2823,8 +1960,8 @@ end interface
 
 interface MPI_Type_ub
 
-subroutine MPI_Type_ub(mtype, ub, ierror)
-  integer, intent(in) :: mtype
+subroutine MPI_Type_ub(datatype, ub, ierror)
+  integer, intent(in) :: datatype
   integer, intent(out) :: ub
   integer, intent(out) :: ierror
 end subroutine MPI_Type_ub
