@@ -11,6 +11,9 @@
  *                         All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2017      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -205,7 +208,10 @@ JNIEXPORT jlongArray JNICALL Java_mpi_Comm_iDup(
     MPI_Comm newcomm;
     MPI_Request request;
     int rc = MPI_Comm_idup((MPI_Comm)comm, &newcomm, &request);
-    ompi_java_exceptionCheck(env, rc);
+    
+    if(ompi_java_exceptionCheck(env, rc))
+        return NULL;
+    
     jlongArray jcr = (*env)->NewLongArray(env, 2);
     jlong *cr = (jlong*)(*env)->GetPrimitiveArrayCritical(env, jcr, NULL);
     cr[0] = (jlong)newcomm;
@@ -332,6 +338,7 @@ JNIEXPORT void JNICALL Java_mpi_Comm_recv(
         jobject buf, jboolean db, jint offset, jint count,
         jlong jType, jint bType, jint source, jint tag, jlongArray jStatus)
 {
+    jboolean exception;
     MPI_Comm     comm = (MPI_Comm)jComm;
     MPI_Datatype type = (MPI_Datatype)jType;
 
@@ -341,10 +348,12 @@ JNIEXPORT void JNICALL Java_mpi_Comm_recv(
 
     MPI_Status status;
     int rc = MPI_Recv(ptr, count, type, source, tag, comm, &status);
-    ompi_java_exceptionCheck(env, rc);
+    exception = ompi_java_exceptionCheck(env, rc);
 
     ompi_java_releaseWritePtr(ptr,item,env,buf,db,offset,count,type,bType);
-    ompi_java_status_set(env, jStatus, &status);
+    
+    if(!exception)
+        ompi_java_status_set(env, jStatus, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_Comm_sendRecv(
@@ -355,6 +364,7 @@ JNIEXPORT void JNICALL Java_mpi_Comm_sendRecv(
         jlong rjType, jint rBType, jint source, jint rTag,
         jlongArray jStatus)
 {
+    jboolean exception;
     MPI_Comm     comm  = (MPI_Comm)jComm;
     MPI_Datatype sType = (MPI_Datatype)sjType;
     MPI_Datatype rType = (MPI_Datatype)rjType;
@@ -369,10 +379,12 @@ JNIEXPORT void JNICALL Java_mpi_Comm_sendRecv(
     int rc = MPI_Sendrecv(sPtr, sCount, sType, dest, sTag,
                           rPtr, rCount, rType, source, rTag, comm, &status);
 
-    ompi_java_exceptionCheck(env, rc);
+    exception = ompi_java_exceptionCheck(env, rc);
     ompi_java_releaseReadPtr(sPtr, sItem, sBuf, sdb);
     ompi_java_releaseWritePtr(rPtr,rItem,env,rBuf,rdb,rOff,rCount,rType,rBType);
-    ompi_java_status_set(env, jStatus, &status);
+    
+    if(!exception)
+        ompi_java_status_set(env, jStatus, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_Comm_sendRecvReplace(
@@ -392,8 +404,9 @@ JNIEXPORT void JNICALL Java_mpi_Comm_sendRecvReplace(
     int rc = MPI_Sendrecv_replace(ptr, count, type, dest,
                                   sTag, source, rTag, comm, &status);
 
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, jStatus, &status);
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, jStatus, &status);
+
     ompi_java_releaseWritePtr(ptr,item,env,buf,db,offset,count,type,bType);
 }
 
@@ -662,8 +675,9 @@ JNIEXPORT void JNICALL Java_mpi_Comm_probe(
 {
     MPI_Status status;
     int rc = MPI_Probe(source, tag, (MPI_Comm)comm, &status);
-    ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(env, jStatus, &status);
+    
+    if(!ompi_java_exceptionCheck(env, rc))
+        ompi_java_status_set(env, jStatus, &status);
 }
 
 JNIEXPORT jint JNICALL Java_mpi_Comm_getTopology(
@@ -695,6 +709,13 @@ JNIEXPORT jlong JNICALL Java_mpi_Comm_getErrhandler(
     return (jlong)errhandler;
 }
 
+JNIEXPORT void JNICALL Java_mpi_Comm_callErrhandler(
+        JNIEnv *env, jobject jthis, jlong comm, jint errorCode)
+{
+    int rc = MPI_Comm_call_errhandler((MPI_Comm)comm, errorCode);
+    ompi_java_exceptionCheck(env, rc);
+}
+
 static int commCopyAttr(MPI_Comm oldcomm, int keyval, void *extraState,
                         void *attrValIn, void *attrValOut, int *flag)
 {
@@ -1628,8 +1649,8 @@ JNIEXPORT void JNICALL Java_mpi_Comm_allToAllw(
 
 JNIEXPORT jlong JNICALL Java_mpi_Comm_iAllToAllw(
 		JNIEnv *env, jobject jthis, jlong jComm,
-		jobject sendBuf, jintArray sCount, jintArray sDispls, jlongArray sTypes,
-		jobject recvBuf, jintArray rCount, jintArray rDispls, jlongArray rTypes)
+				jobject sendBuf, jintArray sCount, jintArray sDispls, jlongArray sTypes,
+				jobject recvBuf, jintArray rCount, jintArray rDispls, jlongArray rTypes)
 {
 	MPI_Comm     comm  = (MPI_Comm)jComm;
 
