@@ -16,9 +16,7 @@
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
- *
  * Additional copyrights may follow
- *
  * $HEADER$
  */
 
@@ -164,6 +162,8 @@ struct opal_class_t {
     size_t cls_sizeof;              /**< size of an object instance */
 };
 
+extern int opal_class_init_epoch;
+
 /**
  * For static initializations of OBJects.
  *
@@ -318,12 +318,14 @@ static inline opal_object_t *opal_obj_new_debug(opal_class_t* type, const char*
  * to NULL.
  *
  * @param object        Pointer to the object
+ *
+ *
  */
 #if OPAL_ENABLE_DEBUG
 #define OBJ_RELEASE(object)                                             \
     do {                                                                \
-        assert(NULL != ((opal_object_t *) (object))->obj_class);        \
         assert(OPAL_OBJ_MAGIC_ID == ((opal_object_t *) (object))->obj_magic_id); \
+        assert(NULL != ((opal_object_t *) (object))->obj_class);        \
         if (0 == opal_obj_update((opal_object_t *) (object), -1)) {     \
             OBJ_SET_MAGIC_ID((object), 0);                              \
             opal_obj_run_destructors((opal_object_t *) (object));       \
@@ -358,8 +360,8 @@ do {                                                            \
 
 #define OBJ_CONSTRUCT_INTERNAL(object, type)                        \
 do {                                                                \
-    OBJ_SET_MAGIC_ID((object), OPAL_OBJ_MAGIC_ID);              \
-    if (0 == (type)->cls_initialized) {                             \
+    OBJ_SET_MAGIC_ID((object), OPAL_OBJ_MAGIC_ID);                  \
+    if (opal_class_init_epoch != (type)->cls_initialized) {         \
         opal_class_initialize((type));                              \
     }                                                               \
     ((opal_object_t *) (object))->obj_class = (type);               \
@@ -483,7 +485,7 @@ static inline opal_object_t *opal_obj_new(opal_class_t * cls)
 #else
     object = (opal_object_t *) malloc(cls->cls_sizeof);
 #endif
-    if (0 == cls->cls_initialized) {
+    if (opal_class_init_epoch != cls->cls_initialized) {
         opal_class_initialize(cls);
     }
     if (NULL != object) {
