@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2014 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -12,6 +12,9 @@
  *                         All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2014      NVIDIA Corporation.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -52,6 +55,8 @@ BEGIN_C_DECLS
 #define CONVERTOR_STATE_ALLOC      0x04000000
 #define CONVERTOR_COMPLETED        0x08000000
 #define CONVERTOR_CUDA_UNIFIED     0x10000000
+#define CONVERTOR_HAS_REMOTE_SIZE  0x20000000
+#define CONVERTOR_SKIP_CUDA_INIT   0x40000000
 
 union dt_elem_desc;
 typedef struct opal_convertor_t opal_convertor_t;
@@ -70,7 +75,7 @@ struct dt_stack_t {
     int32_t           index;    /**< index in the element description */
     int16_t           type;     /**< the type used for the last pack/unpack (original or OPAL_DATATYPE_UINT1) */
     size_t            count;    /**< number of times we still have to do it */
-    OPAL_PTRDIFF_TYPE disp;     /**< actual displacement depending on the count field */
+    ptrdiff_t         disp;     /**< actual displacement depending on the count field */
 };
 typedef struct dt_stack_t dt_stack_t;
 
@@ -184,9 +189,16 @@ static inline int32_t opal_convertor_need_buffers( const opal_convertor_t* pConv
     return 1;
 }
 
+/**
+ * Update the size of the remote datatype representation. The size will
+ * depend on the configuration of the master convertor. In homogeneous
+ * environments, the local and remote sizes are identical.
+ */
+size_t
+opal_convertor_compute_remote_size( opal_convertor_t* pConv );
 
-/*
- *
+/**
+ * Return the local size of the convertor (count times the size of the datatype).
  */
 static inline void opal_convertor_get_packed_size( const opal_convertor_t* pConv,
                                                    size_t* pSize )
@@ -195,16 +207,25 @@ static inline void opal_convertor_get_packed_size( const opal_convertor_t* pConv
 }
 
 
-/*
- *
+/**
+ * Return the remote size of the convertor (count times the remote size of the
+ * datatype). On homogeneous environments the local and remote sizes are
+ * identical.
  */
 static inline void opal_convertor_get_unpacked_size( const opal_convertor_t* pConv,
                                                      size_t* pSize )
 {
+    if( pConv->flags & CONVERTOR_HOMOGENEOUS ) {
+        *pSize = pConv->local_size;
+        return;
+    }
+    if( 0 == (CONVERTOR_HAS_REMOTE_SIZE & pConv->flags) ) {
+        assert(! (pConv->flags & CONVERTOR_SEND));
+        opal_convertor_compute_remote_size( (opal_convertor_t*)pConv);
+    }
     *pSize = pConv->remote_size;
 }
 
-
 /**
  * Return the current absolute position of the next pack/unpack. This function is
  * mostly useful for contiguous datatypes, when we need to get the pointer to the
@@ -277,6 +298,7 @@ opal_convertor_raw( opal_convertor_t* convertor,  /* [IN/OUT] */
                     uint32_t* iov_count,          /* [IN/OUT] */
                     size_t* length );             /* [OUT]    */
 
+
 /*
  * Upper level does not need to call the _nocheck function directly.
  */
