@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -499,6 +499,9 @@ int opal_dss_pack_pstat(opal_buffer_t *buffer, const void *src,
         if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->num_threads, 1, OPAL_INT16))) {
             return ret;
         }
+        if (OPAL_SUCCESS != (ret = opal_dss_pack_float(buffer, &ptr[i]->pss, 1, OPAL_FLOAT))) {
+            return ret;
+        }
         if (OPAL_SUCCESS != (ret = opal_dss_pack_float(buffer, &ptr[i]->vsize, 1, OPAL_FLOAT))) {
             return ret;
         }
@@ -769,6 +772,10 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
             }
             break;
         case OPAL_UINT8:
+        case OPAL_PERSIST:
+        case OPAL_SCOPE:
+        case OPAL_DATA_RANGE:
+        case OPAL_PROC_STATE:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.uint8, 1, OPAL_UINT8))) {
                 return ret;
             }
@@ -779,6 +786,7 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
             }
             break;
         case OPAL_UINT32:
+        case OPAL_INFO_DIRECTIVES:
             if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.uint32, 1, OPAL_UINT32))) {
                 return ret;
             }
@@ -823,296 +831,13 @@ int opal_dss_pack_value(opal_buffer_t *buffer, const void *src,
                 return ret;
             }
             break;
-        case OPAL_FLOAT_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.fval_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.fval_array.data,
-                                            ptr[i]->data.fval_array.size,
-                                            OPAL_FLOAT))) {
-                return ret;
-            }
-            break;
-        case OPAL_DOUBLE_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.dval_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.dval_array.data,
-                                            ptr[i]->data.dval_array.size,
-                                            OPAL_DOUBLE))) {
-                return ret;
-            }
-            break;
-        case OPAL_STRING_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.string_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.string_array.data,
-                                            ptr[i]->data.string_array.size,
-                                            OPAL_STRING))) {
-                return ret;
-            }
-            break;
-        case OPAL_BOOL_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.flag_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.flag_array.data,
-                                            ptr[i]->data.flag_array.size,
-                                            OPAL_BOOL))) {
-                return ret;
-            }
-            break;
-        case OPAL_SIZE_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.size_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.size_array.data,
-                                            ptr[i]->data.size_array.size,
-                                            OPAL_SIZE))) {
-                return ret;
-            }
-            break;
-        case OPAL_BYTE_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.byte_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.byte_array.data,
-                                            ptr[i]->data.byte_array.size,
-                                            OPAL_BYTE))) {
-                return ret;
-            }
-            break;
-        case OPAL_INT_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.integer_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.integer_array.data,
-                                            ptr[i]->data.integer_array.size,
-                                            OPAL_INT))) {
-                return ret;
-            }
-            break;
-        case OPAL_INT8_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.int8_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.int8_array.data,
-                                            ptr[i]->data.int8_array.size,
-                                            OPAL_INT8))) {
-                return ret;
-            }
-            break;
-        case OPAL_INT16_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.int16_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.int16_array.data,
-                                            ptr[i]->data.int16_array.size,
-                                            OPAL_INT16))) {
-                return ret;
-            }
-            break;
-        case OPAL_INT32_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.int32_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.int32_array.data,
-                                            ptr[i]->data.int32_array.size,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            break;
-        case OPAL_INT64_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.int64_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.int64_array.data,
-                                            ptr[i]->data.int64_array.size,
-                                            OPAL_INT64))) {
-                return ret;
-            }
-            break;
-        case OPAL_UINT_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.uint_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.uint_array.data,
-                                            ptr[i]->data.uint_array.size,
-                                            OPAL_UINT))) {
-                return ret;
-            }
-            break;
-        case OPAL_UINT8_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.uint8_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.uint8_array.data,
-                                            ptr[i]->data.uint8_array.size,
-                                            OPAL_UINT8))) {
-                return ret;
-            }
-            break;
-        case OPAL_UINT16_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.uint16_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.uint16_array.data,
-                                            ptr[i]->data.uint16_array.size,
-                                            OPAL_UINT16))) {
-                return ret;
-            }
-            break;
-        case OPAL_UINT32_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.uint32_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.uint32_array.data,
-                                            ptr[i]->data.uint32_array.size,
-                                            OPAL_UINT32))) {
-                return ret;
-            }
-            break;
-        case OPAL_UINT64_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.uint64_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.uint64_array.data,
-                                            ptr[i]->data.uint64_array.size,
-                                            OPAL_UINT64))) {
-                return ret;
-            }
-            break;
-        case OPAL_PID_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.pid_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.pid_array.data,
-                                            ptr[i]->data.pid_array.size,
-                                            OPAL_PID))) {
-                return ret;
-            }
-            break;
-        case OPAL_TIMEVAL_ARRAY:
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            &ptr[i]->data.tv_array.size,
-                                            1,
-                                            OPAL_INT32))) {
-                return ret;
-            }
-            if (OPAL_SUCCESS !=
-                (ret = opal_dss_pack_buffer(buffer,
-                                            ptr[i]->data.tv_array.data,
-                                            ptr[i]->data.tv_array.size,
-                                            OPAL_TIMEVAL))) {
+        case OPAL_STATUS:
+            if (OPAL_SUCCESS != (ret = opal_dss_pack_buffer(buffer, &ptr[i]->data.status, 1, OPAL_INT))) {
                 return ret;
             }
             break;
         default:
-            opal_output(0, "PACK-OPAL-VALUE: UNSUPPORTED TYPE %d", (int)ptr[i]->type);
+            opal_output(0, "PACK-OPAL-VALUE: UNSUPPORTED TYPE %d FOR KEY %s", (int)ptr[i]->type, ptr[i]->key);
             return OPAL_ERROR;
         }
     }
@@ -1240,3 +965,19 @@ int opal_dss_pack_vpid(opal_buffer_t *buffer, const void *src,
     return ret;
 }
 
+/*
+ * STATUS
+ */
+int opal_dss_pack_status(opal_buffer_t *buffer, const void *src,
+                         int32_t num_vals, opal_data_type_t type)
+{
+    int ret;
+
+    /* Turn around and pack the real type */
+    ret = opal_dss_pack_buffer(buffer, src, num_vals, OPAL_INT);
+    if (OPAL_SUCCESS != ret) {
+        OPAL_ERROR_LOG(ret);
+    }
+
+    return ret;
+}
