@@ -1,8 +1,13 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2013-2015 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2013-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Bull SAS.  All rights reserved.
+ * Copyright (c) 2015      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -144,7 +149,7 @@ static int mca_base_pvar_default_get_value (const mca_base_pvar_t *pvar, void *v
     /* not used */
     (void) obj_handle;
 
-    memmove (value, pvar->ctx, var_type_sizes[pvar->type]);
+    memmove (value, pvar->ctx, ompi_var_type_sizes[pvar->type]);
 
     return OPAL_SUCCESS;
 }
@@ -154,7 +159,7 @@ static int mca_base_pvar_default_set_value (mca_base_pvar_t *pvar, const void *v
     /* not used */
     (void) obj_handle;
 
-    memmove (pvar->ctx, value, var_type_sizes[pvar->type]);
+    memmove (pvar->ctx, value, ompi_var_type_sizes[pvar->type]);
 
     return OPAL_SUCCESS;
 }
@@ -298,6 +303,7 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
                 }
             }
 
+            pvar->pvar_index = pvar_count;
             opal_hash_table_set_value_ptr (&mca_base_pvar_index_hash, pvar->name, strlen (pvar->name),
                                            (void *)(uintptr_t) pvar->pvar_index);
 
@@ -332,7 +338,6 @@ int mca_base_pvar_register (const char *project, const char *framework, const ch
     }
 
     pvar->ctx        = ctx;
-    pvar->pvar_index = pvar_count;
 
     return pvar->pvar_index;
 }
@@ -343,9 +348,8 @@ int mca_base_component_pvar_register (const mca_base_component_t *component, con
                                       int bind, mca_base_pvar_flag_t flags, mca_base_get_value_fn_t get_value,
                                       mca_base_set_value_fn_t set_value, mca_base_notify_fn_t notify, void *ctx)
 {
-    /* XXX -- component_update -- We will stash the project name in the component */
     /* invalidate this variable if the component's group is deregistered */
-    return mca_base_pvar_register(NULL, component->mca_type_name, component->mca_component_name,
+    return mca_base_pvar_register(component->mca_project_name, component->mca_type_name, component->mca_component_name,
                                   name, description, verbosity, var_class, type, enumerator, bind,
                                   flags | MCA_BASE_PVAR_FLAG_IWG, get_value, set_value, notify, ctx);
 }
@@ -477,7 +481,7 @@ int mca_base_pvar_handle_alloc (mca_base_pvar_session_t *session, int index, voi
 
         /* get the size of this datatype since read functions will expect an
            array of datatype not mca_base_pvar_value_t's. */
-        datatype_size = var_type_sizes[pvar->type];
+        datatype_size = ompi_var_type_sizes[pvar->type];
         if (0 == datatype_size) {
             ret = OPAL_ERROR;
             break;
@@ -685,7 +689,7 @@ int mca_base_pvar_handle_read_value (mca_base_pvar_handle_t *handle, void *value
     if (mca_base_pvar_is_sum (handle->pvar) || mca_base_pvar_is_watermark (handle->pvar) ||
         !mca_base_pvar_handle_is_running (handle)) {
         /* read the value cached in the handle. */
-        memmove (value, handle->current_value, handle->count * var_type_sizes[handle->pvar->type]);
+        memmove (value, handle->current_value, handle->count * ompi_var_type_sizes[handle->pvar->type]);
     } else {
         /* read the value directly from the variable. */
         ret = handle->pvar->get_value (handle->pvar, value, handle->obj_handle);
@@ -706,14 +710,17 @@ int mca_base_pvar_handle_write_value (mca_base_pvar_handle_t *handle, const void
         return OPAL_ERR_PERM;
     }
 
-    /* TODO -- actually write the variable. this will likely require a pvar lock */
+    /* write the value directly from the variable. */
+    ret = handle->pvar->set_value (handle->pvar, value, handle->obj_handle);
 
     ret = mca_base_pvar_handle_update (handle);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
 
-    memmove (handle->current_value, value, handle->count * var_type_sizes[handle->pvar->type]);
+    memmove (handle->current_value, value, handle->count * ompi_var_type_sizes[handle->pvar->type]);
+    /* read the value directly from the variable. */
+    ret = handle->pvar->set_value (handle->pvar, value, handle->obj_handle);
 
     return OPAL_SUCCESS;
 }
@@ -792,7 +799,7 @@ int mca_base_pvar_handle_reset (mca_base_pvar_handle_t *handle)
     /* reset this handle to a state analagous to when it was created */
     if (mca_base_pvar_is_sum (handle->pvar)) {
         /* reset the running sum to 0 */
-        memset (handle->current_value, 0, handle->count * var_type_sizes[handle->pvar->type]);
+        memset (handle->current_value, 0, handle->count * ompi_var_type_sizes[handle->pvar->type]);
 
         if (mca_base_pvar_handle_is_running (handle)) {
             ret = handle->pvar->get_value (handle->pvar, handle->last_value, handle->obj_handle);
@@ -872,7 +879,7 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
             }
         }
 
-        (void)asprintf(out[0] + line++, "%stype:%s", tmp, var_type_names[pvar->type]);
+        (void)asprintf(out[0] + line++, "%stype:%s", tmp, ompi_var_type_names[pvar->type]);
         free(tmp);  // release tmp storage
     } else {
         /* there will be at most three lines in the pretty print case */
@@ -882,7 +889,7 @@ int mca_base_pvar_dump(int index, char ***out, mca_base_var_dump_type_t output_t
         }
 
         (void)asprintf (out[0] + line++, "performance \"%s\" (type: %s, class: %s)", full_name,
-                        var_type_names[pvar->type], pvar_class_names[pvar->var_class]);
+                        ompi_var_type_names[pvar->type], pvar_class_names[pvar->var_class]);
 
         if (pvar->description) {
             (void)asprintf(out[0] + line++, "%s", pvar->description);
