@@ -10,12 +10,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2017 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2008-2018 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2012-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -66,10 +67,9 @@ static char *mca_base_var_override_file = NULL;
 static char *mca_base_var_file_prefix = NULL;
 static char *mca_base_envar_file_prefix = NULL;
 static char *mca_base_param_file_path = NULL;
-static char *mca_base_env_list = NULL;
-#define MCA_BASE_ENV_LIST_SEP_DEFAULT ";"
-static char *mca_base_env_list_sep = MCA_BASE_ENV_LIST_SEP_DEFAULT;
-static char *mca_base_env_list_internal = NULL;
+char *mca_base_env_list = NULL;
+char *mca_base_env_list_sep = MCA_BASE_ENV_LIST_SEP_DEFAULT;
+char *mca_base_env_list_internal = NULL;
 static bool mca_base_var_suppress_override_warning = false;
 static opal_list_t mca_base_var_file_values;
 static opal_list_t mca_base_envar_file_values;
@@ -79,7 +79,7 @@ static int mca_base_var_count = 0;
 
 static opal_hash_table_t mca_base_var_index_hash;
 
-const char *var_type_names[] = {
+const char *ompi_var_type_names[] = {
     "int",
     "unsigned_int",
     "unsigned_long",
@@ -91,7 +91,7 @@ const char *var_type_names[] = {
     "double"
 };
 
-const size_t var_type_sizes[] = {
+const size_t ompi_var_type_sizes[] = {
     sizeof (int),
     sizeof (unsigned),
     sizeof (unsigned long),
@@ -103,7 +103,7 @@ const size_t var_type_sizes[] = {
     sizeof (double)
 };
 
-const char *var_source_names[] = {
+static const char *var_source_names[] = {
     "default",
     "command line",
     "environment",
@@ -130,7 +130,6 @@ static const char *info_lvl_strings[] = {
  */
 static int fixup_files(char **file_list, char * path, bool rel_path_search, char sep);
 static int read_files (char *file_list, opal_list_t *file_values, char sep);
-static int mca_base_var_cache_files (bool rel_path_search);
 static int var_set_initial (mca_base_var_t *var, mca_base_var_t *original);
 static int var_get (int vari, mca_base_var_t **var_out, bool original);
 static int var_value_string (mca_base_var_t *var, char **value_string);
@@ -242,7 +241,6 @@ static char *append_filename_to_list(const char *filename)
 int mca_base_var_init(void)
 {
     int ret;
-    char *name = NULL;
 
     if (!mca_base_var_initialized) {
         /* Init the value array for the param storage */
@@ -282,41 +280,6 @@ int mca_base_var_init(void)
 
         mca_base_var_initialized = true;
 
-        mca_base_var_cache_files(false);
-
-        /* register the envar-forwarding params */
-        (void)mca_base_var_register ("opal", "mca", "base", "env_list",
-                                     "Set SHELL env variables",
-                                     MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
-                                     MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list);
-
-        mca_base_env_list_sep = MCA_BASE_ENV_LIST_SEP_DEFAULT;
-        (void)mca_base_var_register ("opal", "mca", "base", "env_list_delimiter",
-                                     "Set SHELL env variables delimiter. Default: semicolon ';'",
-                                     MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
-                                     MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list_sep);
-
-        /* Set OMPI_MCA_mca_base_env_list variable, it might not be set before
-         * if mca variable was taken from amca conf file. Need to set it
-         * here because mca_base_var_process_env_list is called from schizo_ompi.c
-         * only when this env variable was set.
-         */
-        if (NULL != mca_base_env_list) {
-            (void) mca_base_var_env_name ("mca_base_env_list", &name);
-            if (NULL != name) {
-                opal_setenv(name, mca_base_env_list, false, &environ);
-                free(name);
-            }
-        }
-
-        /* Register internal MCA variable mca_base_env_list_internal. It can be set only during
-         * parsing of amca conf file and contains SHELL env variables specified via -x there.
-         * Its format is the same as for mca_base_env_list.
-         */
-        (void)mca_base_var_register ("opal", "mca", "base", "env_list_internal",
-                "Store SHELL env variables from amca conf file",
-                MCA_BASE_VAR_TYPE_STRING, NULL, 0, MCA_BASE_VAR_FLAG_INTERNAL, OPAL_INFO_LVL_3,
-                MCA_BASE_VAR_SCOPE_READONLY, &mca_base_env_list_internal);
     }
 
     return OPAL_SUCCESS;
@@ -366,7 +329,7 @@ static void process_env_list(char *env_list, char ***argv, char sep)
     opal_argv_free(tokens);
 }
 
-int mca_base_var_process_env_list(char ***argv)
+int mca_base_var_process_env_list(char *list, char ***argv)
 {
     char sep;
     sep = ';';
@@ -379,7 +342,9 @@ int mca_base_var_process_env_list(char ***argv)
             return OPAL_SUCCESS;
         }
     }
-    if (NULL != mca_base_env_list) {
+    if (NULL != list) {
+        process_env_list(list, argv, sep);
+    } else if (NULL != mca_base_env_list) {
         process_env_list(mca_base_env_list, argv, sep);
     }
 
@@ -417,7 +382,7 @@ static void resolve_relative_paths(char **file_prefix, char *file_path, bool rel
     }
 }
 
-static int mca_base_var_cache_files(bool rel_path_search)
+int mca_base_var_cache_files(bool rel_path_search)
 {
     char *tmp;
     int ret;
@@ -425,7 +390,7 @@ static int mca_base_var_cache_files(bool rel_path_search)
     /* We may need this later */
     home = (char*)opal_home_directory();
 
-    if(NULL == cwd) {
+    if (NULL == cwd) {
         cwd = (char *) malloc(sizeof(char) * MAXPATHLEN);
         if( NULL == (cwd = getcwd(cwd, MAXPATHLEN) )) {
             opal_output(0, "Error: Unable to get the current working directory\n");
@@ -450,7 +415,7 @@ static int mca_base_var_cache_files(bool rel_path_search)
                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_2,
                                  MCA_BASE_VAR_SCOPE_READONLY, &mca_base_var_files);
     free (tmp);
-    if (OPAL_SUCCESS != ret) {
+    if (0 > ret) {
         return ret;
     }
 
@@ -806,7 +771,7 @@ int mca_base_var_set_value (int vari, const void *value, size_t size, mca_base_v
     }
 
     if (MCA_BASE_VAR_TYPE_STRING != var->mbv_type && MCA_BASE_VAR_TYPE_VERSION_STRING != var->mbv_type) {
-        memmove (var->mbv_storage, value, var_type_sizes[var->mbv_type]);
+        memmove (var->mbv_storage, value, ompi_var_type_sizes[var->mbv_type]);
     } else {
         var_set_string (var, (char *) value);
     }
@@ -1926,13 +1891,13 @@ static char *source_name(mca_base_var_t *var)
 
 static int var_value_string (mca_base_var_t *var, char **value_string)
 {
-    const mca_base_var_storage_t *value;
+    const mca_base_var_storage_t *value=NULL;
     int ret;
 
     assert (MCA_BASE_VAR_TYPE_MAX > var->mbv_type);
 
     ret = mca_base_var_get_value(var->mbv_index, &value, NULL, NULL);
-    if (OPAL_SUCCESS !=ret) {
+    if (OPAL_SUCCESS != ret || NULL == value) {
         return ret;
     }
 
@@ -2125,7 +2090,8 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
         asprintf(out[0] + line++, "%ssource:%s", tmp, source_string);
 
         /* Output whether it's read only or writable */
-        asprintf(out[0] + line++, "%sstatus:%s", tmp, VAR_IS_DEFAULT_ONLY(var[0]) ? "read-only" : "writeable");
+        asprintf(out[0] + line++, "%sstatus:%s", tmp,
+                VAR_IS_SETTABLE(var[0]) ? "writeable" : "read-only");
 
         /* Output the info level of this parametere */
         asprintf(out[0] + line++, "%slevel:%d", tmp, var->mbv_info_lvl + 1);
@@ -2153,7 +2119,7 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
         /* Is this variable deprecated? */
         asprintf(out[0] + line++, "%sdeprecated:%s", tmp, VAR_IS_DEPRECATED(var[0]) ? "yes" : "no");
 
-        asprintf(out[0] + line++, "%stype:%s", tmp, var_type_names[var->mbv_type]);
+        asprintf(out[0] + line++, "%stype:%s", tmp, ompi_var_type_names[var->mbv_type]);
 
         /* Does this parameter have any synonyms or is it a synonym? */
         if (VAR_IS_SYNONYM(var[0])) {
@@ -2184,7 +2150,7 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
         asprintf (out[0], "%s \"%s\" (current value: \"%s\", data source: %s, level: %d %s, type: %s",
                   VAR_IS_DEFAULT_ONLY(var[0]) ? "informational" : "parameter",
                   full_name, value_string, source_string, var->mbv_info_lvl + 1,
-                  info_lvl_strings[var->mbv_info_lvl], var_type_names[var->mbv_type]);
+                  info_lvl_strings[var->mbv_info_lvl], ompi_var_type_names[var->mbv_type]);
 
         tmp = out[0][0];
         if (VAR_IS_DEPRECATED(var[0])) {
