@@ -41,6 +41,10 @@
 #include "opal/mca/btl/btl.h"
 #include "opal/mca/btl/base/btl_base_error.h"
 
+#if OPAL_ENABLE_FT_CR == 1
+#include "opal/runtime/opal_cr.h"
+#endif
+
 #include "btl_openib_ini.h"
 
 #include "btl_openib.h"
@@ -107,7 +111,7 @@ mca_btl_openib_module_t mca_btl_openib_module = {
         .btl_get = mca_btl_openib_get,
         .btl_dump = mca_btl_base_dump,
         .btl_register_error = mca_btl_openib_register_error_cb, /* error call back registration */
-        .btl_ft_event = NULL,
+        .btl_ft_event = mca_btl_openib_ft_event,
         .btl_register_mem = mca_btl_openib_register_mem,
         .btl_deregister_mem = mca_btl_openib_deregister_mem,
 #if HAVE_DECL_IBV_ATOMIC_HCA
@@ -222,7 +226,7 @@ static int adjust_cq(mca_btl_openib_device_t *device, const int cq)
         rc = ibv_resize_cq(device->ib_cq[cq], cq_size);
         /* For ConnectX the resize CQ is not implemented and verbs returns -ENOSYS
          * but should return ENOSYS. So it is reason for abs */
-        if(rc && ENOSYS != abs(rc)) {
+        if(rc && ENOSYS != abs(rc) && EOPNOTSUPP != abs(rc)) {
             BTL_ERROR(("cannot resize completion queue, error: %d", rc));
             return OPAL_ERROR;
         }
@@ -1954,3 +1958,53 @@ static int mca_btl_openib_deregister_mem (mca_btl_base_module_t *btl, mca_btl_ba
     return OPAL_SUCCESS;
 }
 
+#if OPAL_ENABLE_FT_CR == 0
+int mca_btl_openib_ft_event(int state) {
+    return OPAL_SUCCESS;
+}
+#else
+int mca_btl_openib_ft_event(int state) {
+    int i;
+
+    if(OPAL_CRS_CHECKPOINT == state) {
+        /* Continue must reconstruct the routes (including modex), since we
+         * have to tear down the devices completely. */
+        opal_cr_continue_like_restart = true;
+
+        /*
+         * To keep the node from crashing we need to call ibv_close_device
+         * before the checkpoint is taken. To do this we need to tear
+         * everything down, and rebuild it all on continue/restart. :(
+         */
+
+        /* Shutdown all modules
+         * - Do this backwards since the openib_finalize function also loops
+         *   over this variable.
+         */
+        for (i = 0; i < mca_btl_openib_component.ib_num_btls; ++i ) {
+            mca_btl_openib_finalize_resources( &(mca_btl_openib_component.openib_btls[i])->super);
+        }
+
+        mca_btl_openib_component.devices_count = 0;
+        mca_btl_openib_component.ib_num_btls = 0;
+        OBJ_DESTRUCT(&mca_btl_openib_component.ib_procs);
+
+        opal_btl_openib_connect_base_finalize();
+    }
+    else if(OPAL_CRS_CONTINUE == state) {
+        ; /* Cleared by forcing the modex, no work needed */
+    }
+    else if(OPAL_CRS_RESTART == state) {
+        ;
+    }
+    else if(OPAL_CRS_TERM == state ) {
+        ;
+    }
+    else {
+        ;
+    }
+
+    return OPAL_SUCCESS;
+}
+
+#endif /* OPAL_ENABLE_FT_CR */
