@@ -20,8 +20,6 @@
  * Copyright (c) 2014      Bull SAS.  All rights reserved.
  * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2015      Mellanox Technologies. All rights reserved.
- *
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -850,6 +848,15 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
 extern void mca_btl_openib_frag_progress_pending_put_get(
         struct mca_btl_base_endpoint_t*, const int);
 
+/**
+ * Fault Tolerance Event Notification Function
+ *
+ * @param state (IN)  Checkpoint State
+ * @return OPAL_SUCCESS or failure status
+ */
+extern int mca_btl_openib_ft_event(int state);
+
+
 /**
  * Show an error during init, particularly when running out of
  * registered memory.
