@@ -54,8 +54,12 @@
 #endif
 #include <ctype.h>
 #include <limits.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 
 #include "opal/mca/event/event.h"
+#include "opal/util/ethtool.h"
 #include "opal/util/if.h"
 #include "opal/util/output.h"
 #include "opal/util/argv.h"
@@ -85,6 +89,9 @@
 #include "opal/mca/common/cuda/common_cuda.h"
 #endif /* OPAL_CUDA_SUPPORT */
 
+#define MCA_BTL_TCP_BTL_BANDWIDTH 100
+#define MCA_BTL_TCP_BTL_LATENCY 100
+
 /*
  * Local functions
  */
@@ -246,8 +253,20 @@ static int mca_btl_tcp_component_register(void)
     mca_btl_tcp_param_register_int ("free_list_num", NULL, 8, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_num);
     mca_btl_tcp_param_register_int ("free_list_max", NULL, -1, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_max);
     mca_btl_tcp_param_register_int ("free_list_inc", NULL, 32, OPAL_INFO_LVL_5,  &mca_btl_tcp_component.tcp_free_list_inc);
-    mca_btl_tcp_param_register_int ("sndbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_sndbuf);
-    mca_btl_tcp_param_register_int ("rcvbuf", NULL, 128*1024, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_rcvbuf);
+    mca_btl_tcp_param_register_int ("sndbuf",
+                                    "The size of the send buffer socket option for each connection.  "
+                                    "Modern TCP stacks generally are smarter than a fixed size and in some "
+                                    "situations setting a buffer size explicitly can actually lower "
+                                    "performance.  0 means the tcp btl will not try to set a send buffer "
+                                    "size.",
+                                    0, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_sndbuf);
+    mca_btl_tcp_param_register_int ("rcvbuf",
+                                    "The size of the receive buffer socket option for each connection.  "
+                                    "Modern TCP stacks generally are smarter than a fixed size and in some "
+                                    "situations setting a buffer size explicitly can actually lower "
+                                    "performance.  0 means the tcp btl will not try to set a send buffer "
+                                    "size.",
+                                    0, OPAL_INFO_LVL_4, &mca_btl_tcp_component.tcp_rcvbuf);
     mca_btl_tcp_param_register_int ("endpoint_cache",
         "The size of the internal cache for each TCP connection. This cache is"
         " used to reduce the number of syscalls, by replacing them with memcpy."
@@ -299,7 +318,12 @@ static int mca_btl_tcp_component_register(void)
     mca_btl_tcp_module.super.btl_rndv_eager_limit = 64*1024;
     mca_btl_tcp_module.super.btl_max_send_size = 128*1024;
     mca_btl_tcp_module.super.btl_rdma_pipeline_send_length = 128*1024;
-    mca_btl_tcp_module.super.btl_rdma_pipeline_frag_size = INT_MAX;
+    /* Some OSes have hard coded limits on how many bytes can be manipulated
+     * by each writev operation.  Force a reasonable limit, to prevent overflowing
+     * a signed 32-bit integer (limit comes from BSD and OS X). We remove 1k to
+     * make some room for our internal headers.
+     */
+    mca_btl_tcp_module.super.btl_rdma_pipeline_frag_size = ((1UL<<31) - 1024);
     mca_btl_tcp_module.super.btl_min_rdma_pipeline_size = 0;
     mca_btl_tcp_module.super.btl_flags = MCA_BTL_FLAGS_PUT |
                                        MCA_BTL_FLAGS_SEND_INPLACE |
@@ -308,12 +332,19 @@ static int mca_btl_tcp_component_register(void)
                                        MCA_BTL_FLAGS_HETEROGENEOUS_RDMA |
                                        MCA_BTL_FLAGS_SEND;
 
-    mca_btl_tcp_module.super.btl_bandwidth = 100;
-    mca_btl_tcp_module.super.btl_latency = 100;
+    /* Bandwidth and latency initially set to 0. May be overridden during
+     * mca_btl_tcp_create().
+     */
+    mca_btl_tcp_module.super.btl_bandwidth = 0;
+    mca_btl_tcp_module.super.btl_latency = 0;
 
     mca_btl_base_param_register(&mca_btl_tcp_component.super.btl_version,
                                 &mca_btl_tcp_module.super);
-
+    if (mca_btl_tcp_module.super.btl_rdma_pipeline_frag_size > ((1UL<<31) - 1024) ) {
+        /* Assume a hard limit. A test in configure would be a better solution, but until then
+         * kicking-in the pipeline RDMA for extremely large data is good enough. */
+        mca_btl_tcp_module.super.btl_rdma_pipeline_frag_size = ((1UL<<31) - 1024);
+    }
     mca_btl_tcp_param_register_int ("disable_family", NULL, 0, OPAL_INFO_LVL_2,  &mca_btl_tcp_component.tcp_disable_family);
 
     return mca_btl_tcp_component_verify();
@@ -373,9 +404,8 @@ static int mca_btl_tcp_component_open(void)
 
 static int mca_btl_tcp_component_close(void)
 {
-    opal_list_item_t *item;
+    mca_btl_tcp_event_t *event, *next;
 
-#if MCA_BTL_TCP_SUPPORT_PROGRESS_THREAD
     /**
      * If we have a progress thread we should shut it down before
      * moving forward with the TCP tearing down process.
@@ -412,7 +442,6 @@ static int mca_btl_tcp_component_close(void)
 
     OBJ_DESTRUCT(&mca_btl_tcp_ready_frag_mutex);
     OBJ_DESTRUCT(&mca_btl_tcp_ready_frag_pending_queue);
-#endif
 
     if (NULL != mca_btl_tcp_component.tcp_btls) {
         free(mca_btl_tcp_component.tcp_btls);
@@ -433,12 +462,13 @@ static int mca_btl_tcp_component_close(void)
 
     /* remove all pending events. Do not lock the tcp_events list as
        the event themselves will unregister during the destructor. */
-    while( NULL != (item = opal_list_remove_first(&mca_btl_tcp_component.tcp_events)) ) {
-        mca_btl_tcp_event_t* event = (mca_btl_tcp_event_t*)item;
+    OPAL_LIST_FOREACH_SAFE(event, next, &mca_btl_tcp_component.tcp_events, mca_btl_tcp_event_t) {
         opal_event_del(&event->event);
         OBJ_RELEASE(event);
     }
 
+    opal_proc_table_remove_value(&mca_btl_tcp_component.tcp_procs, opal_proc_local_get()->proc_name);
+
     /* release resources */
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_procs);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_eager);
@@ -470,6 +500,7 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
             return OPAL_ERR_OUT_OF_RESOURCE;
         memcpy(btl, &mca_btl_tcp_module, sizeof(mca_btl_tcp_module));
         OBJ_CONSTRUCT(&btl->tcp_endpoints, opal_list_t);
+        OBJ_CONSTRUCT(&btl->tcp_endpoints_mutex, opal_mutex_t);
         mca_btl_tcp_component.tcp_btls[mca_btl_tcp_component.tcp_num_btls++] = btl;
 
         /* initialize the btl */
@@ -499,6 +530,27 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
         /* allow user to override/specify latency ranking */
         sprintf(param, "latency_%s:%d", if_name, i);
         mca_btl_tcp_param_register_uint(param, NULL, btl->super.btl_latency, OPAL_INFO_LVL_5, &btl->super.btl_latency);
+
+        /* Only attempt to auto-detect bandwidth and/or latency if it is 0.
+         *
+         * If detection fails to return anything other than 0, set a default
+         * bandwidth and latency.
+         */
+        if (0 == btl->super.btl_bandwidth) {
+            unsigned int speed = opal_ethtool_get_speed(if_name);
+            btl->super.btl_bandwidth = (speed == 0) ? MCA_BTL_TCP_BTL_BANDWIDTH : speed;
+            if (i > 0) {
+                btl->super.btl_bandwidth >>= 1;
+            }
+        }
+        /* We have no runtime btl latency detection mechanism. Just set a default. */
+        if (0 == btl->super.btl_latency) {
+            btl->super.btl_latency = MCA_BTL_TCP_BTL_LATENCY;
+            if (i > 0) {
+                btl->super.btl_latency <<= 1;
+            }
+        }
+
 #if 0 && OPAL_ENABLE_DEBUG
         BTL_OUTPUT(("interface %s instance %i: bandwidth %d latency %d\n", if_name, i,
                     btl->super.btl_bandwidth, btl->super.btl_latency));
@@ -689,7 +741,9 @@ static int mca_btl_tcp_component_create_instances(void)
         char* if_name = *argv;
         int if_index = opal_ifnametokindex(if_name);
         if(if_index < 0) {
-            BTL_ERROR(("invalid interface \"%s\"", if_name));
+            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                           true, "include", opal_process_info.nodename,
+                           if_name, "Unknown interface name");
             ret = OPAL_ERR_NOT_FOUND;
             goto cleanup;
         }
@@ -816,13 +870,18 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         freeaddrinfo (res);
 
 #ifdef IPV6_V6ONLY
-        /* in case of AF_INET6, disable v4-mapped addresses */
+        /* If this OS supports the "IPV6_V6ONLY" constant, then set it
+           on this socket.  It specifies that *only* V6 connections
+           should be accepted on this socket (vs. allowing incoming
+           both V4 and V6 connections -- which is actually defined
+           behavior for V6<-->V4 interop stuff).  See
+           https://github.com/open-mpi/ompi/commit/95d7e08a6617530d57b6700c57738b351bfccbf8 for some
+           more details. */
         if (AF_INET6 == af_family) {
             int flg = 1;
             if (setsockopt (sd, IPPROTO_IPV6, IPV6_V6ONLY,
                             (char *) &flg, sizeof (flg)) < 0) {
-                opal_output(0,
-                    "mca_btl_tcp_create_listen: unable to disable v4-mapped addresses\n");
+                BTL_ERROR(("mca_btl_tcp_create_listen: unable to set IPV6_V6ONLY\n"));
             }
         }
 #endif /* IPV6_V6ONLY */
@@ -864,6 +923,10 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 #else
             ((struct sockaddr_in*) &inaddr)->sin_port = htons(port + index);
 #endif  /* OPAL_ENABLE_IPV6 */
+            opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                "btl:tcp: Attempting to bind to %s port %d",
+                                (AF_INET == af_family) ? "AF_INET" : "AF_INET6",
+                                port + index);
             if(bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
                 if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
                     continue;
@@ -873,6 +936,10 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
                 CLOSE_THE_SOCKET(sd);
                 return OPAL_ERROR;
             }
+            opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                "btl:tcp: Successfully bound to %s port %d",
+                                (AF_INET == af_family) ? "AF_INET" : "AF_INET6",
+                                port + index);
             goto socket_binded;
         }
 #if OPAL_ENABLE_IPV6
@@ -903,11 +970,19 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
     if (AF_INET6 == af_family) {
         mca_btl_tcp_component.tcp6_listen_port = ((struct sockaddr_in6*) &inaddr)->sin6_port;
         mca_btl_tcp_component.tcp6_listen_sd = sd;
+        opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                            "btl:tcp: my listening v6 socket port is %d",
+                            ntohs(mca_btl_tcp_component.tcp6_listen_port));
     } else
 #endif
     {
+        char str[16];
         mca_btl_tcp_component.tcp_listen_port = ((struct sockaddr_in*) &inaddr)->sin_port;
         mca_btl_tcp_component.tcp_listen_sd = sd;
+        inet_ntop(AF_INET, &(((struct sockaddr_in*)&inaddr)->sin_addr), str, sizeof(str));
+        opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                            "btl:tcp: my listening v4 socket is %s:%u",
+                            str, ntohs(mca_btl_tcp_component.tcp_listen_port));
     }
 
     /* setup listen backlog to maximum allowed by kernel */
@@ -920,15 +995,20 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     /* set socket up to be non-blocking, otherwise accept could block */
     if((flags = fcntl(sd, F_GETFL, 0)) < 0) {
-        BTL_ERROR(("fcntl(F_GETFL) failed: %s (%d)",
-                   strerror(opal_socket_errno), opal_socket_errno));
+        opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                       true, opal_process_info.nodename,
+                       getpid(), "fcntl(sd, F_GETFL, 0)",
+                       strerror(opal_socket_errno), opal_socket_errno);
         CLOSE_THE_SOCKET(sd);
         return OPAL_ERROR;
     } else {
         flags |= O_NONBLOCK;
         if(fcntl(sd, F_SETFL, flags) < 0) {
-            BTL_ERROR(("fcntl(F_SETFL) failed: %s (%d)",
-                       strerror(opal_socket_errno), opal_socket_errno));
+            opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                           true, opal_process_info.nodename,
+                           getpid(),
+                           "fcntl(sd, F_SETFL, flags & O_NONBLOCK)",
+                           strerror(opal_socket_errno), opal_socket_errno);
             CLOSE_THE_SOCKET(sd);
             return OPAL_ERROR;
         }
@@ -1041,6 +1121,7 @@ static int mca_btl_tcp_component_exchange(void)
      size_t current_addr = 0;
 
      if(mca_btl_tcp_component.tcp_num_btls != 0) {
+         char ifn[32];
          mca_btl_tcp_addr_t *addrs = (mca_btl_tcp_addr_t *)malloc(size);
          memset(addrs, 0, size);
 
@@ -1058,6 +1139,9 @@ static int mca_btl_tcp_component_exchange(void)
                      continue;
                  }
 
+                 opal_ifindextoname(index, ifn, sizeof(ifn));
+                 opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                     "btl:tcp: examining interface %s", ifn);
                  if (OPAL_SUCCESS !=
                      opal_ifindextoaddr(index, (struct sockaddr*) &my_ss,
                                         sizeof (my_ss))) {
@@ -1081,6 +1165,8 @@ static int mca_btl_tcp_component_exchange(void)
                      addrs[current_addr].addr_ifkindex =
                          opal_ifindextokindex (index);
                      current_addr++;
+                     opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                         "btl:tcp: using ipv4 interface %s", ifn);
                  } else
 #endif
                  if ((AF_INET == my_ss.ss_family) &&
@@ -1096,11 +1182,13 @@ static int mca_btl_tcp_component_exchange(void)
                      addrs[current_addr].addr_ifkindex =
                          opal_ifindextokindex (index);
                      current_addr++;
+                     opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                         "btl:tcp: using ipv6 interface %s", ifn);
                  }
              } /* end of for opal_ifbegin() */
          } /* end of for tcp_num_btls */
          OPAL_MODEX_SEND(rc, OPAL_PMIX_GLOBAL,
-                         &mca_btl_tcp_component.super.btl_version, 
+                         &mca_btl_tcp_component.super.btl_version,
                          addrs, xfer_size);
          free(addrs);
      } /* end if */
@@ -1259,45 +1347,158 @@ static void mca_btl_tcp_component_recv_handler(int sd, short flags, void* user)
     struct sockaddr_storage addr;
     opal_socklen_t addr_len = sizeof(addr);
     mca_btl_tcp_proc_t* btl_proc;
-    int retval;
+    bool sockopt = true;
+    size_t retval, len = strlen(mca_btl_tcp_magic_id_string);
+    mca_btl_tcp_endpoint_hs_msg_t hs_msg;
+    struct timeval save, tv;
+    socklen_t rcvtimeo_save_len = sizeof(save);
+    char str[128];
+
+    /* Note, Socket will be in blocking mode during intial handshake
+     * hence setting SO_RCVTIMEO to say 2 seconds here to avoid waiting 
+     * forever when connecting to older versions (that reply to the
+     * handshake with only the guid) or when the remote side isn't OMPI
+     */
+
+    /* get the current timeout value so we can reset to it */
+    if (0 != getsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, (void*)&save, &rcvtimeo_save_len)) {
+        if (ENOPROTOOPT == errno) {
+            sockopt = false;
+        } else {
+            opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                                "Cannot get current recv timeout value of the socket"
+                                "Local_host:%s PID:%d",
+                                opal_process_info.nodename, getpid());
+            return;
+        }
+    } else {
+        tv.tv_sec = 2;
+        tv.tv_usec = 0;
+        if (0 != setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv))) {
+            opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                                "Cannot set new recv timeout value of the socket"
+                                "Local_host:%s PID:%d",
+                                opal_process_info.nodename, getpid());
+           return;
+       }
+    }
 
     OBJ_RELEASE(event);
+    retval = mca_btl_tcp_recv_blocking(sd, (void *)&hs_msg, sizeof(hs_msg));
 
-    /* recv the process identifier */
-    retval = recv(sd, (char *)&guid, sizeof(guid), 0);
-    if(retval != sizeof(guid)) {
+    /* An unknown process attempted to connect to Open MPI via TCP.
+     * Open MPI uses a "magic" string to trivially verify that the connecting
+     * process is a fellow Open MPI process.  An MPI process accepted a TCP
+     * connection but did not receive the correct magic string.  This might
+     * indicate an Open MPI version mismatch between different MPI processes
+     * in the same job, or it may indicate that some other agent is
+     * mistakenly attempting to connect to Open MPI's TCP listening sockets.
+
+     * This attempted connection will be ignored; your MPI job may or may not
+     * continue properly.
+     */
+     if (sizeof(hs_msg) != retval) {
+         opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                            "process did not receive full connect ACK "
+                            "Local_host:%s PID:%d String_received:%s Test_fail:%s",
+                            opal_process_info.nodename,
+                            getpid(),
+                            (retval > 0) ? hs_msg.magic_id : "<nothing>",
+                            "handshake message length");
+
+         /* The other side probably isn't OMPI, so just hang up */
+         CLOSE_THE_SOCKET(sd);
+         return;
+    }
+
+    guid = hs_msg.guid;
+    if (0 != strncmp(hs_msg.magic_id, mca_btl_tcp_magic_id_string, len)) {
+        opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                            "process did not receive right magic string. "
+                            "Local_host:%s PID:%d String_received:%s Test_fail:%s",
+                            opal_process_info.nodename,
+                            getpid(), hs_msg.magic_id,
+                            "string value");
+        /* The other side probably isn't OMPI, so just hang up */
         CLOSE_THE_SOCKET(sd);
         return;
     }
+
+    if (sockopt) {
+       /* reset RECVTIMEO option to its original state */
+       if (0 != setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &save, sizeof(save))) {
+           opal_output_verbose(20, opal_btl_base_framework.framework_output,
+                               "Cannot reset recv timeout value"
+                               "Local_host:%s PID:%d",
+                               opal_process_info.nodename, getpid());
+          return;
+       }
+    }
+
     OPAL_PROCESS_NAME_NTOH(guid);
 
     /* now set socket up to be non-blocking */
     if((flags = fcntl(sd, F_GETFL, 0)) < 0) {
-        BTL_ERROR(("fcntl(F_GETFL) failed: %s (%d)",
-                   strerror(opal_socket_errno), opal_socket_errno));
+        opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                       true, opal_process_info.nodename,
+                       getpid(), "fcntl(sd, F_GETFL, 0)",
+                       strerror(opal_socket_errno), opal_socket_errno);
+        CLOSE_THE_SOCKET(sd);
     } else {
         flags |= O_NONBLOCK;
         if(fcntl(sd, F_SETFL, flags) < 0) {
-            BTL_ERROR(("fcntl(F_SETFL) failed: %s (%d)",
-                       strerror(opal_socket_errno), opal_socket_errno));
+            opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                           true, opal_process_info.nodename,
+                           getpid(),
+                           "fcntl(sd, F_SETFL, flags & O_NONBLOCK)",
+                           strerror(opal_socket_errno), opal_socket_errno);
+            CLOSE_THE_SOCKET(sd);
         }
     }
 
     /* lookup the corresponding process */
     btl_proc = mca_btl_tcp_proc_lookup(&guid);
     if(NULL == btl_proc) {
+        opal_show_help("help-mpi-btl-tcp.txt",
+                       "server accept cannot find guid",
+                       true, opal_process_info.nodename,
+                       getpid());
         CLOSE_THE_SOCKET(sd);
         return;
     }
 
     /* lookup peer address */
     if(getpeername(sd, (struct sockaddr*)&addr, &addr_len) != 0) {
-        BTL_ERROR(("getpeername() failed: %s (%d)",
-                   strerror(opal_socket_errno), opal_socket_errno));
+        opal_show_help("help-mpi-btl-tcp.txt",
+                       "server getpeername failed",
+                       true, opal_process_info.nodename,
+                       getpid(),
+                       strerror(opal_socket_errno), opal_socket_errno);
         CLOSE_THE_SOCKET(sd);
         return;
     }
 
     /* are there any existing peer instances willing to accept this connection */
     (void)mca_btl_tcp_proc_accept(btl_proc, (struct sockaddr*)&addr, sd);
+
+    switch (addr.ss_family) {
+        case AF_INET:
+            inet_ntop(AF_INET, &(((struct sockaddr_in*) &addr)->sin_addr), str, sizeof(str));
+            break;
+
+    #if OPAL_ENABLE_IPV6
+        case AF_INET6:
+            inet_ntop(AF_INET6, &(((struct sockaddr_in6*) &addr)->sin6_addr), str, sizeof(str));
+            break;
+    #endif
+
+        default:
+            BTL_ERROR(("Got an accept() from an unknown address family -- this shouldn't happen"));
+            CLOSE_THE_SOCKET(sd);
+            return;
+
+    }
+    opal_output_verbose(10, opal_btl_base_framework.framework_output,
+                        "btl:tcp: now connected to %s, process %s", str,
+                        OPAL_NAME_PRINT(btl_proc->proc_opal->proc_name));
 }
