@@ -62,6 +62,11 @@
 #include "btl_tcp_frag.h"
 #include "btl_tcp_addr.h"
 
+/*
+ * Magic ID string send during connect/accept handshake
+ */
+
+const char mca_btl_tcp_magic_id_string[MCA_BTL_TCP_MAGIC_STRING_LENGTH] = "OPAL-TCP-BTL";
 
 /*
  * Initialize state of the endpoint instance.
@@ -150,7 +155,6 @@ mca_btl_tcp_endpoint_dump(int level,
     opal_socklen_t obtlen;
     opal_socklen_t addrlen = sizeof(inaddr);
     mca_btl_tcp_frag_t* item;
-    mca_btl_tcp_proc_t* this_proc = mca_btl_tcp_proc_local();
 
     used += snprintf(&outmsg[used], DEBUG_LENGTH - used, "%s: ", msg);
     if (used >= DEBUG_LENGTH) goto out;
@@ -280,7 +284,7 @@ out:
     opal_output_verbose(level, opal_btl_base_framework.framework_output,
                         "[%s:%d:%s][%s -> %s] %s",
                         fname, lineno, funcname,
-                        (NULL != this_proc ? OPAL_NAME_PRINT(mca_btl_tcp_proc_local()->proc_opal->proc_name) : "unknown"),
+                        OPAL_NAME_PRINT(opal_proc_local_get()->proc_name),
                         (NULL != btl_endpoint->endpoint_proc ? OPAL_NAME_PRINT(btl_endpoint->endpoint_proc->proc_opal->proc_name) : "unknown remote"),
                         outmsg);
 }
@@ -300,18 +304,17 @@ static inline void mca_btl_tcp_endpoint_event_init(mca_btl_base_endpoint_t* btl_
 
     opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_recv_event,
                     btl_endpoint->endpoint_sd,
-                    OPAL_EV_READ|OPAL_EV_PERSIST,
+                    OPAL_EV_READ | OPAL_EV_PERSIST,
                     mca_btl_tcp_endpoint_recv_handler,
                     btl_endpoint );
     /**
-     * The send event should be non persistent until the endpoint is
-     * completely connected. This means, when the event is created it
-     * will be fired only once, and when the endpoint is marked as
-     * CONNECTED the event should be recreated with the correct flags.
+     * In the multi-threaded case, the send event must be persistent in order
+     * to avoid missing the connection notification in send_handler due to
+     * a local handling of the peer process (which holds the lock).
      */
     opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_send_event,
                     btl_endpoint->endpoint_sd,
-                    OPAL_EV_WRITE,
+                    OPAL_EV_WRITE | OPAL_EV_PERSIST,
                     mca_btl_tcp_endpoint_send_handler,
                     btl_endpoint);
 }
@@ -373,49 +376,42 @@ int mca_btl_tcp_endpoint_send(mca_btl_base_endpoint_t* btl_endpoint, mca_btl_tcp
 
 
 /*
- * A blocking send on a non-blocking socket. Used to send the small amount of connection
- * information that identifies the endpoints endpoint.
+ * A blocking send on a non-blocking socket. Used to send the small
+ * amount of connection information that identifies the endpoints endpoint.
  */
 static int
 mca_btl_tcp_endpoint_send_blocking(mca_btl_base_endpoint_t* btl_endpoint,
-                                   void* data, size_t size)
+                                   const void* data, size_t size)
 {
-    unsigned char* ptr = (unsigned char*)data;
-    size_t cnt = 0;
-    while(cnt < size) {
-        int retval = send(btl_endpoint->endpoint_sd, (const char *)ptr+cnt, size-cnt, 0);
-        if(retval < 0) {
-            if(opal_socket_errno != EINTR && opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-                BTL_ERROR(("send(%d, %p, %lu/%lu) failed: %s (%d)",
-                           btl_endpoint->endpoint_sd, data, cnt, size,
-                           strerror(opal_socket_errno), opal_socket_errno));
-                btl_endpoint->endpoint_state = MCA_BTL_TCP_FAILED;
-                mca_btl_tcp_endpoint_close(btl_endpoint);
-                return -1;
-            }
-            continue;
-        }
-        cnt += retval;
+    int ret = mca_btl_tcp_send_blocking(btl_endpoint->endpoint_sd, data, size);
+    if (ret < 0) {
+        mca_btl_tcp_endpoint_close(btl_endpoint);
     }
-    return cnt;
+    return ret;
 }
 
-
 /*
  * Send the globally unique identifier for this process to a endpoint on
  * a newly connected socket.
  */
-
-static int mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_endpoint)
+static int 
+mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_endpoint)
 {
-    /* send process identifier to remote endpoint */
-    mca_btl_tcp_proc_t* btl_proc = mca_btl_tcp_proc_local();
-    opal_process_name_t guid = btl_proc->proc_opal->proc_name;
-
+    opal_process_name_t guid = opal_proc_local_get()->proc_name;
     OPAL_PROCESS_NAME_HTON(guid);
-    if(mca_btl_tcp_endpoint_send_blocking(btl_endpoint, &guid, sizeof(guid)) !=
-          sizeof(guid)) {
-        return OPAL_ERR_UNREACH;
+    
+    mca_btl_tcp_endpoint_hs_msg_t hs_msg;
+    strcpy(hs_msg.magic_id, mca_btl_tcp_magic_id_string);
+    hs_msg.guid = guid;
+    
+    if(sizeof(hs_msg) != 
+       mca_btl_tcp_endpoint_send_blocking(btl_endpoint, 
+                                          &hs_msg, sizeof(hs_msg))) {
+         opal_show_help("help-mpi-btl-tcp.txt", "client handshake fail",
+                       true, opal_process_info.nodename,
+                       sizeof(hs_msg),
+                       "connect ACK failed to send magic-id and guid");
+          return OPAL_ERR_UNREACH;
     }
     return OPAL_SUCCESS;
 }
@@ -423,7 +419,6 @@ static int mca_btl_tcp_endpoint_send_connect_ack(mca_btl_base_endpoint_t* btl_en
 static void *mca_btl_tcp_endpoint_complete_accept(int fd, int flags, void *context)
 {
     mca_btl_base_endpoint_t* btl_endpoint = (mca_btl_base_endpoint_t*)context;
-    mca_btl_tcp_proc_t* this_proc = mca_btl_tcp_proc_local();
     struct timeval now = {0, 0};
     int cmpval;
 
@@ -452,7 +447,7 @@ static void *mca_btl_tcp_endpoint_complete_accept(int fd, int flags, void *conte
     }
 
     cmpval = opal_compare_proc(btl_endpoint->endpoint_proc->proc_opal->proc_name,
-                               this_proc->proc_opal->proc_name);
+                               opal_proc_local_get()->proc_name);
     if((btl_endpoint->endpoint_sd < 0) ||
        (btl_endpoint->endpoint_state != MCA_BTL_TCP_CONNECTED &&
         cmpval < 0)) {
@@ -468,6 +463,10 @@ static void *mca_btl_tcp_endpoint_complete_accept(int fd, int flags, void *conte
         mca_btl_tcp_endpoint_event_init(btl_endpoint);
         MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, true, "event_add(recv) [endpoint_accept]");
         opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
+        if( mca_btl_tcp_event_base == opal_sync_event_base ) {
+            /* If no progress thread then raise the awarness of the default progress engine */
+            opal_progress_event_users_increment();
+        }
         mca_btl_tcp_endpoint_connected(btl_endpoint);
 
         MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, true, "accepted");
@@ -517,6 +516,10 @@ void mca_btl_tcp_endpoint_close(mca_btl_base_endpoint_t* btl_endpoint)
     btl_endpoint->endpoint_retries++;
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, false, "event_del(recv) [close]");
     opal_event_del(&btl_endpoint->endpoint_recv_event);
+    if( mca_btl_tcp_event_base == opal_sync_event_base ) {
+        /* If no progress thread then lower the awarness of the default progress engine */
+        opal_progress_event_users_decrement();
+    }
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, false, "event_del(send) [close]");
     opal_event_del(&btl_endpoint->endpoint_send_event);
 
@@ -561,13 +564,6 @@ static void mca_btl_tcp_endpoint_connected(mca_btl_base_endpoint_t* btl_endpoint
     btl_endpoint->endpoint_retries = 0;
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, true, "READY [endpoint_connected]");
 
-    /* Create the send event in a persistent manner. */
-    opal_event_set(mca_btl_tcp_event_base, &btl_endpoint->endpoint_send_event,
-                    btl_endpoint->endpoint_sd,
-                    OPAL_EV_WRITE | OPAL_EV_PERSIST,
-                    mca_btl_tcp_endpoint_send_handler,
-                    btl_endpoint );
-
     if(opal_list_get_size(&btl_endpoint->endpoint_frags) > 0) {
         if(NULL == btl_endpoint->endpoint_send_frag)
             btl_endpoint->endpoint_send_frag = (mca_btl_tcp_frag_t*)
@@ -578,55 +574,27 @@ static void mca_btl_tcp_endpoint_connected(mca_btl_base_endpoint_t* btl_endpoint
 }
 
 
-/*
- * A blocking recv on a non-blocking socket. Used to receive the small
- * amount of connection information that identifies the remote endpoint (guid).
- */
-static int mca_btl_tcp_endpoint_recv_blocking(mca_btl_base_endpoint_t* btl_endpoint, void* data, size_t size)
-{
-    unsigned char* ptr = (unsigned char*)data;
-    size_t cnt = 0;
-    while(cnt < size) {
-        int retval = recv(btl_endpoint->endpoint_sd, (char *)ptr+cnt, size-cnt, 0);
-
-        /* remote closed connection */
-        if(retval == 0) {
-            mca_btl_tcp_endpoint_close(btl_endpoint);
-            return cnt;
-        }
-
-        /* socket is non-blocking so handle errors */
-        if(retval < 0) {
-            if(opal_socket_errno != EINTR && opal_socket_errno != EAGAIN && opal_socket_errno != EWOULDBLOCK) {
-                BTL_ERROR(("recv(%d, %lu/%lu) failed: %s (%d)",
-                           btl_endpoint->endpoint_sd, cnt, size, strerror(opal_socket_errno), opal_socket_errno));
-                btl_endpoint->endpoint_state = MCA_BTL_TCP_FAILED;
-                mca_btl_tcp_endpoint_close(btl_endpoint);
-                return -1;
-            }
-            continue;
-        }
-        cnt += retval;
-    }
-    return cnt;
-}
-
-
 /*
  *  Receive the endpoints globally unique process identification from a newly
  *  connected socket and verify the expected response. If so, move the
  *  socket to a connected state.
+ *
+ *  NOTE: The return codes from this function are checked in
+ *  mca_btl_tcp_endpoint_recv_handler().  Don't change them here
+ *  without also changing the handling in _recv_handler()!
  */
 static int mca_btl_tcp_endpoint_recv_connect_ack(mca_btl_base_endpoint_t* btl_endpoint)
 {
-    size_t s;
-    opal_process_name_t guid;
+    size_t retval, len = strlen(mca_btl_tcp_magic_id_string);;
     mca_btl_tcp_proc_t* btl_proc = btl_endpoint->endpoint_proc;
+    opal_process_name_t guid;
 
-    s = mca_btl_tcp_endpoint_recv_blocking(btl_endpoint,
-                                           &guid, sizeof(opal_process_name_t));
-    if (s != sizeof(opal_process_name_t)) {
-        if (0 == s) {
+    mca_btl_tcp_endpoint_hs_msg_t hs_msg;
+    retval = mca_btl_tcp_recv_blocking(btl_endpoint->endpoint_sd, &hs_msg, sizeof(hs_msg));
+
+    if (sizeof(hs_msg) != retval) {
+        mca_btl_tcp_endpoint_close(btl_endpoint);
+        if (0 == retval) {
             /* If we get zero bytes, the peer closed the socket. This
                can happen when the two peers started the connection
                protocol simultaneously. Just report the problem
@@ -635,10 +603,19 @@ static int mca_btl_tcp_endpoint_recv_connect_ack(mca_btl_base_endpoint_t* btl_en
         }
         opal_show_help("help-mpi-btl-tcp.txt", "client handshake fail",
                        true, opal_process_info.nodename,
-                       getpid(),
-                       "did not receive entire connect ACK from peer");
-        return OPAL_ERR_UNREACH;
+                       getpid(), "did not receive entire connect ACK from peer");
+        
+        return OPAL_ERR_BAD_PARAM;
     }
+    if (0 != strncmp(hs_msg.magic_id, mca_btl_tcp_magic_id_string, len)) {
+        opal_show_help("help-mpi-btl-tcp.txt", "server did not receive magic string",
+                       true, opal_process_info.nodename,
+                       getpid(), "client", hs_msg.magic_id,
+                       "string value");
+        return OPAL_ERR_BAD_PARAM;
+    }
+
+    guid = hs_msg.guid;
     OPAL_PROCESS_NAME_NTOH(guid);
     /* compare this to the expected values */
     /* TODO: this deserve a little bit more thinking as we are not supposed
@@ -719,30 +696,48 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
 
     /* setup the socket as non-blocking */
     if((flags = fcntl(btl_endpoint->endpoint_sd, F_GETFL, 0)) < 0) {
-        BTL_ERROR(("fcntl(F_GETFL) failed: %s (%d)",
-                   strerror(opal_socket_errno), opal_socket_errno));
+        opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                       true, opal_process_info.nodename,
+                       getpid(), "fcntl(sd, F_GETFL, 0)",
+                       strerror(opal_socket_errno), opal_socket_errno);
+        /* Upper layer will handler the error */
+        return OPAL_ERR_UNREACH;
     } else {
         flags |= O_NONBLOCK;
-        if(fcntl(btl_endpoint->endpoint_sd, F_SETFL, flags) < 0)
-            BTL_ERROR(("fcntl(F_SETFL) failed: %s (%d)",
-                       strerror(opal_socket_errno), opal_socket_errno));
+        if(fcntl(btl_endpoint->endpoint_sd, F_SETFL, flags) < 0) {
+            opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                           true, opal_process_info.nodename,
+                           getpid(),
+                           "fcntl(sd, F_SETFL, flags & O_NONBLOCK)",
+                           strerror(opal_socket_errno), opal_socket_errno);
+            /* Upper layer will handler the error */
+            return OPAL_ERR_UNREACH;
+        }
     }
 
     /* start the connect - will likely fail with EINPROGRESS */
     mca_btl_tcp_proc_tosocks(btl_endpoint->endpoint_addr, &endpoint_addr);
 
-    opal_output_verbose(20, opal_btl_base_framework.framework_output,
+    opal_output_verbose(10, opal_btl_base_framework.framework_output,
                         "btl: tcp: attempting to connect() to %s address %s on port %d",
                         OPAL_NAME_PRINT(btl_endpoint->endpoint_proc->proc_opal->proc_name),
                         opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
                         ntohs(btl_endpoint->endpoint_addr->addr_port));
 
     if(0 == connect(btl_endpoint->endpoint_sd, (struct sockaddr*)&endpoint_addr, addrlen)) {
+        opal_output_verbose(10, opal_btl_base_framework.framework_output,
+                            "btl:tcp: connect() to %s:%d completed",
+                            opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
+                            ntohs(((struct sockaddr_in*) &endpoint_addr)->sin_port));
         /* send our globally unique process identifier to the endpoint */
         if((rc = mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint)) == OPAL_SUCCESS) {
             btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECT_ACK;
             MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, true, "event_add(recv) [start_connect]");
             opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
+            if( mca_btl_tcp_event_base == opal_sync_event_base ) {
+                /* If no progress thread then raise the awarness of the default progress engine */
+                opal_progress_event_users_increment();
+            }
             return OPAL_SUCCESS;
         }
         /* We connected to the peer, but he close the socket before we got a chance to send our guid */
@@ -753,6 +748,8 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
             btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECTING;
             MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, true, "event_add(send) [start_connect]");
             MCA_BTL_TCP_ACTIVATE_EVENT(&btl_endpoint->endpoint_send_event, 0);
+            opal_output_verbose(30, opal_btl_base_framework.framework_output,
+                                "btl:tcp: would block, so allowing background progress");
             return OPAL_SUCCESS;
         }
     }
@@ -776,49 +773,69 @@ static int mca_btl_tcp_endpoint_start_connect(mca_btl_base_endpoint_t* btl_endpo
  * later. Otherwise, send this processes identifier to the endpoint on the
  * newly connected socket.
  */
-static void mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_endpoint)
+static int mca_btl_tcp_endpoint_complete_connect(mca_btl_base_endpoint_t* btl_endpoint)
 {
     int so_error = 0;
     opal_socklen_t so_length = sizeof(so_error);
     struct sockaddr_storage endpoint_addr;
 
+    /* Delete the send event notification, as the next step is waiting for the ack
+     * from the peer. Once this ack is received we will deal with the send notification
+     * accordingly.
+     */
     opal_event_del(&btl_endpoint->endpoint_send_event);
 
     mca_btl_tcp_proc_tosocks(btl_endpoint->endpoint_addr, &endpoint_addr);
 
     /* check connect completion status */
     if(getsockopt(btl_endpoint->endpoint_sd, SOL_SOCKET, SO_ERROR, (char *)&so_error, &so_length) < 0) {
-        BTL_ERROR(("getsockopt() to %s failed: %s (%d)",
+        opal_show_help("help-mpi-btl-tcp.txt", "socket flag fail",
+                       true, opal_process_info.nodename,
+                       getpid(), "fcntl(sd, F_GETFL, 0)",
+                       strerror(opal_socket_errno), opal_socket_errno);
+        BTL_ERROR(("getsockopt() to %s:%d failed: %s (%d)",
                    opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
+                   ((struct sockaddr_in*) &endpoint_addr)->sin_port,
                    strerror(opal_socket_errno), opal_socket_errno));
         mca_btl_tcp_endpoint_close(btl_endpoint);
-        return;
+        return OPAL_ERROR;
     }
     if(so_error == EINPROGRESS || so_error == EWOULDBLOCK) {
-        return;
+        return OPAL_SUCCESS;
     }
     if(so_error != 0) {
-        BTL_ERROR(("connect() to %s failed: %s (%d)",
-                   opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
-                   strerror(so_error), so_error));
+        char *msg;
+        asprintf(&msg, "connect() to %s:%d failed",
+                 opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
+                 ntohs(((struct sockaddr_in*) &endpoint_addr)->sin_port));
+        opal_show_help("help-mpi-btl-tcp.txt", "client connect fail",
+                       true, opal_process_info.nodename,
+                       getpid(), msg,
+                       strerror(opal_socket_errno), opal_socket_errno);
+        free(msg);
         mca_btl_tcp_endpoint_close(btl_endpoint);
-        return;
+        return OPAL_ERROR;
     }
 
-    /* Do not unregister from receiving send event notifications, instead
-     * leave the event to trigger once more, and then it will get automatically
-     * deleted as no send fragments are available.
-     */
+    opal_output_verbose(10, opal_btl_base_framework.framework_output,
+                        "btl:tcp: connect() to %s:%d completed (complete_connect), sending connect ACK",
+                        opal_net_get_hostname((struct sockaddr*) &endpoint_addr),
+                        ntohs(((struct sockaddr_in*) &endpoint_addr)->sin_port));
 
-     if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) == OPAL_SUCCESS) {
+    if(mca_btl_tcp_endpoint_send_connect_ack(btl_endpoint) == OPAL_SUCCESS) {
         btl_endpoint->endpoint_state = MCA_BTL_TCP_CONNECT_ACK;
         opal_event_add(&btl_endpoint->endpoint_recv_event, 0);
+        if( mca_btl_tcp_event_base == opal_sync_event_base ) {
+            /* If no progress thread then raise the awarness of the default progress engine */
+            opal_progress_event_users_increment();
+        }
         MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, false, "event_add(recv) [complete_connect]");
-        return;
+        return OPAL_SUCCESS;
     }
     MCA_BTL_TCP_ENDPOINT_DUMP(1, btl_endpoint, false, " [complete_connect]");
     btl_endpoint->endpoint_state = MCA_BTL_TCP_FAILED;
     mca_btl_tcp_endpoint_close(btl_endpoint);
+    return OPAL_ERROR;
 }
 
 
@@ -867,6 +884,26 @@ static void mca_btl_tcp_endpoint_recv_handler(int sd, short flags, void* user)
                 OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_send_lock);
                 MCA_BTL_TCP_ENDPOINT_DUMP(10, btl_endpoint, true, "connected");
             }
+            else if (OPAL_ERR_BAD_PARAM == rc) {
+                /* If we get a BAD_PARAM, it means that it probably wasn't
+                   an OMPI process on the other end of the socket (e.g.,
+                   the magic string ID failed).  So we can probably just
+                   close the socket and ignore this connection. */
+                CLOSE_THE_SOCKET(sd);
+            }
+            else {
+                /* Otherwise, it probably *was* an OMPI peer process on
+                   the other end, and something bad has probably
+                   happened.  */
+                mca_btl_tcp_module_t *m = btl_endpoint->endpoint_btl;
+
+                /* Fail up to the PML */
+                if (NULL != m->tcp_error_cb) {
+                    m->tcp_error_cb((mca_btl_base_module_t*) m, MCA_BTL_ERROR_FLAGS_FATAL,
+                        btl_endpoint->endpoint_proc->proc_opal,
+                        "TCP ACK is neither SUCCESS nor ERR (something bad has probably happened)");
+                }
+            }
             OPAL_THREAD_UNLOCK(&btl_endpoint->endpoint_recv_lock);
             return;
         }
