@@ -17,8 +17,6 @@
  * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
  * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2015      Mellanox Technologies.  All rights reserved.
- *
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -60,6 +58,8 @@ static int vader_add_procs(struct mca_btl_base_module_t* btl,
                            struct mca_btl_base_endpoint_t** peers,
                            struct opal_bitmap_t* reachability);
 
+static int vader_ft_event (int state);
+
 mca_btl_vader_t mca_btl_vader = {
     {
         &mca_btl_vader_component.super,
@@ -73,7 +73,7 @@ mca_btl_vader_t mca_btl_vader = {
         .btl_sendi = mca_btl_vader_sendi,
         .btl_dump = mca_btl_base_dump,
         .btl_register_error = vader_register_error_cb,
-        .btl_ft_event = NULL
+        .btl_ft_event = vader_ft_event
     }
 };
 
@@ -440,7 +440,6 @@ static struct mca_btl_base_descriptor_t *vader_prepare_src (struct mca_btl_base_
 {
     const size_t total_size = reserve + *size;
     mca_btl_vader_frag_t *frag;
-    unsigned char *fbox;
     void *data_ptr;
     int rc;
 
@@ -506,19 +505,6 @@ static struct mca_btl_base_descriptor_t *vader_prepare_src (struct mca_btl_base_
             frag->base.des_segment_count = 2;
         } else {
 #endif
-
-            /* inline send */
-            if (OPAL_LIKELY(MCA_BTL_DES_FLAGS_BTL_OWNERSHIP & flags)) {
-                /* try to reserve a fast box for this transfer only if the
-                 * fragment does not belong to the caller */
-                fbox = mca_btl_vader_reserve_fbox (endpoint, total_size);
-                if (OPAL_LIKELY(fbox)) {
-                    frag->segments[0].seg_addr.pval = fbox;
-                }
-
-                frag->fbox = fbox;
-            }
-
             /* NTH: the covertor adds some latency so we bypass it here */
             memcpy ((void *)((uintptr_t)frag->segments[0].seg_addr.pval + reserve), data_ptr, *size);
             frag->segments[0].seg_len = total_size;
@@ -533,6 +519,16 @@ static struct mca_btl_base_descriptor_t *vader_prepare_src (struct mca_btl_base_
     return &frag->base;
 }
 
+/**
+ * Fault Tolerance Event Notification Function
+ * @param state Checkpoint Stae
+ * @return OPAL_SUCCESS or failure status
+ */
+static int vader_ft_event (int state)
+{
+    return OPAL_SUCCESS;
+}
+
 static void mca_btl_vader_endpoint_constructor (mca_btl_vader_endpoint_t *ep)
 {
     OBJ_CONSTRUCT(&ep->pending_frags, opal_list_t);
