@@ -3,6 +3,7 @@
  * Copyright (c) 2016      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  *
+ * Copyright (c) 2016-2017 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,6 +20,7 @@
 
 BEGIN_C_DECLS
 
+#include <opal_config.h>
 #include MCA_hwloc_external_header
 
 /* If the including file requested it, also include the hwloc verbs
@@ -41,6 +43,11 @@ BEGIN_C_DECLS
 #    endif
 #endif
 
+#if HWLOC_API_VERSION < 0x00010b00
+#define HWLOC_OBJ_NUMANODE HWLOC_OBJ_NODE
+#define HWLOC_OBJ_PACKAGE HWLOC_OBJ_SOCKET
+#endif
+
 END_C_DECLS
 
 #endif /* MCA_OPAL_HWLOC_EXTERNAL_H */
