@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
@@ -27,6 +27,7 @@
 #include "opal/dss/dss.h"
 #include "opal/util/error.h"
 #include "opal/util/proc.h"
+#include "opal/hash_string.h"
 
 #include "opal/mca/pmix/pmix_types.h"
 #include "opal/mca/pmix/pmix_server.h"
@@ -42,6 +43,14 @@ extern int opal_pmix_base_exchange(opal_value_t *info,
                                    opal_pmix_pdata_t *pdat,
                                    int timeout);
 
+/*
+ * Count the fash for the the external RM
+ */
+#define OPAL_HASH_JOBID( str, hash ){               \
+    OPAL_HASH_STR( str, hash );                     \
+    hash &= ~(0x8000);                              \
+}
+
 /**
  * Provide a simplified macro for sending data via modex
  * to other processes. The macro requires four arguments:
@@ -152,6 +161,47 @@ extern int opal_pmix_base_exchange(opal_value_t *info,
         OPAL_LIST_DESTRUCT(&(_ilist));                                                   \
     } while(0);
 
+/**
+ * Provide a simplified macro for retrieving modex data
+ * from another process when we want the PMIx module
+ * to request it from the server if not found, but do not
+ * want the server to go find it if the server doesn't
+ * already have it:
+ *
+ * r - the integer return status from the modex op (int)
+ * s - string key (char*)
+ * p - pointer to the opal_process_name_t of the proc that posted
+ *     the data (opal_process_name_t*)
+ * d - pointer to a location wherein the data object
+ *     is to be returned
+ * t - the expected data type
+ */
+#define OPAL_MODEX_RECV_VALUE_IMMEDIATE(r, s, p, d, t)                                   \
+    do {                                                                                 \
+        opal_value_t *_kv, *_info;                                                       \
+        opal_list_t _ilist;                                                              \
+        opal_output_verbose(1, opal_pmix_verbose_output,                                \
+                            "%s[%s:%d] MODEX RECV VALUE IMMEDIATE FOR PROC %s KEY %s",   \
+                            OPAL_NAME_PRINT(OPAL_PROC_MY_NAME),                          \
+                            __FILE__, __LINE__,                                          \
+                            OPAL_NAME_PRINT(*(p)), (s));                                \
+        OBJ_CONSTRUCT(&(_ilist), opal_list_t);                                           \
+        _info = OBJ_NEW(opal_value_t);                                                   \
+        _info->key = strdup(OPAL_PMIX_IMMEDIATE);                                        \
+        _info->type = OPAL_BOOL;                                                         \
+        _info->data.flag = true;                                                         \
+        opal_list_append(&(_ilist), &(_info)->super);                                    \
+        if (OPAL_SUCCESS == ((r) = opal_pmix.get((p), (s), &(_ilist), &(_kv)))) {        \
+            if (NULL == _kv) {                                                           \
+                (r) = OPAL_ERR_NOT_FOUND;                                                \
+            } else {                                                                     \
+                (r) = opal_value_unload(_kv, (void**)(d), (t));                          \
+                OBJ_RELEASE(_kv);                                                        \
+            }                                                                            \
+        }                                                                                \
+        OPAL_LIST_DESTRUCT(&(_ilist));                                                   \
+    } while(0);
+
 /**
  * Provide a simplified macro for retrieving modex data
  * from another process:
@@ -275,7 +325,7 @@ extern int opal_pmix_base_exchange(opal_value_t *info,
  * If the information is not found, or the server connection fails, then
  * an appropriate error constant will be returned.
  */
-typedef int (*opal_pmix_base_module_init_fn_t)(void);
+typedef int (*opal_pmix_base_module_init_fn_t)(opal_list_t *ilist);
 
 /* Finalize the PMIx client, closing the connection to the local server.
  * An error code will be returned if, for some reason, the connection
@@ -517,6 +567,9 @@ typedef int (*opal_pmix_base_module_resolve_nodes_fn_t)(opal_jobid_t jobid, char
 
 /************************************************************
  *                       SERVER APIs                        *
+ *                                                          *
+ * These are calls that go down (or "south") from the ORTE  *
+ * daemon into the PMIx server library                      *
  ************************************************************/
 
 /* Initialize the server support library - must pass the callback
@@ -595,7 +648,9 @@ typedef int (*opal_pmix_base_module_server_register_nspace_fn_t)(opal_jobid_t jo
  * can be freed. Note that the server will automatically
  * purge all info relating to any clients it has from
  * this nspace */
-typedef void (*opal_pmix_base_module_server_deregister_nspace_fn_t)(opal_jobid_t jobid);
+typedef void (*opal_pmix_base_module_server_deregister_nspace_fn_t)(opal_jobid_t jobid,
+                                                                    opal_pmix_op_cbfunc_t cbfunc,
+                                                                    void *cbdata);
 
 /* Register a client process with the PMIx server library. The
  * expected user ID and group ID of the child process helps the
@@ -623,7 +678,9 @@ typedef int (*opal_pmix_base_module_server_register_client_fn_t)(const opal_proc
  * so there is no need to call this function during normal
  * finalize operations. Instead, this is provided for use
  * during exception operations */
-typedef void (*opal_pmix_base_module_server_deregister_client_fn_t)(const opal_process_name_t *proc);
+typedef void (*opal_pmix_base_module_server_deregister_client_fn_t)(const opal_process_name_t *proc,
+                                                                    opal_pmix_op_cbfunc_t cbfunc,
+                                                                    void *cbdata);
 
 /* Setup the environment of a child process to be forked
  * by the host so it can correctly interact with the PMIx
@@ -645,44 +702,55 @@ typedef int (*opal_pmix_base_module_server_dmodex_request_fn_t)(const opal_proce
                                                                 opal_pmix_modex_cbfunc_t cbfunc,
                                                                 void *cbdata);
 
-/* Report an error to a process for notification via any
- * registered errhandler. The errhandler registration can be
+/* Report an event to a process for notification via any
+ * registered event handler. The handler registration can be
  * called by both the server and the client application. On the
- * server side, the errhandler is used to report errors detected
+ * server side, the handler is used to report events detected
  * by PMIx to the host server for handling. On the client side,
- * the errhandler is used to notify the process of errors
+ * the handler is used to notify the process of events
  * reported by the server - e.g., the failure of another process.
  *
  * This function allows the host server to direct the server
- * convenience library to notify all indicated local procs of
- * an error. The error can be local, or anywhere in the cluster.
- * The status indicates the error being reported.
+ * convenience library to notify all registered local procs of
+ * an event. The event can be local, or anywhere in the cluster.
+ * The status indicates the event being reported.
  *
- * The first array  of procs informs the server library as to which
- * processes should be alerted - e.g., the processes that are in
- * a directly-affected job or are connected to one that is affected.
- * Passing a NULL for this array will indicate that all local procs
- * are to be notified.
- *
- * The second array identifies the processes that will be impacted
- * by the error. This could consist of a single process, or a number
- * of processes.
+ * The source parameter informs the handler of the source that
+ * generated the event. This will be NULL if the event came
+ * from the external resource manager.
  *
  * The info array contains any further info the RM can and/or chooses
  * to provide.
  *
  * The callback function will be called upon completion of the
- * notify_error function's actions. Note that any messages will
+ * notify_event function's actions. Note that any messages will
  * have been queued, but may not have been transmitted by this
  * time. Note that the caller is required to maintain the input
- * data until the callback function has been executed! */
-typedef int (*opal_pmix_base_module_server_notify_error_fn_t)(int status,
-                                                              opal_list_t *procs,
-                                                              opal_list_t *error_procs,
+ * data until the callback function has been executed if this
+ * function returns OPAL_SUCCESS! */
+typedef int (*opal_pmix_base_module_server_notify_event_fn_t)(int status,
+                                                              const opal_process_name_t *source,
                                                               opal_list_t *info,
                                                               opal_pmix_op_cbfunc_t cbfunc, void *cbdata);
 
 
+/************************************************************
+ *                         TOOL APIs                        *
+ ************************************************************/
+/* Initialize the PMIx tool support
+ * When called the library will check for the required connection
+ * information of the local server and will establish the connection.
+ * The connection info can be provided either in the environment or
+ * in the list of attributes. If the information is not found, or the
+ * server connection fails, then an appropriate error constant will
+ * be returned.
+ */
+typedef int (*opal_pmix_base_module_tool_init_fn_t)(opal_list_t *ilist);
+
+/* Finalize the PMIx tool support */
+typedef int (*opal_pmix_base_module_tool_fini_fn_t)(void);
+
+
 /************************************************************
  *                       UTILITY APIs                       *
  ************************************************************/
@@ -690,90 +758,76 @@ typedef int (*opal_pmix_base_module_server_notify_error_fn_t)(int status,
 /* get the version of the embedded library */
 typedef const char* (*opal_pmix_base_module_get_version_fn_t)(void);
 
-/* Register an errhandler to report errors. Three types of errors
+/* Register an event handler to report event. Three types of events
  * can be reported:
  *
  * (a) those that occur within the client library, but are not
  *     reportable via the API itself (e.g., loss of connection to
- *     the server). These errors typically occur during behind-the-scenes
+ *     the server). These events typically occur during behind-the-scenes
  *     non-blocking operations.
  *
- * (b) job-related errors such as the failure of another process in
+ * (b) job-related events such as the failure of another process in
  *     the job or in any connected job, impending failure of hardware
  *     within the job's usage footprint, etc.
  *
  * (c) system notifications that are made available by the local
  *     administrators
  *
- * By default, only errors that directly affect the process and/or
+ * By default, only events that directly affect the process and/or
  * any process to which it is connected (via the PMIx_Connect call)
  * will be reported. Options to modify that behavior can be provided
  * in the info array
  *
  * Both the client application and the resource manager can register
- * err handlers for specific errors. PMIx client/server calls the registered
- * err handler upon receiving error notify notification (via PMIx_Notify_error)
+ * event handlers for specific events. PMIx client/server calls the registered
+ * event handler upon receiving event notify notification (via PMIx_Notify_event)
  * from the other end (Resource Manager/Client application).
  *
- * Multiple err handlers can be registered for different errors. PMIX returns
- * an integer reference to each register handler in the callback fn. The caller
- * must retain the reference in order to deregister the errhandler.
+ * Multiple event handlers can be registered for different events. PMIX returns
+ * a size_t reference to each register handler in the callback fn. The caller
+ * must retain the reference in order to deregister the evhandler.
  * Modification of the notification behavior can be accomplished by
- * deregistering the current errhandler, and then registering it
+ * deregistering the current evhandler, and then registering it
  * using a new set of info values.
  *
+ * A NULL for event_codes indicates registration as a default event handler
+ *
  * See pmix_types.h for a description of the notification function */
-typedef void (*opal_pmix_base_module_register_fn_t)(opal_list_t *info,
-                                                    opal_pmix_notification_fn_t errhandler,
-                                                    opal_pmix_errhandler_reg_cbfunc_t cbfunc,
+typedef void (*opal_pmix_base_module_register_fn_t)(opal_list_t *event_codes,
+                                                    opal_list_t *info,
+                                                    opal_pmix_notification_fn_t evhandler,
+                                                    opal_pmix_evhandler_reg_cbfunc_t cbfunc,
                                                     void *cbdata);
 
-/* deregister the errhandler
- * errhandler_ref is the reference returned by PMIx for the errhandler
- * to pmix_errhandler_reg_cbfunc_t */
-typedef void (*opal_pmix_base_module_deregister_fn_t)(int errhandler,
+/* deregister the evhandler
+ * evhandler_ref is the reference returned by PMIx for the evhandler
+ * to pmix_evhandler_reg_cbfunc_t */
+typedef void (*opal_pmix_base_module_deregister_fn_t)(size_t evhandler,
                                                       opal_pmix_op_cbfunc_t cbfunc,
                                                       void *cbdata);
 
-/* Report an error to a process for notification via any
- * registered errhandler. The errhandler registration can be
- * called by both the server and the client application. On the
- * server side, the errhandler is used to report errors detected
- * by PMIx to the host server for handling. On the client side,
- * the errhandler is used to notify the process of errors
- * reported by the server - e.g., the failure of another process.
- *
- * This function allows the host server to direct the server
- * convenience library to notify all indicated local procs of
- * an error. The error can be local, or anywhere in the cluster.
- * The status indicates the error being reported.
+/* Report an event for notification via any
+ * registered evhandler. On the PMIx
+ * server side, this is used to report events detected
+ * by PMIx to the host server for handling and/or distribution.
  *
  * The client application can also call this function to notify the
- * resource manager of an error it encountered. It can request the host
- * server to notify the indicated processes about the error.
+ * resource manager of an event it detected. It can specify the
+ * range over which that notification should occur.
  *
- * The first array  of procs informs the server library as to which
- * processes should be alerted - e.g., the processes that are in
- * a directly-affected job or are connected to one that is affected.
- * Passing a NULL for this array will indicate that all local procs
- * are to be notified.
- *
- * The second array identifies the processes that will be impacted
- * by the error. This could consist of a single process, or a number
- * of processes.
- *
- * The info array contains any further info the RM can and/or chooses
+ * The info array contains any further info the caller can and/or chooses
  * to provide.
  *
  * The callback function will be called upon completion of the
- * notify_error function's actions. Note that any messages will
+ * notify_event function's actions. Note that any messages will
  * have been queued, but may not have been transmitted by this
  * time. Note that the caller is required to maintain the input
- * data until the callback function has been executed!
+ * data until the callback function has been executed if it
+ * returns OPAL_SUCCESS!
 */
-typedef int (*opal_pmix_base_module_notify_error_fn_t)(int status,
-                                                       opal_list_t *procs,
-                                                       opal_list_t *error_procs,
+typedef int (*opal_pmix_base_module_notify_event_fn_t)(int status,
+                                                       const opal_process_name_t *source,
+                                                       opal_pmix_data_range_t range,
                                                        opal_list_t *info,
                                                        opal_pmix_op_cbfunc_t cbfunc, void *cbdata);
 
@@ -790,10 +844,36 @@ typedef const char* (*opal_pmix_base_module_get_nspace_fn_t)(opal_jobid_t jobid)
 /* register a jobid-to-nspace pair */
 typedef void (*opal_pmix_base_module_register_jobid_fn_t)(opal_jobid_t jobid, const char *nspace);
 
+/* query information from the system */
+typedef void (*opal_pmix_base_module_query_fn_t)(opal_list_t *queries,
+                                                 opal_pmix_info_cbfunc_t cbfunc, void *cbdata);
+
+/* log data to the system */
+typedef void (*opal_pmix_base_log_fn_t)(opal_list_t *info,
+                                        opal_pmix_op_cbfunc_t cbfunc, void *cbdata);
+
+/* allocation */
+typedef int (*opal_pmix_base_alloc_fn_t)(opal_pmix_alloc_directive_t directive,
+                                         opal_list_t *info,
+                                         opal_pmix_info_cbfunc_t cbfunc, void *cbdata);
+
+/* job control */
+typedef int (*opal_pmix_base_job_control_fn_t)(opal_list_t *targets,
+                                               opal_list_t *directives,
+                                               opal_pmix_info_cbfunc_t cbfunc, void *cbdata);
+
+/* monitoring */
+typedef int (*opal_pmix_base_process_monitor_fn_t)(opal_list_t *monitor,
+                                                   opal_list_t *directives,
+                                                   opal_pmix_info_cbfunc_t cbfunc, void *cbdata);
+
+typedef bool (*opal_pmix_base_legacy_get_fn_t)(void);
+
 /*
  * the standard public API data structure
  */
 typedef struct {
+    opal_pmix_base_legacy_get_fn_t                          legacy_get;
     /* client APIs */
     opal_pmix_base_module_init_fn_t                         init;
     opal_pmix_base_module_fini_fn_t                         finalize;
@@ -819,6 +899,11 @@ typedef struct {
     opal_pmix_base_module_disconnect_nb_fn_t                disconnect_nb;
     opal_pmix_base_module_resolve_peers_fn_t                resolve_peers;
     opal_pmix_base_module_resolve_nodes_fn_t                resolve_nodes;
+    opal_pmix_base_module_query_fn_t                        query;
+    opal_pmix_base_log_fn_t                                 log;
+    opal_pmix_base_alloc_fn_t                               allocate;
+    opal_pmix_base_job_control_fn_t                         job_control;
+    opal_pmix_base_process_monitor_fn_t                     monitor;
     /* server APIs */
     opal_pmix_base_module_server_init_fn_t                  server_init;
     opal_pmix_base_module_server_finalize_fn_t              server_finalize;
@@ -830,12 +915,15 @@ typedef struct {
     opal_pmix_base_module_server_deregister_client_fn_t     server_deregister_client;
     opal_pmix_base_module_server_setup_fork_fn_t            server_setup_fork;
     opal_pmix_base_module_server_dmodex_request_fn_t        server_dmodex_request;
-    opal_pmix_base_module_server_notify_error_fn_t          server_notify_error;
+    opal_pmix_base_module_server_notify_event_fn_t          server_notify_event;
+    /* tool APIs */
+    opal_pmix_base_module_tool_init_fn_t                    tool_init;
+    opal_pmix_base_module_tool_fini_fn_t                    tool_finalize;
     /* Utility APIs */
     opal_pmix_base_module_get_version_fn_t                  get_version;
-    opal_pmix_base_module_register_fn_t                     register_errhandler;
-    opal_pmix_base_module_deregister_fn_t                   deregister_errhandler;
-    opal_pmix_base_module_notify_error_fn_t                 notify_error;
+    opal_pmix_base_module_register_fn_t                     register_evhandler;
+    opal_pmix_base_module_deregister_fn_t                   deregister_evhandler;
+    opal_pmix_base_module_notify_event_fn_t                 notify_event;
     opal_pmix_base_module_store_fn_t                        store_local;
     opal_pmix_base_module_get_nspace_fn_t                   get_nspace;
     opal_pmix_base_module_register_jobid_fn_t               register_jobid;
