@@ -14,6 +14,7 @@
  *                         All rights reserved
  * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      Cisco Systems, Inc.  All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,12 +28,17 @@
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
+#include "opal/mca/base/mca_base_pvar.h"
 #include "opal/mca/rcache/rcache.h"
 #include "opal/mca/rcache/base/base.h"
 #include "opal/memoryhooks/memory.h"
 #include "opal/constants.h"
 #include "rcache_base_mem_cb.h"
 
+/* two-level macro for stringifying a number */
+#define STRINGIFYX(x) #x
+#define STRINGIFY(x) STRINGIFYX(x)
+
 /*
  * The following file was created by configure.  It contains extern
  * statements and the definition of an array of pointers to each
@@ -59,12 +65,19 @@ static void mca_rcache_base_registration_constructor( mca_rcache_base_registrati
 OBJ_CLASS_INSTANCE(mca_rcache_base_registration_t, opal_free_list_item_t,
                    mca_rcache_base_registration_constructor, NULL);
 
+#define TREE_ITEMS_MIN 2048
+#define TREE_ITEMS_MAX 16384
+#define TREE_ITEMS_INC 2048
 
 /*
  * Global variables
  */
 opal_list_t mca_rcache_base_modules = {{0}};
-
+opal_free_list_t mca_rcache_base_vma_tree_items = {{{0}}};
+bool mca_rcache_base_vma_tree_items_inited = false;
+unsigned int mca_rcache_base_vma_tree_items_min = TREE_ITEMS_MIN;
+int mca_rcache_base_vma_tree_items_max = TREE_ITEMS_MAX;
+unsigned int mca_rcache_base_vma_tree_items_inc = TREE_ITEMS_INC;
 
 OBJ_CLASS_INSTANCE(mca_rcache_base_selected_module_t, opal_list_item_t, NULL, NULL);
 
@@ -100,8 +113,11 @@ static int mca_rcache_base_close(void)
            time between now and end of application (even post main()!) */
         (void) mca_base_framework_close (&opal_memory_base_framework);
     }
-    /* All done */
 
+    OBJ_DESTRUCT(&mca_rcache_base_vma_tree_items);
+    mca_rcache_base_vma_tree_items_inited = false;
+
+    /* All done */
     /* Close all remaining available components */
     return mca_base_framework_components_close(&opal_rcache_base_framework, NULL);
 }
@@ -117,11 +133,42 @@ static int mca_rcache_base_open(mca_base_open_flag_t flags)
 
     OBJ_CONSTRUCT(&mca_rcache_base_modules, opal_list_t);
 
+    /* the free list is only initialized when a VMA tree is created */
+    OBJ_CONSTRUCT(&mca_rcache_base_vma_tree_items, opal_free_list_t);
+
      /* Open up all available components */
     return mca_base_framework_components_open(&opal_rcache_base_framework, flags);
 }
 
-MCA_BASE_FRAMEWORK_DECLARE(opal, rcache, "OPAL Registration Cache", NULL,
+static int mca_rcache_base_register_mca_variables (mca_base_register_flag_t flags)
+{
+
+    mca_rcache_base_vma_tree_items_min = TREE_ITEMS_MIN;
+    (void) mca_base_framework_var_register (&opal_rcache_base_framework, "vma_tree_items_min",
+                                            "Minimum number of VMA tree items to allocate (default: "
+                                            STRINGIFY(TREE_ITEMS_MIN) ")", MCA_BASE_VAR_TYPE_UNSIGNED_INT,
+                                            NULL, MCA_BASE_VAR_BIND_NO_OBJECT, 0, OPAL_INFO_LVL_6,
+                                            MCA_BASE_VAR_SCOPE_READONLY, &mca_rcache_base_vma_tree_items_min);
+
+    mca_rcache_base_vma_tree_items_max = TREE_ITEMS_MAX;
+    (void) mca_base_framework_var_register (&opal_rcache_base_framework, "vma_tree_items_max",
+                                            "Maximum number of VMA tree items to allocate (default: "
+                                            STRINGIFY(TREE_ITEMS_MAX) ", -1: unlimited)", MCA_BASE_VAR_TYPE_INT,
+                                            NULL, MCA_BASE_VAR_BIND_NO_OBJECT, 0, OPAL_INFO_LVL_6,
+                                            MCA_BASE_VAR_SCOPE_READONLY, &mca_rcache_base_vma_tree_items_max);
+
+    mca_rcache_base_vma_tree_items_inc = TREE_ITEMS_INC;
+    (void) mca_base_framework_var_register (&opal_rcache_base_framework, "vma_tree_items_inc",
+                                            "Number of VMA tree items to allocate at a time (default: "
+                                            STRINGIFY(TREE_ITEMS_INC) ")", MCA_BASE_VAR_TYPE_UNSIGNED_INT,
+                                            NULL, MCA_BASE_VAR_BIND_NO_OBJECT, 0, OPAL_INFO_LVL_6,
+                                            MCA_BASE_VAR_SCOPE_READONLY, &mca_rcache_base_vma_tree_items_inc);
+
+    return OPAL_SUCCESS;
+}
+
+MCA_BASE_FRAMEWORK_DECLARE(opal, rcache, "OPAL Registration Cache",
+                           mca_rcache_base_register_mca_variables,
                            mca_rcache_base_open, mca_rcache_base_close,
                            mca_rcache_base_static_components, 0);
 
