@@ -15,9 +15,11 @@
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2016 Intel, Inc. All rights reserved
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -35,12 +37,12 @@
 
 #include "opal/util/malloc.h"
 #include "opal/util/arch.h"
-#include "opal/util/opal_environ.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
 #include "opal/util/proc.h"
 #include "opal/memoryhooks/memory.h"
 #include "opal/mca/base/base.h"
+#include "opal/mca/base/mca_base_var.h"
 #include "opal/runtime/opal.h"
 #include "opal/util/net.h"
 #include "opal/datatype/opal_datatype.h"
@@ -49,13 +51,20 @@
 #include "opal/mca/patcher/base/base.h"
 #include "opal/mca/memcpy/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
-#include "opal/mca/sec/base/base.h"
+#include "opal/mca/reachable/base/base.h"
 #include "opal/mca/timer/base/base.h"
 #include "opal/mca/memchecker/base/base.h"
+#include "opal/mca/if/base/base.h"
 #include "opal/dss/dss.h"
 #include "opal/mca/shmem/base/base.h"
+#if OPAL_ENABLE_FT_CR    == 1
+#include "opal/mca/compress/base/base.h"
+#endif
 #include "opal/threads/threads.h"
 
+#include "opal/runtime/opal_cr.h"
+#include "opal/mca/crs/base/base.h"
+
 #include "opal/runtime/opal_progress.h"
 #include "opal/mca/event/base/base.h"
 #include "opal/mca/backtrace/base/base.h"
@@ -65,6 +74,7 @@
 #include "opal/util/stacktrace.h"
 #include "opal/util/keyval_parse.h"
 #include "opal/util/sys_limits.h"
+#include "opal/util/timings.h"
 
 #if OPAL_CC_USE_PRAGMA_IDENT
 #pragma ident OPAL_IDENT_STRING
@@ -244,8 +254,59 @@ opal_err2str(int errnum, const char **errmsg)
     case OPAL_ERR_SERVER_NOT_AVAIL:
         retval = "Server not available";
         break;
+    case OPAL_ERR_IN_PROCESS:
+        retval = "Operation in process";
+        break;
+    case OPAL_ERR_DEBUGGER_RELEASE:
+        retval = "Release debugger";
+        break;
+    case OPAL_ERR_HANDLERS_COMPLETE:
+        retval = "Event handlers complete";
+        break;
+    case OPAL_ERR_PARTIAL_SUCCESS:
+        retval = "Partial success";
+        break;
+    case OPAL_ERR_PROC_ABORTED:
+        retval = "Process abnormally terminated";
+        break;
+    case OPAL_ERR_PROC_REQUESTED_ABORT:
+        retval = "Process requested abort";
+        break;
+    case OPAL_ERR_PROC_ABORTING:
+        retval = "Process is aborting";
+        break;
+    case OPAL_ERR_NODE_DOWN:
+        retval = "Node has gone down";
+        break;
+    case OPAL_ERR_NODE_OFFLINE:
+        retval = "Node has gone offline";
+        break;
+    case OPAL_ERR_JOB_TERMINATED:
+        retval = "Job terminated";
+        break;
+    case OPAL_ERR_PROC_RESTART:
+        retval = "Process restarted";
+        break;
+    case OPAL_ERR_PROC_CHECKPOINT:
+        retval = "Process checkpoint";
+        break;
+    case OPAL_ERR_PROC_MIGRATE:
+        retval = "Process migrate";
+        break;
+    case OPAL_ERR_EVENT_REGISTRATION:
+        retval = "Event registration";
+        break;
+    case OPAL_ERR_HEARTBEAT_ALERT:
+        retval = "Heartbeat not received";
+        break;
+    case OPAL_ERR_FILE_ALERT:
+        retval = "File alert - proc may have stalled";
+        break;
+    case OPAL_ERR_MODEL_DECLARED:
+        retval = "Model declared";
+        break;
     default:
-        retval = NULL;
+        retval = "UNRECOGNIZED";
     }
 
     *errmsg = retval;
@@ -287,6 +348,7 @@ opal_init_util(int* pargc, char*** pargv)
     int ret;
     char *error = NULL;
     char hostname[OPAL_MAXHOSTNAMELEN];
+    OPAL_TIMING_ENV_INIT(otmng);
 
     if( ++opal_util_initialized != 1 ) {
         if( opal_util_initialized < 1 ) {
@@ -295,15 +357,7 @@ opal_init_util(int* pargc, char*** pargv)
         return OPAL_SUCCESS;
     }
 
-#if OPAL_NO_LIB_DESTRUCTOR
-    if (opal_init_called) {
-        /* can't use show_help here */
-        fprintf (stderr, "opal_init_util: attempted to initialize after finalize without compiler "
-                 "support for either __attribute__(destructor) or linker support for -fini -- process "
-                 "will likely abort\n");
-        return OPAL_ERR_NOT_SUPPORTED;
-    }
-#endif
+    opal_thread_set_main();
 
     opal_init_called = true;
 
@@ -317,6 +371,8 @@ opal_init_util(int* pargc, char*** pargv)
     /* initialize the memory allocator */
     opal_malloc_init();
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_malloc_init");
+
     /* initialize the output system */
     opal_output_init();
 
@@ -330,6 +386,8 @@ opal_init_util(int* pargc, char*** pargv)
     /* initialize the help system */
     opal_show_help_init();
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_show_help_init");
+
     /* register handler for errnum -> string converstion */
     if (OPAL_SUCCESS !=
         (ret = opal_error_register("OPAL",
@@ -348,11 +406,23 @@ opal_init_util(int* pargc, char*** pargv)
     // details)
     opal_init_psm();
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_init_psm");
+
     /* Setup the parameter system */
     if (OPAL_SUCCESS != (ret = mca_base_var_init())) {
         error = "mca_base_var_init";
         goto return_error;
     }
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_var_init");
+
+    /* read any param files that were provided */
+    if (OPAL_SUCCESS != (ret = mca_base_var_cache_files(false))) {
+        error = "failed to cache files";
+        goto return_error;
+    }
+
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_var_cache");
+
 
     /* register params for opal */
     if (OPAL_SUCCESS != (ret = opal_register_params())) {
@@ -365,6 +435,8 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_net_init");
+
     /* pretty-print stack handlers */
     if (OPAL_SUCCESS != (ret = opal_util_register_stackhandlers())) {
         error = "opal_util_register_stackhandlers";
@@ -387,12 +459,16 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_arch_init");
+
     /* initialize the datatype engine */
     if (OPAL_SUCCESS != (ret = opal_datatype_init ())) {
         error = "opal_datatype_init";
         goto return_error;
     }
 
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_datatype_init");
+
     /* Initialize the data storage service. */
     if (OPAL_SUCCESS != (ret = opal_dss_open())) {
         error = "opal_dss_open";
@@ -405,6 +481,15 @@ opal_init_util(int* pargc, char*** pargv)
         goto return_error;
     }
 
+    /* initialize if framework */
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_if_base_framework, 0))) {
+        fprintf(stderr, "opal_if_base_open() failed -- process will likely abort (%s:%d, returned %d instead of OPAL_SUCCESS)\n",
+                __FILE__, __LINE__, ret);
+        return ret;
+    }
+
+    OPAL_TIMING_ENV_NEXT(otmng, "opal_if_init");
+
     return OPAL_SUCCESS;
 
  return_error:
@@ -452,15 +537,6 @@ opal_init(int* pargc, char*** pargv)
         goto return_error;
     }
 
-    /* open the memory manager components.  Memory hooks may be
-       triggered before this (any time after mem_free_init(),
-       actually).  This is a hook available for memory manager hooks
-       without good initialization routine support */
-    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_memory_base_framework, 0))) {
-        error = "opal_memory_base_open";
-        goto return_error;
-    }
-
     /* initialize the memory manager / tracker */
     if (OPAL_SUCCESS != (ret = opal_mem_hooks_init())) {
         error = "opal_mem_hooks_init";
@@ -491,6 +567,8 @@ opal_init(int* pargc, char*** pargv)
 
     /*
      * Need to start the event and progress engines if none else is.
+     * opal_cr_init uses the progress engine, so it is lumped together
+     * into this set as well.
      */
     /*
      * Initialize the event library
@@ -521,13 +599,41 @@ opal_init(int* pargc, char*** pargv)
         goto return_error;
     }
 
-    /* initialize the security framework */
-    if( OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_sec_base_framework, 0)) ) {
-        error = "opal_sec_base_open";
+    /* Load reachable framework */
+    if (OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_reachable_base_framework, 0))){
+        error = "opal_reachable_base_framework";
         goto return_error;
     }
-    if( OPAL_SUCCESS != (ret = opal_sec_base_select()) ) {
-        error = "opal_sec_base_select";
+    if (OPAL_SUCCESS != (ret = opal_reachable_base_select())) {
+        error = "opal_reachable_base_select";
+        goto return_error;
+    }
+
+#if OPAL_ENABLE_FT_CR    == 1
+    /*
+     * Initialize the compression framework
+     * Note: Currently only used in C/R so it has been marked to only
+     *       initialize when C/R is enabled. If other places in the code
+     *       wish to use this framework, it is safe to remove the protection.
+     */
+    if( OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_compress_base_framework, 0)) ) {
+        error = "opal_compress_base_open";
+        goto return_error;
+    }
+    if( OPAL_SUCCESS != (ret = opal_compress_base_select()) ) {
+        error = "opal_compress_base_select";
+        goto return_error;
+    }
+#endif
+
+    /*
+     * Initalize the checkpoint/restart functionality
+     * Note: Always do this so we can detect if the user
+     * attempts to checkpoint a non checkpointable job,
+     * otherwise the tools may hang or not clean up properly.
+     */
+    if (OPAL_SUCCESS != (ret = opal_cr_init() ) ) {
+        error = "opal_cr_init";
         goto return_error;
     }
 
