@@ -13,7 +13,7 @@
  *                         reserved.
  * Copyright (c) 2008-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
+ * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2014      Hochschule Esslingen.  All rights reserved.
  * Copyright (c) 2015      Mellanox Technologies, Inc.
@@ -75,14 +75,12 @@ OPAL_DECLSPEC extern bool opal_abort_print_stack;
  */
 OPAL_DECLSPEC extern int opal_abort_delay;
 
-/**
- * Ratio of calls to high-priority to low-priority progress functions.
- * Must be a power of two.
- */
-OPAL_DECLSPEC extern unsigned int opal_progress_lp_call_ratio;
-
 #if OPAL_ENABLE_DEBUG
 extern bool opal_progress_debug;
 #endif
 
+#if OPAL_ENABLE_FT_CR == 1
+extern bool opal_base_distill_checkpoint_ready;
+#endif
+
 #endif
