@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -9,11 +10,13 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2012      Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2012-2017 Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * Copyright (c) 2012-2015 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -33,6 +36,7 @@
 #include "opal/util/argv.h"
 #include "opal/util/cmd_line.h"
 #include "opal/util/output.h"
+#include "opal/util/opal_environ.h"
 
 #include "opal/mca/base/mca_base_var.h"
 #include "opal/constants.h"
@@ -53,7 +57,7 @@
 /*
  * Description of a command line option
  */
-struct cmd_line_option_t {
+struct ompi_cmd_line_option_t {
     opal_list_item_t super;
 
     char clo_short_name;
@@ -67,19 +71,20 @@ struct cmd_line_option_t {
     char *clo_mca_param_env_var;
     void *clo_variable_dest;
     bool clo_variable_set;
+    opal_cmd_line_otype_t clo_otype;
 };
-typedef struct cmd_line_option_t cmd_line_option_t;
-static void option_constructor(cmd_line_option_t *cmd);
-static void option_destructor(cmd_line_option_t *cmd);
+typedef struct ompi_cmd_line_option_t ompi_cmd_line_option_t;
+static void option_constructor(ompi_cmd_line_option_t *cmd);
+static void option_destructor(ompi_cmd_line_option_t *cmd);
 
-OBJ_CLASS_INSTANCE(cmd_line_option_t,
+OBJ_CLASS_INSTANCE(ompi_cmd_line_option_t,
                    opal_list_item_t,
                    option_constructor, option_destructor);
 
 /*
  * An option that was used in the argv that was parsed
  */
-struct cmd_line_param_t {
+struct ompi_cmd_line_param_t {
     opal_list_item_t super;
 
     /* Note that clp_arg points to storage "owned" by someone else; it
@@ -91,7 +96,7 @@ struct cmd_line_param_t {
     /* Pointer to the existing option.  This is also by reference; it
        should not be free()ed. */
 
-    cmd_line_option_t *clp_option;
+    ompi_cmd_line_option_t *clp_option;
 
     /* This argv array is a list of all the parameters of this option.
        It is owned by this parameter, and should be freed when this
@@ -100,10 +105,10 @@ struct cmd_line_param_t {
     int clp_argc;
     char **clp_argv;
 };
-typedef struct cmd_line_param_t cmd_line_param_t;
-static void param_constructor(cmd_line_param_t *cmd);
-static void param_destructor(cmd_line_param_t *cmd);
-OBJ_CLASS_INSTANCE(cmd_line_param_t,
+typedef struct ompi_cmd_line_param_t ompi_cmd_line_param_t;
+static void param_constructor(ompi_cmd_line_param_t *cmd);
+static void param_destructor(ompi_cmd_line_param_t *cmd);
+OBJ_CLASS_INSTANCE(ompi_cmd_line_param_t,
                    opal_list_item_t,
                    param_constructor, param_destructor);
 
@@ -133,11 +138,13 @@ static int split_shorts(opal_cmd_line_t *cmd,
                         char *token, char **args,
                         int *output_argc, char ***output_argv,
                         int *num_args_used, bool ignore_unknown);
-static cmd_line_option_t *find_option(opal_cmd_line_t *cmd,
+static ompi_cmd_line_option_t *find_option(opal_cmd_line_t *cmd,
                                       const char *option_name) __opal_attribute_nonnull__(1) __opal_attribute_nonnull__(2);
-static int set_dest(cmd_line_option_t *option, char *sval);
-static void fill(const cmd_line_option_t *a, char result[3][BUFSIZ]);
+static int set_dest(ompi_cmd_line_option_t *option, char *sval);
+static void fill(const ompi_cmd_line_option_t *a, char result[3][BUFSIZ]);
 static int qsort_callback(const void *a, const void *b);
+static opal_cmd_line_otype_t get_help_otype(opal_cmd_line_t *cmd);
+static char *build_parsable(ompi_cmd_line_option_t *option);
 
 
 /*
@@ -146,7 +153,7 @@ static int qsort_callback(const void *a, const void *b);
 int opal_cmd_line_create(opal_cmd_line_t *cmd,
                          opal_cmd_line_init_t *table)
 {
-    int i, ret = OPAL_SUCCESS;
+    int ret = OPAL_SUCCESS;
 
     /* Check bozo case */
 
@@ -155,8 +162,19 @@ int opal_cmd_line_create(opal_cmd_line_t *cmd,
     }
     OBJ_CONSTRUCT(cmd, opal_cmd_line_t);
 
-    /* Ensure we got a table */
+    if (NULL != table) {
+        ret = opal_cmd_line_add(cmd, table);
+    }
+    return ret;
+}
 
+/* Add a table to an existing cmd line object */
+int opal_cmd_line_add(opal_cmd_line_t *cmd,
+                      opal_cmd_line_init_t *table)
+{
+    int i, ret;
+
+    /* Ensure we got a table */
     if (NULL == table) {
         return OPAL_SUCCESS;
     }
@@ -164,9 +182,7 @@ int opal_cmd_line_create(opal_cmd_line_t *cmd,
     /* Loop through the table */
 
     for (i = 0; ; ++i) {
-
         /* Is this the end? */
-
         if ('\0' == table[i].ocl_cmd_short_name &&
             NULL == table[i].ocl_cmd_single_dash_name &&
             NULL == table[i].ocl_cmd_long_name) {
@@ -174,16 +190,14 @@ int opal_cmd_line_create(opal_cmd_line_t *cmd,
         }
 
         /* Nope -- it's an entry.  Process it. */
-
         ret = make_opt(cmd, &table[i]);
         if (OPAL_SUCCESS != ret) {
             return ret;
         }
     }
 
-    return ret;
+    return OPAL_SUCCESS;
 }
-
 /*
  * Append a command line entry to the previously constructed command line
  */
@@ -231,12 +245,12 @@ int opal_cmd_line_make_opt3(opal_cmd_line_t *cmd, char short_name,
  * Parse a command line according to a pre-built OPAL command line
  * handle.
  */
-int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
+int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown, bool ignore_unknown_option,
                         int argc, char **argv)
 {
     int i, j, orig, ret;
-    cmd_line_option_t *option;
-    cmd_line_param_t *param;
+    ompi_cmd_line_option_t *option;
+    ompi_cmd_line_param_t *param;
     bool is_unknown_option;
     bool is_unknown_token;
     bool is_option;
@@ -245,6 +259,7 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
     int num_args_used;
     bool have_help_option = false;
     bool printed_error = false;
+    bool help_without_arg = false;
 
     /* Bozo check */
 
@@ -337,7 +352,7 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
 
                     if (NULL != option) {
                         opal_argv_delete(&cmd->lcl_argc,
-			                 &cmd->lcl_argv, i,
+                                         &cmd->lcl_argv, i,
                                          1 + num_args_used);
                         opal_argv_insert(&cmd->lcl_argv, i, shortsv);
                         cmd->lcl_argc = opal_argv_count(cmd->lcl_argv);
@@ -371,7 +386,7 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
                    (insertted by split_shorts()), then print an error
                    and return. */
 
-                param = OBJ_NEW(cmd_line_param_t);
+                param = OBJ_NEW(ompi_cmd_line_param_t);
                 if (NULL == param) {
                     opal_mutex_unlock(&cmd->lcl_mutex);
                     return OPAL_ERR_OUT_OF_RESOURCE;
@@ -384,10 +399,17 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
                    recognized */
 
                 for (j = 0; j < option->clo_num_params; ++j, ++i) {
-
-                    /* If we run out of parameters, error */
-
+                    /* If we run out of parameters, error, unless its a help request
+                       which can have 0 or 1 arguments */
                     if (i >= cmd->lcl_argc) {
+                    /* If this is a help request, can have no arguments */
+                        if((NULL != option->clo_single_dash_name &&
+                           0 == strcmp(option->clo_single_dash_name, "h")) ||
+                           (NULL != option->clo_long_name &&
+                           0 == strcmp(option->clo_long_name, "help"))) {
+                            help_without_arg = true;
+                            continue;
+                        }
                         fprintf(stderr, "%s: Error: option \"%s\" did not "
                                 "have enough parameters (%d)\n",
                                 cmd->lcl_argv[0],
@@ -444,10 +466,11 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
                     }
                 }
 
-                /* If there are no options to this command, see if we
-                   need to set a boolean value to "true". */
+                /* If there are no options to this command or it is
+                   a help request with no argument, see if we need to
+                   set a boolean value to "true". */
 
-                if (0 == option->clo_num_params) {
+                if (0 == option->clo_num_params || help_without_arg) {
                     if (OPAL_SUCCESS != (ret = set_dest(option, "1"))) {
                         opal_mutex_unlock(&cmd->lcl_mutex);
                         return ret;
@@ -468,7 +491,7 @@ int opal_cmd_line_parse(opal_cmd_line_t *cmd, bool ignore_unknown,
            into the tail.  If we're not ignoring unknowns, then print
            an error and return. */
         if (is_unknown_option || is_unknown_token) {
-            if (!ignore_unknown || is_unknown_option) {
+            if (!ignore_unknown || (is_unknown_option && !ignore_unknown_option)) {
                 fprintf(stderr, "%s: Error: unknown option \"%s\"\n",
                         cmd->lcl_argv[0], cmd->lcl_argv[i]);
                 printed_error = true;
@@ -513,7 +536,8 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
     char *ret, temp[MAX_WIDTH * 2], line[MAX_WIDTH * 2];
     char *start, *desc, *ptr;
     opal_list_item_t *item;
-    cmd_line_option_t *option, **sorted;
+    ompi_cmd_line_option_t *option, **sorted;
+    opal_cmd_line_otype_t otype;
 
     /* Thread serialization */
 
@@ -527,148 +551,137 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
 
     /* First, take the original list and sort it */
 
-    sorted = (cmd_line_option_t**)malloc(sizeof(cmd_line_option_t *) *
+    sorted = (ompi_cmd_line_option_t**)malloc(sizeof(ompi_cmd_line_option_t *) *
                                          opal_list_get_size(&cmd->lcl_options));
     if (NULL == sorted) {
         opal_mutex_unlock(&cmd->lcl_mutex);
         return NULL;
     }
-    for (i = 0, item = opal_list_get_first(&cmd->lcl_options);
-         opal_list_get_end(&cmd->lcl_options) != item;
-         ++i, item = opal_list_get_next(item)) {
-        sorted[i] = (cmd_line_option_t *) item;
+    i = 0;
+    OPAL_LIST_FOREACH(item, &cmd->lcl_options, opal_list_item_t) {
+        sorted[i++] = (ompi_cmd_line_option_t *) item;
     }
-    qsort(sorted, i, sizeof(cmd_line_option_t*), qsort_callback);
+    qsort(sorted, i, sizeof(ompi_cmd_line_option_t*), qsort_callback);
+
+    /* Find if a help argument was passed, and return its type if it was. */
+
+    otype = get_help_otype(cmd);
 
     /* Now go through the sorted array and make the strings */
 
     for (j = 0; j < opal_list_get_size(&cmd->lcl_options); ++j) {
         option = sorted[j];
-        if (NULL != option->clo_description) {
-            bool filled = false;
+        if(otype == OPAL_CMD_LINE_OTYPE_PARSABLE) {
+            ret = build_parsable(option);
+            opal_argv_append(&argc, &argv, ret);
+            free(ret);
+            ret = NULL;
+        } else if(otype == OPAL_CMD_LINE_OTYPE_NULL || option->clo_otype == otype) {
+            if (NULL != option->clo_description) {
+                bool filled = false;
 
-            /* Build up the output line */
+                /* Build up the output line */
 
-            memset(line, 0, sizeof(line));
-            if ('\0' != option->clo_short_name) {
-                line[0] = '-';
-                line[1] = option->clo_short_name;
-                filled = true;
-            } else {
-                line[0] = ' ';
-                line[1] = ' ';
-            }
-            if (NULL != option->clo_single_dash_name) {
-                line[2] = (filled) ? '|' : ' ';
-                strncat(line, "-", sizeof(line) - 1);
-                strncat(line, option->clo_single_dash_name, sizeof(line) - 1);
-                filled = true;
-            }
-            if (NULL != option->clo_long_name) {
-                if (filled) {
-                    strncat(line, "|", sizeof(line) - 1);
+                memset(line, 0, sizeof(line));
+                if ('\0' != option->clo_short_name) {
+                    line[0] = '-';
+                    line[1] = option->clo_short_name;
+                    filled = true;
                 } else {
+                    line[0] = ' ';
+                    line[1] = ' ';
+                }
+                if (NULL != option->clo_single_dash_name) {
+                    line[2] = (filled) ? '|' : ' ';
+                    strncat(line, "-", sizeof(line) - 1);
+                    strncat(line, option->clo_single_dash_name, sizeof(line) - 1);
+                    filled = true;
+                }
+                if (NULL != option->clo_long_name) {
+                    if (filled) {
+                        strncat(line, "|", sizeof(line) - 1);
+                    } else {
+                        strncat(line, " ", sizeof(line) - 1);
+                    }
+                    strncat(line, "--", sizeof(line) - 1);
+                    strncat(line, option->clo_long_name, sizeof(line) - 1);
+                }
+                strncat(line, " ", sizeof(line) - 1);
+                for (i = 0; (int)i < option->clo_num_params; ++i) {
+                    len = sizeof(temp);
+                    snprintf(temp, len, "<arg%d> ", (int)i);
+                    strncat(line, temp, sizeof(line) - 1);
+                }
+                if (option->clo_num_params > 0) {
                     strncat(line, " ", sizeof(line) - 1);
                 }
-                strncat(line, "--", sizeof(line) - 1);
-                strncat(line, option->clo_long_name, sizeof(line) - 1);
-            }
-                   strncat(line, " ", sizeof(line) - 1);
-            for (i = 0; (int)i < option->clo_num_params; ++i) {
-                len = sizeof(temp);
-                snprintf(temp, len, "<arg%d> ", (int)i);
-                strncat(line, temp, sizeof(line) - 1);
-            }
-            if (option->clo_num_params > 0) {
-                strncat(line, " ", sizeof(line) - 1);
-            }
 
-            /* If we're less than param width, then start adding the
-               description to this line.  Otherwise, finish this line
-               and start adding the description on the next line. */
+                /* If we're less than param width, then start adding the
+                   description to this line.  Otherwise, finish this line
+                   and start adding the description on the next line. */
 
-            if (strlen(line) > PARAM_WIDTH) {
-                opal_argv_append(&argc, &argv, line);
-
-                /* Now reset the line to be all blanks up to
-                   PARAM_WIDTH so that we can start adding the
-                   description */
-
-                memset(line, ' ', PARAM_WIDTH);
-                line[PARAM_WIDTH] = '\0';
-            } else {
-
-                /* Add enough blanks to the end of the line so that we
-                   can start adding the description */
-
-                for (i = strlen(line); i < PARAM_WIDTH; ++i) {
-                    line[i] = ' ';
-                }
-                line[i] = '\0';
-            }
-
-            /* Loop over adding the description to the array, breaking
-               the string at most at MAX_WIDTH characters.  We need a
-               modifyable description (for simplicity), so strdup the
-               clo_description (because it's likely a compiler
-               constant, and may barf if we write temporary \0's in
-               the middle). */
-
-            desc = strdup(option->clo_description);
-            if (NULL == desc) {
-                free(sorted);
-                opal_mutex_unlock(&cmd->lcl_mutex);
-                return strdup("");
-            }
-            start = desc;
-            len = strlen(desc);
-            do {
-
-                /* Trim off leading whitespace */
-
-                while (isspace(*start) && start < desc + len) {
-                    ++start;
-                }
-                if (start >= desc + len) {
-                    break;
-                }
-
-                /* Last line */
-
-                if (strlen(start) < (MAX_WIDTH - PARAM_WIDTH)) {
-                    strncat(line, start, sizeof(line) - 1);
+                if (strlen(line) > PARAM_WIDTH) {
                     opal_argv_append(&argc, &argv, line);
-                    break;
+
+                    /* Now reset the line to be all blanks up to
+                       PARAM_WIDTH so that we can start adding the
+                       description */
+
+                    memset(line, ' ', PARAM_WIDTH);
+                    line[PARAM_WIDTH] = '\0';
+                } else {
+
+                    /* Add enough blanks to the end of the line so that we
+                       can start adding the description */
+
+                    for (i = strlen(line); i < PARAM_WIDTH; ++i) {
+                        line[i] = ' ';
+                    }
+                    line[i] = '\0';
                 }
 
-                /* We have more than 1 line's worth left -- find this
-                   line's worth and add it to the array.  Then reset
-                   and loop around to get the next line's worth. */
+                /* Loop over adding the description to the array, breaking
+                   the string at most at MAX_WIDTH characters.  We need a
+                   modifyable description (for simplicity), so strdup the
+                   clo_description (because it's likely a compiler
+                   constant, and may barf if we write temporary \0's in
+                   the middle). */
 
-                for (ptr = start + (MAX_WIDTH - PARAM_WIDTH);
-                     ptr > start; --ptr) {
-                    if (isspace(*ptr)) {
-                        *ptr = '\0';
-                        strncat(line, start, sizeof(line) - 1);
-                        opal_argv_append(&argc, &argv, line);
+                desc = strdup(option->clo_description);
+                if (NULL == desc) {
+                    free(sorted);
+                    opal_mutex_unlock(&cmd->lcl_mutex);
+                    return strdup("");
+                }
+                start = desc;
+                len = strlen(desc);
+                do {
 
-                        start = ptr + 1;
-                        memset(line, ' ', PARAM_WIDTH);
-                        line[PARAM_WIDTH] = '\0';
+                    /* Trim off leading whitespace */
+
+                    while (isspace(*start) && start < desc + len) {
+                        ++start;
+                    }
+                    if (start >= desc + len) {
                         break;
                     }
-                }
 
-                /* If we got all the way back to the beginning of the
-                   string, then go forward looking for a whitespace
-                   and break there. */
+                    /* Last line */
+
+                    if (strlen(start) < (MAX_WIDTH - PARAM_WIDTH)) {
+                        strncat(line, start, sizeof(line) - 1);
+                        opal_argv_append(&argc, &argv, line);
+                        break;
+                    }
+
+                    /* We have more than 1 line's worth left -- find this
+                       line's worth and add it to the array.  Then reset
+                       and loop around to get the next line's worth. */
 
-                if (ptr == start) {
                     for (ptr = start + (MAX_WIDTH - PARAM_WIDTH);
-                         ptr < start + len; ++ptr) {
+                         ptr > start; --ptr) {
                         if (isspace(*ptr)) {
                             *ptr = '\0';
-
                             strncat(line, start, sizeof(line) - 1);
                             opal_argv_append(&argc, &argv, line);
 
@@ -679,19 +692,45 @@ char *opal_cmd_line_get_usage_msg(opal_cmd_line_t *cmd)
                         }
                     }
 
-                    /* If we reached the end of the string with no
-                       whitespace, then just add it on and be done */
+                    /* If we got all the way back to the beginning of the
+                       string, then go forward looking for a whitespace
+                       and break there. */
 
-                    if (ptr >= start + len) {
-                        strncat(line, start, sizeof(line) - 1);
-                        opal_argv_append(&argc, &argv, line);
-                        start = desc + len + 1;
+                    if (ptr == start) {
+                        for (ptr = start + (MAX_WIDTH - PARAM_WIDTH);
+                             ptr < start + len; ++ptr) {
+                            if (isspace(*ptr)) {
+                                *ptr = '\0';
+
+                                strncat(line, start, sizeof(line) - 1);
+                                opal_argv_append(&argc, &argv, line);
+
+                                start = ptr + 1;
+                                memset(line, ' ', PARAM_WIDTH);
+                                line[PARAM_WIDTH] = '\0';
+                                break;
+                            }
+                        }
+
+                        /* If we reached the end of the string with no
+                           whitespace, then just add it on and be done */
+
+                        if (ptr >= start + len) {
+                            strncat(line, start, sizeof(line) - 1);
+                            opal_argv_append(&argc, &argv, line);
+                            start = desc + len + 1;
+                        }
                     }
-                }
-            } while (start < desc + len);
-            free(desc);
+                } while (start < desc + len);
+                free(desc);
+            }
         }
     }
+    if(otype == OPAL_CMD_LINE_OTYPE_NULL || otype == OPAL_CMD_LINE_OTYPE_GENERAL) {
+        char *argument_line = "\nFor additional mpirun arguments, run 'mpirun --help <category>'\n\nThe following categories exist: general (Defaults to this option), debug,\n    output, input, mapping, ranking, binding, devel (arguments useful to OMPI\n    Developers), compatibility (arguments supported for backwards compatibility),\n    launch (arguments to modify launch options), and dvm (Distributed Virtual\n    Machine arguments).";
+
+        opal_argv_append(&argc, &argv, argument_line);
+    }
     if (NULL != argv) {
         ret = opal_argv_join(argv, '\n');
         opal_argv_free(argv);
@@ -723,9 +762,8 @@ bool opal_cmd_line_is_taken(opal_cmd_line_t *cmd, const char *opt)
 int opal_cmd_line_get_ninsts(opal_cmd_line_t *cmd, const char *opt)
 {
     int ret;
-    opal_list_item_t *item;
-    cmd_line_param_t *param;
-    cmd_line_option_t *option;
+    ompi_cmd_line_param_t *param;
+    ompi_cmd_line_option_t *option;
 
     /* Thread serialization */
 
@@ -737,10 +775,7 @@ int opal_cmd_line_get_ninsts(opal_cmd_line_t *cmd, const char *opt)
     ret = 0;
     option = find_option(cmd, opt);
     if (NULL != option) {
-        for (item = opal_list_get_first(&cmd->lcl_params);
-             opal_list_get_end(&cmd->lcl_params) != item;
-             item = opal_list_get_next(item)) {
-            param = (cmd_line_param_t *) item;
+        OPAL_LIST_FOREACH(param, &cmd->lcl_params, ompi_cmd_line_param_t) {
             if (param->clp_option == option) {
                 ++ret;
             }
@@ -765,9 +800,8 @@ char *opal_cmd_line_get_param(opal_cmd_line_t *cmd, const char *opt, int inst,
                               int idx)
 {
     int num_found;
-    opal_list_item_t *item;
-    cmd_line_param_t *param;
-    cmd_line_option_t *option;
+    ompi_cmd_line_param_t *param;
+    ompi_cmd_line_option_t *option;
 
     /* Thread serialization */
 
@@ -784,11 +818,8 @@ char *opal_cmd_line_get_param(opal_cmd_line_t *cmd, const char *opt, int inst,
            parameter index greater than we will have */
 
         if (idx < option->clo_num_params) {
-            for (item = opal_list_get_first(&cmd->lcl_params);
-                 opal_list_get_end(&cmd->lcl_params) != item;
-                 item = opal_list_get_next(item)) {
-                param = (cmd_line_param_t *) item;
-                if (param->clp_option == option) {
+            OPAL_LIST_FOREACH(param, &cmd->lcl_params, ompi_cmd_line_param_t) {
+                if (param->clp_argc > 0 && param->clp_option == option) {
                     if (num_found == inst) {
                         opal_mutex_unlock(&cmd->lcl_mutex);
                         return param->clp_argv[idx];
@@ -850,7 +881,7 @@ int opal_cmd_line_get_tail(opal_cmd_line_t *cmd, int *tailc, char ***tailv)
  * Static functions
  **************************************************************************/
 
-static void option_constructor(cmd_line_option_t *o)
+static void option_constructor(ompi_cmd_line_option_t *o)
 {
     o->clo_short_name = '\0';
     o->clo_single_dash_name = NULL;
@@ -862,10 +893,11 @@ static void option_constructor(cmd_line_option_t *o)
     o->clo_mca_param_env_var = NULL;
     o->clo_variable_dest = NULL;
     o->clo_variable_set = false;
+    o->clo_otype = OPAL_CMD_LINE_OTYPE_NULL;
 }
 
 
-static void option_destructor(cmd_line_option_t *o)
+static void option_destructor(ompi_cmd_line_option_t *o)
 {
     if (NULL != o->clo_single_dash_name) {
         free(o->clo_single_dash_name);
@@ -882,7 +914,7 @@ static void option_destructor(cmd_line_option_t *o)
 }
 
 
-static void param_constructor(cmd_line_param_t *p)
+static void param_constructor(ompi_cmd_line_param_t *p)
 {
     p->clp_arg = NULL;
     p->clp_option = NULL;
@@ -891,7 +923,7 @@ static void param_constructor(cmd_line_param_t *p)
 }
 
 
-static void param_destructor(cmd_line_param_t *p)
+static void param_destructor(ompi_cmd_line_param_t *p)
 {
     if (NULL != p->clp_argv) {
         opal_argv_free(p->clp_argv);
@@ -905,7 +937,7 @@ static void cmd_line_constructor(opal_cmd_line_t *cmd)
        only thread that has this instance), there's no need to lock it
        right now. */
 
-    OBJ_CONSTRUCT(&cmd->lcl_mutex, opal_mutex_t);
+    OBJ_CONSTRUCT(&cmd->lcl_mutex, opal_recursive_mutex_t);
 
     /* Initialize the lists */
 
@@ -951,7 +983,7 @@ static void cmd_line_destructor(opal_cmd_line_t *cmd)
 
 static int make_opt(opal_cmd_line_t *cmd, opal_cmd_line_init_t *e)
 {
-    cmd_line_option_t *option;
+    ompi_cmd_line_option_t *option;
 
     /* Bozo checks */
 
@@ -965,9 +997,20 @@ static int make_opt(opal_cmd_line_t *cmd, opal_cmd_line_init_t *e)
         return OPAL_ERR_BAD_PARAM;
     }
 
-    /* Allocate and fill an option item */
+    /* see if the option already exists */
+    if (NULL != e->ocl_cmd_single_dash_name &&
+        NULL != find_option(cmd, e->ocl_cmd_single_dash_name)) {
+        opal_output(0, "Duplicate cmd line entry %s", e->ocl_cmd_single_dash_name);
+        return OPAL_ERR_BAD_PARAM;
+    }
+    if (NULL != e->ocl_cmd_long_name &&
+        NULL != find_option(cmd, e->ocl_cmd_long_name)) {
+        opal_output(0, "Duplicate cmd line entry %s", e->ocl_cmd_long_name);
+        return OPAL_ERR_BAD_PARAM;
+    }
 
-    option = OBJ_NEW(cmd_line_option_t);
+    /* Allocate and fill an option item */
+    option = OBJ_NEW(ompi_cmd_line_option_t);
     if (NULL == option) {
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
@@ -991,6 +1034,8 @@ static int make_opt(opal_cmd_line_t *cmd, opal_cmd_line_init_t *e)
                                      &option->clo_mca_param_env_var);
     }
 
+    option->clo_otype = e->ocl_otype;
+
     /* Append the item, serializing thread access */
 
     opal_mutex_lock(&cmd->lcl_mutex);
@@ -1043,7 +1088,7 @@ static int split_shorts(opal_cmd_line_t *cmd, char *token, char **args,
                         int *num_args_used, bool ignore_unknown)
 {
     int i, j, len;
-    cmd_line_option_t *option;
+    ompi_cmd_line_option_t *option;
     char fake_token[3];
     int num_args;
 
@@ -1104,20 +1149,16 @@ static int split_shorts(opal_cmd_line_t *cmd, char *token, char **args,
 }
 
 
-static cmd_line_option_t *find_option(opal_cmd_line_t *cmd,
+static ompi_cmd_line_option_t *find_option(opal_cmd_line_t *cmd,
                                       const char *option_name)
 {
-    opal_list_item_t *item;
-    cmd_line_option_t *option;
+    ompi_cmd_line_option_t *option;
 
     /* Iterate through the list of options hanging off the
        opal_cmd_line_t and see if we find a match in either the short
        or long names */
 
-    for (item = opal_list_get_first(&cmd->lcl_options);
-         opal_list_get_end(&cmd->lcl_options) != item;
-         item = opal_list_get_next(item)) {
-        option = (cmd_line_option_t *) item;
+    OPAL_LIST_FOREACH(option, &cmd->lcl_options, ompi_cmd_line_option_t) {
         if ((NULL != option->clo_long_name &&
              0 == strcmp(option_name, option->clo_long_name)) ||
             (NULL != option->clo_single_dash_name &&
@@ -1134,11 +1175,10 @@ static cmd_line_option_t *find_option(opal_cmd_line_t *cmd,
 }
 
 
-static int set_dest(cmd_line_option_t *option, char *sval)
+static int set_dest(ompi_cmd_line_option_t *option, char *sval)
 {
     int ival = atol(sval);
     long lval = strtoul(sval, NULL, 10);
-    char *str = NULL;
     size_t i;
 
     /* Set MCA param.  We do this in the environment because the MCA
@@ -1156,17 +1196,14 @@ static int set_dest(cmd_line_option_t *option, char *sval)
         case OPAL_CMD_LINE_TYPE_STRING:
         case OPAL_CMD_LINE_TYPE_INT:
         case OPAL_CMD_LINE_TYPE_SIZE_T:
-            asprintf(&str, "%s=%s", option->clo_mca_param_env_var, sval);
+            opal_setenv(option->clo_mca_param_env_var, sval, true, &environ);
             break;
         case OPAL_CMD_LINE_TYPE_BOOL:
-            asprintf(&str, "%s=1", option->clo_mca_param_env_var);
+            opal_setenv(option->clo_mca_param_env_var, "1", true, &environ);
             break;
         default:
             break;
         }
-        if (NULL != str) {
-            putenv(str);
-        }
     }
 
     /* Set variable */
@@ -1242,7 +1279,7 @@ static int set_dest(cmd_line_option_t *option, char *sval)
 /*
  * Helper function to qsort_callback
  */
-static void fill(const cmd_line_option_t *a, char result[3][BUFSIZ])
+static void fill(const ompi_cmd_line_option_t *a, char result[3][BUFSIZ])
 {
     int i = 0;
 
@@ -1269,8 +1306,8 @@ static int qsort_callback(const void *aa, const void *bb)
 {
     int ret, i;
     char str1[3][BUFSIZ], str2[3][BUFSIZ];
-    const cmd_line_option_t *a = *((const cmd_line_option_t**) aa);
-    const cmd_line_option_t *b = *((const cmd_line_option_t**) bb);
+    const ompi_cmd_line_option_t *a = *((const ompi_cmd_line_option_t**) aa);
+    const ompi_cmd_line_option_t *b = *((const ompi_cmd_line_option_t**) bb);
 
     /* Icky comparison of command line options.  There are multiple
        forms of each command line option, so we first have to check
@@ -1290,3 +1327,80 @@ static int qsort_callback(const void *aa, const void *bb)
 
     return 0;
 }
+
+
+/*
+ * Helper function to find the option type specified in the help
+ * command.
+ */
+static opal_cmd_line_otype_t get_help_otype(opal_cmd_line_t *cmd)
+{
+    /* Initialize to NULL, if it remains so, the user asked for
+       "full" help output */
+    opal_cmd_line_otype_t otype = OPAL_CMD_LINE_OTYPE_NULL;
+    char *arg;
+
+    arg = opal_cmd_line_get_param(cmd, "help", 0, 0);
+
+    /* If not "help", check for "h" */
+    if(NULL == arg) {
+        arg = opal_cmd_line_get_param(cmd, "h", 0, 0);
+    }
+
+    /* If arg is still NULL, give them the General info by default */
+    if(NULL == arg) {
+        arg = "general";
+    }
+
+    if (0 == strcmp(arg, "debug")) {
+        otype = OPAL_CMD_LINE_OTYPE_DEBUG;
+    } else if (0 == strcmp(arg, "output")) {
+        otype = OPAL_CMD_LINE_OTYPE_OUTPUT;
+    } else if (0 == strcmp(arg, "input")) {
+        otype = OPAL_CMD_LINE_OTYPE_INPUT;
+    } else if (0 == strcmp(arg, "mapping")) {
+        otype = OPAL_CMD_LINE_OTYPE_MAPPING;
+    } else if (0 == strcmp(arg, "ranking")) {
+        otype = OPAL_CMD_LINE_OTYPE_RANKING;
+    } else if (0 == strcmp(arg, "binding")) {
+        otype = OPAL_CMD_LINE_OTYPE_BINDING;
+    } else if (0 == strcmp(arg, "devel")) {
+        otype = OPAL_CMD_LINE_OTYPE_DEVEL;
+    } else if (0 == strcmp(arg, "compatibility")) {
+        otype = OPAL_CMD_LINE_OTYPE_COMPAT;
+    } else if (0 == strcmp(arg, "launch")) {
+        otype = OPAL_CMD_LINE_OTYPE_LAUNCH;
+    } else if (0 == strcmp(arg, "dvm")) {
+        otype = OPAL_CMD_LINE_OTYPE_DVM;
+    } else if (0 == strcmp(arg, "general")) {
+        otype = OPAL_CMD_LINE_OTYPE_GENERAL;
+    } else if (0 == strcmp(arg, "parsable")) {
+        otype = OPAL_CMD_LINE_OTYPE_PARSABLE;
+    }
+
+    return otype;
+}
+
+/*
+ * Helper function to build a parsable string for the help
+ * output.
+ */
+static char *build_parsable(ompi_cmd_line_option_t *option) {
+    char *line;
+    int length;
+
+    length = snprintf(NULL, 0, "%c:%s:%s:%d:%s\n", option->clo_short_name, option->clo_single_dash_name,
+                      option->clo_long_name, option->clo_num_params, option->clo_description);
+
+    line = (char *)malloc(length * sizeof(char));
+
+    if('\0' == option->clo_short_name) {
+        snprintf(line, length, "0:%s:%s:%d:%s\n", option->clo_single_dash_name, option->clo_long_name,
+                 option->clo_num_params, option->clo_description);
+    } else {
+        snprintf(line, length, "%c:%s:%s:%d:%s\n", option->clo_short_name, option->clo_single_dash_name,
+                 option->clo_long_name, option->clo_num_params, option->clo_description);
+    }
+
+    return line;
+}
