@@ -12,7 +12,7 @@
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2009-2015 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2015 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013      Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -358,6 +358,30 @@ opal_net_addr_isipv4public(const struct sockaddr *addr)
     return false;
 }
 
+bool
+opal_net_addr_isipv6linklocal(const struct sockaddr *addr)
+{
+    struct sockaddr_in6 if_addr;
+
+    switch (addr->sa_family) {
+#if OPAL_ENABLE_IPV6
+        case AF_INET6:
+            if_addr.sin6_family = AF_INET6;
+            if (1 != inet_pton(AF_INET6, "fe80::0000", &if_addr.sin6_addr)) {
+                return false;
+            }
+            return opal_net_samenetwork(addr, (struct sockaddr*)&if_addr, 64);
+#endif
+        case AF_INET:
+            return false;
+        default:
+            opal_output (0,
+                         "unhandled sa_family %d passed to opal_net_addr_isipv6linklocal\n",
+                         addr->sa_family);
+    }
+
+    return false;
+}
 
 char*
 opal_net_get_hostname(const struct sockaddr *addr)
