@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014-2015 Intel, Inc. All rights reserved
+ * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -33,6 +33,7 @@
 #include "orte/util/name_fns.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/show_help.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rml/rml.h"
@@ -506,7 +507,9 @@ static void process_opens(int fd, short args, void *cbdata)
     orte_process_name_t daemon;
     opal_list_t lt;
     opal_namelist_t *nm;
-    
+
+    ORTE_ACQUIRE_OBJECT(dfs);
+
     /* get the scheme to determine if we can process locally or not */
     if (NULL == (scheme = opal_uri_get_scheme(dfs->uri))) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
@@ -615,7 +618,8 @@ static void process_opens(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&daemon),
                         filename);
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(&daemon, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          &daemon, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -660,7 +664,7 @@ static void dfs_open(char *uri,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_opens);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_opens, ORTE_SYS_PRI);
 }
 
 static void process_close(int fd, short args, void *cbdata)
@@ -671,6 +675,8 @@ static void process_close(int fd, short args, void *cbdata)
     opal_buffer_t *buffer;
     int rc;
 
+    ORTE_ACQUIRE_OBJECT(close_dfs);
+
     opal_output_verbose(1, orte_dfs_base_framework.framework_output,
                         "%s closing fd %d",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -721,7 +727,8 @@ static void process_close(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&trk->host_daemon),
                         trk->local_fd);
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(&trk->host_daemon, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          &trk->host_daemon, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -755,7 +762,7 @@ static void dfs_close(int fd,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_close);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_close, ORTE_SYS_PRI);
 }
 
 static void process_sizes(int fd, short args, void *cbdata)
@@ -767,6 +774,8 @@ static void process_sizes(int fd, short args, void *cbdata)
     int rc;
     struct stat buf;
 
+    ORTE_ACQUIRE_OBJECT(size_dfs);
+
     opal_output_verbose(1, orte_dfs_base_framework.framework_output,
                         "%s processing get_size on fd %d",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -843,7 +852,8 @@ static void process_sizes(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&trk->host_daemon),
                         trk->local_fd);
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(&trk->host_daemon, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          &trk->host_daemon, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -878,7 +888,7 @@ static void dfs_get_file_size(int fd,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_sizes);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_sizes, ORTE_SYS_PRI);
 }
 
 
@@ -892,6 +902,8 @@ static void process_seeks(int fd, short args, void *cbdata)
     int rc;
     struct stat buf;
 
+    ORTE_ACQUIRE_OBJECT(seek_dfs);
+
     opal_output_verbose(1, orte_dfs_base_framework.framework_output,
                         "%s processing seek on fd %d",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -997,7 +1009,8 @@ static void process_seeks(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&trk->host_daemon),
                         trk->local_fd);
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(&trk->host_daemon, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          &trk->host_daemon, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -1031,7 +1044,7 @@ static void dfs_seek(int fd, long offset, int whence,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_seeks);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_seeks, ORTE_SYS_PRI);
 }
 
 static void process_reads(int fd, short args, void *cbdata)
@@ -1044,6 +1057,8 @@ static void process_reads(int fd, short args, void *cbdata)
     int64_t i64;
     int rc;
 
+    ORTE_ACQUIRE_OBJECT(read_dfs);
+
     /* look in our local records for this fd */
     trk = NULL;
     for (item = opal_list_get_first(&active_files);
@@ -1109,7 +1124,8 @@ static void process_reads(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&trk->host_daemon),
                         trk->local_fd);
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(&trk->host_daemon, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          &trk->host_daemon, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -1140,7 +1156,7 @@ static void dfs_read(int fd, uint8_t *buffer,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_reads);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_reads, ORTE_SYS_PRI);
 }
 
 static void process_posts(int fd, short args, void *cbdata)
@@ -1149,6 +1165,8 @@ static void process_posts(int fd, short args, void *cbdata)
     opal_buffer_t *buffer;
     int rc;
 
+    ORTE_ACQUIRE_OBJECT(dfs);
+
     /* we will get confirmation in our receive function, so
      * add this request to our list */
     dfs->id = req_id++;
@@ -1176,7 +1194,8 @@ static void process_posts(int fd, short args, void *cbdata)
         goto error;
     }
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_DAEMON, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          ORTE_PROC_MY_DAEMON, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -1206,7 +1225,7 @@ static void dfs_post_file_map(opal_buffer_t *bo,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_posts);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_posts, ORTE_SYS_PRI);
 }
 
 static void process_getfm(int fd, short args, void *cbdata)
@@ -1215,6 +1234,8 @@ static void process_getfm(int fd, short args, void *cbdata)
     opal_buffer_t *buffer;
     int rc;
 
+    ORTE_ACQUIRE_OBJECT(dfs);
+
     /* we will get confirmation in our receive function, so
      * add this request to our list */
     dfs->id = req_id++;
@@ -1237,7 +1258,8 @@ static void process_getfm(int fd, short args, void *cbdata)
         goto error;
     }
     /* send it */
-    if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_DAEMON, buffer,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          ORTE_PROC_MY_DAEMON, buffer,
                                           ORTE_RML_TAG_DFS_CMD,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -1268,7 +1290,7 @@ static void dfs_get_file_map(orte_process_name_t *target,
     dfs->cbdata = cbdata;
 
     /* post it for processing */
-    ORTE_DFS_POST_REQUEST(dfs, process_getfm);
+    ORTE_THREADSHIFT(dfs, orte_event_base, process_getfm, ORTE_SYS_PRI);
 }
 
 static void dfs_load_file_maps(orte_jobid_t jobid,
@@ -1291,4 +1313,3 @@ static void dfs_purge_file_maps(orte_jobid_t jobid,
         cbfunc(cbdata);
     }
 }
-
