@@ -12,6 +12,9 @@
  *                         All rights reserved.
  * Copyright (c) 2011-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2016-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,6 +33,7 @@
 
 #include "opal/dss/dss.h"
 
+#include "orte/util/compress.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/error_strings.h"
 #include "orte/mca/errmgr/errmgr.h"
@@ -39,7 +43,7 @@
 #include "orte/mca/routed/routed.h"
 #include "orte/mca/state/state.h"
 #include "orte/util/name_fns.h"
-#include "orte/util/nidmap.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_globals.h"
 
 #include "orte/mca/grpcomm/grpcomm.h"
@@ -68,9 +72,15 @@ static void gccon(orte_grpcomm_caddy_t *p)
     p->cbfunc = NULL;
     p->cbdata = NULL;
 }
+static void gcdes(orte_grpcomm_caddy_t *p)
+{
+    if (NULL != p->buf) {
+        OBJ_RELEASE(p->buf);
+    }
+}
 static OBJ_CLASS_INSTANCE(orte_grpcomm_caddy_t,
                           opal_object_t,
-                          gccon, NULL);
+                          gccon, gcdes);
 
 int orte_grpcomm_API_xcast(orte_grpcomm_signature_t *sig,
                            orte_rml_tag_t tag,
@@ -132,7 +142,9 @@ static void allgather_stub(int fd, short args, void *cbdata)
     int rc;
     orte_grpcomm_base_active_t *active;
     orte_grpcomm_coll_t *coll;
-    void *seq_number;
+    uint32_t *seq_number;
+
+    ORTE_ACQUIRE_OBJECT(cd);
 
     OPAL_OUTPUT_VERBOSE((1, orte_grpcomm_base_framework.framework_output,
                          "%s grpcomm:base:allgather stub",
@@ -141,11 +153,12 @@ static void allgather_stub(int fd, short args, void *cbdata)
     /* retrieve an existing tracker, create it if not
      * already found. The allgather module is responsible
      * for releasing it upon completion of the collective */
-    ret = opal_hash_table_get_value_ptr(&orte_grpcomm_base.sig_table, (void *)cd->sig->signature, cd->sig->sz * sizeof(orte_process_name_t), &seq_number);
+    ret = opal_hash_table_get_value_ptr(&orte_grpcomm_base.sig_table, (void *)cd->sig->signature, cd->sig->sz * sizeof(orte_process_name_t), (void **)&seq_number);
     if (OPAL_ERR_NOT_FOUND == ret) {
-        cd->sig->seq_num = 0;
+        seq_number = (uint32_t *)malloc(sizeof(uint32_t));
+        *seq_number = 0;
     } else if (OPAL_SUCCESS == ret) {
-        cd->sig->seq_num = *((uint32_t *)(seq_number)) + 1;
+        *seq_number = *seq_number + 1;
     } else {
         OPAL_OUTPUT((orte_grpcomm_base_framework.framework_output,
                      "%s rpcomm:base:allgather cannot get signature from hash table",
@@ -154,7 +167,7 @@ static void allgather_stub(int fd, short args, void *cbdata)
         OBJ_RELEASE(cd);
         return;
     }
-    ret = opal_hash_table_set_value_ptr(&orte_grpcomm_base.sig_table, (void *)cd->sig->signature, cd->sig->sz * sizeof(orte_process_name_t), (void *)&cd->sig->seq_num);
+    ret = opal_hash_table_set_value_ptr(&orte_grpcomm_base.sig_table, (void *)cd->sig->signature, cd->sig->sz * sizeof(orte_process_name_t), (void *)seq_number);
     if (OPAL_SUCCESS != ret) {
         OPAL_OUTPUT((orte_grpcomm_base_framework.framework_output,
                      "%s rpcomm:base:allgather cannot add new signature to hash table",
@@ -164,6 +177,12 @@ static void allgather_stub(int fd, short args, void *cbdata)
         return;
     }
     coll = orte_grpcomm_base_get_tracker(cd->sig, true);
+    if (NULL == coll) {
+        OBJ_RELEASE(cd->sig);
+        OBJ_RELEASE(cd);
+        return;
+    }
+    OBJ_RELEASE(cd->sig);
     coll->cbfunc = cd->cbfunc;
     coll->cbdata = cd->cbdata;
 
@@ -200,6 +219,7 @@ int orte_grpcomm_API_allgather(orte_grpcomm_signature_t *sig,
     cd->cbdata = cbdata;
     opal_event_set(orte_event_base, &cd->ev, -1, OPAL_EV_WRITE, allgather_stub, cd);
     opal_event_set_priority(&cd->ev, ORTE_MSG_PRI);
+    ORTE_POST_OBJECT(cd);
     opal_event_active(&cd->ev, OPAL_EV_WRITE, 1);
     return ORTE_SUCCESS;
 }
@@ -211,6 +231,7 @@ orte_grpcomm_coll_t* orte_grpcomm_base_get_tracker(orte_grpcomm_signature_t *sig
     orte_namelist_t *nm;
     opal_list_t children;
     size_t n;
+    char *routed;
 
     /* search the existing tracker list to see if this already exists */
     OPAL_LIST_FOREACH(coll, &orte_grpcomm_base.ongoing, orte_grpcomm_coll_t) {
@@ -257,30 +278,41 @@ orte_grpcomm_coll_t* orte_grpcomm_base_get_tracker(orte_grpcomm_signature_t *sig
         ORTE_ERROR_LOG(rc);
         return NULL;
     }
-    /* cycle thru the array of daemons and compare them to our
-     * children in the routing tree, counting the ones that match
-     * so we know how many daemons we should receive contributions from */
-    OBJ_CONSTRUCT(&children, opal_list_t);
-    orte_routed.get_routing_list(&children);
-    while (NULL != (nm = (orte_namelist_t*)opal_list_remove_first(&children))) {
+
+    /* get the routed module for our conduit */
+    routed = orte_rml.get_routed(orte_coll_conduit);
+    if (NULL == routed) {
+        /* this conduit is not routed, so we expect all daemons
+         * to directly participate */
+        coll->nexpected = coll->ndmns;
+    } else {
+        /* cycle thru the array of daemons and compare them to our
+         * children in the routing tree, counting the ones that match
+         * so we know how many daemons we should receive contributions from */
+        OBJ_CONSTRUCT(&children, opal_list_t);
+        orte_routed.get_routing_list(routed, &children);
+        while (NULL != (nm = (orte_namelist_t*)opal_list_remove_first(&children))) {
+            for (n=0; n < coll->ndmns; n++) {
+                if (nm->name.vpid == coll->dmns[n]) {
+                    coll->nexpected++;
+                    break;
+                }
+            }
+            OBJ_RELEASE(nm);
+        }
+        OPAL_LIST_DESTRUCT(&children);
+
+        /* see if I am in the array of participants - note that I may
+         * be in the rollup tree even though I'm not participating
+         * in the collective itself */
         for (n=0; n < coll->ndmns; n++) {
-            if (nm->name.vpid == coll->dmns[n]) {
+            if (coll->dmns[n] == ORTE_PROC_MY_NAME->vpid) {
                 coll->nexpected++;
                 break;
             }
         }
-        OBJ_RELEASE(nm);
-    }
-    OPAL_LIST_DESTRUCT(&children);
-    /* see if I am in the array of participants - note that I may
-     * be in the rollup tree even though I'm not participating
-     * in the collective itself */
-    for (n=0; n < coll->ndmns; n++) {
-        if (coll->dmns[n] == ORTE_PROC_MY_NAME->vpid) {
-            coll->nexpected++;
-            break;
-        }
     }
+
     return coll;
 }
 
@@ -304,8 +336,9 @@ static int create_dmns(orte_grpcomm_signature_t *sig,
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          (NULL == sig->signature) ? "NULL" : "NON-NULL"));
 
-    /* if NULL == procs, then all daemons are participating */
-    if (NULL == sig->signature) {
+    /* if NULL == procs, or the target jobid is our own,
+     * then all daemons are participating */
+    if (NULL == sig->signature || ORTE_PROC_MY_NAME->jobid == sig->signature[0].jobid) {
         *ndmns = orte_process_info.num_procs;
         *dmns = NULL;
         return ORTE_SUCCESS;
@@ -324,7 +357,7 @@ static int create_dmns(orte_grpcomm_signature_t *sig,
             *dmns = NULL;
             return ORTE_ERR_NOT_FOUND;
         }
-        if (NULL == jdata->map) {
+        if (NULL == jdata->map || 0 == jdata->map->num_nodes) {
             /* we haven't generated a job map yet - if we are the HNP,
              * then we should only involve ourselves. Otherwise, we have
              * no choice but to abort to avoid hangs */
@@ -335,12 +368,12 @@ static int create_dmns(orte_grpcomm_signature_t *sig,
                 *dmns = dns;
                 return ORTE_SUCCESS;
             }
+            ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
             ORTE_FORCED_TERMINATE(ORTE_ERR_NOT_FOUND);
             *ndmns = 0;
             *dmns = NULL;
             return ORTE_ERR_NOT_FOUND;
         }
-        /* get the array */
         dns = (orte_vpid_t*)malloc(jdata->map->num_nodes * sizeof(vpid));
         nds = 0;
         for (i=0; i < jdata->map->nodes->size && (int)nds < jdata->map->num_nodes; i++) {
@@ -376,10 +409,10 @@ static int create_dmns(orte_grpcomm_signature_t *sig,
                 *dmns = NULL;
                 return ORTE_ERR_NOT_FOUND;
             }
-            opal_output_verbose(5, orte_grpcomm_base_framework.framework_output,
+            OPAL_OUTPUT_VERBOSE((5, orte_grpcomm_base_framework.framework_output,
                                 "%s sign: GETTING PROC OBJECT FOR %s",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                ORTE_NAME_PRINT(&sig->signature[n]));
+                                ORTE_NAME_PRINT(&sig->signature[n])));
             if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(jdata->procs, sig->signature[n].vpid))) {
                 ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                 OPAL_LIST_DESTRUCT(&ds);
@@ -441,28 +474,87 @@ static int pack_xcast(orte_grpcomm_signature_t *sig,
                       orte_rml_tag_t tag)
 {
     int rc;
+    opal_buffer_t data;
+    int8_t flag;
+    uint8_t *cmpdata;
+    size_t cmplen;
+
+    /* setup an intermediate buffer */
+    OBJ_CONSTRUCT(&data, opal_buffer_t);
 
     /* pass along the signature */
-    if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &sig, 1, ORTE_SIGNATURE))) {
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(&data, &sig, 1, ORTE_SIGNATURE))) {
         ORTE_ERROR_LOG(rc);
-        goto CLEANUP;
+        OBJ_DESTRUCT(&data);
+        return rc;
     }
     /* pass the final tag */
-    if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &tag, 1, ORTE_RML_TAG))) {
+    if (ORTE_SUCCESS != (rc = opal_dss.pack(&data, &tag, 1, ORTE_RML_TAG))) {
         ORTE_ERROR_LOG(rc);
-        goto CLEANUP;
+        OBJ_DESTRUCT(&data);
+        return rc;
     }
 
     /* copy the payload into the new buffer - this is non-destructive, so our
      * caller is still responsible for releasing any memory in the buffer they
      * gave to us
      */
-    if (ORTE_SUCCESS != (rc = opal_dss.copy_payload(buffer, message))) {
+    if (ORTE_SUCCESS != (rc = opal_dss.copy_payload(&data, message))) {
         ORTE_ERROR_LOG(rc);
-        goto CLEANUP;
+        OBJ_DESTRUCT(&data);
+        return rc;
     }
 
-CLEANUP:
+    /* see if we want to compress this message */
+    if (orte_util_compress_block((uint8_t*)data.base_ptr, data.bytes_used,
+                                 &cmpdata, &cmplen)) {
+        /* the data was compressed - mark that we compressed it */
+        flag = 1;
+        if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &flag, 1, OPAL_INT8))) {
+            ORTE_ERROR_LOG(rc);
+            free(cmpdata);
+            OBJ_DESTRUCT(&data);
+            return rc;
+        }
+        /* pack the compressed length */
+        if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &cmplen, 1, OPAL_SIZE))) {
+            ORTE_ERROR_LOG(rc);
+            free(cmpdata);
+            OBJ_DESTRUCT(&data);
+            return rc;
+        }
+        /* pack the uncompressed length */
+        if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &data.bytes_used, 1, OPAL_SIZE))) {
+            ORTE_ERROR_LOG(rc);
+            free(cmpdata);
+            OBJ_DESTRUCT(&data);
+            return rc;
+        }
+        /* pack the compressed info */
+        if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, cmpdata, cmplen, OPAL_UINT8))) {
+            ORTE_ERROR_LOG(rc);
+            free(cmpdata);
+            OBJ_DESTRUCT(&data);
+            return rc;
+        }
+        OBJ_DESTRUCT(&data);
+        free(cmpdata);
+    } else {
+        /* mark that it was not compressed */
+        flag = 0;
+        if (ORTE_SUCCESS != (rc = opal_dss.pack(buffer, &flag, 1, OPAL_INT8))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_DESTRUCT(&data);
+            free(cmpdata);
+            return rc;
+        }
+        /* transfer the payload across */
+        opal_dss.copy_payload(buffer, &data);
+        OBJ_DESTRUCT(&data);
+    }
+
+    OPAL_OUTPUT_VERBOSE((1, orte_grpcomm_base_framework.framework_output,
+                         "MSG SIZE: %lu", buffer->bytes_used));
     return ORTE_SUCCESS;
 }
 
