@@ -15,7 +15,11 @@
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2013-2016 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2017      Rutgers, The State University of New Jersey.
+ *                         All rights reserved.
+ * Copyright (c) 2017      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -123,6 +127,7 @@
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/rtc/rtc.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/threads.h"
 
 #include "orte/mca/odls/base/base.h"
 #include "orte/mca/odls/base/odls_private.h"
@@ -144,11 +149,8 @@ static int orte_odls_default_restart_proc(orte_proc_t *child);
 static void send_error_show_help(int fd, int exit_status,
                                  const char *file, const char *topic, ...)
     __opal_attribute_noreturn__;
-static int do_child(orte_app_context_t* context,
-                    orte_proc_t *child,
-                    char **environ_copy,
-                    orte_job_t *jobdat, int write_fd,
-                    orte_iof_base_io_conf_t opts)
+
+static int do_child(orte_odls_spawn_caddy_t *cd, int write_fd)
     __opal_attribute_noreturn__;
 
 
@@ -156,79 +158,15 @@ static int do_child(orte_app_context_t* context,
  * Module
  */
 orte_odls_base_module_t orte_odls_default_module = {
-    orte_odls_base_default_get_add_procs_data,
-    orte_odls_default_launch_local_procs,
-    orte_odls_default_kill_local_procs,
-    orte_odls_default_signal_local_procs,
-    orte_odls_base_default_deliver_message,
-    orte_odls_default_restart_proc
+    .get_add_procs_data = orte_odls_base_default_get_add_procs_data,
+    .launch_local_procs = orte_odls_default_launch_local_procs,
+    .kill_local_procs = orte_odls_default_kill_local_procs,
+    .signal_local_procs = orte_odls_default_signal_local_procs,
+    .restart_proc = orte_odls_default_restart_proc
 };
 
 
-static bool odls_default_child_died(orte_proc_t *child)
-{
-    time_t end;
-    pid_t ret;
-
-    /* Because of rounding in time (which returns whole seconds) we
-     * have to add 1 to our wait number: this means that we wait
-     * somewhere between (target) and (target)+1 seconds.  Otherwise,
-     * the default 1s actually means 'somwhere between 0 and 1s'. */
-    end = time(NULL) + orte_odls_globals.timeout_before_sigkill + 1;
-    do {
-        OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
-                             "%s odls:default:WAITPID CHECKING PID %d WITH TIMEOUT %d SECONDS",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid),
-                             orte_odls_globals.timeout_before_sigkill + 1));
-        ret = waitpid(child->pid, &child->exit_code, WNOHANG);
-        if (child->pid == ret) {
-            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
-                                 "%s odls:default:WAITPID INDICATES PROC %d IS DEAD",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
-            /* It died -- return success */
-            return true;
-        } else if (0 == ret) {
-            /* with NOHANG specified, if a process has already exited
-             * while waitpid was registered, then waitpid returns 0
-             * as there is no error - this is a race condition problem
-             * that occasionally causes us to incorrectly report a proc
-             * as refusing to die. Unfortunately, errno may not be reset
-             * by waitpid in this case, so we cannot check it.
-             *
-             * (note the previous fix to this, to return 'process dead'
-             * here, fixes the race condition at the cost of reporting
-             * all live processes have immediately died!  Better to
-             * occasionally report a dead process as still living -
-             * which will occasionally trip the timeout for cases that
-             * are right on the edge.)
-             */
-            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
-                                 "%s odls:default:WAITPID INDICATES PID %d MAY HAVE ALREADY EXITED",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
-            /* Do nothing, process still alive */
-        } else if (-1 == ret && ECHILD == errno) {
-            /* The pid no longer exists, so we'll call this "good
-               enough for government work" */
-            OPAL_OUTPUT_VERBOSE((20, orte_odls_base_framework.framework_output,
-                                 "%s odls:default:WAITPID INDICATES PID %d NO LONGER EXISTS",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
-            return true;
-        }
-
-        /* Bogus delay for 1 msec - let's actually give the CPU some time
-         * to quit the other process (sched_yield() -- even if we have it
-         * -- changed behavior in 2.6.3x Linux flavors to be undesirable)
-         * Don't use select on a bogus file descriptor here as it has proven
-         * unreliable and sometimes immediately returns - we really, really
-         * -do- want to wait a bit!
-         */
-        usleep(1000);
-    } while (time(NULL) < end);
-
-    /* The child didn't die, so return false */
-    return false;
-}
-
+/* deliver a signal to a specified pid. */
 static int odls_default_kill_local(pid_t pid, int signum)
 {
     pid_t pgrp;
@@ -243,9 +181,10 @@ static int odls_default_kill_local(pid_t pid, int signum)
          * child processes our child may have
          * started
          */
-        pid = pgrp;
+        pid = -pgrp;
     }
 #endif
+
     if (0 != kill(pid, signum)) {
         if (ESRCH != errno) {
             OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
@@ -265,7 +204,7 @@ int orte_odls_default_kill_local_procs(opal_pointer_array_t *procs)
     int rc;
 
     if (ORTE_SUCCESS != (rc = orte_odls_base_default_kill_local_procs(procs,
-                                    odls_default_kill_local, odls_default_child_died))) {
+                                            odls_default_kill_local))) {
         ORTE_ERROR_LOG(rc);
         return rc;
     }
@@ -354,9 +293,8 @@ static void send_error_show_help(int fd, int exit_status,
     exit(exit_status);
 }
 
-/* close all open file descriptors w/ exception of stdin/stdout/stderr,
-   the pipe used for the IOF INTERNAL messages, and the pipe up to
-   the parent. */
+/* close all open file descriptors w/ exception of stdin/stdout/stderr
+   and the pipe up to the parent. */
 static int close_open_file_descriptors(int write_fd,
                                       orte_iof_base_io_conf_t opts) {
     DIR *dir = opendir("/proc/self/fd");
@@ -373,7 +311,11 @@ static int close_open_file_descriptors(int write_fd,
             closedir(dir);
             return ORTE_ERR_TYPE_MISMATCH;
         }
-        if (fd >=3 && fd != opts.p_internal[1] && fd != write_fd) {
+        if (fd >=3 &&
+#if OPAL_PMIX_V1
+            fd != opts.p_internal[1] &&
+#endif
+            fd != write_fd) {
             close(fd);
         }
     }
@@ -381,28 +323,23 @@ static int close_open_file_descriptors(int write_fd,
     return ORTE_SUCCESS;
 }
 
-static int do_child(orte_app_context_t* context,
-                    orte_proc_t *child,
-                    char **environ_copy,
-                    orte_job_t *jobdat, int write_fd,
-                    orte_iof_base_io_conf_t opts)
+static int do_child(orte_odls_spawn_caddy_t *cd, int write_fd)
 {
-    int i, rc;
+    int i;
     sigset_t sigs;
     long fd, fdmax = sysconf(_SC_OPEN_MAX);
-    char *param, *msg;
+    char dir[MAXPATHLEN];
 
 #if HAVE_SETPGID
-    /* Set a new process group for this child, so that a
-       SIGSTOP can be sent to it without being sent to the
-       orted. */
+    /* Set a new process group for this child, so that any
+     * signals we send to it will reach any children it spawns */
     setpgid(0, 0);
 #endif
 
     /* Setup the pipe to be close-on-exec */
     opal_fd_set_cloexec(write_fd);
 
-    if (NULL != child) {
+    if (NULL != cd->child) {
         /* setup stdout/stderr so that any error messages that we
            may print out will get displayed back at orterun.
 
@@ -416,22 +353,21 @@ static int do_child(orte_app_context_t* context,
            always outputs a nice, single message indicating what
            happened
         */
-        if (ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
-            if (ORTE_SUCCESS != (i = orte_iof_base_setup_child(&opts,
-                                                               &environ_copy))) {
+        if (ORTE_FLAG_TEST(cd->jdata, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
+            if (ORTE_SUCCESS != (i = orte_iof_base_setup_child(&cd->opts, &cd->env))) {
                 ORTE_ERROR_LOG(i);
                 send_error_show_help(write_fd, 1,
                                      "help-orte-odls-default.txt",
                                      "iof setup failed",
-                                     orte_process_info.nodename, context->app);
+                                     orte_process_info.nodename, cd->app->app);
                 /* Does not return */
             }
         }
 
         /* now set any child-level controls such as binding */
-        orte_rtc.set(jobdat, child, &environ_copy, write_fd);
+        orte_rtc.set(cd->jdata, cd->child, &cd->env, write_fd);
 
-    } else if (!ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
+    } else if (!ORTE_FLAG_TEST(cd->jdata, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
         /* tie stdin/out/err/internal to /dev/null */
         int fdnull;
         for (i=0; i < 3; i++) {
@@ -441,41 +377,35 @@ static int do_child(orte_app_context_t* context,
             }
             close(fdnull);
         }
+#if OPAL_PMIX_V1
         fdnull = open("/dev/null", O_RDONLY, 0);
-        if (fdnull > opts.p_internal[1]) {
-            dup2(fdnull, opts.p_internal[1]);
+        if (fdnull > cd->opts.p_internal[1]) {
+            dup2(fdnull, cd->opts.p_internal[1]);
         }
         close(fdnull);
+#endif
     }
 
-    /* if the user requested it, set the system resource limits */
-    if (OPAL_SUCCESS != (rc = opal_util_init_sys_limits(&msg))) {
-        send_error_show_help(write_fd, 1, "help-orte-odls-default.txt",
-                             "set limit",
-                             orte_process_info.nodename, context->app,
-                             __FILE__, __LINE__, msg);
-    }
-    /* ensure we only do this once */
-    (void) mca_base_var_env_name("opal_set_max_sys_limits", &param);
-    opal_unsetenv(param, &environ_copy);
-    free(param);
-
     /* close all open file descriptors w/ exception of stdin/stdout/stderr,
        the pipe used for the IOF INTERNAL messages, and the pipe up to
        the parent. */
-    if (ORTE_SUCCESS != close_open_file_descriptors(write_fd, opts)) {
+    if (ORTE_SUCCESS != close_open_file_descriptors(write_fd, cd->opts)) {
         // close *all* file descriptors -- slow
         for(fd=3; fd<fdmax; fd++) {
-            if (fd != opts.p_internal[1] && fd != write_fd) {
+            if (
+#if OPAL_PMIX_V1
+                fd != cd->opts.p_internal[1] &&
+#endif
+                fd != write_fd) {
                 close(fd);
             }
         }
     }
 
-    if (context->argv == NULL) {
-        context->argv = malloc(sizeof(char*)*2);
-        context->argv[0] = strdup(context->app);
-        context->argv[1] = NULL;
+    if (cd->argv == NULL) {
+        cd->argv = malloc(sizeof(char*)*2);
+        cd->argv[0] = strdup(cd->app->app);
+        cd->argv[1] = NULL;
     }
 
     /* Set signal handlers back to the default.  Do this close to
@@ -498,39 +428,46 @@ static int do_child(orte_app_context_t* context,
     sigprocmask(0, 0, &sigs);
     sigprocmask(SIG_UNBLOCK, &sigs, 0);
 
-    /* Exec the new executable */
+    /* take us to the correct wdir */
+    if (NULL != cd->wdir) {
+        if (0 != chdir(cd->wdir)) {
+            send_error_show_help(write_fd, 1,
+                                 "help-orterun.txt",
+                                 "orterun:wdir-not-found",
+                                 "orted",
+                                 cd->wdir,
+                                 orte_process_info.nodename,
+                                 (NULL == cd->child) ? 0 : cd->child->app_rank);
+            /* Does not return */
+        }
+    }
 
-    execve(context->app, context->argv, environ_copy);
+    /* Exec the new executable */
+    execve(cd->cmd, cd->argv, cd->env);
+    getcwd(dir, sizeof(dir));
     send_error_show_help(write_fd, 1,
                          "help-orte-odls-default.txt", "execve error",
-                         orte_process_info.nodename, context->app, strerror(errno));
+                         orte_process_info.nodename, dir, cd->app->app, strerror(errno));
     /* Does not return */
 }
 
 
-static int do_parent(orte_app_context_t* context,
-                     orte_proc_t *child,
-                     char **environ_copy,
-                     orte_job_t *jobdat, int read_fd,
-                     orte_iof_base_io_conf_t opts)
+static int do_parent(orte_odls_spawn_caddy_t *cd, int read_fd)
 {
     int rc;
     orte_odls_pipe_err_msg_t msg;
     char file[ORTE_ODLS_MAX_FILE_LEN + 1], topic[ORTE_ODLS_MAX_TOPIC_LEN + 1], *str = NULL;
 
-    if (NULL != child && ORTE_FLAG_TEST(jobdat, ORTE_JOB_FLAG_FORWARD_OUTPUT)) {
-        /* connect endpoints IOF */
-        rc = orte_iof_base_setup_parent(&child->name, &opts);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
-            close(read_fd);
-
-            if (NULL != child) {
-                child->state = ORTE_PROC_STATE_UNDEF;
-            }
-            return rc;
-        }
+    if (cd->opts.connect_stdin) {
+        close(cd->opts.p_stdin[0]);
     }
+    close(cd->opts.p_stdout[1]);
+    if( !orte_iof_base.redirect_app_stderr_to_stdout ) {
+        close(cd->opts.p_stderr[1]);
+    }
+#if OPAL_PMIX_V1
+    close(cd->opts.p_internal[1]);
+#endif
 
     /* Block reading a message from the pipe */
     while (1) {
@@ -546,18 +483,18 @@ static int do_parent(orte_app_context_t* context,
             ORTE_ERROR_LOG(rc);
             close(read_fd);
 
-            if (NULL != child) {
-                child->state = ORTE_PROC_STATE_UNDEF;
+            if (NULL != cd->child) {
+                cd->child->state = ORTE_PROC_STATE_UNDEF;
             }
             return rc;
         }
 
         /* Otherwise, we got a warning or error message from the child */
-        if (NULL != child) {
+        if (NULL != cd->child) {
             if (msg.fatal) {
-                ORTE_FLAG_UNSET(child, ORTE_PROC_FLAG_ALIVE);
+                ORTE_FLAG_UNSET(cd->child, ORTE_PROC_FLAG_ALIVE);
             } else {
-                ORTE_FLAG_SET(child, ORTE_PROC_FLAG_ALIVE);
+                ORTE_FLAG_SET(cd->child, ORTE_PROC_FLAG_ALIVE);
             }
         }
 
@@ -567,10 +504,10 @@ static int do_parent(orte_app_context_t* context,
             if (OPAL_SUCCESS != rc) {
                 orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
-                               orte_process_info.nodename, context->app,
+                               orte_process_info.nodename, cd->app->app,
                                "opal_fd_read", __FILE__, __LINE__);
-                if (NULL != child) {
-                    child->state = ORTE_PROC_STATE_UNDEF;
+                if (NULL != cd->child) {
+                    cd->child->state = ORTE_PROC_STATE_UNDEF;
                 }
                 return rc;
             }
@@ -581,10 +518,10 @@ static int do_parent(orte_app_context_t* context,
             if (OPAL_SUCCESS != rc) {
                 orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
-                               orte_process_info.nodename, context->app,
+                               orte_process_info.nodename, cd->app->app,
                                "opal_fd_read", __FILE__, __LINE__);
-                if (NULL != child) {
-                    child->state = ORTE_PROC_STATE_UNDEF;
+                if (NULL != cd->child) {
+                    cd->child->state = ORTE_PROC_STATE_UNDEF;
                 }
                 return rc;
             }
@@ -595,10 +532,10 @@ static int do_parent(orte_app_context_t* context,
             if (NULL == str) {
                 orte_show_help("help-orte-odls-default.txt", "syscall fail",
                                true,
-                               orte_process_info.nodename, context->app,
+                               orte_process_info.nodename, cd->app->app,
                                "opal_fd_read", __FILE__, __LINE__);
-                if (NULL != child) {
-                    child->state = ORTE_PROC_STATE_UNDEF;
+                if (NULL != cd->child) {
+                    cd->child->state = ORTE_PROC_STATE_UNDEF;
                 }
                 return rc;
             }
@@ -619,9 +556,9 @@ static int do_parent(orte_app_context_t* context,
            closed, indicating that the child launched
            successfully). */
         if (msg.fatal) {
-            if (NULL != child) {
-                child->state = ORTE_PROC_STATE_FAILED_TO_START;
-                ORTE_FLAG_UNSET(child, ORTE_PROC_FLAG_ALIVE);
+            if (NULL != cd->child) {
+                cd->child->state = ORTE_PROC_STATE_FAILED_TO_START;
+                ORTE_FLAG_UNSET(cd->child, ORTE_PROC_FLAG_ALIVE);
             }
             close(read_fd);
             return ORTE_ERR_FAILED_TO_START;
@@ -631,9 +568,9 @@ static int do_parent(orte_app_context_t* context,
     /* If we got here, it means that the pipe closed without
        indication of a fatal error, meaning that the child process
        launched successfully. */
-    if (NULL != child) {
-        child->state = ORTE_PROC_STATE_RUNNING;
-        ORTE_FLAG_SET(child, ORTE_PROC_FLAG_ALIVE);
+    if (NULL != cd->child) {
+        cd->child->state = ORTE_PROC_STATE_RUNNING;
+        ORTE_FLAG_SET(cd->child, ORTE_PROC_FLAG_ALIVE);
     }
     close(read_fd);
 
@@ -644,38 +581,12 @@ static int do_parent(orte_app_context_t* context,
 /**
  *  Fork/exec the specified processes
  */
-static int odls_default_fork_local_proc(orte_app_context_t* context,
-                                        orte_proc_t *child,
-                                        char **environ_copy,
-                                        orte_job_t *jobdat)
+static int odls_default_fork_local_proc(void *cdptr)
 {
-    orte_iof_base_io_conf_t opts;
-    int rc, p[2];
+    orte_odls_spawn_caddy_t *cd = (orte_odls_spawn_caddy_t*)cdptr;
+    int p[2];
     pid_t pid;
-
-    if (NULL != child) {
-        /* should pull this information from MPIRUN instead of going with
-         default */
-        opts.usepty = OPAL_ENABLE_PTY_SUPPORT;
-
-        /* do we want to setup stdin? */
-        if (NULL != child &&
-            (jobdat->stdin_target == ORTE_VPID_WILDCARD ||
-             child->name.vpid == jobdat->stdin_target)) {
-            opts.connect_stdin = true;
-        } else {
-            opts.connect_stdin = false;
-        }
-
-        if (ORTE_SUCCESS != (rc = orte_iof_base_setup_prefork(&opts))) {
-            ORTE_ERROR_LOG(rc);
-            if (NULL != child) {
-                child->state = ORTE_PROC_STATE_FAILED_TO_START;
-                child->exit_code = rc;
-            }
-            return rc;
-        }
-    }
+    orte_proc_t *child = cd->child;
 
     /* A pipe is used to communicate between the parent and child to
        indicate whether the exec ultimately succeeded or failed.  The
@@ -711,15 +622,12 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
 
     if (pid == 0) {
         close(p[0]);
-#if HAVE_SETPGID
-        setpgid(0, 0);
-#endif
-        do_child(context, child, environ_copy, jobdat, p[1], opts);
+        do_child(cd, p[1]);
         /* Does not return */
     }
 
     close(p[1]);
-    return do_parent(context, child, environ_copy, jobdat, p[0], opts);
+    return do_parent(cd, p[0]);
 }
 
 
@@ -751,21 +659,28 @@ int orte_odls_default_launch_local_procs(opal_buffer_t *data)
  * Send a signal to a pid.  Note that if we get an error, we set the
  * return value and let the upper layer print out the message.
  */
-static int send_signal(pid_t pid, int signal)
+static int send_signal(pid_t pd, int signal)
 {
     int rc = ORTE_SUCCESS;
+    pid_t pid;
+
+    if (orte_odls_globals.signal_direct_children_only) {
+        pid = pd;
+    } else {
+#if HAVE_SETPGID
+        /* send to the process group so that any children of our children
+         * also receive the signal*/
+        pid = -pd;
+#else
+        pid = pd;
+#endif
+    }
 
     OPAL_OUTPUT_VERBOSE((1, orte_odls_base_framework.framework_output,
                          "%s sending signal %d to pid %ld",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          signal, (long)pid));
 
-#if HAVE_SETPGID
-    /* Send the signal to the process group rather than the
-       process.  The child is the leader of its process group. */
-    pid = -pid;
-#endif
-
     if (kill(pid, signal) != 0) {
         switch(errno) {
             case EINVAL:
