@@ -12,7 +12,7 @@
  * Copyright (c) 2006-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2010-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,13 +28,19 @@
 #include "opal/class/opal_list.h"
 
 #include "orte/mca/rml/base/base.h"
-
+#include "orte/util/threads.h"
 #include "oob_tcp.h"
 #include "oob_tcp_hdr.h"
 
+/* forward declare */
+struct mca_oob_tcp_peer_t;
+
 /* tcp structure for sending a message */
 typedef struct {
     opal_list_item_t super;
+    opal_event_t ev;
+    struct mca_oob_tcp_peer_t *peer;
+    bool activate;
     mca_oob_tcp_hdr_t hdr;
     orte_rml_send_t *msg;
     char *data;
@@ -74,27 +80,11 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_recv_t);
  */
 #define MCA_OOB_TCP_QUEUE_MSG(p, s, f)                                  \
     do {                                                                \
-        /* if there is no message on-deck, put this one there */        \
-        if (NULL == (p)->send_msg) {                                    \
-            (p)->send_msg = (s);                                        \
-        } else {                                                        \
-            /* add it to the queue */                                   \
-            opal_list_append(&(p)->send_queue, &(s)->super);            \
-        }                                                               \
-        if ((f)) {                                                      \
-            /* if we aren't connected, then start connecting */         \
-            if (MCA_OOB_TCP_CONNECTED != (p)->state) {                  \
-                (p)->state = MCA_OOB_TCP_CONNECTING;                    \
-                ORTE_ACTIVATE_TCP_CONN_STATE((p), mca_oob_tcp_peer_try_connect); \
-            } else {                                                    \
-                /* ensure the send event is active */                   \
-                if (!(p)->send_ev_active) {                             \
-                    opal_event_add(&(p)->send_event, 0);                \
-                    (p)->send_ev_active = true;                         \
-                }                                                       \
-            }                                                           \
-        }                                                               \
-    }while(0);
+        (s)->peer = (struct mca_oob_tcp_peer_t*)(p);                    \
+        (s)->activate = (f);                                            \
+        ORTE_THREADSHIFT((s), (p)->ev_base,                             \
+                         mca_oob_tcp_queue_msg, ORTE_MSG_PRI);          \
+    } while(0)
 
 /* queue a message to be sent by one of our modules - must
  * provide the following params:
@@ -104,40 +94,45 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_recv_t);
  */
 #define MCA_OOB_TCP_QUEUE_SEND(m, p)                                    \
     do {                                                                \
-        mca_oob_tcp_send_t *msg;                                        \
+        mca_oob_tcp_send_t *_s;                                         \
         int i;                                                          \
         opal_output_verbose(5, orte_oob_base_framework.framework_output, \
                             "%s:[%s:%d] queue send to %s",              \
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
-                            __FILE__, __LINE__,                         \
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),        \
+                             __FILE__, __LINE__,                        \
                             ORTE_NAME_PRINT(&((m)->dst)));              \
-        msg = OBJ_NEW(mca_oob_tcp_send_t);                              \
+        _s = OBJ_NEW(mca_oob_tcp_send_t);                              \
         /* setup the header */                                          \
-        msg->hdr.origin = (m)->origin;                                  \
-        msg->hdr.dst = (m)->dst;                                        \
-        msg->hdr.type = MCA_OOB_TCP_USER;                               \
-        msg->hdr.tag = (m)->tag;                                        \
+        _s->hdr.origin = (m)->origin;                                  \
+        _s->hdr.dst = (m)->dst;                                        \
+        _s->hdr.type = MCA_OOB_TCP_USER;                               \
+        _s->hdr.tag = (m)->tag;                                        \
+        _s->hdr.seq_num = (m)->seq_num;                                \
+        if (NULL != (m)->routed) {                                      \
+            (void)strncpy(_s->hdr.routed, (m)->routed,                 \
+                          ORTE_MAX_RTD_SIZE);                           \
+        }                                                               \
         /* point to the actual message */                               \
-        msg->msg = (m);                                                 \
+        _s->msg = (m);                                                 \
         /* set the total number of bytes to be sent */                  \
         if (NULL != (m)->buffer) {                                      \
-            msg->hdr.nbytes = (m)->buffer->bytes_used;                  \
+            _s->hdr.nbytes = (m)->buffer->bytes_used;                  \
         } else if (NULL != (m)->iov) {                                  \
-            msg->hdr.nbytes = 0;                                        \
+            _s->hdr.nbytes = 0;                                        \
             for (i=0; i < (m)->count; i++) {                            \
-                msg->hdr.nbytes += (m)->iov[i].iov_len;                 \
+                _s->hdr.nbytes += (m)->iov[i].iov_len;                 \
             }                                                           \
         } else {                                                        \
-            msg->hdr.nbytes = (m)->count;                               \
+            _s->hdr.nbytes = (m)->count;                               \
         }                                                               \
         /* prep header for xmission */                                  \
-        MCA_OOB_TCP_HDR_HTON(&msg->hdr);                                \
+        MCA_OOB_TCP_HDR_HTON(&_s->hdr);                                \
         /* start the send with the header */                            \
-        msg->sdptr = (char*)&msg->hdr;                                  \
-        msg->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
+        _s->sdptr = (char*)&_s->hdr;                                  \
+        _s->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
         /* add to the msg queue for this peer */                        \
-        MCA_OOB_TCP_QUEUE_MSG((p), msg, true);                          \
-    }while(0);
+        MCA_OOB_TCP_QUEUE_MSG((p), _s, true);                          \
+    } while(0)
 
 /* queue a message to be sent by one of our modules upon completing
  * the connection process - must provide the following params:
@@ -147,40 +142,45 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_recv_t);
  */
 #define MCA_OOB_TCP_QUEUE_PENDING(m, p)                                 \
     do {                                                                \
-        mca_oob_tcp_send_t *msg;                                        \
+        mca_oob_tcp_send_t *_s;                                        \
         int i;                                                          \
         opal_output_verbose(5, orte_oob_base_framework.framework_output, \
                             "%s:[%s:%d] queue pending to %s",           \
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
                             __FILE__, __LINE__,                         \
                             ORTE_NAME_PRINT(&((m)->dst)));              \
-        msg = OBJ_NEW(mca_oob_tcp_send_t);                              \
+        _s = OBJ_NEW(mca_oob_tcp_send_t);                              \
         /* setup the header */                                          \
-        msg->hdr.origin = (m)->origin;                                  \
-        msg->hdr.dst = (m)->dst;                                        \
-        msg->hdr.type = MCA_OOB_TCP_USER;                               \
-        msg->hdr.tag = (m)->tag;                                        \
+        _s->hdr.origin = (m)->origin;                                  \
+        _s->hdr.dst = (m)->dst;                                        \
+        _s->hdr.type = MCA_OOB_TCP_USER;                               \
+        _s->hdr.tag = (m)->tag;                                        \
+        _s->hdr.seq_num = (m)->seq_num;                                \
+        if (NULL != (m)->routed) {                                      \
+            (void)strncpy(_s->hdr.routed, (m)->routed,                 \
+                          ORTE_MAX_RTD_SIZE);                           \
+        }                                                               \
         /* point to the actual message */                               \
-        msg->msg = (m);                                                 \
+        _s->msg = (m);                                                 \
         /* set the total number of bytes to be sent */                  \
         if (NULL != (m)->buffer) {                                      \
-            msg->hdr.nbytes = (m)->buffer->bytes_used;                  \
+            _s->hdr.nbytes = (m)->buffer->bytes_used;                  \
         } else if (NULL != (m)->iov) {                                  \
-            msg->hdr.nbytes = 0;                                        \
+            _s->hdr.nbytes = 0;                                        \
             for (i=0; i < (m)->count; i++) {                            \
-                msg->hdr.nbytes += (m)->iov[i].iov_len;                 \
+                _s->hdr.nbytes += (m)->iov[i].iov_len;                 \
             }                                                           \
         } else {                                                        \
-            msg->hdr.nbytes = (m)->count;                               \
+            _s->hdr.nbytes = (m)->count;                               \
         }                                                               \
         /* prep header for xmission */                                  \
-        MCA_OOB_TCP_HDR_HTON(&msg->hdr);                                \
+        MCA_OOB_TCP_HDR_HTON(&_s->hdr);                                \
         /* start the send with the header */                            \
-        msg->sdptr = (char*)&msg->hdr;                                  \
-        msg->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
+        _s->sdptr = (char*)&_s->hdr;                                  \
+        _s->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
         /* add to the msg queue for this peer */                        \
-        MCA_OOB_TCP_QUEUE_MSG((p), msg, false);                         \
-    }while(0);
+        MCA_OOB_TCP_QUEUE_MSG((p), _s, false);                         \
+    } while(0)
 
 /* queue a message for relay by one of our modules - must
  * provide the following params:
@@ -190,30 +190,32 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_recv_t);
 */
 #define MCA_OOB_TCP_QUEUE_RELAY(m, p)                                   \
     do {                                                                \
-        mca_oob_tcp_send_t *msg;                                        \
+        mca_oob_tcp_send_t *_s;                                        \
         opal_output_verbose(5, orte_oob_base_framework.framework_output, \
                             "%s:[%s:%d] queue relay to %s",             \
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
                             __FILE__, __LINE__,                         \
                             ORTE_NAME_PRINT(&((p)->name)));             \
-        msg = OBJ_NEW(mca_oob_tcp_send_t);                              \
+        _s = OBJ_NEW(mca_oob_tcp_send_t);                              \
         /* setup the header */                                          \
-        msg->hdr.origin = (m)->hdr.origin;                              \
-        msg->hdr.dst = (m)->hdr.dst;                                    \
-        msg->hdr.type = MCA_OOB_TCP_USER;                               \
-        msg->hdr.tag = (m)->hdr.tag;                                    \
+        _s->hdr.origin = (m)->hdr.origin;                              \
+        _s->hdr.dst = (m)->hdr.dst;                                    \
+        _s->hdr.type = MCA_OOB_TCP_USER;                               \
+        _s->hdr.tag = (m)->hdr.tag;                                    \
+        (void)strncpy(_s->hdr.routed, (m)->hdr.routed,                 \
+                      ORTE_MAX_RTD_SIZE);                               \
         /* point to the actual message */                               \
-        msg->data = (m)->data;                                          \
+        _s->data = (m)->data;                                          \
         /* set the total number of bytes to be sent */                  \
-        msg->hdr.nbytes = (m)->hdr.nbytes;                              \
+        _s->hdr.nbytes = (m)->hdr.nbytes;                              \
         /* prep header for xmission */                                  \
-        MCA_OOB_TCP_HDR_HTON(&msg->hdr);                                \
+        MCA_OOB_TCP_HDR_HTON(&_s->hdr);                                \
         /* start the send with the header */                            \
-        msg->sdptr = (char*)&msg->hdr;                                  \
-        msg->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
+        _s->sdptr = (char*)&_s->hdr;                                  \
+        _s->sdbytes = sizeof(mca_oob_tcp_hdr_t);                       \
         /* add to the msg queue for this peer */                        \
-        MCA_OOB_TCP_QUEUE_MSG((p), msg, true);                          \
-    }while(0);
+        MCA_OOB_TCP_QUEUE_MSG((p), _s, true);                          \
+    } while(0)
 
 /* State machine for processing message */
 typedef struct {
@@ -233,10 +235,8 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_msg_op_t);
                             ORTE_NAME_PRINT(&((ms)->dst)));             \
         mop = OBJ_NEW(mca_oob_tcp_msg_op_t);                            \
         mop->msg = (ms);                                                \
-        opal_event_set(mca_oob_tcp_module.ev_base, &mop->ev, -1,        \
-                       OPAL_EV_WRITE, (cbfunc), mop);                   \
-        opal_event_set_priority(&mop->ev, ORTE_MSG_PRI);                \
-        opal_event_active(&mop->ev, OPAL_EV_WRITE, 1);                  \
+        ORTE_THREADSHIFT(mop, (ms)->peer->ev_base,                      \
+                         (cbfunc), ORTE_MSG_PRI);                       \
     } while(0);
 
 typedef struct {
@@ -281,28 +281,9 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_msg_error_t);
         mop->hop.jobid = (h)->jobid;                                    \
         mop->hop.vpid = (h)->vpid;                                      \
         /* this goes to the OOB framework, so use that event base */    \
-        opal_event_set(orte_event_base, &mop->ev, -1,                   \
-                       OPAL_EV_WRITE, (cbfunc), mop);                   \
-        opal_event_set_priority(&mop->ev, ORTE_MSG_PRI);                \
-        opal_event_active(&mop->ev, OPAL_EV_WRITE, 1);                  \
-    } while(0);
-
-#define ORTE_ACTIVATE_TCP_POST_RESEND(mop, cbfunc)                      \
-    do {                                                                \
-        mca_oob_tcp_msg_error_t *mp;                                    \
-        opal_output_verbose(5, orte_oob_base_framework.framework_output, \
-                            "%s:[%s:%d] post resend to %s",             \
-                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
-                            __FILE__, __LINE__,                         \
-                            ORTE_NAME_PRINT(&((mop)->hop)));            \
-        mp = OBJ_NEW(mca_oob_tcp_msg_error_t);                          \
-        mp->snd = (mop)->snd;                                           \
-        mp->hop = (mop)->hop;                                           \
-        opal_event_set(mca_oob_tcp_module.ev_base, &mp->ev, -1,         \
-                       OPAL_EV_WRITE, (cbfunc), mp);                    \
-        opal_event_set_priority(&mp->ev, ORTE_MSG_PRI);                 \
-        opal_event_active(&mp->ev, OPAL_EV_WRITE, 1);                   \
-    } while(0);
+        ORTE_THREADSHIFT(mop, orte_oob_base.ev_base,                    \
+                         (cbfunc), ORTE_MSG_PRI);                       \
+    } while(0)
 
 #define ORTE_ACTIVATE_TCP_NO_ROUTE(r, h, c)                             \
     do {                                                                \
@@ -316,11 +297,10 @@ OBJ_CLASS_DECLARATION(mca_oob_tcp_msg_error_t);
         mop->rmsg = (r);                                                \
         mop->hop.jobid = (h)->jobid;                                    \
         mop->hop.vpid = (h)->vpid;                                      \
-        /* this goes to the OOB framework, so use that event base */    \
-        opal_event_set(orte_event_base, &mop->ev, -1,                   \
-                       OPAL_EV_WRITE, (c), mop);                        \
-        opal_event_set_priority(&mop->ev, ORTE_MSG_PRI);                \
-        opal_event_active(&mop->ev, OPAL_EV_WRITE, 1);                  \
-    } while(0);
+        /* this goes to the component, so use the framework             \
+         * event base */                                                \
+        ORTE_THREADSHIFT(mop, orte_oob_base.ev_base,                    \
+                         (c), ORTE_MSG_PRI);                            \
+    } while(0)
 
 #endif /* _MCA_OOB_TCP_SENDRECV_H_ */
