@@ -4,6 +4,7 @@
  *                         reserved.
  *               2014      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2015-2016 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -106,8 +107,6 @@ static int mca_oob_ud_send_self (orte_rml_send_t *msg)
 
     req->rml_msg->status = ORTE_SUCCESS;
 
-    ORTE_RML_SEND_COMPLETE(req->rml_msg);
-
     return size;
 }
 
@@ -130,7 +129,7 @@ int mca_oob_ud_process_send_nb(int fd, short args, void *cbdata)
     }
 
     /* if we have a route to this peer, then we can reach it */
-    hop = orte_routed.get_route(&op->msg->dst);
+    hop = orte_routed.get_route(NULL, &op->msg->dst);
     if (ORTE_JOBID_INVALID == hop.jobid ||
         ORTE_VPID_INVALID == hop.vpid) {
         ORTE_ERROR_LOG(ORTE_ERR_UNREACH);
@@ -161,6 +160,7 @@ int mca_oob_ud_process_send_nb(int fd, short args, void *cbdata)
     send_req->req_target = op->msg->dst;
     send_req->req_origin = op->msg->origin;
     send_req->req_tag    = op->msg->tag;
+    send_req->req_seq_num  = op->msg->seq_num;
 
     if (op->msg->data != NULL) {
         size = op->msg->count;
@@ -227,6 +227,7 @@ int mca_oob_ud_process_send_nb(int fd, short args, void *cbdata)
 
     req_msg->hdr->msg_origin   = op->msg->origin;
     req_msg->hdr->msg_target   = op->msg->dst;
+    req_msg->hdr->msg_seq_num  = op->msg->seq_num;
 
     req_msg->hdr->msg_data.req.data_len = size;
     req_msg->hdr->msg_data.req.mtu      = port->mtu;
