@@ -13,7 +13,7 @@
  * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -55,13 +55,14 @@
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
-#include "orte/util/show_help.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/path.h"
 #include "opal/util/basename.h"
 
 #include "orte/runtime/orte_globals.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/show_help.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rmaps/rmaps.h"
@@ -185,6 +186,9 @@ static void launch_daemons(int fd, short args, void *cbdata)
     orte_std_cntr_t nnode;
     orte_job_t *daemons;
     orte_state_caddy_t *state = (orte_state_caddy_t*)cbdata;
+    char *ltmp;
+
+    ORTE_ACQUIRE_OBJECT(state);
 
     /* if we are launching debugger daemons, then just go
      * do it - no new daemons will be launched
@@ -288,41 +292,42 @@ static void launch_daemons(int fd, short args, void *cbdata)
     opal_argv_append(&argc, &argv, "-e");
     opal_argv_append(&argc, &argv, "OMPI_NO_USE_CRAY_PMI=1");
 
-    /* create nodelist */
-    nodelist_argv = NULL;
-    nodelist_argc = 0;
-
-    for (nnode=0; nnode < map->nodes->size; nnode++) {
-        if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, nnode))) {
-            continue;
-        }
-
-        /* if the daemon already exists on this node, then
-         * don't include it
-         */
-        if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_DAEMON_LAUNCHED)) {
-            continue;
-        }
-
-        /* otherwise, add it to the list of nodes upon which
-         * we need to launch a daemon
-         */
-        opal_argv_append(&nodelist_argc, &nodelist_argv, node->name);
-    }
-    if (0 == opal_argv_count(nodelist_argv)) {
-        orte_show_help("help-plm-alps.txt", "no-hosts-in-list", true);
-        rc = ORTE_ERR_FAILED_TO_START;
-        goto cleanup;
-    }
-    nodelist_flat = opal_argv_join(nodelist_argv, ',');
-    opal_argv_free(nodelist_argv);
-
     /* if we are using all allocated nodes, then alps
      * doesn't need a nodelist, or if running without a batch scheduler
      */
     if ((map->num_new_daemons < orte_num_allocated_nodes) || (orte_num_allocated_nodes == 0)) {
+        /* create nodelist */
+        nodelist_argv = NULL;
+        nodelist_argc = 0;
+
+        for (nnode=0; nnode < map->nodes->size; nnode++) {
+            if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, nnode))) {
+                continue;
+            }
+
+            /* if the daemon already exists on this node, then
+             * don't include it
+             */
+            if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_DAEMON_LAUNCHED)) {
+                continue;
+            }
+
+            /* otherwise, add it to the list of nodes upon which
+             * we need to launch a daemon
+             */
+            opal_argv_append(&nodelist_argc, &nodelist_argv, node->name);
+        }
+        if (0 == opal_argv_count(nodelist_argv)) {
+            orte_show_help("help-plm-alps.txt", "no-hosts-in-list", true);
+            rc = ORTE_ERR_FAILED_TO_START;
+            goto cleanup;
+        }
+        nodelist_flat = opal_argv_join(nodelist_argv, ',');
+        opal_argv_free(nodelist_argv);
+
         opal_argv_append(&argc, &argv, "-L");
         opal_argv_append(&argc, &argv, nodelist_flat);
+        free(nodelist_flat);
     }
 
 
@@ -336,9 +341,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
     /* Add basic orted command line options, including debug flags */
     orte_plm_base_orted_append_basic_args(&argc, &argv,
                                           NULL,
-                                          &proc_vpid_index,
-                                          nodelist_flat);
-    free(nodelist_flat);
+                                          &proc_vpid_index);
 
     /* tell the new daemons the base of the name list so they can compute
      * their own name on the other end
