@@ -12,8 +12,8 @@
  * Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -66,7 +66,7 @@ static void reset_usage(orte_node_t *node, orte_jobid_t jobid)
                         node->name, node->num_procs);
 
     /* start by clearing any existing info */
-    opal_hwloc_base_clear_usage(node->topology);
+    opal_hwloc_base_clear_usage(node->topology->topo);
 
     /* cycle thru the procs on the node and record
      * their usage in the topology
@@ -133,7 +133,6 @@ static int bind_upwards(orte_job_t *jdata,
     orte_job_map_t *map;
     orte_proc_t *proc;
     hwloc_obj_t obj;
-    hwloc_cpuset_t cpus;
     unsigned int idx, ncpus;
     opal_hwloc_obj_data_t *data;
     hwloc_obj_t locale;
@@ -172,11 +171,13 @@ static int bind_upwards(orte_job_t *jdata,
                                 hwloc_obj_type_string(target),
                                 hwloc_obj_type_string(obj->type));
             if (target == obj->type) {
+#if HWLOC_API_VERSION < 0x20000
                 if (HWLOC_OBJ_CACHE == target && cache_level != obj->attr->cache.depth) {
                     continue;
                 }
+#endif
                 /* get its index */
-                if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology, obj, OPAL_HWLOC_AVAILABLE))) {
+                if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology->topo, obj, OPAL_HWLOC_AVAILABLE))) {
                     ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
                     return ORTE_ERR_SILENT;
                 }
@@ -184,7 +185,7 @@ static int bind_upwards(orte_job_t *jdata,
                 data = (opal_hwloc_obj_data_t*)obj->userdata;
                 data->num_bound++;
                 /* get the number of cpus under this location */
-                if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology, obj))) {
+                if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology->topo, obj))) {
                     orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                     return ORTE_ERR_SILENT;
                 }
@@ -210,8 +211,7 @@ static int bind_upwards(orte_job_t *jdata,
                     }
                 }
                 /* bind it here */
-                cpus = opal_hwloc_base_get_available_cpus(node->topology, obj);
-                hwloc_bitmap_list_asprintf(&cpu_bitmap, cpus);
+                hwloc_bitmap_list_asprintf(&cpu_bitmap, obj->cpuset);
                 orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
                 /* record the location */
                 orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
@@ -250,7 +250,6 @@ static int bind_downwards(orte_job_t *jdata,
     orte_job_map_t *map;
     orte_proc_t *proc;
     hwloc_obj_t trg_obj, nxt_obj;
-    hwloc_cpuset_t cpus;
     unsigned int ncpus;
     opal_hwloc_obj_data_t *data;
     int total_cpus;
@@ -287,7 +286,7 @@ static int bind_downwards(orte_job_t *jdata,
          * or if it is some depth below it, so we have to conduct a bit
          * of a search. Let hwloc find the min usage one for us.
          */
-        trg_obj = opal_hwloc_base_find_min_bound_target_under_obj(node->topology, locale,
+        trg_obj = opal_hwloc_base_find_min_bound_target_under_obj(node->topology->topo, locale,
                                                                   target, cache_level);
         if (NULL == trg_obj) {
             /* there aren't any such targets under this object */
@@ -310,7 +309,7 @@ static int bind_downwards(orte_job_t *jdata,
             }
             trg_obj = nxt_obj;
             /* get the number of cpus under this location */
-            ncpus = opal_hwloc_base_get_npus(node->topology, trg_obj);
+            ncpus = opal_hwloc_base_get_npus(node->topology->topo, trg_obj);
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "%s GOT %d CPUS",
                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ncpus);
@@ -344,8 +343,7 @@ static int bind_downwards(orte_job_t *jdata,
                 }
             }
             /* bind the proc here */
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, trg_obj);
-            hwloc_bitmap_or(totalcpuset, totalcpuset, cpus);
+            hwloc_bitmap_or(totalcpuset, totalcpuset, trg_obj->cpuset);
             /* track total #cpus */
             total_cpus += ncpus;
             /* move to the next location, in case we need it */
@@ -363,13 +361,13 @@ static int bind_downwards(orte_job_t *jdata,
         if (4 < opal_output_get_verbosity(orte_rmaps_base_framework.framework_output)) {
             char tmp1[1024], tmp2[1024];
             if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1),
-                                                               node->topology, totalcpuset)) {
+                                                               node->topology->topo, totalcpuset)) {
                 opal_output(orte_rmaps_base_framework.framework_output,
                             "%s PROC %s ON %s IS NOT BOUND",
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                             ORTE_NAME_PRINT(&proc->name), node->name);
             } else {
-                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), node->topology, totalcpuset);
+                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), node->topology->topo, totalcpuset);
                 opal_output(orte_rmaps_base_framework.framework_output,
                             "%s BOUND PROC %s[%s] TO %s: %s",
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -395,7 +393,6 @@ static int bind_in_place(orte_job_t *jdata,
     orte_job_map_t *map;
     orte_node_t *node;
     orte_proc_t *proc;
-    hwloc_cpuset_t cpus;
     unsigned int idx, ncpus;
     struct hwloc_topology_support *support;
     opal_hwloc_obj_data_t *data;
@@ -414,11 +411,14 @@ static int bind_in_place(orte_job_t *jdata,
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -461,7 +461,7 @@ static int bind_in_place(orte_job_t *jdata,
          * on this node, just silently skip it - we will not bind
          */
         if (!OPAL_BINDING_POLICY_IS_SET(map->binding) &&
-            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology->topo, HWLOC_OBJ_CORE)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "Unable to bind-to core by default on node %s as no cores detected",
                                 node->name);
@@ -484,18 +484,19 @@ static int bind_in_place(orte_job_t *jdata,
                 continue;
             }
             /* bozo check */
+            locale = NULL;
             if (!orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, (void**)&locale, OPAL_PTR)) {
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-locale", true, ORTE_NAME_PRINT(&proc->name));
                 return ORTE_ERR_SILENT;
             }
             /* get the index of this location */
-            if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology, locale, OPAL_HWLOC_AVAILABLE))) {
+            if (UINT_MAX == (idx = opal_hwloc_base_get_obj_idx(node->topology->topo, locale, OPAL_HWLOC_AVAILABLE))) {
                 ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
                 return ORTE_ERR_SILENT;
             }
             data = (opal_hwloc_obj_data_t*)locale->userdata;
             /* get the number of cpus under this location */
-            if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology, locale))) {
+            if (0 == (ncpus = opal_hwloc_base_get_npus(node->topology->topo, locale))) {
                 orte_show_help("help-orte-rmaps-base.txt", "rmaps:no-available-cpus", true, node->name);
                 return ORTE_ERR_SILENT;
             }
@@ -510,7 +511,7 @@ static int bind_in_place(orte_job_t *jdata,
                 found = false;
                 while (NULL != (sib = sib->next_cousin)) {
                     data = (opal_hwloc_obj_data_t*)sib->userdata;
-                    ncpus = opal_hwloc_base_get_npus(node->topology, sib);
+                    ncpus = opal_hwloc_base_get_npus(node->topology->topo, sib);
                     if (data->num_bound < ncpus) {
                         found = true;
                         locale = sib;
@@ -525,7 +526,7 @@ static int bind_in_place(orte_job_t *jdata,
                     sib = locale;
                     while (NULL != (sib = sib->prev_cousin)) {
                         data = (opal_hwloc_obj_data_t*)sib->userdata;
-                        ncpus = opal_hwloc_base_get_npus(node->topology, sib);
+                        ncpus = opal_hwloc_base_get_npus(node->topology->topo, sib);
                         if (data->num_bound < ncpus) {
                             found = true;
                             locale = sib;
@@ -562,8 +563,7 @@ static int bind_in_place(orte_job_t *jdata,
                                 ORTE_NAME_PRINT(&proc->name),
                                 hwloc_obj_type_string(locale->type), idx);
             /* bind the proc here */
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, locale);
-            hwloc_bitmap_list_asprintf(&cpu_bitmap, cpus);
+            hwloc_bitmap_list_asprintf(&cpu_bitmap, locale->cpuset);
             orte_set_attribute(&proc->attributes, ORTE_PROC_CPU_BITMAP, ORTE_ATTR_GLOBAL, cpu_bitmap, OPAL_STRING);
             /* update the location, in case it changed */
             orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_BOUND, ORTE_ATTR_LOCAL, locale, OPAL_PTR);
@@ -584,7 +584,7 @@ static int bind_in_place(orte_job_t *jdata,
 
 static int bind_to_cpuset(orte_job_t *jdata)
 {
-    /* bind each process to opal_hwloc_base_cpu_set */
+    /* bind each process to opal_hwloc_base_cpu_list */
     int i, j;
     orte_job_map_t *map;
     orte_node_t *node;
@@ -595,9 +595,9 @@ static int bind_to_cpuset(orte_job_t *jdata)
     char *cpu_bitmap;
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                        "mca:rmaps: bind job %s to cpuset %s",
+                        "mca:rmaps: bind job %s to cpus %s",
                         ORTE_JOBID_PRINT(jdata->jobid),
-                        opal_hwloc_base_cpu_set);
+                        opal_hwloc_base_cpu_list);
     /* initialize */
     map = jdata->map;
 
@@ -605,11 +605,14 @@ static int bind_to_cpuset(orte_job_t *jdata)
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -642,7 +645,7 @@ static int bind_to_cpuset(orte_job_t *jdata)
                 }
             }
         }
-        root = hwloc_get_root_obj(node->topology);
+        root = hwloc_get_root_obj(node->topology->topo);
         if (NULL == root->userdata) {
             /* something went wrong */
             ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
@@ -725,16 +728,13 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         hwb = HWLOC_OBJ_SOCKET;
         break;
     case OPAL_BIND_TO_L3CACHE:
-        hwb = HWLOC_OBJ_CACHE;
-        clvl = 3;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(3, hwb, clvl);
         break;
     case OPAL_BIND_TO_L2CACHE:
-        hwb = HWLOC_OBJ_CACHE;
-        clvl = 2;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(2, hwb, clvl);
         break;
     case OPAL_BIND_TO_L1CACHE:
-        hwb = HWLOC_OBJ_CACHE;
-        clvl = 1;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(1, hwb, clvl);
         break;
     case OPAL_BIND_TO_CORE:
         hwb = HWLOC_OBJ_CORE;
@@ -762,16 +762,13 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
         hwm = HWLOC_OBJ_SOCKET;
         break;
     case ORTE_MAPPING_BYL3CACHE:
-        hwm = HWLOC_OBJ_CACHE;
-        clvm = 3;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(3, hwm, clvm);
         break;
     case ORTE_MAPPING_BYL2CACHE:
-        hwm = HWLOC_OBJ_CACHE;
-        clvm = 2;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(2, hwm, clvm);
         break;
     case ORTE_MAPPING_BYL1CACHE:
-        hwm = HWLOC_OBJ_CACHE;
-        clvm = 1;
+        OPAL_HWLOC_MAKE_OBJ_CACHE(1, hwm, clvm);
         break;
     case ORTE_MAPPING_BYCORE:
         hwm = HWLOC_OBJ_CORE;
@@ -834,18 +831,24 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
      * basis because different nodes could potentially have different
      * topologies, with different relative depths for the two levels
      */
- execute:
+  execute:
     /* initialize */
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "mca:rmaps: computing bindings for job %s",
+                        ORTE_JOBID_PRINT(jdata->jobid));
 
     for (i=0; i < jdata->map->nodes->size; i++) {
         if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, i))) {
             continue;
         }
+        if (!orte_no_vm && (int)ORTE_PROC_MY_NAME->vpid != node->index) {
+            continue;
+        }
         if (!orte_do_not_launch) {
             /* if we don't want to launch, then we are just testing the system,
              * so ignore questions about support capabilities
              */
-            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology);
+            support = (struct hwloc_topology_support*)hwloc_topology_get_support(node->topology->topo);
             /* check if topology supports cpubind - have to be careful here
              * as Linux doesn't currently support thread-level binding. This
              * may change in the future, though, and it isn't clear how hwloc
@@ -888,7 +891,7 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
          * on this node, just silently skip it - we will not bind
          */
         if (!OPAL_BINDING_POLICY_IS_SET(jdata->map->binding) &&
-            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology, HWLOC_OBJ_CORE)) {
+            HWLOC_TYPE_DEPTH_UNKNOWN == hwloc_get_type_depth(node->topology->topo, HWLOC_OBJ_CORE)) {
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "Unable to bind-to core by default on node %s as no cores detected",
                                 node->name);
@@ -908,28 +911,33 @@ int orte_rmaps_base_compute_bindings(orte_job_t *jdata)
             }
         } else {
             /* determine the relative depth on this node */
+#if HWLOC_API_VERSION < 0x20000
             if (HWLOC_OBJ_CACHE == hwb) {
                 /* must use a unique function because blasted hwloc
                  * just doesn't deal with caches very well...sigh
                  */
-                bind_depth = hwloc_get_cache_type_depth(node->topology, clvl, -1);
-            } else {
-                bind_depth = hwloc_get_type_depth(node->topology, hwb);
-            }
+                bind_depth = hwloc_get_cache_type_depth(node->topology->topo, clvl, (hwloc_obj_cache_type_t)-1);
+            } else
+#endif
+                bind_depth = hwloc_get_type_depth(node->topology->topo, hwb);
             if (0 > bind_depth) {
                 /* didn't find such an object */
                 orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:no-objects",
                                true, hwloc_obj_type_string(hwb), node->name);
                 return ORTE_ERR_SILENT;
             }
+#if HWLOC_API_VERSION < 0x20000
             if (HWLOC_OBJ_CACHE == hwm) {
                 /* must use a unique function because blasted hwloc
                  * just doesn't deal with caches very well...sigh
                  */
-                map_depth = hwloc_get_cache_type_depth(node->topology, clvm, -1);
-            } else {
-                map_depth = hwloc_get_type_depth(node->topology, hwm);
-            }
+                map_depth = hwloc_get_cache_type_depth(node->topology->topo, clvm, (hwloc_obj_cache_type_t)-1);
+            } else
+#else
+                /* do something with clvm to silence compiler warnings */
+                ++clvm;
+#endif
+                map_depth = hwloc_get_type_depth(node->topology->topo, hwm);
             if (0 > map_depth) {
                 /* didn't find such an object */
                 orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:no-objects",
