@@ -12,8 +12,10 @@
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -140,8 +142,8 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
                                      orte_app_context_t *app, orte_mapping_policy_t policy,
                                      bool initial_map, bool silent)
 {
-    opal_list_item_t *item, *next;
-    orte_node_t *node, *nd, *nptr;
+    opal_list_item_t *item;
+    orte_node_t *node, *nd, *nptr, *next;
     orte_std_cntr_t num_slots;
     orte_std_cntr_t i;
     int rc;
@@ -167,6 +169,7 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         /* if the app provided a dash-host, and we are not treating
          * them as requested or "soft" locations, then use those nodes
          */
+        hosts = NULL;
         if (!orte_soft_locations &&
             orte_get_attribute(&app->attributes, ORTE_APP_DASH_HOST, (void**)&hosts, OPAL_STRING)) {
             OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
@@ -252,13 +255,12 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
         /* find the nodes in our node array and assemble them
          * in daemon order if the vm was launched
          */
-        while (NULL != (item = opal_list_remove_first(&nodes))) {
-            nptr = (orte_node_t*)item;
+        for (i=0; i < orte_node_pool->size; i++) {
             nd = NULL;
-            for (i=0; i < orte_node_pool->size; i++) {
-                if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
-                    continue;
-                }
+            if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, i))) {
+                continue;
+            }
+            OPAL_LIST_FOREACH_SAFE(nptr, next, &nodes, orte_node_t) {
                 if (0 != strcmp(node->name, nptr->name)) {
                     OPAL_OUTPUT_VERBOSE((10, orte_rmaps_base_framework.framework_output,
                                          "NODE %s DOESNT MATCH NODE %s",
@@ -331,37 +333,16 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
                     /* reset us back to the end for the next node */
                     nd = (orte_node_t*)opal_list_get_last(allocated_nodes);
                 }
+                opal_list_remove_item(&nodes, (opal_list_item_t*)nptr);
+                OBJ_RELEASE(nptr);
             }
-            OBJ_RELEASE(nptr);
         }
         OBJ_DESTRUCT(&nodes);
         /* now prune for usage and compute total slots */
         goto complete;
     }
 
- addknown:
-    /* if the hnp was allocated, include it unless flagged not to */
-    if (orte_hnp_is_allocated && !(ORTE_GET_MAPPING_DIRECTIVE(policy) & ORTE_MAPPING_NO_USE_LOCAL)) {
-        if (NULL != (node = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0))) {
-            if (ORTE_NODE_STATE_DO_NOT_USE == node->state) {
-                OPAL_OUTPUT_VERBOSE((10, orte_rmaps_base_framework.framework_output,
-                                     "HNP IS MARKED NO_USE"));
-                /* clear this for future use, but don't include it */
-                node->state = ORTE_NODE_STATE_UP;
-            } else if (ORTE_NODE_STATE_NOT_INCLUDED != node->state) {
-                OBJ_RETAIN(node);
-                if (initial_map) {
-                    /* if this is the first app_context we
-                     * are getting for an initial map of a job,
-                     * then mark all nodes as unmapped
-                     */
-                    ORTE_FLAG_UNSET(node, ORTE_NODE_FLAG_MAPPED);
-                }
-                opal_list_append(allocated_nodes, &node->super);
-            }
-        }
-    }
-
+  addknown:
     /* add everything in the node pool that can be used - add them
      * in daemon order, which may be different than the order in the
      * node pool. Since an empty list is passed into us, the list at
@@ -369,8 +350,14 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
      * node obviously has a daemon on it (us!)
      */
     if (0 == opal_list_get_size(allocated_nodes)) {
-        /* the list is empty */
-        nd = NULL;
+        /* the list is empty - if the HNP is allocated, then add it */
+        if (orte_hnp_is_allocated) {
+            nd = (orte_node_t*)opal_pointer_array_get_item(orte_node_pool, 0);
+            OBJ_RETAIN(nd);
+            opal_list_append(allocated_nodes, &nd->super);
+        } else {
+            nd = NULL;
+        }
     } else {
         nd = (orte_node_t*)opal_list_get_last(allocated_nodes);
     }
@@ -412,10 +399,10 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
                  * are getting for an initial map of a job,
                  * then mark all nodes as unmapped
                  */
-                    ORTE_FLAG_UNSET(node, ORTE_NODE_FLAG_MAPPED);
+                ORTE_FLAG_UNSET(node, ORTE_NODE_FLAG_MAPPED);
             }
             if (NULL == nd || NULL == nd->daemon ||
-		NULL == node->daemon ||
+                NULL == node->daemon ||
                 nd->daemon->name.vpid < node->daemon->name.vpid) {
                 /* just append to end */
                 opal_list_append(allocated_nodes, &node->super);
@@ -475,56 +462,71 @@ int orte_rmaps_base_get_target_nodes(opal_list_t *allocated_nodes, orte_std_cntr
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          (int)opal_list_get_size(allocated_nodes)));
 
- complete:
+  complete:
+    num_slots = 0;
     /* remove all nodes that are already at max usage, and
      * compute the total number of allocated slots while
-     * we do so */
-    num_slots = 0;
-    item  = opal_list_get_first(allocated_nodes);
-    while (item != opal_list_get_end(allocated_nodes)) {
-        /** save the next pointer in case we remove this node */
-        next  = opal_list_get_next(item);
-        /** check to see if this node is fully used - remove if so */
-        node = (orte_node_t*)item;
-        if (0 != node->slots_max && node->slots_inuse > node->slots_max) {
-            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
-                                 "%s Removing node %s: max %d inuse %d",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 node->name, node->slots_max, node->slots_inuse));
-            opal_list_remove_item(allocated_nodes, item);
-            OBJ_RELEASE(item);  /* "un-retain" it */
-        } else if (node->slots <= node->slots_inuse &&
-                   (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(policy))) {
-            /* remove the node as fully used */
-            OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
-                                 "%s Removing node %s slots %d inuse %d",
-                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 node->name, node->slots, node->slots_inuse));
-            opal_list_remove_item(allocated_nodes, item);
-            OBJ_RELEASE(item);  /* "un-retain" it */
-        } else if (node->slots > node->slots_inuse) {
+     * we do so - can ignore this if we are mapping debugger
+     * daemons as they do not count against the allocation */
+    if (ORTE_MAPPING_DEBUGGER & ORTE_GET_MAPPING_DIRECTIVE(policy)) {
+        num_slots = opal_list_get_size(allocated_nodes);    // tell the mapper there is one slot/node for debuggers
+    } else {
+        item  = opal_list_get_first(allocated_nodes);
+        OPAL_LIST_FOREACH_SAFE(node, next, allocated_nodes, orte_node_t) {
+            /* if the hnp was not allocated, or flagged not to be used,
+             * then remove it here */
+            if (!orte_hnp_is_allocated || (ORTE_GET_MAPPING_DIRECTIVE(policy) & ORTE_MAPPING_NO_USE_LOCAL)) {
+                if (0 == node->index) {
+                    opal_list_remove_item(allocated_nodes, &node->super);
+                    OBJ_RELEASE(node);  /* "un-retain" it */
+                    continue;
+                }
+            }
+            /** check to see if this node is fully used - remove if so */
+            if (0 != node->slots_max && node->slots_inuse > node->slots_max) {
+                OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
+                                     "%s Removing node %s: max %d inuse %d",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                     node->name, node->slots_max, node->slots_inuse));
+                opal_list_remove_item(allocated_nodes, &node->super);
+                OBJ_RELEASE(node);  /* "un-retain" it */
+                continue;
+            }
+            if (node->slots <= node->slots_inuse &&
+                       (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(policy))) {
+                /* remove the node as fully used */
+                OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
+                                     "%s Removing node %s slots %d inuse %d",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                     node->name, node->slots, node->slots_inuse));
+                opal_list_remove_item(allocated_nodes, &node->super);
+                OBJ_RELEASE(node);  /* "un-retain" it */
+                continue;
+            }
+            if (node->slots > node->slots_inuse) {
                 /* add the available slots */
                 OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
                                      "%s node %s has %d slots available",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      node->name, node->slots - node->slots_inuse));
                 num_slots += node->slots - node->slots_inuse;
-        } else if (!(ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(policy))) {
+                continue;
+            }
+            if (!(ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(policy))) {
                 /* nothing needed to do here - we don't add slots to the
                  * count as we don't have any available. Just let the mapper
                  * do what it needs to do to meet the request
                  */
                 OPAL_OUTPUT_VERBOSE((5, orte_rmaps_base_framework.framework_output,
-                                     "%s node %s is fully used, but available for oversubscrition",
+                                     "%s node %s is fully used, but available for oversubscription",
                                      ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                                      node->name));
-        } else {
-            /* if we cannot use it, remove it from list */
-            opal_list_remove_item(allocated_nodes, item);
-            OBJ_RELEASE(item);  /* "un-retain" it */
+            } else {
+                /* if we cannot use it, remove it from list */
+                opal_list_remove_item(allocated_nodes, &node->super);
+                OBJ_RELEASE(node);  /* "un-retain" it */
+            }
         }
-        /** go on to next item */
-        item = next;
     }
 
     /* Sanity check to make sure we have resources available */
@@ -581,9 +583,13 @@ orte_proc_t* orte_rmaps_base_setup_proc(orte_job_t *jdata,
 
     OBJ_RETAIN(node);  /* maintain accounting on object */
     proc->node = node;
-    node->num_procs++;
-    if (node->slots_inuse < node->slots) {
-        node->slots_inuse += orte_rmaps_base.cpus_per_rank;
+    /* if this is a debugger job, then it doesn't count against
+     * available slots - otherwise, it does */
+    if (!ORTE_FLAG_TEST(jdata, ORTE_JOB_FLAG_DEBUGGER_DAEMON)) {
+        node->num_procs++;
+        if (node->slots_inuse < node->slots) {
+            ++node->slots_inuse;
+        }
     }
     if (0 > (rc = opal_pointer_array_add(node->procs, (void*)proc))) {
         ORTE_ERROR_LOG(rc);
