@@ -2,8 +2,8 @@
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -33,9 +33,11 @@
 #include "rmaps_ppr.h"
 
 static int ppr_mapper(orte_job_t *jdata);
+static int assign_locations(orte_job_t *jdata);
 
 orte_rmaps_base_module_t orte_rmaps_ppr_module = {
-    ppr_mapper
+    .map_job = ppr_mapper,
+    .assign_locations = assign_locations
 };
 
 /* RHC: will eventually remove this
@@ -110,8 +112,10 @@ static int ppr_mapper(orte_job_t *jdata)
         ORTE_MAPPING_PPR != ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
         /* not for us */
         opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                            "mca:rmaps:ppr: job %s not using ppr mapper",
-                            ORTE_JOBID_PRINT(jdata->jobid));
+                            "mca:rmaps:ppr: job %s not using ppr mapper PPR %s policy %s",
+                            ORTE_JOBID_PRINT(jdata->jobid),
+                            (NULL == jdata->map->ppr) ? "NULL" : jdata->map->ppr,
+                            (ORTE_MAPPING_PPR == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) ? "PPRSET" : "PPR NOTSET");
         return ORTE_ERR_TAKE_NEXT_OPTION;
     }
 
@@ -263,7 +267,7 @@ static int ppr_mapper(orte_job_t *jdata)
              item = opal_list_get_next(item)) {
             node = (orte_node_t*)item;
             /* bozo check */
-            if (NULL == node->topology) {
+            if (NULL == node->topology || NULL == node->topology->topo) {
                 orte_show_help("help-orte-rmaps-ppr.txt", "ppr-topo-missing",
                                true, node->name);
                 rc = ORTE_ERR_SILENT;
@@ -271,19 +275,16 @@ static int ppr_mapper(orte_job_t *jdata)
             }
             /* add the node to the map, if needed */
             if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-                if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                    ORTE_ERROR_LOG(rc);
-                    goto error;
-                }
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-                OBJ_RETAIN(node);  /* maintain accounting on object */
+                OBJ_RETAIN(node);
+                opal_pointer_array_add(jdata->map->nodes, node);
                 jdata->map->num_nodes++;
             }
             /* if we are mapping solely at the node level, just put
              * that many procs on this node
              */
             if (OPAL_HWLOC_NODE_LEVEL == start) {
-                obj = hwloc_get_root_obj(node->topology);
+                obj = hwloc_get_root_obj(node->topology->topo);
                 for (j=0; j < ppr[start] && nprocs_mapped < total_procs; j++) {
                     if (NULL == (proc = orte_rmaps_base_setup_proc(jdata, node, idx))) {
                         rc = ORTE_ERR_OUT_OF_RESOURCE;
@@ -294,7 +295,7 @@ static int ppr_mapper(orte_job_t *jdata)
                 }
             } else {
                 /* get the number of lowest resources on this node */
-                nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology,
+                nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology->topo,
                                                            lowest, cache_level,
                                                            OPAL_HWLOC_AVAILABLE);
 
@@ -302,7 +303,7 @@ static int ppr_mapper(orte_job_t *jdata)
                  * recording the locale of each proc so we know its cpuset
                  */
                 for (i=0; i < nobjs; i++) {
-                    obj = opal_hwloc_base_get_obj_by_type(node->topology,
+                    obj = opal_hwloc_base_get_obj_by_type(node->topology->topo,
                                                           lowest, cache_level,
                                                           i, OPAL_HWLOC_AVAILABLE);
                     for (j=0; j < ppr[start] && nprocs_mapped < total_procs; j++) {
@@ -325,46 +326,49 @@ static int ppr_mapper(orte_job_t *jdata)
                 }
             }
 
-            /* set the total slots used */
-            if ((int)node->num_procs <= node->slots) {
-                node->slots_inuse = (int)node->num_procs;
-            } else {
-                node->slots_inuse = node->slots;
-            }
-
-            /* if no-oversubscribe was specified, check to see if
-             * we have violated the total slot specification - regardless,
-             * if slots_max was given, we are not allowed to violate it!
-             */
-            if ((node->slots < (int)node->num_procs) ||
-                (0 < node->slots_max && node->slots_max < (int)node->num_procs)) {
-                if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
-                    orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                   true, node->num_procs, app->app);
-                    ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
-                    rc = ORTE_ERR_SILENT;
-                    goto error;
+            if (!(ORTE_MAPPING_DEBUGGER & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping))) {
+                /* set the total slots used */
+                if ((int)node->num_procs <= node->slots) {
+                    node->slots_inuse = (int)node->num_procs;
+                } else {
+                    node->slots_inuse = node->slots;
                 }
-                /* flag the node as oversubscribed so that sched-yield gets
-                 * properly set
+
+                /* if no-oversubscribe was specified, check to see if
+                 * we have violated the total slot specification - regardless,
+                 * if slots_max was given, we are not allowed to violate it!
                  */
-                ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
-                /* check for permission */
-                if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
-                    /* if we weren't given a directive either way, then we will error out
-                     * as the #slots were specifically given, either by the host RM or
-                     * via hostfile/dash-host */
-                    if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
+                if ((node->slots < (int)node->num_procs) ||
+                    (0 < node->slots_max && node->slots_max < (int)node->num_procs)) {
+                    if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
+                                       true, node->num_procs, app->app);
                         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
-                        return ORTE_ERR_SILENT;
-                    } else if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
-                        /* if we were explicitly told not to oversubscribe, then don't */
-                        orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
-                        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
-                        return ORTE_ERR_SILENT;
+                        rc = ORTE_ERR_SILENT;
+                        goto error;
+                    }
+                    /* flag the node as oversubscribed so that sched-yield gets
+                     * properly set
+                     */
+                    ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+                    ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
+                    /* check for permission */
+                    if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
+                        /* if we weren't given a directive either way, then we will error out
+                         * as the #slots were specifically given, either by the host RM or
+                         * via hostfile/dash-host */
+                        if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
+                            orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
+                                           true, app->num_procs, app->app);
+                            ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                            return ORTE_ERR_SILENT;
+                        } else if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
+                            /* if we were explicitly told not to oversubscribe, then don't */
+                            orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
+                                           true, app->num_procs, app->app);
+                            ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
+                            return ORTE_ERR_SILENT;
+                        }
                     }
                 }
             }
@@ -386,11 +390,6 @@ static int ppr_mapper(orte_job_t *jdata)
             rc = ORTE_ERR_SILENT;
             goto error;
         }
-        /* compute vpids and add proc objects to the job */
-        if (ORTE_SUCCESS != (rc = orte_rmaps_base_compute_vpids(jdata, app, &node_list))) {
-            ORTE_ERROR_LOG(rc);
-            goto error;
-        }
 
         /* track the total number of processes we mapped - must update
          * this AFTER we compute vpids so that computation is done
@@ -405,7 +404,7 @@ static int ppr_mapper(orte_job_t *jdata)
     }
     return ORTE_SUCCESS;
 
- error:
+  error:
     while (NULL != (item = opal_list_remove_first(&node_list))) {
         OBJ_RELEASE(item);
     }
@@ -444,7 +443,7 @@ static void prune(orte_jobid_t jobid,
     hwloc_obj_type_t lvl;
     unsigned cache_level = 0, k;
     int nprocs;
-    hwloc_cpuset_t avail, cpus, childcpus;
+    hwloc_cpuset_t avail;
     int n, limit, nmax, nunder, idx, idxmax = 0;
     orte_proc_t *proc, *pptr, *procmax;
     opal_hwloc_level_t ll;
@@ -483,7 +482,7 @@ static void prune(orte_jobid_t jobid,
     }
 
     /* get the number of resources at this level on this node */
-    nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology,
+    nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology->topo,
                                                lvl, cache_level,
                                                OPAL_HWLOC_AVAILABLE);
 
@@ -491,11 +490,11 @@ static void prune(orte_jobid_t jobid,
      * underneath it and check against the limit
      */
     for (i=0; i < nobjs; i++) {
-        obj = opal_hwloc_base_get_obj_by_type(node->topology,
+        obj = opal_hwloc_base_get_obj_by_type(node->topology->topo,
                                               lvl, cache_level,
                                               i, OPAL_HWLOC_AVAILABLE);
         /* get the available cpuset */
-        avail = opal_hwloc_base_get_available_cpus(node->topology, obj);
+        avail = obj->cpuset;
 
         /* look at the intersection of this object's cpuset and that
          * of each proc in the job/app - if they intersect, then count this proc
@@ -515,8 +514,7 @@ static void prune(orte_jobid_t jobid,
                 ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                 return;
             }
-            cpus = opal_hwloc_base_get_available_cpus(node->topology, locale);
-            if (hwloc_bitmap_intersects(avail, cpus)) {
+            if (hwloc_bitmap_intersects(avail, locale->cpuset)) {
                 nprocs++;
             }
         }
@@ -541,7 +539,7 @@ static void prune(orte_jobid_t jobid,
              * have only one child, then return this
              * object
              */
-            top = find_split(node->topology, obj);
+            top = find_split(node->topology->topo, obj);
             hwloc_obj_type_snprintf(dang, 64, top, 1);
             opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps:ppr: SPLIT AT LEVEL %s", dang);
@@ -553,7 +551,6 @@ static void prune(orte_jobid_t jobid,
             /* find the child with the most procs underneath it */
             for (k=0; k < top->arity && limit < nprocs; k++) {
                 /* get this object's available cpuset */
-                childcpus = opal_hwloc_base_get_available_cpus(node->topology, top->children[k]);
                 nunder = 0;
                 pptr = NULL;
                 for (n=0; n < node->procs->size; n++) {
@@ -569,8 +566,7 @@ static void prune(orte_jobid_t jobid,
                         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                         return;
                     }
-                    cpus = opal_hwloc_base_get_available_cpus(node->topology, locale);
-                    if (hwloc_bitmap_intersects(childcpus, cpus)) {
+                    if (hwloc_bitmap_intersects(top->children[k]->cpuset, locale->cpuset)) {
                         nunder++;
                         if (NULL == pptr) {
                             /* save the location of the first proc under this object */
@@ -618,3 +614,126 @@ static void prune(orte_jobid_t jobid,
  error:
     opal_output(0, "INFINITE LOOP");
 }
+
+static int assign_locations(orte_job_t *jdata)
+{
+    int i, j, m, n;
+    mca_base_component_t *c=&mca_rmaps_ppr_component.base_version;
+    orte_node_t *node;
+    orte_proc_t *proc;
+    orte_app_context_t *app;
+    hwloc_obj_type_t level;
+    hwloc_obj_t obj;
+    unsigned int cache_level=0;
+    int ppr, cnt, nobjs, nprocs_mapped;
+    char **ppr_req, **ck;
+
+    if (NULL == jdata->map->last_mapper ||
+        0 != strcasecmp(jdata->map->last_mapper, c->mca_component_name)) {
+        /* a mapper has been specified, and it isn't me */
+        opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                            "mca:rmaps:ppr: job %s not using ppr assign: %s",
+                            ORTE_JOBID_PRINT(jdata->jobid),
+                            (NULL == jdata->map->last_mapper) ? "NULL" : jdata->map->last_mapper);
+        return ORTE_ERR_TAKE_NEXT_OPTION;
+    }
+
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "mca:rmaps:ppr: assigning locations for job %s with ppr %s policy %s",
+                        ORTE_JOBID_PRINT(jdata->jobid), jdata->map->ppr,
+                        orte_rmaps_base_print_mapping(jdata->map->mapping));
+
+    /* pickup the object level */
+    if (ORTE_MAPPING_BYNODE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_MACHINE;
+    } else if (ORTE_MAPPING_BYHWTHREAD == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_PU;
+    } else if (ORTE_MAPPING_BYCORE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_CORE;
+    } else if (ORTE_MAPPING_BYSOCKET == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_SOCKET;
+    } else if (ORTE_MAPPING_BYL1CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_L1CACHE;
+        cache_level = 1;
+    } else if (ORTE_MAPPING_BYL2CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_L2CACHE;
+        cache_level = 2;
+    } else if (ORTE_MAPPING_BYL3CACHE == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_L3CACHE;
+        cache_level = 3;
+    } else if (ORTE_MAPPING_BYNUMA == ORTE_GET_MAPPING_POLICY(jdata->map->mapping)) {
+        level = HWLOC_OBJ_NUMANODE;
+    } else {
+        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+        return ORTE_ERR_TAKE_NEXT_OPTION;
+    }
+
+    /* get the ppr value */
+    ppr_req = opal_argv_split(jdata->map->ppr, ',');
+    ck = opal_argv_split(ppr_req[0], ':');
+    ppr = strtol(ck[0], NULL, 10);
+    opal_argv_free(ck);
+    opal_argv_free(ppr_req);
+
+    /* start assigning procs to objects, filling each object as we go until
+     * all procs are assigned. */
+    for (n=0; n < jdata->apps->size; n++) {
+        if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, n))) {
+            continue;
+        }
+        nprocs_mapped = 0;
+        for (m=0; m < jdata->map->nodes->size; m++) {
+            if (NULL == (node = (orte_node_t*)opal_pointer_array_get_item(jdata->map->nodes, m))) {
+                continue;
+            }
+            if (NULL == node->topology || NULL == node->topology->topo) {
+                orte_show_help("help-orte-rmaps-ppr.txt", "ppr-topo-missing",
+                               true, node->name);
+                return ORTE_ERR_SILENT;
+            }
+            if (HWLOC_OBJ_MACHINE == level) {
+                obj = hwloc_get_root_obj(node->topology->topo);
+                for (j=0; j < node->procs->size; j++) {
+                    if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+                        continue;
+                    }
+                    if (proc->name.jobid != jdata->jobid) {
+                        continue;
+                    }
+                    orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
+                }
+            } else {
+                /* get the number of resources on this node at this level */
+                nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology->topo,
+                                                           level, cache_level,
+                                                           OPAL_HWLOC_AVAILABLE);
+
+                /* map the specified number of procs to each such resource on this node,
+                 * recording the locale of each proc so we know its cpuset
+                 */
+                for (i=0; i < nobjs; i++) {
+                    cnt = 0;
+                    obj = opal_hwloc_base_get_obj_by_type(node->topology->topo,
+                                                          level, cache_level,
+                                                          i, OPAL_HWLOC_AVAILABLE);
+                    for (j=0; j < node->procs->size && cnt < ppr && nprocs_mapped < app->num_procs; j++) {
+                        if (NULL == (proc = (orte_proc_t*)opal_pointer_array_get_item(node->procs, j))) {
+                            continue;
+                        }
+                        if (proc->name.jobid != jdata->jobid) {
+                            continue;
+                        }
+                        /* if we already assigned it, then skip */
+                        if (orte_get_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, NULL, OPAL_PTR)) {
+                            continue;
+                        }
+                        nprocs_mapped++;
+                        cnt++;
+                        orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
+                    }
+                }
+            }
+        }
+    }
+    return ORTE_SUCCESS;
+}
