@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2016 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -43,7 +43,7 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                          orte_std_cntr_t num_slots,
                          orte_vpid_t num_procs)
 {
-    int rc, i, nprocs_mapped;
+    int i, nprocs_mapped;
     orte_node_t *node;
     orte_proc_t *proc;
     int num_procs_to_assign, extra_procs_to_assign=0, nxtra_nodes=0;
@@ -56,10 +56,10 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                         ORTE_JOBID_PRINT(jdata->jobid), (int)num_slots, (unsigned long)num_procs);
 
     /* check to see if we can map all the procs */
-    if (num_slots < ((int)app->num_procs * orte_rmaps_base.cpus_per_rank)) {
+    if (num_slots < (int)app->num_procs) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                           true, app->num_procs, app->app);
+                           true, app->num_procs, app->app, orte_process_info.nodename);
             ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
             return ORTE_ERR_SILENT;
         }
@@ -76,8 +76,8 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
         /* get the root object as we are not assigning
          * locale here except at the node level
          */
-        if (NULL != node->topology) {
-            obj = hwloc_get_root_obj(node->topology);
+        if (NULL != node->topology && NULL != node->topology->topo) {
+            obj = hwloc_get_root_obj(node->topology->topo);
         }
         if (node->slots <= node->slots_inuse) {
             opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
@@ -85,11 +85,8 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                                 node->name);
             continue;
         }
-        /* assign a number of procs equal to the number of available
-         * slots divided by the number of cpus/rank the user
-         * requested
-         */
-        num_procs_to_assign = (node->slots - node->slots_inuse) / orte_rmaps_base.cpus_per_rank;
+        /* assign a number of procs equal to the number of available slots */
+        num_procs_to_assign = node->slots - node->slots_inuse;
         opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:rr:slot assigning %d procs to node %s",
                             (int)num_procs_to_assign, node->name);
@@ -97,12 +94,9 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
         for (i=0; i < num_procs_to_assign && nprocs_mapped < app->num_procs; i++) {
             /* add this node to the map - do it only once */
             if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-                if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                    ORTE_ERROR_LOG(rc);
-                    return rc;
-                }
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-                OBJ_RETAIN(node);  /* maintain accounting on object */
+                OBJ_RETAIN(node);
+                opal_pointer_array_add(jdata->map->nodes, node);
                 ++(jdata->map->num_nodes);
             }
             if (NULL == (proc = orte_rmaps_base_setup_proc(jdata, node, app->idx))) {
@@ -146,18 +140,15 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
         /* get the root object as we are not assigning
          * locale except at the node level
          */
-        if (NULL != node->topology) {
-            obj = hwloc_get_root_obj(node->topology);
+        if (NULL != node->topology && NULL != node->topology->topo) {
+            obj = hwloc_get_root_obj(node->topology->topo);
         }
 
         /* add this node to the map - do it only once */
         if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-            if (ORTE_SUCCESS > (rc = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
-            }
             ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-            OBJ_RETAIN(node);  /* maintain accounting on object */
+            OBJ_RETAIN(node);
+            opal_pointer_array_add(jdata->map->nodes, node);
             ++(jdata->map->num_nodes);
         }
         if (add_one) {
@@ -168,7 +159,7 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                 --nxtra_nodes;
             }
         }
-        num_procs_to_assign = ((node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank) + extra_procs_to_assign;
+        num_procs_to_assign = node->slots - node->slots_inuse + extra_procs_to_assign;
         opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:rr:slot adding up to %d procs to node %s",
                             num_procs_to_assign, node->name);
@@ -182,11 +173,12 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots < ((int)node->num_procs * orte_rmaps_base.cpus_per_rank)) {
+        if (node->slots < (int)node->num_procs) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
             ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+            ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
             /* check for permission */
             if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
                 /* if we weren't given a directive either way, then we will error out
@@ -194,13 +186,13 @@ int orte_rmaps_rr_byslot(orte_job_t *jdata,
                  * via hostfile/dash-host */
                 if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
                     orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                   true, app->num_procs, app->app);
+                                   true, app->num_procs, app->app, orte_process_info.nodename);
                     ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                     return ORTE_ERR_SILENT;
                 } else if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                     /* if we were explicitly told not to oversubscribe, then don't */
                     orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                   true, app->num_procs, app->app);
+                                   true, app->num_procs, app->app, orte_process_info.nodename);
                     ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                     return ORTE_ERR_SILENT;
                 }
@@ -223,7 +215,7 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
     int j, nprocs_mapped, nnodes;
     orte_node_t *node;
     orte_proc_t *proc;
-    int num_procs_to_assign, navg, idx;
+    int num_procs_to_assign, navg;
     int extra_procs_to_assign=0, nxtra_nodes=0;
     hwloc_obj_t obj=NULL;
     float balance;
@@ -236,10 +228,10 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                         (int)num_slots, (unsigned long)num_procs);
 
     /* quick check to see if we can map all the procs */
-    if (num_slots < ((int)app->num_procs * orte_rmaps_base.cpus_per_rank)) {
+    if (num_slots < (int)app->num_procs) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                           true, app->num_procs, app->app);
+                           true, app->num_procs, app->app, orte_process_info.nodename);
             ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
             return ORTE_ERR_SILENT;
         }
@@ -290,17 +282,14 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
             /* get the root object as we are not assigning
              * locale except at the node level
              */
-            if (NULL != node->topology) {
-                obj = hwloc_get_root_obj(node->topology);
+            if (NULL != node->topology && NULL != node->topology->topo) {
+                obj = hwloc_get_root_obj(node->topology->topo);
             }
             /* add this node to the map, but only do so once */
             if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-                if (ORTE_SUCCESS > (idx = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                    ORTE_ERROR_LOG(idx);
-                    return idx;
-                }
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-                OBJ_RETAIN(node);  /* maintain accounting on object */
+                OBJ_RETAIN(node);
+                opal_pointer_array_add(jdata->map->nodes, node);
                 ++(jdata->map->num_nodes);
             }
             if (oversubscribed) {
@@ -336,8 +325,8 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                     }
                 }
                 /* if slots < avg + extra (adjusted for cpus/proc), then try to take all */
-                if ((node->slots - node->slots_inuse) < ((navg + extra_procs_to_assign) * orte_rmaps_base.cpus_per_rank)) {
-                    num_procs_to_assign = (node->slots - node->slots_inuse)/orte_rmaps_base.cpus_per_rank;
+                if ((node->slots - node->slots_inuse) < (navg + extra_procs_to_assign)) {
+                    num_procs_to_assign = node->slots - node->slots_inuse;
                     /* if we can't take any proc, skip following steps */
                     if (num_procs_to_assign == 0) {
                         continue;
@@ -361,16 +350,17 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                     return ORTE_ERR_OUT_OF_RESOURCE;
                 }
                 nprocs_mapped++;
-            orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
+                orte_set_attribute(&proc->attributes, ORTE_PROC_HWLOC_LOCALE, ORTE_ATTR_LOCAL, obj, OPAL_PTR);
             }
             /* not all nodes are equal, so only set oversubscribed for
              * this node if it is in that state
              */
-            if (node->slots < ((int)node->num_procs * orte_rmaps_base.cpus_per_rank)) {
+            if (node->slots < (int)node->num_procs) {
                 /* flag the node as oversubscribed so that sched-yield gets
                  * properly set
                  */
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+                ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
                 /* check for permission */
                 if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
                     /* if we weren't given a directive either way, then we will error out
@@ -378,13 +368,13 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
                      * via hostfile/dash-host */
                     if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
+                                       true, app->num_procs, app->app, orte_process_info.nodename);
                         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                         return ORTE_ERR_SILENT;
                     } else if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                         /* if we were explicitly told not to oversubscribe, then don't */
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
+                                       true, app->num_procs, app->app, orte_process_info.nodename);
                         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                         return ORTE_ERR_SILENT;
                     }
@@ -403,8 +393,8 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
             /* get the root object as we are not assigning
              * locale except at the node level
              */
-            if (NULL != node->topology) {
-                obj = hwloc_get_root_obj(node->topology);
+            if (NULL != node->topology && NULL != node->topology->topo) {
+                obj = hwloc_get_root_obj(node->topology->topo);
             }
 
            OPAL_OUTPUT_VERBOSE((20, orte_rmaps_base_framework.framework_output,
@@ -418,11 +408,12 @@ int orte_rmaps_rr_bynode(orte_job_t *jdata,
             /* not all nodes are equal, so only set oversubscribed for
              * this node if it is in that state
              */
-            if (node->slots < ((int)node->num_procs * orte_rmaps_base.cpus_per_rank)) {
+            if (node->slots < (int)node->num_procs) {
                 /* flag the node as oversubscribed so that sched-yield gets
                  * properly set
                  */
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+                ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
             }
             if (nprocs_mapped == app->num_procs) {
                 /* we are done */
@@ -456,7 +447,6 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
     orte_node_t *node;
     orte_proc_t *proc;
     int nprocs, start;
-    int idx;
     hwloc_obj_t obj=NULL;
     unsigned int nobjs;
     bool add_one;
@@ -488,10 +478,10 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                         (int)num_slots, (unsigned long)num_procs);
 
     /* quick check to see if we can map all the procs */
-    if (num_slots < (app->num_procs * orte_rmaps_base.cpus_per_rank)) {
+    if (num_slots < app->num_procs) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                           true, app->num_procs, app->app);
+                           true, app->num_procs, app->app, orte_process_info.nodename);
             ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
             return ORTE_ERR_SILENT;
         }
@@ -507,14 +497,14 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
     do {
         add_one = false;
         OPAL_LIST_FOREACH(node, node_list, orte_node_t) {
-            if (NULL == node->topology) {
+            if (NULL == node->topology || NULL == node->topology->topo) {
                 orte_show_help("help-orte-rmaps-ppr.txt", "ppr-topo-missing",
                                true, node->name);
                 return ORTE_ERR_SILENT;
             }
             start = 0;
             /* get the number of objects of this type on this node */
-            nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology, target, cache_level, OPAL_HWLOC_AVAILABLE);
+            nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology->topo, target, cache_level, OPAL_HWLOC_AVAILABLE);
             if (0 == nobjs) {
                 continue;
             }
@@ -528,7 +518,7 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                 start = (jdata->bkmark_obj + 1) % nobjs;
             }
             /* compute the number of procs to go on this node */
-            nprocs = (node->slots - node->slots_inuse) / orte_rmaps_base.cpus_per_rank;
+            nprocs = node->slots - node->slots_inuse;
             opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                                 "mca:rmaps:rr: calculated nprocs %d", nprocs);
             if (nprocs < 1) {
@@ -547,12 +537,9 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
             }
             /* add this node to the map, if reqd */
             if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-                if (ORTE_SUCCESS > (idx = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                    ORTE_ERROR_LOG(idx);
-                    return idx;
-                }
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-                OBJ_RETAIN(node);  /* maintain accounting on object */
+                OBJ_RETAIN(node);
+                opal_pointer_array_add(jdata->map->nodes, node);
                 ++(jdata->map->num_nodes);
             }
             nmapped = 0;
@@ -564,13 +551,13 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                     opal_output_verbose(20, orte_rmaps_base_framework.framework_output,
                                         "mca:rmaps:rr: assigning proc to object %d", (i+start) % nobjs);
                     /* get the hwloc object */
-                    if (NULL == (obj = opal_hwloc_base_get_obj_by_type(node->topology, target, cache_level, (i+start) % nobjs, OPAL_HWLOC_AVAILABLE))) {
+                    if (NULL == (obj = opal_hwloc_base_get_obj_by_type(node->topology->topo, target, cache_level, (i+start) % nobjs, OPAL_HWLOC_AVAILABLE))) {
                         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                         return ORTE_ERR_NOT_FOUND;
                     }
-                    if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology, obj)) {
+                    if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology->topo, obj)) {
                         orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low", true,
-                                       orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology, obj),
+                                       orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology->topo, obj),
                                        orte_rmaps_base_print_mapping(orte_rmaps_base.mapping));
                         return ORTE_ERR_SILENT;
                     }
@@ -591,6 +578,7 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                  * properly set
                  */
                 ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+                ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
                 /* check for permission */
                 if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
                     /* if we weren't given a directive either way, then we will error out
@@ -598,13 +586,13 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                      * via hostfile/dash-host */
                     if (!(ORTE_MAPPING_SUBSCRIBE_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping))) {
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
+                                       true, app->num_procs, app->app, orte_process_info.nodename);
                         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                         return ORTE_ERR_SILENT;
                     } else if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
                         /* if we were explicitly told not to oversubscribe, then don't */
                         orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                                       true, app->num_procs, app->app);
+                                       true, app->num_procs, app->app, orte_process_info.nodename);
                         ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
                         return ORTE_ERR_SILENT;
                     }
@@ -637,7 +625,6 @@ static int byobj_span(orte_job_t *jdata,
     orte_node_t *node;
     orte_proc_t *proc;
     int nprocs, nxtra_objs;
-    int idx;
     hwloc_obj_t obj=NULL;
     unsigned int nobjs;
 
@@ -648,10 +635,10 @@ static int byobj_span(orte_job_t *jdata,
                         (int)num_slots, (unsigned long)num_procs);
 
     /* quick check to see if we can map all the procs */
-    if (num_slots < (int)app->num_procs * orte_rmaps_base.cpus_per_rank) {
+    if (num_slots < (int)app->num_procs) {
         if (ORTE_MAPPING_NO_OVERSUBSCRIBE & ORTE_GET_MAPPING_DIRECTIVE(jdata->map->mapping)) {
             orte_show_help("help-orte-rmaps-base.txt", "orte-rmaps-base:alloc-error",
-                           true, app->num_procs, app->app);
+                           true, app->num_procs, app->app, orte_process_info.nodename);
             ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
             return ORTE_ERR_SILENT;
         }
@@ -662,13 +649,13 @@ static int byobj_span(orte_job_t *jdata,
      */
     nobjs = 0;
     OPAL_LIST_FOREACH(node, node_list, orte_node_t) {
-        if (NULL == node->topology) {
+        if (NULL == node->topology || NULL == node->topology->topo) {
             orte_show_help("help-orte-rmaps-ppr.txt", "ppr-topo-missing",
                            true, node->name);
             return ORTE_ERR_SILENT;
         }
         /* get the number of objects of this type on this node */
-        nobjs += opal_hwloc_base_get_nbobjs_by_type(node->topology, target, cache_level, OPAL_HWLOC_AVAILABLE);
+        nobjs += opal_hwloc_base_get_nbobjs_by_type(node->topology->topo, target, cache_level, OPAL_HWLOC_AVAILABLE);
     }
 
     if (0 == nobjs) {
@@ -698,28 +685,25 @@ static int byobj_span(orte_job_t *jdata,
     OPAL_LIST_FOREACH(node, node_list, orte_node_t) {
         /* add this node to the map, if reqd */
         if (!ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_MAPPED)) {
-            if (ORTE_SUCCESS > (idx = opal_pointer_array_add(jdata->map->nodes, (void*)node))) {
-                ORTE_ERROR_LOG(idx);
-                return idx;
-            }
             ORTE_FLAG_SET(node, ORTE_NODE_FLAG_MAPPED);
-            OBJ_RETAIN(node);  /* maintain accounting on object */
+            OBJ_RETAIN(node);
+            opal_pointer_array_add(jdata->map->nodes, node);
             ++(jdata->map->num_nodes);
         }
         /* get the number of objects of this type on this node */
-        nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology, target, cache_level, OPAL_HWLOC_AVAILABLE);
+        nobjs = opal_hwloc_base_get_nbobjs_by_type(node->topology->topo, target, cache_level, OPAL_HWLOC_AVAILABLE);
         opal_output_verbose(2, orte_rmaps_base_framework.framework_output,
                             "mca:rmaps:rr:byobj: found %d objs on node %s", nobjs, node->name);
         /* loop through the number of objects */
         for (i=0; i < (int)nobjs && nprocs_mapped < (int)app->num_procs; i++) {
             /* get the hwloc object */
-            if (NULL == (obj = opal_hwloc_base_get_obj_by_type(node->topology, target, cache_level, i, OPAL_HWLOC_AVAILABLE))) {
+            if (NULL == (obj = opal_hwloc_base_get_obj_by_type(node->topology->topo, target, cache_level, i, OPAL_HWLOC_AVAILABLE))) {
                 ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                 return ORTE_ERR_NOT_FOUND;
             }
-            if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology, obj)) {
+            if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology->topo, obj)) {
                 orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low", true,
-                               orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology, obj),
+                               orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology->topo, obj),
                                orte_rmaps_base_print_mapping(orte_rmaps_base.mapping));
                 return ORTE_ERR_SILENT;
             }
@@ -743,11 +727,12 @@ static int byobj_span(orte_job_t *jdata,
         /* not all nodes are equal, so only set oversubscribed for
          * this node if it is in that state
          */
-        if (node->slots < (int)node->num_procs * orte_rmaps_base.cpus_per_rank) {
+        if (node->slots < (int)node->num_procs) {
             /* flag the node as oversubscribed so that sched-yield gets
              * properly set
              */
             ORTE_FLAG_SET(node, ORTE_NODE_FLAG_OVERSUBSCRIBED);
+            ORTE_FLAG_SET(jdata, ORTE_JOB_FLAG_OVERSUBSCRIBED);
         }
         if (nprocs_mapped == app->num_procs) {
             /* we are done */
@@ -757,4 +742,3 @@ static int byobj_span(orte_job_t *jdata,
 
     return ORTE_SUCCESS;
 }
-
