@@ -13,9 +13,9 @@
  * Copyright (c) 2009-2010 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
- * Copyright (c) 2014      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -51,6 +51,8 @@ static char *orte_fork_agent_string = NULL;
 static char *orte_tmpdir_base = NULL;
 static char *orte_local_tmpdir_base = NULL;
 static char *orte_remote_tmpdir_base = NULL;
+static char *orte_top_session_dir = NULL;
+static char *orte_jobfam_session_dir = NULL;
 
 int orte_register_params(void)
 {
@@ -151,6 +153,34 @@ int orte_register_params(void)
         orte_process_info.tmpdir_base = strdup (orte_remote_tmpdir_base);
     }
 
+    orte_top_session_dir = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "top_session_dir",
+                                  "Top of the session directory tree for applications",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
+                                  &orte_top_session_dir);
+
+    if (NULL != orte_top_session_dir) {
+         if (NULL != orte_process_info.top_session_dir) {
+            free(orte_process_info.top_session_dir);
+        }
+        orte_process_info.top_session_dir = strdup(orte_top_session_dir);
+    }
+
+    orte_jobfam_session_dir = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "jobfam_session_dir",
+                                  "The jobfamily session directory for applications",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_ALL_EQ,
+                                  &orte_jobfam_session_dir);
+
+    if (NULL != orte_jobfam_session_dir) {
+        if (NULL != orte_process_info.jobfam_session_dir) {
+            free(orte_process_info.jobfam_session_dir);
+        }
+        orte_process_info.jobfam_session_dir = strdup(orte_jobfam_session_dir);
+    }
+
     orte_prohibited_session_dirs = NULL;
     (void) mca_base_var_register ("orte", "orte", NULL, "no_session_dirs",
                                   "Prohibited locations for session directories (multiple locations separated by ',', default=NULL)",
@@ -277,7 +307,7 @@ int orte_register_params(void)
                                   "Test debugger colaunch after debugger attachment",
                                   MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_debugger_test_daemon);
+                                  &orte_debugger_test_attach);
 
     orte_debugger_check_rate = 0;
     (void) mca_base_var_register ("orte", "orte", NULL, "debugger_check_rate",
@@ -400,6 +430,13 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_retain_aliases);
 
+    orte_hostname_cutoff = 1000;
+    (void) mca_base_var_register ("orte", "orte", NULL, "hostname_cutoff",
+                                  "Pass hostnames to all procs when #nodes is less than cutoff [default:1000]",
+                                  MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                  OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_hostname_cutoff);
+
     /* which alias to use in MPIR_proctab */
     orte_use_hostname_alias = 1;
     (void) mca_base_var_register ("orte", "orte", NULL, "hostname_alias_index",
@@ -459,14 +496,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_timestamp_output);
 
-    /* redirect output into files */
-    orte_output_filename = NULL;
-    (void) mca_base_var_register ("orte", "orte", NULL, "output_filename",
-                                  "Redirect output from application processes into filename.rank [default: NULL]",
-                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_output_filename);
-
     orte_show_resolved_nodenames = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "show_resolved_nodenames",
                                   "Display any node names that are resolved to a different name (default: false)",
@@ -474,21 +503,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_show_resolved_nodenames);
 
-#if 0
-    /* XXX -- option doesn't appear to do anything */
-    mca_base_param_reg_int_name("orte", "hetero_apps",
-                                "Indicates that multiple app_contexts are being provided that are a mix of 32/64 bit binaries (default: false)",
-                                false, false, (int) false, &value);
-    orte_hetero_apps = OPAL_INT_TO_BOOL(value);
-#endif
-
-    orte_hetero_nodes = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "hetero_nodes",
-                                  "Nodes in cluster may differ in topology, so send the topology back from each node [Default = false]",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_hetero_nodes);
-
     /* allow specification of the launch agent */
     orte_launch_agent = "orted";
     (void) mca_base_var_register ("orte", "orte", NULL, "launch_agent",
@@ -556,14 +570,6 @@ int orte_register_params(void)
         orte_map_stddiag_to_stderr = true;
     }
 
-    /* whether or not to forward SIGTSTP and SIGCONT signals */
-    orte_forward_job_control = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "forward_job_control",
-                                  "Forward SIGTSTP (after converting to SIGSTOP) and SIGCONT signals to the application procs [default: no]",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_forward_job_control);
-
     /* whether or not to report launch progress */
     orte_report_launch_progress = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "report_launch_progress",
@@ -650,13 +656,6 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_allowed_exit_without_sync);
 
-    orte_staged_execution = false;
-    (void) mca_base_var_register ("orte", "orte", NULL, "staged_execution",
-                                  "Staged execution is being used",
-                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                  &orte_staged_execution);
-
     orte_report_child_jobs_separately = false;
     (void) mca_base_var_register ("orte", "orte", NULL, "report_child_jobs_separately",
                                   "Return the exit status of the primary job only",
@@ -682,6 +681,15 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_stat_history_size);
 
+    orte_no_vm = false;
+    id = mca_base_var_register ("orte", "orte", NULL, "no_vm",
+                                "Do not build the VM at start to detect topologies",
+                                MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                &orte_no_vm);
+    /* register a synonym for old name */
+    mca_base_var_register_synonym (id, "orte", "state", "novm", "select", MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+
     orte_max_vm_size = -1;
     (void) mca_base_var_register ("orte", "orte", NULL, "max_vm_size",
                                   "Maximum size of virtual machine - used to subdivide allocation",
@@ -745,25 +753,42 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_daemon_cores);
 
-    /* cutoff for full modex */
-    orte_direct_modex_cutoff = UINT32_MAX;
-    id = mca_base_var_register ("orte", "orte", NULL, "direct_modex_cutoff",
-                                "If the number of processes in the application exceeds the provided value,"
-                                "modex will be done upon demand [default: UINT32_MAX]",
-                                MCA_BASE_VAR_TYPE_UNSIGNED_INT, NULL, 0, 0,
-                                OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
-                                &orte_direct_modex_cutoff);
-    /* register a synonym for old name */
-    mca_base_var_register_synonym (id, "ompi", "ompi", "hostname", "cutoff", MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
+    /* get the conduit params */
+    orte_coll_transport = "fabric,ethernet";
+    (void) mca_base_var_register("orte", "orte", "coll", "transports",
+                                 "Comma-separated list of transports to use for ORTE collectives",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &orte_coll_transport);
 
+    orte_mgmt_transport = "oob";
+    (void) mca_base_var_register("orte", "orte", "mgmt", "transports",
+                                 "Comma-separated list of transports to use for ORTE management messages",
+                                 MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_9,
+                                 MCA_BASE_VAR_SCOPE_READONLY, &orte_mgmt_transport);
 
     /* Amount of time to wait for a stack trace to return from the daemons */
     orte_stack_trace_wait_timeout = 30;
     (void) mca_base_var_register ("orte", "orte", NULL, "timeout_for_stack_trace",
-                                  "Seconds to wait for stack traces to return before terminating the job (<= 0 wait forever)",
+                                  "Seconds to wait for stack traces to return before terminating "
+                                  "the job (<= 0 wait forever)",
                                   MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_stack_trace_wait_timeout);
 
+    orte_fwd_mpirun_port = false;
+    (void) mca_base_var_register ("orte", "orte", NULL, "fwd_mpirun_port",
+                                  "Forward the port used by mpirun so all daemons will use it",
+                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                  OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_fwd_mpirun_port);
+
+    /* register the URI of the UNIVERSAL data server */
+    orte_data_server_uri = NULL;
+    (void) mca_base_var_register ("orte", "pmix", NULL, "server_uri",
+                                  "URI of a session-level keyval server for publish/lookup operations",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_3, MCA_BASE_VAR_SCOPE_ALL,
+                                  &orte_data_server_uri);
+
     return ORTE_SUCCESS;
 }
