@@ -1,5 +1,5 @@
-/* -*- C -*-
- *
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
+/*
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
@@ -10,11 +10,11 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2017 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2007-2009 Sun Microsystems, Inc. All rights reserved.
- * Copyright (c) 2007-2013 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -57,6 +57,7 @@
 #include "opal/mca/event/event.h"
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/mca/base/base.h"
+#include "opal/mca/pmix/pmix.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/util/basename.h"
@@ -65,6 +66,7 @@
 #include "opal/util/opal_getcwd.h"
 #include "opal/util/show_help.h"
 #include "opal/util/fd.h"
+#include "opal/util/daemon_init.h"
 
 #include "opal/version.h"
 #include "opal/runtime/opal.h"
@@ -74,6 +76,9 @@
 #include "opal/class/opal_pointer_array.h"
 
 #include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/grpcomm/grpcomm.h"
+#include "orte/mca/odls/odls.h"
+#include "orte/mca/oob/base/base.h"
 #include "orte/mca/rml/rml.h"
 #include "orte/mca/rml/base/rml_contact.h"
 #include "orte/mca/state/state.h"
@@ -81,6 +86,7 @@
 #include "orte/runtime/runtime.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/util/show_help.h"
+#include "orte/util/threads.h"
 
 #include "orte/orted/orted.h"
 
@@ -95,9 +101,13 @@ static bool want_prefix_by_default = (bool) ORTE_WANT_ORTERUN_PREFIX_BY_DEFAULT;
 static struct {
     bool help;
     bool version;
-    char *report_uri;
     char *prefix;
     bool run_as_root;
+    bool set_sid;
+    bool daemonize;
+    bool system_server;
+    char *report_uri;
+    bool remote_connections;
 } myglobals;
 
 static opal_cmd_line_init_t cmd_line_init[] = {
@@ -109,14 +119,18 @@ static opal_cmd_line_init_t cmd_line_init[] = {
       &myglobals.version, OPAL_CMD_LINE_TYPE_BOOL,
       "Print version and exit" },
 
-    { NULL, '\0', "report-uri", "report-uri", 1,
-      &myglobals.report_uri, OPAL_CMD_LINE_TYPE_STRING,
-      "Printout URI on stdout [-], stderr [+], or a file [anything else]" },
-
     { NULL, '\0', "prefix", "prefix", 1,
       &myglobals.prefix, OPAL_CMD_LINE_TYPE_STRING,
       "Prefix to be used to look for ORTE executables" },
 
+    { "orte_daemonize", '\0', NULL, "daemonize", 0,
+      &myglobals.daemonize, OPAL_CMD_LINE_TYPE_BOOL,
+      "Daemonize the orte-dvm into the background" },
+
+    { NULL, '\0', NULL, "set-sid", 0,
+      &myglobals.set_sid, OPAL_CMD_LINE_TYPE_BOOL,
+      "Direct the orte-dvm to separate from the current session"},
+
     { "orte_debug_daemons", '\0', "debug-daemons", "debug-daemons", 0,
       NULL, OPAL_CMD_LINE_TYPE_BOOL,
       "Debug daemons" },
@@ -129,13 +143,49 @@ static opal_cmd_line_init_t cmd_line_init[] = {
       &myglobals.run_as_root, OPAL_CMD_LINE_TYPE_BOOL,
       "Allow execution as root (STRONGLY DISCOURAGED)" },
 
+    /* Specify the launch agent to be used */
+    { "orte_launch_agent", '\0', "launch-agent", "launch-agent", 1,
+      NULL, OPAL_CMD_LINE_TYPE_STRING,
+      "Command used to start processes on remote nodes (default: orted)" },
+
+    /* maximum size of VM - typically used to subdivide an allocation */
+    { "orte_max_vm_size", '\0', "max-vm-size", "max-vm-size", 1,
+      NULL, OPAL_CMD_LINE_TYPE_INT,
+      "Maximum size of VM" },
+
+    /* Set a hostfile */
+    { NULL, '\0', "hostfile", "hostfile", 1,
+      NULL, OPAL_CMD_LINE_TYPE_STRING,
+      "Provide a hostfile" },
+    { NULL, '\0', "machinefile", "machinefile", 1,
+      NULL, OPAL_CMD_LINE_TYPE_STRING,
+      "Provide a hostfile" },
+    { "orte_default_hostfile", '\0', "default-hostfile", "default-hostfile", 1,
+      NULL, OPAL_CMD_LINE_TYPE_STRING,
+      "Provide a default hostfile" },
+
+    { NULL, 'H', "host", "host", 1,
+      NULL, OPAL_CMD_LINE_TYPE_STRING,
+      "List of hosts to invoke processes on" },
+
+    { NULL, '\0', "system-server", "system-server", 0,
+      &myglobals.system_server, OPAL_CMD_LINE_TYPE_BOOL,
+      "Provide a system-level server connection point - only one allowed per node" },
+
+    { NULL, '\0', "report-uri", "report-uri", 1,
+      &myglobals.report_uri, OPAL_CMD_LINE_TYPE_STRING,
+      "Printout URI on stdout [-], stderr [+], or a file [anything else]",
+      OPAL_CMD_LINE_OTYPE_DEBUG },
+
+    { NULL, '\0', "remote-tools", "remote-tools", 0,
+      &myglobals.remote_connections, OPAL_CMD_LINE_TYPE_BOOL,
+      "Enable connections from remote tools" },
+
     /* End of list */
     { NULL, '\0', NULL, NULL, 0,
       NULL, OPAL_CMD_LINE_TYPE_NULL, NULL }
 };
 
-static void notify_requestor(int sd, short args, void *cbdata);
-
 int main(int argc, char *argv[])
 {
     int rc, i, j;
@@ -143,7 +193,6 @@ int main(int argc, char *argv[])
     char *param, *value;
     orte_job_t *jdata=NULL;
     orte_app_context_t *app;
-    char *uri, *ptr;
 
     /* Setup and parse the command line */
     memset(&myglobals, 0, sizeof(myglobals));
@@ -153,7 +202,7 @@ int main(int argc, char *argv[])
 
     opal_cmd_line_create(&cmd_line, cmd_line_init);
     mca_base_cmd_line_setup(&cmd_line);
-    if (OPAL_SUCCESS != (rc = opal_cmd_line_parse(&cmd_line, true,
+    if (OPAL_SUCCESS != (rc = opal_cmd_line_parse(&cmd_line, true, false,
                                                   argc, argv)) ) {
         if (OPAL_ERR_SILENT != rc) {
             fprintf(stderr, "%s: command line error (%s)\n", argv[0],
@@ -184,18 +233,22 @@ int main(int argc, char *argv[])
      * exit with a giant warning flag
      */
     if (0 == geteuid() && !myglobals.run_as_root) {
+        /* show_help is not yet available, so print an error manually */
         fprintf(stderr, "--------------------------------------------------------------------------\n");
         if (myglobals.help) {
-            fprintf(stderr, "%s cannot provide the help message when run as root\n", orte_basename);
+            fprintf(stderr, "%s cannot provide the help message when run as root.\n\n", orte_basename);
         } else {
-            /* show_help is not yet available, so print an error manually */
-            fprintf(stderr, "%s has detected an attempt to run as root.\n", orte_basename);
+            fprintf(stderr, "%s has detected an attempt to run as root.\n\n", orte_basename);
         }
-        fprintf(stderr, " This is *strongly* discouraged as any mistake (e.g., in defining TMPDIR) or bug can\n");
-        fprintf(stderr, "result in catastrophic damage to the OS file system, leaving\n");
-        fprintf(stderr, "your system in an unusable state.\n\n");
+
+        fprintf(stderr, "Running at root is *strongly* discouraged as any mistake (e.g., in\n");
+        fprintf(stderr, "defining TMPDIR) or bug can result in catastrophic damage to the OS\n");
+        fprintf(stderr, "file system, leaving your system in an unusable state.\n\n");
+
+        fprintf(stderr, "We strongly suggest that you run %s as a non-root user.\n\n", orte_basename);
+
         fprintf(stderr, "You can override this protection by adding the --allow-run-as-root\n");
-        fprintf(stderr, "option to your cmd line. However, we reiterate our strong advice\n");
+        fprintf(stderr, "option to your command line.  However, we reiterate our strong advice\n");
         fprintf(stderr, "against doing so - please do so at your own risk.\n");
         fprintf(stderr, "--------------------------------------------------------------------------\n");
         exit(1);
@@ -241,6 +294,20 @@ int main(int argc, char *argv[])
         exit(0);
     }
 
+    if (myglobals.system_server) {
+        /* we should act as system-level PMIx server */
+        opal_setenv(OPAL_MCA_PREFIX"pmix_system_server", "1", true, &environ);
+    }
+    /* always act as session-level PMIx server */
+    opal_setenv(OPAL_MCA_PREFIX"pmix_session_server", "1", true, &environ);
+    /* if we were asked to report a uri, set the MCA param to do so */
+    if (NULL != myglobals.report_uri) {
+        opal_setenv("PMIX_MCA_ptl_tcp_report_uri", myglobals.report_uri, true, &environ);
+    }
+    if (myglobals.remote_connections) {
+        opal_setenv("PMIX_MCA_ptl_tcp_remote_connections", "1", true, &environ);
+    }
+
     /* Setup MCA params */
     orte_register_params();
 
@@ -252,6 +319,22 @@ int main(int argc, char *argv[])
      */
     orte_launch_environ = opal_argv_copy(environ);
 
+#if defined(HAVE_SETSID)
+    /* see if we were directed to separate from current session */
+    if (myglobals.set_sid) {
+        setsid();
+    }
+#endif
+
+    /* detach from controlling terminal
+     * otherwise, remain attached so output can get to us
+     */
+    if(!orte_debug_flag &&
+       !orte_debug_daemons_flag &&
+       myglobals.daemonize) {
+        opal_daemon_init(NULL);
+    }
+
     /* Intialize our Open RTE environment */
     if (ORTE_SUCCESS != (rc = orte_init(&argc, &argv, ORTE_PROC_MASTER))) {
         /* cannot call ORTE_ERROR_LOG as it could be the errmgr
@@ -264,43 +347,7 @@ int main(int argc, char *argv[])
      */
     opal_finalize();
 
-    /* check for request to report uri */
-    uri = orte_rml.get_contact_info();
-    if (NULL != myglobals.report_uri) {
-        FILE *fp;
-        if (0 == strcmp(myglobals.report_uri, "-")) {
-            /* if '-', then output to stdout */
-            printf("VMURI: %s\n", uri);
-        } else if (0 == strcmp(myglobals.report_uri, "+")) {
-            /* if '+', output to stderr */
-            fprintf(stderr, "VMURI: %s\n", uri);
-        } else if (0 == strncasecmp(myglobals.report_uri, "file:", strlen("file:"))) {
-            ptr = strchr(myglobals.report_uri, ':');
-            ++ptr;
-            fp = fopen(ptr, "w");
-            if (NULL == fp) {
-                orte_show_help("help-orterun.txt", "orterun:write_file", false,
-                               orte_basename, "pid", ptr);
-                exit(0);
-            }
-            fprintf(fp, "%s\n", uri);
-            fclose(fp);
-        } else {
-            fp = fopen(myglobals.report_uri, "w");
-            if (NULL == fp) {
-                orte_show_help("help-orterun.txt", "orterun:write_file", false,
-                               orte_basename, "pid", myglobals.report_uri);
-                exit(0);
-            }
-            fprintf(fp, "%s\n", uri);
-            fclose(fp);
-        }
-        free(uri);
-    } else {
-        printf("VMURI: %s\n", uri);
-    }
-
-    /* get the daemon job object - was created by ess/hnp component */
+     /* get the daemon job object - was created by ess/hnp component */
     if (NULL == (jdata = orte_get_job_data_object(ORTE_PROC_MY_NAME->jobid))) {
         orte_show_help("help-orterun.txt", "bad-job-object", true,
                        orte_basename);
@@ -415,15 +462,6 @@ int main(int argc, char *argv[])
     orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_DAEMON,
                             ORTE_RML_PERSISTENT, orte_daemon_recv, NULL);
 
-    /* override the notify_completed state so we can send a message
-     * back to anyone who submits a job to us telling them the job
-     * completed */
-    if (ORTE_SUCCESS != (rc = orte_state.set_job_state_callback(ORTE_JOB_STATE_NOTIFY_COMPLETED, notify_requestor))) {
-        ORTE_ERROR_LOG(rc);
-        ORTE_UPDATE_EXIT_STATUS(rc);
-        exit(orte_exit_status);
-    }
-
     /* spawn the DVM - we skip the initial steps as this
      * isn't a user-level application */
     ORTE_ACTIVATE_JOB_STATE(jdata, ORTE_JOB_STATE_ALLOCATE);
@@ -432,6 +470,7 @@ int main(int argc, char *argv[])
     while (orte_event_base_active) {
         opal_event_loop(orte_event_base, OPAL_EVLOOP_ONCE);
     }
+    ORTE_ACQUIRE_OBJECT(orte_event_base_active);
 
     /* cleanup and leave */
     orte_finalize();
@@ -441,41 +480,3 @@ int main(int argc, char *argv[])
     }
     exit(orte_exit_status);
 }
-
-static void send_callback(int status, orte_process_name_t *peer,
-                          opal_buffer_t* buffer, orte_rml_tag_t tag,
-                          void* cbdata)
-
-{
-    orte_job_t *jdata = (orte_job_t*)cbdata;
-
-    OBJ_RELEASE(buffer);
-    /* cleanup the job object */
-    opal_pointer_array_set_item(orte_job_data, ORTE_LOCAL_JOBID(jdata->jobid), NULL);
-    OBJ_RELEASE(jdata);
-}
-static void notify_requestor(int sd, short args, void *cbdata)
-{
-    orte_state_caddy_t *caddy = (orte_state_caddy_t*)cbdata;
-    orte_job_t *jdata = caddy->jdata;
-    orte_proc_t *pptr;
-    int ret;
-    opal_buffer_t *reply;
-
-    /* notify the requestor */
-    reply = OBJ_NEW(opal_buffer_t);
-    /* see if there was any problem */
-    if (orte_get_attribute(&jdata->attributes, ORTE_JOB_ABORTED_PROC, (void**)&pptr, OPAL_PTR) && NULL != pptr) {
-        ret = pptr->exit_code;
-    } else {
-        ret = 0;
-    }
-    opal_dss.pack(reply, &ret, 1, OPAL_INT);
-    orte_rml.send_buffer_nb(&jdata->originator, reply, ORTE_RML_TAG_TOOL, send_callback, jdata);
-
-    /* we cannot cleanup the job object as we might
-     * hit an error during transmission, so clean it
-     * up in the send callback */
-    OBJ_RELEASE(caddy);
-}
-
