@@ -43,6 +43,9 @@
 #define SPML_UCX_PUT_DEBUG    0
 #endif
 
+static
+spml_ucx_mkey_t * mca_spml_ucx_get_mkey_slow(int pe, void *va, void **rva);
+
 mca_spml_ucx_t mca_spml_ucx = {
     {
         /* Init mca_spml_base_module_t */
@@ -64,6 +67,7 @@ mca_spml_ucx_t mca_spml_ucx = {
                                every spml */
         mca_spml_ucx_rmkey_unpack,
         mca_spml_ucx_rmkey_free,
+        mca_spml_ucx_rmkey_ptr,
         mca_spml_ucx_memuse_hook,
         (void*)&mca_spml_ucx
     },
@@ -73,7 +77,9 @@ mca_spml_ucx_t mca_spml_ucx = {
     NULL,   /* ucp_peers */
     0,      /* using_mem_hooks */
     1,      /* num_disconnect */
-    0       /* heap_reg_nb */
+    0,      /* heap_reg_nb */
+    0,      /* enabled */
+    mca_spml_ucx_get_mkey_slow
 };
 
 int mca_spml_ucx_enable(bool enable)
@@ -165,6 +171,7 @@ int mca_spml_ucx_del_procs(ompi_proc_t** procs, size_t nprocs)
     mca_spml_ucx_waitall(dreqs, &num_reqs);
     free(dreqs);
 
+    opal_pmix.fence(NULL, 0);
     free(mca_spml_ucx.ucp_peers);
     return OSHMEM_SUCCESS;
 }
@@ -244,7 +251,7 @@ static void dump_address(int pe, char *addr, size_t len)
 
 static char spml_ucx_transport_ids[1] = { 0 };
 
-int mca_spml_ucx_add_procs(ompi_proc_t **procs, size_t nprocs)
+int mca_spml_ucx_add_procs(ompi_proc_t** procs, size_t nprocs)
 {
     size_t i, n;
     int rc = OSHMEM_ERROR;
@@ -269,7 +276,7 @@ int mca_spml_ucx_add_procs(ompi_proc_t **procs, size_t nprocs)
     dump_address(my_rank, (char *)wk_local_addr, wk_addr_len);
 
     rc = oshmem_shmem_xchng(wk_local_addr, wk_addr_len, nprocs,
-            (void **)&wk_raddrs, &wk_roffs, &wk_rsizes);
+                            (void **)&wk_raddrs, &wk_roffs, &wk_rsizes);
     if (rc != OSHMEM_SUCCESS) {
         goto error;
     }
@@ -284,13 +291,14 @@ int mca_spml_ucx_add_procs(ompi_proc_t **procs, size_t nprocs)
         ep_params.field_mask = UCP_EP_PARAM_FIELD_REMOTE_ADDRESS;
         ep_params.address    = (ucp_address_t *)(wk_raddrs + wk_roffs[i]);
 
-        err = ucp_ep_create(mca_spml_ucx.ucp_worker, 
-                            &ep_params,
+        err = ucp_ep_create(mca_spml_ucx.ucp_worker, &ep_params,
                             &mca_spml_ucx.ucp_peers[i].ucp_conn);
         if (UCS_OK != err) {
-            SPML_ERROR("ucp_ep_create failed: %s", ucs_status_string(err));
+            SPML_ERROR("ucp_ep_create(proc=%d/%d) failed: %s", n, nprocs,
+                       ucs_status_string(err));
             goto error2;
         }
+
         OSHMEM_PROC_DATA(procs[i])->num_transports = 1;
         OSHMEM_PROC_DATA(procs[i])->transport_ids = spml_ucx_transport_ids;
     }
@@ -327,6 +335,7 @@ error:
 }
 
 
+static
 spml_ucx_mkey_t * mca_spml_ucx_get_mkey_slow(int pe, void *va, void **rva)
 {
     sshmem_mkey_t *r_mkey;
@@ -352,6 +361,23 @@ void mca_spml_ucx_rmkey_free(sshmem_mkey_t *mkey)
     ucp_rkey_destroy(ucx_mkey->rkey);
 }
 
+void *mca_spml_ucx_rmkey_ptr(const void *dst_addr, sshmem_mkey_t *mkey, int pe)
+{
+#if (((UCP_API_MAJOR >= 1) && (UCP_API_MINOR >= 3)) || (UCP_API_MAJOR >= 2))
+    void *rva;
+    ucs_status_t err;
+    spml_ucx_mkey_t *ucx_mkey = (spml_ucx_mkey_t *)(mkey->spml_context);
+
+    err = ucp_rkey_ptr(ucx_mkey->rkey, (uint64_t)dst_addr, &rva);
+    if (UCS_OK != err) {
+        return NULL;
+    }
+    return rva;
+#else
+    return NULL;
+#endif
+}
+
 static void mca_spml_ucx_cache_mkey(sshmem_mkey_t *mkey, uint32_t segno, int dst_pe)
 {
     ucp_peer_t *peer;
@@ -419,13 +445,14 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
                                          int *count)
 {
     sshmem_mkey_t *mkeys;
-    ucs_status_t err;
+    ucs_status_t status;
     spml_ucx_mkey_t   *ucx_mkey;
     size_t len;
-    int my_pe = oshmem_my_proc_id();
     ucp_mem_map_params_t mem_map_params;
-    int seg;
+    int segno;
+    map_segment_t *mem_seg;
     unsigned flags;
+    int my_pe = oshmem_my_proc_id();
 
     *count = 0;
     mkeys = (sshmem_mkey_t *) calloc(1, sizeof(*mkeys));
@@ -433,31 +460,38 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
         return NULL;
     }
 
-    seg = memheap_find_segnum(addr);
+    segno   = memheap_find_segnum(addr);
+    mem_seg = memheap_find_seg(segno);
 
-    ucx_mkey = &mca_spml_ucx.ucp_peers[my_pe].mkeys[seg].key;
+    ucx_mkey = &mca_spml_ucx.ucp_peers[my_pe].mkeys[segno].key;
     mkeys[0].spml_context = ucx_mkey;
 
-    flags = 0;
-    if (mca_spml_ucx.heap_reg_nb && memheap_is_va_in_segment(addr, HEAP_SEG_INDEX)) {
-        flags = UCP_MEM_MAP_NONBLOCK;
+    /* if possible use mem handle already created by ucx allocator */
+    if (MAP_SEGMENT_ALLOC_UCX != mem_seg->type) {
+        flags = 0;
+        if (mca_spml_ucx.heap_reg_nb && memheap_is_va_in_segment(addr, HEAP_SEG_INDEX)) {
+            flags = UCP_MEM_MAP_NONBLOCK;
+        }
+
+        mem_map_params.field_mask = UCP_MEM_MAP_PARAM_FIELD_ADDRESS |
+                                    UCP_MEM_MAP_PARAM_FIELD_LENGTH |
+                                    UCP_MEM_MAP_PARAM_FIELD_FLAGS;
+        mem_map_params.address    = addr;
+        mem_map_params.length     = size;
+        mem_map_params.flags      = flags;
+
+        status = ucp_mem_map(mca_spml_ucx.ucp_context, &mem_map_params, &ucx_mkey->mem_h);
+        if (UCS_OK != status) {
+            goto error_out;
+        }
+
+    } else {
+        ucx_mkey->mem_h = (ucp_mem_h)mem_seg->context;
     }
 
-    mem_map_params.field_mask = UCP_MEM_MAP_PARAM_FIELD_ADDRESS |
-                                UCP_MEM_MAP_PARAM_FIELD_LENGTH |
-                                UCP_MEM_MAP_PARAM_FIELD_FLAGS;
-    mem_map_params.address    = addr;
-    mem_map_params.length     = size;
-    mem_map_params.flags      = flags;
-
-    err = ucp_mem_map(mca_spml_ucx.ucp_context, &mem_map_params, &ucx_mkey->mem_h);
-    if (UCS_OK != err) {
-        goto error_out;
-    }
-
-    err = ucp_rkey_pack(mca_spml_ucx.ucp_context, ucx_mkey->mem_h, 
-            &mkeys[0].u.data, &len); 
-    if (UCS_OK != err) {
+    status = ucp_rkey_pack(mca_spml_ucx.ucp_context, ucx_mkey->mem_h, 
+                           &mkeys[0].u.data, &len); 
+    if (UCS_OK != status) {
         goto error_unmap;
     }
     if (len >= 0xffff) {
@@ -467,18 +501,18 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
         oshmem_shmem_abort(-1);
     }
 
-    err = ucp_ep_rkey_unpack(mca_spml_ucx.ucp_peers[oshmem_group_self->my_pe].ucp_conn,
-                             mkeys[0].u.data,
-                             &ucx_mkey->rkey);
-    if (UCS_OK != err) {
+    status = ucp_ep_rkey_unpack(mca_spml_ucx.ucp_peers[oshmem_group_self->my_pe].ucp_conn,
+                                mkeys[0].u.data,
+                                &ucx_mkey->rkey);
+    if (UCS_OK != status) {
         SPML_ERROR("failed to unpack rkey");
         goto error_unmap;
     }
 
     mkeys[0].len     = len;
-    mkeys[0].va_base = mem_map_params.address;
+    mkeys[0].va_base = addr;
     *count = 1;
-    mca_spml_ucx_cache_mkey(&mkeys[0], seg, my_pe);
+    mca_spml_ucx_cache_mkey(&mkeys[0], segno, my_pe);
     return mkeys;
 
 error_unmap:
@@ -492,6 +526,7 @@ error_out:
 int mca_spml_ucx_deregister(sshmem_mkey_t *mkeys)
 {
     spml_ucx_mkey_t   *ucx_mkey;
+    map_segment_t *mem_seg;
 
     MCA_SPML_CALL(fence());
     if (!mkeys)
@@ -500,8 +535,13 @@ int mca_spml_ucx_deregister(sshmem_mkey_t *mkeys)
     if (!mkeys[0].spml_context) 
         return OSHMEM_SUCCESS;
 
-    ucx_mkey = (spml_ucx_mkey_t *)mkeys[0].spml_context;
-    ucp_mem_unmap(mca_spml_ucx.ucp_context, ucx_mkey->mem_h);
+    mem_seg  = memheap_find_va(mkeys[0].va_base);
+    ucx_mkey = (spml_ucx_mkey_t*)mkeys[0].spml_context;
+    
+    if (MAP_SEGMENT_ALLOC_UCX != mem_seg->type) {
+        ucp_mem_unmap(mca_spml_ucx.ucp_context, ucx_mkey->mem_h);
+    }
+    ucp_rkey_destroy(ucx_mkey->rkey);
 
     if (0 < mkeys[0].len) {
         ucp_rkey_buffer_release(mkeys[0].u.data);
@@ -516,7 +556,7 @@ int mca_spml_ucx_get(void *src_addr, size_t size, void *dst_addr, int src)
     ucs_status_t status;
     spml_ucx_mkey_t *ucx_mkey;
 
-    ucx_mkey = mca_spml_ucx_get_mkey(src, src_addr, &rva);
+    ucx_mkey = mca_spml_ucx_get_mkey(src, src_addr, &rva, &mca_spml_ucx);
     status = ucp_get(mca_spml_ucx.ucp_peers[src].ucp_conn, dst_addr, size,
                      (uint64_t)rva, ucx_mkey->rkey);
 
@@ -529,7 +569,7 @@ int mca_spml_ucx_get_nb(void *src_addr, size_t size, void *dst_addr, int src, vo
     ucs_status_t status;
     spml_ucx_mkey_t *ucx_mkey;
 
-    ucx_mkey = mca_spml_ucx_get_mkey(src, src_addr, &rva);
+    ucx_mkey = mca_spml_ucx_get_mkey(src, src_addr, &rva, &mca_spml_ucx);
     status = ucp_get_nbi(mca_spml_ucx.ucp_peers[src].ucp_conn, dst_addr, size,
                      (uint64_t)rva, ucx_mkey->rkey);
 
@@ -542,7 +582,7 @@ int mca_spml_ucx_put(void* dst_addr, size_t size, void* src_addr, int dst)
     ucs_status_t status;
     spml_ucx_mkey_t *ucx_mkey;
 
-    ucx_mkey = mca_spml_ucx_get_mkey(dst, dst_addr, &rva);
+    ucx_mkey = mca_spml_ucx_get_mkey(dst, dst_addr, &rva, &mca_spml_ucx);
     status = ucp_put(mca_spml_ucx.ucp_peers[dst].ucp_conn, src_addr, size,
                      (uint64_t)rva, ucx_mkey->rkey);
 
@@ -555,7 +595,7 @@ int mca_spml_ucx_put_nb(void* dst_addr, size_t size, void* src_addr, int dst, vo
     ucs_status_t status;
     spml_ucx_mkey_t *ucx_mkey;
 
-    ucx_mkey = mca_spml_ucx_get_mkey(dst, dst_addr, &rva);
+    ucx_mkey = mca_spml_ucx_get_mkey(dst, dst_addr, &rva, &mca_spml_ucx);
     status = ucp_put_nbi(mca_spml_ucx.ucp_peers[dst].ucp_conn, src_addr, size,
                      (uint64_t)rva, ucx_mkey->rkey);
 
@@ -581,7 +621,7 @@ int mca_spml_ucx_quiet(void)
 
     err = ucp_worker_flush(mca_spml_ucx.ucp_worker);
     if (UCS_OK != err) {
-         SPML_ERROR("fence failed: %s", ucs_status_string(err));
+         SPML_ERROR("quiet failed: %s", ucs_status_string(err));
          oshmem_shmem_abort(-1);
          return OSHMEM_ERROR;
     }
